/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Arrays;

import cz.cuni.mff.been.benchmarkmanager.gui.SchedulerInfo;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;

public class AnalysisAdapter extends UnicastRemoteObject 
		implements AnalysisEntityInterface {

	transient private final AnalysisManager manager;

	private static AnalysisAdapter analysisAdapter; 

	AnalysisAdapter(AnalysisManager manager) throws RemoteException {
		super();
		this.manager = manager;
	}
	
	static AnalysisEntityInterface getInstance(AnalysisManager manager) 
	throws RemoteException {
		if (analysisAdapter == null) {
			analysisAdapter = new AnalysisAdapter(manager);
		}

		return analysisAdapter;
	}
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.AnalysisAdapterInterface#getEntity(cz.cuni.mff.been.common.id.AID)
	 */
	public AnalysisEntity getEntity(AID id) 
	throws RemoteException {
		return EntityFactory.createEntity(findAnalysis(id));
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.AnalysisAdapterInterface#getEntities()
	 */
	public AnalysisEntity[] getEntities() throws RemoteException {
		AnalysisMetadata[] values = manager.listAnalyses();
		AnalysisEntity[] entities = new AnalysisEntity[values.length];
		int i=0;
		for (AnalysisMetadata analysis : values) {
			entities[i++] = EntityFactory.createEntity(analysis);
		}
		return entities;
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.AnalysisAdapterInterface#getSubentityInterface(cz.cuni.mff.been.common.id.AID)
	 */
	public ExperimentEntityInterface getSubentityInterface(AID id) 
	throws RemoteException {
		return new ExperimentAdapter(findAnalysis(id));
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.AnalysisAdapterInterface#deleteEntity(cz.cuni.mff.been.common.id.AID)
	 */
	public void deleteEntity(AID id) throws RemoteException, BenchmarkManagerException{
		manager.deleteAnalysis(id);
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.AnalysisAdapterInterface#getTasks(cz.cuni.mff.been.common.id.AID)
	 */
	public TaskDescriptor[] getAllTasks(AID id) 
	throws RemoteException {
		ExperimentEntityInterface expInterface = getSubentityInterface(id);
		ArrayList<TaskDescriptor> result = new ArrayList<TaskDescriptor>();
		for (ExperimentEntity entity : expInterface.getEntities()) {
			result.addAll(Arrays.asList(expInterface.getAllTasks(entity.getId())));
		}
		
		return (TaskDescriptor[]) result.toArray(new TaskDescriptor[result.size()]);
	}

	/**
	 * On analysis level this is the same as {@link #getAllTasks(AID)}
	 */
	public TaskDescriptor[] getTasks(AID id)
	throws RemoteException {
		return getAllTasks(id);
	}
	
	
	
	/**
	 * Reschedules given analysis
	 * @param analysis	analysis to reschedule
	 * @param newSchedule	new scheduling
	 * 
	 * @throws RemoteException when something in RMI goes bad
	 * @throws IllegalArgumentException when AID is invalid or scheduler time is overdue
	 */
	public void reschedule(AID analysis, SchedulerInfo newSchedule) 
	throws RemoteException, IllegalArgumentException, BenchmarkManagerException {
		manager.reschedule(analysis, newSchedule);
	}
	
	private AnalysisMetadata findAnalysis(AID id) 
	throws IllegalArgumentException {
		AnalysisMetadata analysis = manager.getAnalysis(id);
		if (analysis == null) {
			throw new IllegalArgumentException("Analysis with given ID(" + id + "does not exist");
		}
		
		return analysis;
	}
	
}
