/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;

import cz.cuni.mff.been.common.IdentifiableList;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.EID;

/**
 * Class containing information about specific benchmark analysis.
 * 
 * @author Michal Tomcanyi
 */
public class AnalysisMetadata implements Serializable, Identifiable<AID> {
	
	
	/** Analysis identifier. */
	private final AID id;
	/** Analysis name. */
	private String name;
	/** Analysis description. */
	private String description;
    
	/** List of experiments in the analysis	*/
	/*
	 * Transient, because we don't want to lose time every time when analysis metadata is saved
	 * In restore the list will be constructed from experiment metadata saved under the analysis
	 */
	transient private IdentifiableList<EID> experiments = new IdentifiableList<EID>();
	
	/**
	 * Allocates a new <code>AnalysisMetadata</code> object.
	 */
	public AnalysisMetadata(AID id) {
		this.id = id;
	}
	
	/** @return Analysis identifier */
	public AID getID() {
		return id;
	}

	/** @return Analysis description */
	public String getDescription() {
		return description;
	}

	/** @return Human-readable name of analysis*/
	public String getName() {
		return name;
	}
	
	/** Sets user defined description of the analysis */
	public void setDescription(String description) {
		this.description = description;
	}

	/** Sets human-readable name for the analysis */
	public void setName(String name) {
		assert name != null && name.length() > 0 : "Analysis name must be non-empty";
		this.name = name;
	}

	@Override
	public String toString() {
		return "AnalysisMetadata(" + id +	"," + name + ")";
	}

	/** @return false */
	public boolean isRegression() {
		return false;
	}

	public void addExperiment(ExperimentMetadata experiment) {
		experiments.add(experiment);
	}
	
	public void removeExperiment(ExperimentMetadata experiment) {
		experiments.remove(experiment);
	}
	
	public ExperimentMetadata[] getExperiments() {
		return (ExperimentMetadata[]) experiments.toArray(new ExperimentMetadata[experiments.size()]);
	}
	
	/**
	 * Searches for an experiment with given id
	 * @param eid	id to search for
	 * @return	experiment with given ID or <code>null</code> when no found
	 */
	public ExperimentMetadata getExperiment(EID eid) {
		return (ExperimentMetadata) experiments.findByID(eid);
	}
	
	/**
	 * @return status of analysis
	 */
	public EntityStatus getStatus() {
		for (ExperimentMetadata experiment : getExperiments()) {
			if (experiment.getStatus(experiment.getID()) == EntityStatus.RUNNING) {
				return EntityStatus.RUNNING;
			}
		}
		
		return EntityStatus.FINISHED;
	}
	
	private void readObject(ObjectInputStream is) throws IOException, ClassNotFoundException  {
		is.defaultReadObject();
		this.experiments = new IdentifiableList<EID>();
	}

}
