/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.InetAddress;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Level;
import org.java.plugin.util.ExtendedProperties;

import cz.cuni.mff.been.benchmarkmanager.gui.BenchmarkManagerGUIInterface;
import cz.cuni.mff.been.benchmarkmanager.gui.Role;
import cz.cuni.mff.been.benchmarkmanager.gui.SchedulerInfo;
import cz.cuni.mff.been.benchmarkmanager.gui.Screen;
import cz.cuni.mff.been.common.BeenException;
import cz.cuni.mff.been.common.BeenUtils;
import cz.cuni.mff.been.common.ComponentInitializationException;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.UploadThread;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BRID;
import cz.cuni.mff.been.common.id.IDManagerInterface;
import cz.cuni.mff.been.common.id.OID;
import cz.cuni.mff.been.common.id.PEID;
import cz.cuni.mff.been.common.id.PersistentIDManager;
import cz.cuni.mff.been.hostmanager.HostManagerException;
import cz.cuni.mff.been.hostmanager.HostManagerInterface;
import cz.cuni.mff.been.hostmanager.HostManagerService;
import cz.cuni.mff.been.hostmanager.ValueNotFoundException;
import cz.cuni.mff.been.hostmanager.ValueTypeIncorrectException;
import cz.cuni.mff.been.hostruntime.TasksPortInterface;
import cz.cuni.mff.been.logging.LogLevel;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;
import cz.cuni.mff.been.task.Task;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;
import cz.cuni.mff.been.taskmanager.data.TaskState;


public final class BenchmarkManager extends UnicastRemoteObject 
							  implements BenchmarkManagerGUIInterface, BenchmarkManagerLocalInterface {
	
	/** System property name storing reference to log4j configuration file */
	static final String LOG4J_CONFIG_KEY = "log4j.configuration";
	
	/** BEEN log level to use for BM task */
	public static final String LOG_LEVEL_KEY = "cz.cuni.mff.been.benchmarkmanager.logLevel";
	
	/**
	 * Plugin uploader thread
	 * 
	 * @author Michal Tomcanyi
	 */
	private class PluginUploadThread extends UploadThread {

		public PluginUploadThread(UploadHandle handle, InetAddress ip, int port) {
			super(handle, ip, port);
		}

		@Override
		protected void processFile() {
			File tempFile = getTemporaryFile();
			try {
				pluginManager.registerPlugin(tempFile);
				setStatus(UploadStatus.ACCEPTED);
			} catch (IOException e) {
				addErrorMessage(e.getMessage());
				addErrorMessage("Check structure of plugin file and plugin descriptor");
				addErrorMessage("Plugin file:" + tempFile.getName());
				e.printStackTrace();
				setStatus(UploadStatus.REJECTED);
			} catch (BeenException e) {
				addErrorMessage(e.getMessage());
				addErrorMessage("Check structure of plugin file and plugin descriptor");
				addErrorMessage("Plugin file:" + tempFile.getName());
				e.printStackTrace();
				setStatus(UploadStatus.REJECTED);
			}
		}
		
		@Override
		protected void uploadFailed() {
			logError("Plugin upload failed: " + errorMessages.getFirst());
		}
		
		@Override
		protected String getPrefix() {
			// give some meaningful name for plugin files
			return "plugin";
		}
		
		@Override
		protected String getSuffix() {
			// need to be zip or jar so the file is detected by JPF scanner
			return ".jar";
		}

		@Override
		protected File getTempDirectory() {
			// use BM's temp directory for upload
			return new File(task.getTempDirectory());
		}
	}
	
	/**
	 * Shared instance
	 */
	private static BenchmarkManager instance = null;
	
	private JPFPluginManager pluginManager;
	private IDManagerInterface idManager;
	private AnalysisManager analysisManager;
	private PluginConfiguratorManager configManager;

	private ExtendedProperties bootProperties;
	
	/** Benchmark Manager's task. */
	private Task task;

	private Map<UploadHandle,PluginUploadThread> uploaders = 
		Collections.synchronizedMap(new HashMap<UploadHandle,PluginUploadThread>());
	
	public static BenchmarkManager getInstance() 
	throws RemoteException,ComponentInitializationException {
		if (instance == null) {
			// create shared BM instance
			instance = new BenchmarkManager();
		}

		return instance;
		
	}

	public BenchmarkManager() 
	throws RemoteException,ComponentInitializationException {
		super();
		this.task = Task.getTaskHandle();
		
		try {
			/*
			 * Read configuration file
			 */
			configure();
			
			/*
			 * Create directory from which benchmark manager can be restored 
			 */
			File restoreRootDir = new File(task.getWorkingDirectory(),"restore");
			if (!restoreRootDir.exists() && !restoreRootDir.mkdirs()) {
				throw new ComponentInitializationException("Unable to create 'restore' directory: " + restoreRootDir.getPath());
			}
			
			/*
			 * Initialize subcomponents
			 */
			this.idManager = PersistentIDManager.createInstance(restoreRootDir);
			// initialize plugin loader either according to config or to the current directory
			final String defaultPluginsDirectory = task.getTaskDirectory() + File.separator + "plugins";
			final String jpfWorkingDirectory = task.getWorkingDirectory();
			this.pluginManager = JPFPluginManager.createInstance(this, bootProperties,defaultPluginsDirectory,jpfWorkingDirectory,restoreRootDir);
			this.analysisManager = AnalysisManager.createInstance(this,restoreRootDir);
			this.configManager = new PluginConfiguratorManager(this);
		} catch (FileNotFoundException e) {
			throw new ComponentInitializationException(e);
		} catch (IOException e) {
		  throw new ComponentInitializationException(e);
		} catch (ClassNotFoundException e) {
			throw new ComponentInitializationException(e);
		}
		
	}

	/**
	 * Sets up JPF runtime, deploys default plugins and initializes plugin system
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private void configure() 
	throws FileNotFoundException, IOException  {
		final String taskDir = task.getTaskDirectory();
		
		File bootFile = new File(taskDir, "boot.properties"); 
		if (!bootFile.exists()) {
			throw new FileNotFoundException("boot.properties file missing in package!");
		}

		// create extended properties as in JPF's Boot.java
		bootProperties = new ExtendedProperties(System.getProperties());
		final FileInputStream stream = new FileInputStream(bootFile);
		bootProperties.load(stream);
		stream.close();
		
		// set log4j configuration property in system properties (as in Boot.java)
		if (System.getProperty(LOG4J_CONFIG_KEY) == null) {
			// not set, must use our log4j configuration
			File log4jconfig = new File(taskDir, "log4j.properties");
			System.setProperty(LOG4J_CONFIG_KEY, log4jconfig.toURI().toURL().toExternalForm());
		}
		
		// set log level as specified in properties or use INFO by default
		final String logLevelName = bootProperties.getProperty(LOG_LEVEL_KEY, "INFO");
		Level log4jLevel = Level.toLevel(logLevelName.trim());
		task.setLogLevel(LogLevel.getInstance(log4jLevel));
	}
 	
	public PluginsIntegrityReport getPluginsIntegrityReport() 
	throws RemoteException {
		return pluginManager.getIntegrityReport();
	}
	
	// -------------- Interface BenchmarkManagerInterface ----------------
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.gui.BenchmarkManagerInterface#getAvailableAnalysis()
	 */

	/** 
	 *  @return <code>PluginEntry</code> elements
	 */
	public BeenPluginDescriptor[] getAvailablePlugins() {
		return pluginManager.getAvailablePlugins();
	}

	public TaskDescriptor[] beginTaskGeneration(BRID brid) 
	throws RemoteException, BenchmarkManagerException {
		TaskDescriptor[] result = startGenerator(brid);
		BeenUtils.sortTasks(result);
		return result;
	}

	public AnalysisEntity createAnalysis(String name, String description, boolean isRegression) 
	throws BenchmarkManagerException, RemoteException {
		return analysisManager.createAnalysis(name, description, isRegression);
	}	
	
	/**
	 * @param analysis	idenitifer of analysis 
	 * @return	analysis metadata
	 * @throws RemoteException
	 * @throws BenchmarkManagerException when analysis with given identifier does not exist
	 */
	public AnalysisMetadata getAnalysis(AID analysis) 
	throws RemoteException, BenchmarkManagerException {
		return analysisManager.getAnalysis(analysis);
	}
	
	public Role[] getBenchmarkRoles(BRID brid) 
	throws BenchmarkManagerException, RemoteException, HostManagerException,
		ValueNotFoundException, ValueTypeIncorrectException{
		StandardRole[] allRoles = expandRoles(brid);
		List<Role>  benchRoles = new LinkedList<Role>();
		for (StandardRole role : allRoles) {
			if (role.isBenchmarking()) {
				benchRoles.add(new Role(role,new String[0]));
			}
		}
		
		return (Role[]) benchRoles.toArray(new Role[benchRoles.size()]);
	}
	
	public void setBenchmarkRoles(BRID session, Role[] roles) 
	throws RemoteException, IllegalArgumentException {
		ExperimentMetadata experiment = configManager.getConfigurator(session).getBenchmarkExperiment();
		experiment.setUserHosts(roles);
	}
	
	
	public ExperimentEntity getConfiguredExperiment(BRID brid)
	throws RemoteException, BenchmarkManagerException {
		ExperimentMetadata experiment = null;
		experiment = configManager.getConfigurator(brid).getBenchmarkExperiment();
		return new ExperimentEntity(experiment);
	}
	
	public CallbackRScripts getAnalysisCallbackRScripts(BRID brid) 
	throws RemoteException, IllegalArgumentException {
		return configManager.getConfigurator(brid).getDefaultRScripts();
	}

	public void startExperiment(BRID session, SchedulerInfo schedule, CallbackRScripts scripts) 
	throws BenchmarkManagerException, RemoteException {
		
		
		ExperimentMetadata experiment = configManager.getConfigurator(session).getBenchmarkExperiment();
		if (experiment == null) {
			throw new BenchmarkManagerException("Experiment does not exist for session " + session);
		}

		// saves callback scripts to experiment metadata
		experiment.setScripts(scripts);
		
		// scheduling model experiment for regression analysis
		if (schedule != null) {
			analysisManager.registerModelExperiment(experiment,schedule);
		} else {
			analysisManager.addExperiment(experiment);
			analysisManager.reportResultsMetadata(experiment);
			analysisManager.dispatchExperiment(experiment);
		}
		
		// configurator may be removed, because experiment was sucessfully started
		configManager.removeConfigurator(session);
	}
	
	public void runExperiment(ExperimentMetadata experiment) 
	throws BenchmarkManagerException {
		try {
			TaskDescriptor[] tasks = experiment.getAllTasks();
			TasksPortInterface tasksPort = task.getTasksPort();
			tasksPort.newContext(experiment.getContextID(), 
					experiment.getName(), 
					experiment.getDescription(), 
					new OID[] { experiment.getAnalysis(), experiment.getID() });
			tasksPort.runTasks(tasks);
			tasksPort.endContext(experiment.getContextID());
		} catch (RemoteException e) {
			throw new BenchmarkManagerException("TaskManager not available", e);
		}
	}
	
	public Screen getFirstConfiguratorScreen(BRID session, AID analysisID, 
			String experimentName, String experimentDescription, PEID pluginID) 
	throws RemoteException, BenchmarkManagerException  {
		boolean isRegression = analysisManager.getAnalysis(analysisID).isRegression();
		ExperimentMetadata experiment = 
			analysisManager.createExperiment(analysisID, experimentName, experimentDescription, pluginID);
		return configManager.getFirstScreen(session, experiment, isRegression);
	}

	public Screen getNextConfiguratorScreen(BRID session, Screen currentScreen) 
	throws RemoteException, BenchmarkManagerException {
		Screen result = configManager.getNextScreen(session, currentScreen);
		return result;
	}

	public Screen getPreviousConfiguratorScreen(BRID session, Screen currentScreen) 
	throws RemoteException, BenchmarkManagerException {
		return configManager.getPreviousScreen(session, currentScreen);
	}
	
	public AnalysisEntityInterface getAnalysisInterface() 
	throws RemoteException {
		return AnalysisAdapter.getInstance(analysisManager);
	}

	public UploadHandle beginPluginUpload(InetAddress ip, int port) throws RemoteException {
		UploadHandle handle = UploadHandle.createUploadHandle();
		PluginUploadThread uploader = new PluginUploadThread(handle,ip,port);
		uploaders.put(handle,uploader);
		uploader.start();
		return handle;
	}

	public boolean deletePlugin(String name) throws RemoteException {
		// TODO Auto-generated method stub
		return false;
	}

	public void endPluginUpload(UploadHandle handle) throws RemoteException {
		uploaders.remove(handle);
	}

	public UploadStatus getPluginUploadStatus(UploadHandle handle) throws RemoteException {
		return getUploader(handle).getStatus();
	}

	public String[] getRejectedPluginUploadErrorMessages(UploadHandle handle) throws RemoteException {
		return getUploader(handle).getErrorMesssages();
	}

	
	//------------------- Local services API ----------------
	
	public IDManagerInterface getIDManager() {
		return this.idManager;
	}
	
	public JPFPluginManager getPluginLoader() {
		return pluginManager;
	}

	/**
	 * Pre-cached lookup service managed by Benchmark Manager
	 * 
	 * @param serviceName name of service to look up
	 * @return remote interface of service
	 * @throws BenchmarkManagerException	when lookup fails, which means service or TaskManager is not available
	 * 
	 */
	public Remote lookup(String serviceName) 
	throws BenchmarkManagerException {
		Remote result;
		try {
			result = task.getTasksPort().serviceFind(serviceName,"main");
			if (result == null) {
				logError("Requested service '" + serviceName + "' is not available");
				throw new BenchmarkManagerException("Service '" + serviceName + "' is not available");
			}
			return result;
		} catch (RemoteException e) {
			logError("Service lookup failed as RMI error occured: " + e.getMessage());
			throw new BenchmarkManagerException("Service lookup failed as RMI error occured",e);
		}
	}
	
	
	/**
	 * Queries task manager for status of given task
	 * 
	 * @param taskId	task identifier
	 * @param contextId	context identifier
	 * @return	status of given task or <code>null</code> when task manager does not know given task
	 */
	public TaskState getTaskStatus(String taskId, String contextId) throws BenchmarkManagerException {
		try {
			return task.getTasksPort().getTaskById(taskId,contextId).getState();
		} catch (RemoteException e) {
			throw new BenchmarkManagerException("Can't query task status",e);
		} catch (IllegalArgumentException e) {
			return null;
		}
	}
	
	public void logDebug(String errorMessage) {
		task.logDebug(errorMessage);
	}

	public void logError(String errorMessage) {
		task.logError(errorMessage);
	}

	public void logFatal(String errorMessage) {
		task.logFatal(errorMessage);
	}

	public void logInfo(String infoMessage) {
		task.logInfo(infoMessage);
	}

	public void logTrace(String infoMessage) {
		task.logTrace(infoMessage);
	}
	
	public void logWarning(String warningMessage) {
		task.logWarning(warningMessage);
	}
	
	public LogLevel getLogLevel() {
		return task.getLogLevel();
	}
	
	
	
	// ------------------------------- Private methods ---------------------------------
	
	private TaskDescriptor[] startGenerator(BRID brid) 
	throws BenchmarkManagerException, IllegalArgumentException {
		ExperimentMetadata experiment = 
			configManager.getConfigurator(brid).getBenchmarkExperiment();
		
		analysisManager.startGenerator(experiment);
		return experiment.getAllTasks();
	}
	
	
	/*
	 * Expands RSL in host roles to a set of matching hosts
	 */
	private StandardRole[] expandRoles(BRID sessionID) 
	throws BenchmarkManagerException, RemoteException, HostManagerException {
		ExperimentMetadata exp = configManager.getConfigurator(sessionID).getBenchmarkExperiment();
		HostManagerInterface hm = (HostManagerInterface) lookup(HostManagerService.SERVICE_NAME);
		for (StandardRole role : exp.getRoles()) {
			role.expand(hm);
		}
		
		return exp.getRoles();
	}

	/**
	 * @param handle handle to uploader
	 * @return uploader for given handle
	 * @throws IllegalArgumentException	when uploader with given handle does not exist
	 */
	private PluginUploadThread getUploader(UploadHandle handle)
	throws IllegalArgumentException {
		PluginUploadThread uploader = uploaders.get(handle);
		if (uploader == null) {
			throw new IllegalArgumentException("Uploader does not exist");
		}
		return uploader;
	}
	
}
