/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import cz.cuni.mff.been.benchmarkmanager.gui.Role;
import cz.cuni.mff.been.common.IdentifiableList;
import cz.cuni.mff.been.common.Pair;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.OID;
import cz.cuni.mff.been.common.id.PEID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.common.rsl.Condition;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;

public class ExperimentMetadata implements Serializable, Identifiable<EID> {

	private EID id;
	private AID aid;
	private PEID plugin;
	private String name;
	private String description;
	private String pluginName;
	private boolean isRerun = false;
	private CallbackRScripts scripts;
	private EntityStatus status;
	
	/**
	 * Hosts of benchmark roles
	 */
	private HashMap<String,StandardRole> roles = new HashMap<String,StandardRole>();
	
	/**
	 * [name,value] pairs of custom benchmark properties
	 */
	private HashMap<String,String> properties = new HashMap<String,String>();

	/**
	 * Holds unioned package configurations of all binaries under this experiment instance
	 * The key is created from binary ident + package name
	 */
	private HashMap<String,String> packages = new HashMap<String,String>();
	
	/**
	 * 1-1 mapping of results for results repostitory
	 * First field is identifier
	 * Second field is hostname  
	 */
	private ArrayList<Pair<String,String>> resultsMapping = new ArrayList<Pair<String,String>>();
	
	
	/**
	 * Number of runs to be performed
	 * Default: 1
	 */
	private int runCount = 1;
	
	/**
	 * Number of binaries to create in each run
	 * Default: 1
	 */
	private int binaryCount = 1;
	
	/**
	 * Samples to create per single run
	 */
	private int sampleCount = 1;
	
	/**
	 * Metadata of binaries created in the experiment
	 */
	private IdentifiableList<BID> binaries = new IdentifiableList<BID>();
	
	/**
	 * Task descriptors generated for experiment
	 */
	private final TaskSequence tasks;
	
	
	public ExperimentMetadata(AID analysisID,
			EID experimentID,
			String name, 
			String description, 
			PEID pluginID,
			String pluginName) {
		assert name != null && name.length() > 0 : "Experiment name must be non-empty";
		this.aid = analysisID;
		this.id = experimentID;
		this.name = name;
		this.description = description;
		this.plugin = pluginID;
		this.pluginName = pluginName;
		this.tasks = new TaskSequence(experimentID);
		this.status = EntityStatus.CONFIGURED;
	}
	
	/**
	 * Constructor intended for creating experiments from model experiment
	 * in regression analysis.
	 * Calls {@link #ExperimentMetadata(AID, EID, String, String, PEID, String)}
	 * with values from model, only experiment identifier is overriden by
	 * value specified as parameter. Other fields are duplicated from the model.
	 * 
	 * @param experimentID	unqiue identifier of newly created experiment
	 * @param model		model experiment registered for regression analysis
	 */
	public ExperimentMetadata(EID experimentID, ExperimentMetadata model) {
		this(model.getAnalysis(),experimentID,model.getName(),
			model.getDescription(),model.getPlugin(),model.getPluginName());
		this.properties.putAll(model.properties);
		this.roles.putAll(model.roles);
		this.packages.putAll(model.packages);
		this.runCount = model.runCount;
		this.binaryCount = model.binaryCount;
		this.sampleCount = model.sampleCount;
		this.scripts = model.scripts;
	}
	
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.ExperimentMetadata#getID()
	 */
	public EID getID() {
		return id;
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getAnalysis()
	 */
	public AID getAnalysis() {
		return aid;
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getPlugin()
	 */
	public PEID getPlugin() {
		return plugin;
	}

	public String getPluginName() {
		return pluginName;
	}
	
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getDescription()
	 */
	public String getDescription() {
		return description;
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getName()
	 */
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * @return tasks stored on experiment level
	 */
	public TaskDescriptor[] getTasks() {
		return tasks.getTasks(getID());
	}
	
	
	/**
	 * @param id	identifier of entity
	 * @return	tasks stored under given entity or empty array if entity was not found or has no tasks
	 */
	public TaskDescriptor[] getTasks(OID id) {
		return tasks.getTasks(id);
	}
	
	/**
	 * @return	all tasks of experiment (recursively)
	 */
	public TaskDescriptor[] getAllTasks() {
		return tasks.getAllTasks();
	}
	
	/**
	 * @param id	identifier of entity
	 * @return	task from entity and its children or empty array is entity has no tasks or was not found
	 */
	public TaskDescriptor[] getAllTasks(OID id) {
		return tasks.getAllTasks(id);
	}
	
	
	/**
	 * @return TaskSequence object holding all descriptors for the experiment
	 */
	public TaskSequence getTaskSequence() {
		return tasks;
	}
	

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getBinaryCount()
	 */
	public int getBinaryCount() {
		return binaryCount;
	}

	public void setBinaryCount(int binaryCount) {
		this.binaryCount = binaryCount;
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getRunCount()
	 */
	public int getRunCount() {
		return runCount;
	}

	/**
	 * Sets number of benchmark runs to create in next generator run
	 * @param runCount
	 */
	public void setRunCount(int runCount) {
		this.runCount = runCount;
	}
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getSamplesCount()
	 */
	public int getSamplesCount() {
		return sampleCount;
	}
	
	public void setSamplesCount(int samplesCount) {
		this.sampleCount = samplesCount;
	}
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getRoleNames()
	 */
	public String[] getRoleNames() {
		return roles.keySet().toArray(new String[roles.size()]); 
	}
	
	/**
	 * @return Indication whether rerun is requested from generator
	 */
	public boolean isRerun() {
		return isRerun;
	}

	/**
	 * Sets given property
	 * @param name	property name
	 * @param value	propery value
	 */
	public void setProperty(String name, String value) {
		properties.put(name,value);
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getProperty(java.lang.String)
	 */
	public String getProperty(String name) {
		return this.properties.get(name);
	}
	
	/**
	 * @return names of all benchmark properties.
	 */
	public String[] getPropertyNames() {
		return (String[]) this.properties.keySet().toArray(
				new String[this.properties.keySet().size()]);
	}
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.BenchmarkExperiment#getProperty(java.lang.String, java.lang.String)
	 */
	public String getProperty(String name, String defaultValue) {
		String val = this.properties.get(name);
		return val != null ? val : defaultValue;
	}

	/**
	 * @return metadata of created registered in experiment
	 */
	public BinaryMetadata[] getBinaries() {
		return (BinaryMetadata[]) binaries.toArray(new BinaryMetadata[binaries.size()]);
	}
	
	/**
	 * 
	 * @param id 	ID of binary to search for
	 * @return	binary with given ID or <code>null</code> when no match
	 */
	public BinaryMetadata getBinary(BID id) {
		return (BinaryMetadata)binaries.findByID(id);
	}
	
	
	/**
	 * @return List of benchmark roles of this experiment
	 */
	public StandardRole[] getRoles() {
		return roles.values().toArray(new StandardRole[roles.size()]);
	}
	
	/**
	 * @param name name of the role
	 * @return role metadata
	 */
	public StandardRole getRole(String name) {
		return roles.get(name);
	}
	
	/**
	 * Adds new <b>non-benchmarking role</b>role to experiment
	 * @param roleName name of role
	 * @param rsl RSL-based specification of host for this role
	 * @param count expected count
	 * @see StandardRole
	 * @see #addBenchmarkingRole(String, Condition, int)
	 */
	public void addRole(String roleName, Condition rsl, int count) {
		createRole(roleName,rsl,false,count);
	}
	
	/**
	 * Adds new <b>benchmarking role</b>role to experiment
	 * @param roleName name of role
	 * @param rsl RSL-based specification of host for this role
	 * @param count expected number of hosts for the role
	 * @see StandardRole
	 * @see #addRole(String, Condition, int)
	 */
	public void addBenchmarkingRole(String roleName, Condition rsl, int count) {
		createRole(roleName,rsl,true,count);
	}
	
	/**
	 * @return Context identifier for this experiment
	 */
	public String getContextID() {
		return  getAnalysis() + "-" + getID();
	}
	

	/**
	 * Counts status of entity with given ID
	 * Works all levels of entity hierarchy
	 * @param id	identifier of given entity
	 * @return	status of given entity
	 */
	public EntityStatus getStatus(OID id) {
		/*
		 * In case we are experiment with configured status,
		 * we have no tasks/children created yet, because
		 * the generator was not started.
		 * When node has no children/tasks, the algoritm returns
		 * "finish" status, which is not correct in this case, so
		 * we must prevent calling getStatus() algorithm.
		 * 
		 * When experiment is dispatched, it has generated tasks, but was not yet
		 * submitted, so task manager returns null for them and status
		 * algorithm assumes they are finished, which is again not correct.
		 * 
		 * Once the experiment is finished, it can't change 
		 * its state again, so we don't need check our status. 
		 */
		synchronized (tasks) {
			switch (status) {
				case CONFIGURED: // must not call status algorithm
				case DISPATCHED: // must not call status algorithm
				case FINISHED:   // need not call status algorithm
						return status;
				default:
						EntityStatus countedStatus = getTaskSequence().getStatus(id); 	
						if (getID().equals(id)) {
							this.status = countedStatus;
						}
						return countedStatus;
			}
		}
		
	}
	
	/**
	 * @return status of whole experiment.
	 */
	public EntityStatus getStatus() {
		return getStatus(getID());
	}
	

	/**
	 * Sets users-specified hosts for benchmarking roles
	 * @param roles	roles with preselected hosts
	 */
	public void setUserHosts(Role[] roles) {
		for (Role role : roles) {
			StandardRole expRole = getRole(role.getName());
			
			if (expRole.isBenchmarking()) {
				((BenchmarkRole)expRole).setUserHosts(role.getSelectedHosts());
			}
		}
	}
	
	/**
	 * Sets ResultsRepository callback scripts
	 * @param scripts	scripts to set
	 */
	public void setScripts(CallbackRScripts scripts) {
		this.scripts = scripts;
	}
	
	/**
	 * @return ResultsRepository callback scripts
	 */
	public CallbackRScripts getScripts() {
		return scripts;
	}
	
	/**
	 * Sets experiment and all its subentities to given status
	 * @param status
	 */
	void setStatus(EntityStatus status) {
		synchronized (tasks) {
	 		this.status = status;
			tasks.setGlobalStatus(status);
		}
	}
	
	/**
	 * Method is used to store binary packages relations in the experiment
	 * @param binary	relevant binary
	 * @param packageName	name of software package
	 * @param metadataFlag	identifier marking the relation
	 */
	void addPackage(BinaryMetadata binary, String packageName, String metadataFlag) {
		packages.put(binary.getID() + packageName, metadataFlag);
	}
	
	/**
	 * Obtains binary package relation flag 
	 * @param binary	relevant binary
	 * @param packageName	name of package
	 * @return Binary relation flag
	 */
	String getPackage(BinaryMetadata binary, String packageName) {
		return packages.get(binary.getID() + packageName);
	}
	
	/**
	 * @return Identifiers of binaries created in this experiment
	 */
	BID[] getBinaryIds() {
		return getTaskSequence().getBinaries();
	}
	
	/**
	 * @return Identifiers of runs created in this experiment
	 */
	RID[] getRunIds() {
		return getTaskSequence().getRuns();
	}
	
	public void addRoleMapping(String role, String host) {
		resultsMapping.add(new Pair<String,String>(role,host));
	}
	
	public List<Pair<String, String>> getResultsMapping() {
		return Collections.unmodifiableList(resultsMapping);
	}
	
	/*
	 * Adds new role into experiment metadata
	 */
	private void createRole(String roleName, Condition rsl, boolean isBenchmarking, int count) {
		StandardRole role = isBenchmarking ?
						new BenchmarkRole(roleName, rsl, count) :
						new StandardRole(roleName,rsl, count);
		roles.put(role.getName(),role);
	}

}
