/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.rmi.RemoteException;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Map.Entry;

import cz.cuni.mff.been.benchmarkmanager.gui.Screen;
import cz.cuni.mff.been.benchmarkmanager.plugins.Configurator;
import cz.cuni.mff.been.benchmarkmanager.plugins.ConfiguratorException;
import cz.cuni.mff.been.common.WorkQueue;
import cz.cuni.mff.been.common.id.BRID;

public class PluginConfiguratorManager {

	private enum NavigationDirection {
		FIRST, NEXT, PREVIOUS
	}

	/**
	 * Timeout in miliseconds after which configurators are forgotten ... 15 minutes
	 */
	private static final long CONFIGURATOR_TIMEOUT = 15*60*1000;  
	
	private class ConfiguratorEntry {
		private final Configurator configurator;
		private final long createdTimestamp = System.currentTimeMillis();
		
		public ConfiguratorEntry(Configurator c) {
			this.configurator = c;
		}
		
		public Configurator getConfigurator() {
			return configurator;
		}
		
		public long getTimestamp() {
			return createdTimestamp;
		}
	}
	
	private class BlockingResultMap {
		private HashMap<BRID, ConfiguratorWork> processed = new HashMap<BRID, ConfiguratorWork>();
		
		public void putResult(BRID key, ConfiguratorWork result) {
			processed.put(key,result);
			processed.notify();
		}
		/**
		 * Method retrieves (and removes) work from the set. Method is blocking until a result for given
		 * key appears. Returns null if given thread has been interrupted
		 * @param key session for which the result is stored
		 * @return	Work object processed by one step of configurator or <code>null</code> if the calling
		 * thread has been interrupted
		 */
		public synchronized ConfiguratorWork getResult(BRID key) {
			while (true) {
				ConfiguratorWork result = processed.remove(key);
				if (result != null) {
					return result;
				}
			
				try {
					wait();
				} catch (InterruptedException e) {
					return null;
				}
			}
			
		}
		
		/**
		 * Sets result under given key. Notifies all sleeping threads on the set
		 * @param key	Key under which add new piece of results
		 * @param result	The results
		 */
		public synchronized void setResult(ConfiguratorWork result) {
			processed.put(result.brid, result);
			notify();
		}
		
	}
	
	private class ConfiguratorWork {
		public final BRID brid;
		public final Screen processedScreen;
		public final Configurator configurator;
		private Screen newScreen;
		private NavigationDirection type;
		private BenchmarkManagerException exception;
		
		public ConfiguratorWork(BRID brid, Screen processed, NavigationDirection type, Configurator config) {
			this.brid = brid;
			this.processedScreen = processed;
			this.type = type;
			this.configurator = config;
		}

		public void setNewScreen(Screen newScreen) {
			this.newScreen = newScreen;
		}
		
		public Screen getNewScreen() {
			return newScreen;
		}
		
		public BenchmarkManagerException getException() {
			return exception;
		}
		
		public void setException(BenchmarkManagerException exception) {
			this.exception = exception;
		}
		
		public boolean isValid() {
			return exception == null;
		}
	}
	
	private class ConfiguratorThread extends Thread {
		private final WorkQueue<ConfiguratorWork> queue;
		private final BlockingResultMap  results;
		public ConfiguratorThread(WorkQueue<ConfiguratorWork> queue, BlockingResultMap results, int counter) {
			super("ConfiguratorThread-" + counter);
			this.queue = queue;
			this.results = results;
			setDaemon(true);
			start();
		}
		
		public void run() {
			while (true) {
				ConfiguratorWork work = queue.getWork();
				Configurator configurator = work.configurator;
				// return when interrupted
				if (isInterrupted()) {
					return;
				}
				try {
					switch(work.type) {
						case FIRST:
								work.setNewScreen(configurator.getFirstScreen());
								break;
						case NEXT:
								work.setNewScreen(configurator.getNextScreen(work.processedScreen));
								break;
						case PREVIOUS:
								work.setNewScreen(configurator.getPreviousScreen(work.processedScreen));
								break;
					}
				} catch (Exception e) {
					// need to check all errors in configurator, otherwise they are not displayed in gui
					// and it skips to review screen instead as null is returned 
					work.setException(new ConfiguratorException(e));
					bm.logError("Error occured in configurator: " + e.getMessage());
					e.printStackTrace();
				} finally {
					results.setResult(work);
				}
			}			
		}
	}

	private class ConfiguratorSweeper extends Thread {
		private final LinkedList<BRID> idsToRemove = new LinkedList<BRID>();
		
		public ConfiguratorSweeper() {
			setDaemon(true);
			start();
		}
		
		@Override
		public void run() {
			while (true) {
				long currentTime = System.currentTimeMillis();
				idsToRemove.clear();
				for (Entry<BRID,ConfiguratorEntry> entry : configurators.entrySet()) {
					ConfiguratorEntry conf = entry.getValue();
					if (currentTime - conf.getTimestamp() > CONFIGURATOR_TIMEOUT) {
						idsToRemove.add(entry.getKey());
					}
				}
				
				for (BRID key : idsToRemove) {
					bm.logDebug("Removing configurator BRID(" + key + ")");
					removeConfigurator(key);
				}
				
				try {
					sleep(CONFIGURATOR_TIMEOUT);
				} catch (InterruptedException e) {
					// ignored
				}
			}

		}
	}
	
	private final BenchmarkManager bm;
	/*
	 * Synchronization needed:
	 * 1. Multiple RMI calls (from GUI) may be requesting adding new configurator
	 * 2. After adding, some of them may be removing old entries 
	 */
	private Map<BRID, ConfiguratorEntry> configurators = Collections.synchronizedMap(new HashMap<BRID, ConfiguratorEntry>());
	private WorkQueue<ConfiguratorWork> workQueue = new WorkQueue<ConfiguratorWork>();
	private BlockingResultMap results = new BlockingResultMap();
	
	public PluginConfiguratorManager(BenchmarkManager bm) {
		this.bm = bm;
		
		// threads running configurators
		for (int i=0; i<2; i++) {
			new ConfiguratorThread(workQueue, results, i);
		}
		
		// thread for sleeping abandoned sessions
		new ConfiguratorSweeper();
	}
	
	public Screen getFirstScreen(BRID session, ExperimentMetadata experiment, boolean isRegression) 
	throws BenchmarkManagerException, RemoteException {
		Configurator c = createConfigurator(session, experiment, isRegression);
		return getScreen(session, new ConfiguratorWork(session, null, NavigationDirection.FIRST, c));
	}

	public Screen getNextScreen(BRID session, Screen currentScreen) 
	throws RemoteException, BenchmarkManagerException {
		Configurator c = getConfigurator(session);
		Screen screen = getScreen(session, new ConfiguratorWork(session, currentScreen, NavigationDirection.NEXT, c));
		return screen;
	}

	public Screen getPreviousScreen(BRID session, Screen currentScreen) 
	throws RemoteException,	BenchmarkManagerException {
		Configurator c = getConfigurator(session);
		return getScreen(session, new ConfiguratorWork(session, currentScreen, NavigationDirection.PREVIOUS, c));
	}

	private Configurator createConfigurator(BRID session, ExperimentMetadata experiment, boolean isRegression) throws BenchmarkManagerException {
		Configurator c = bm.getPluginLoader().createConfigurator(experiment.getPlugin());
		c.setBenchmarkExperiment(experiment);
		c.setRegression(isRegression);
		configurators.put(session, new ConfiguratorEntry(c));
		return c;
	}
	
	private Screen getScreen(BRID session, ConfiguratorWork work) throws BenchmarkManagerException {
		workQueue.addWork(work);
		ConfiguratorWork processed = results.getResult(session);
		if (processed == null) {
			throw new BenchmarkManagerException("Configurator interrupted");
		}
		
		if (processed.isValid()) {
			return work.getNewScreen();
		} else {
			throw processed.getException();
		}
	}
	
	/**
	 * @param session	GUI session for which the configurator stored
	 * @return	configurator for the session
	 * @throws IllegalArgumentException	when no configurator is found for given session
	 */
	public Configurator getConfigurator(BRID session) throws IllegalArgumentException {
		ConfiguratorEntry c = configurators.get(session);
		if (c == null) {
			throw new IllegalArgumentException("Configurator for session '" + session + "' does not exist");
		}

		return c.getConfigurator();
	}

	
	public void removeConfigurator(BRID session) {
		configurators.remove(session);
	}
	
}
