/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager.gui;

import java.util.Arrays;

/**
 * Represents a multiselect control in the plugin configuration wizard screen.
 * 
 * Multiselect control allows user to choose any number of options form a set.
 * Each option is represented by instance of the <code>Option</code> class.
 * 
 * @author David Majda
 * @author Michal Tomcanyi
 */
public class MultiSelect extends Item {
	/** Set of options. */
	private Option[] options;
	/**
	 * Indexes of selected options. Allowed values are
	 * <code>0</code>..<code>options.length - 1</code>.
	 */
	private int[] selectedIndexes;

	/** @return set of options */
	public Option[] getOptions() {
		return options;
	}
		
	/** @return indexes of selected options */
	public int[] getSelectedIndexes() {
		return selectedIndexes;
	}
	
	private void verifySelectedIndexes(Option[] options, int[] selectedIndexes) {
		for (int i = 0; i < selectedIndexes.length; i++) {
			if (selectedIndexes[i] < 0 || options.length <= selectedIndexes[i]) {
				throw new IllegalArgumentException(
					"Invalid selectedIndexes[" + i + "] (" + selectedIndexes[i] + ")."
				);
			}
		}
	}

	/**
	 * Sets the indexes of selected options.
	 * 
	 * @param selectedIndexes indexes of selected options to set
	 * 
	 * @throws IllegalArgumentException if any of <code>selectedIndexes</code> is
	 *          not in range <code>0</code>..<code>options.length - 1</code>
	 */
	public void setSelectedIndexes(int[] selectedIndexes) {
		verifySelectedIndexes(options, selectedIndexes);
		this.selectedIndexes = selectedIndexes;
	}
	
	public String[] getSelectedIds() {
		String[] result = new String[selectedIndexes.length];
		for (int i : selectedIndexes) {
			result[i] = options[selectedIndexes[i]].getId();
		}
		return result;
	}
	
	/**
	 * Selects the options by their ID.
	 * 
	 * @param selectedIds IDs of the options to select
	 * 
	 * @throws IllegalArgumentException if some option does not have the ID 
	 *          specified in <code>selectedIds</code>
	 */
	public void setSelectedIds(String[] selectedIds) {
		int[] newSelectedIndexes = new int[selectedIds.length];
		for (int i = 0; i < selectedIds.length; i++) {
			for (int j = 0; j < options.length; i++) {
				if (options[j].getId().equals(selectedIds[i])) {
					newSelectedIndexes[i] = j;
					break;
				}
			}
			assert selectedIds[i] != null
				: "Invalid selectedIds[" + i + "] (\"" + selectedIds[i] + "\").";
		}
	}

	/**
	 * Allocates a new <code>MultiSelect</code> object.
	 * 
	 * @param label select label
	 * @param options set of options
	 * @param selectedIndexes indexes of selected option
	 * 
	 * @throws IllegalArgumentException if any of <code>selectedIndexes</code> is
	 *          not in range <code>0</code>..<code>options.length - 1</code>
	 */
	public MultiSelect(String label, Option[] options, int[] selectedIndexes) {
		super(label);

		verifySelectedIndexes(options, selectedIndexes);
		
		this.options = options;
		this.selectedIndexes = selectedIndexes;
	}
	
	/**
	 * Returns a string representation of the object.
	 * 
	 * @return a string representation of the object
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "MultiSelect(name=" + getLabel() + ", options="
			+ Arrays.toString(options) + ")";
	}
}
