/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.hostruntime;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import cz.cuni.mff.been.task.TaskException;

/**
 * Stores package configuration, which is read from the "config.xml" file in the
 * package.
 * 
 * @author Antonin Tomecek
 * @author David Majda
 */
class PackageConfiguration {
	/** List of directories or JAR files to append to the Java's class path. */
	private String javaClassPath;

	/** Name of the class to execute. */
	private String javaMainClass;

	/**
	 * Returns the list of directories or JAR files to append to the Java's class
	 * path.
	 * 
	 * @return list of directories or JAR files to append to the Java's class
	 *          path
	 */
	public String getJavaClassPath() {
		return javaClassPath;
	}

	/**
	 * Returns the name of the class to execute.
	 * 
	 * @return name of the class to execute.
	 */
	public String getJavaMainClass() {
		return javaMainClass;
	}

	/**
	 * Custom entity resolver required to load the package configuration DTD.
	 * 
	 * @author Antonin Tomecek
	 * @author David Majda
	 */
	private static class PackageConfigurationEntityResolver
			implements EntityResolver {
		/** Package configuration DTD file location. */
		private String dtdFile;

		/**
		 * Allocates a new <code>PackageConfiguration</code> object.
		 * 
		 * @param dtdFile package configuration DTD file location
		 */
		public PackageConfigurationEntityResolver(String dtdFile) {
			this.dtdFile = dtdFile;
		}

		/**
		 * @see org.xml.sax.EntityResolver#resolveEntity(java.lang.String, java.lang.String)
		 */
		public InputSource resolveEntity(String publicId, String systemId)
				throws SAXException, IOException {
			if (systemId.endsWith("package-configuration.dtd")) {
				return new InputSource(new FileInputStream(dtdFile));
			} else {
				return null;
			}
		}
	}

	/**
	 * Parses the package configuration XML file and builds a DOM document.
	 * 
	 * @param configFile package configuration file
	 * @param dtdFile package configuration DTD file
	 * @return document built form the parsed package configuration file
	 * @throws TaskException if the package configuration file parsing fails 
	 */
	private Document parseConfigFile(String configFile, String dtdFile)
			throws TaskException {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw new TaskException(e);
		}
		builder.setEntityResolver(new PackageConfigurationEntityResolver(dtdFile));
		
		Document document; 
		try {
			document = builder.parse(configFile);
		} catch (SAXException e) {
			throw new TaskException(e);
		} catch (IOException e) {
			throw new TaskException(e);
		}
		
		/* Check that document is not null. If it is, it means it isn't valid XML
		 * file, so report error.
		 */
		if (document == null) {
			throw new TaskException(configFile + ": Not valid XML file.");
		}

		Element documentElement = document.getDocumentElement();

		/* Check that root element is <packageConfiguration>. */
		if (!documentElement.getNodeName().equals("packageConfiguration")) {
			throw new TaskException(configFile + ": Root element must be <packageConfiguration>.");
		}

		/* Check presence of the <java> element. */ 
		NodeList javaElements = documentElement.getElementsByTagName("java");
		Element javaElement;
		if (javaElements.getLength() == 1) {
			javaElement = (Element) javaElements.item(0);   
		} else {
			throw new TaskException(configFile + ": There must be one <java> element present."); 
		}	

		/* Check presence of the <java> element's attributes. */
		if (!javaElement.hasAttribute("classPath")) {
			throw new TaskException(configFile + ": Missing \"classPath\" attribute of the <java> element.");
		}
		if (!javaElement.hasAttribute("mainClass")) {
			throw new TaskException(configFile + ": Missing \"mainClass\" attribute of the <java> element.");
		}

		/* We've passed all the tests now. */
		return document;
	}
	
	/**
	 * Reads the configuration data from DOM document of the XML configuration
	 * file.
	 */
	private void readData(Document document) {
		Element packageElement = document.getDocumentElement();

		NodeList javaElements = packageElement.getElementsByTagName("java");
		assert javaElements.getLength() == 1: "There sohuld be only one \"java\" "
			+ "element in the configuration file.";
		Element javaElement = (Element) javaElements.item(0);
		
		if (File.pathSeparatorChar == ';') {
			javaClassPath = javaElement.getAttribute("classPath").replace(':', File.pathSeparatorChar);
		} else {
			javaClassPath = javaElement.getAttribute("classPath").replace(';', File.pathSeparatorChar);
		}
		javaMainClass = javaElement.getAttribute("mainClass");
	}

	/**
	 * Allocates a new <code>PackageConfiguration</code> object. The data in the
	 * object is read from specified XML file.
	 * 
	 * @param configFile package configuration file
	 * @param dtdFile package configuration DTD file
	 * @throws TaskException if the package configuration file parsing fails
	 *  
	 */
	public PackageConfiguration(String configFile, String dtdFile)
			throws TaskException {
		Document document = parseConfigFile(configFile, dtdFile);
		readData(document);
	}
}
