/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.hostruntime;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Chmod;
import cz.cuni.mff.been.common.anttasks.Delete;
import cz.cuni.mff.been.hostmanager.IllegalOperationException;
import cz.cuni.mff.been.hostmanager.load.LoadMonitorException;
import cz.cuni.mff.been.hostmanager.load.LoadMonitorInterface;
import cz.cuni.mff.been.logging.LogLevel;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.task.Task;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.taskmanager.CheckPoint;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;
import cz.cuni.mff.been.taskmanager.data.TaskExclusivity;
import cz.cuni.mff.been.taskmanager.data.TaskState;

/**
 * Class resving as a representiation of a task in the Host Runtime. Responsible
 * for running the task and taking care of it.
 *
 * @author Antonin Tomecek
 * @author David Majda
 */
public class TaskImplementation extends UnicastRemoteObject
		implements TaskInterface {
	/** Name of the file with standard output of the task process. */
	private static final String STANDARD_OUTPUT_FILE = "standard-output";
	/** Name of the file with error output of the task process. */
	private static final String ERROR_OUTPUT_FILE = "error-output";

	/** Task descriptor of this task. */
	private TaskDescriptor taskDescriptor;
	/** Configuration of the package with this task */
	private PackageConfiguration packageConfiguration;
	
	/** Host Runtime in which this task is running. */
	private HostRuntimeImplementation hostRuntime;
	
	/** Process in which the task is running. */
	private Process process;
	
	/** Directory which contains all the task data. */ 
	private String baseDirectory;

	/** Task directory (task's JAR is extracted there). */
	private String taskDirectory;
	/**
	 * Working directory (results of task's work are stored there, surviving
	 * task's death).
	 */
	private String workingDirectory;
	/** Temporary directory (deleted after the task's death). */
	private String temporaryDirectory;
	/** Service directory (package contents are extracted there). */
	private String serviceDirectory;
		
	/** Class which waits for the termination of the task process. */
	private Thread processTerminationWaiter;
	/** Thread which executes the process. */
	private TaskProcessExecutor executor;
	
	/** Type of the process output. */
	private enum OutputType { STANDARD, ERROR }
	
	/** Determines whether we want to measure detailed load for this task. */
	private boolean measureDetailedLoad;

	/**
	 * Thread which is used to read standard and error output form executed
	 * process and write it to the log file and send it to the Host Runtime
	 * (which sends it further to the Task Manager).
	 * 
	 * The ouput is read and written in <code>BUFFER_SIZE</code> long chunks (or
	 * smaller, if <code>InputStream.read</code> call does not fill entire
	 * buffer).
	 * 
	 * @author David Majda
	 */
	private class ProcessOutputProcessor extends Thread {
		/** Transfer buffer size. */ 
		private static final int BUFFER_SIZE = 4096;
		
		/** Stream to read the logs from. */
		private InputStream inputStream;
		/** File to write the logs to. */
		private String outputFile;
		/** Type of the process output. */ 
		private OutputType outputType;
		/** Task exclusivity. */
		private TaskExclusivity exclusivity;
		
		/**
		 * Allocates a new <code>ProcessOutputProcessor</code> object.
		 * 
		 * @param inputStream stream to read the logs from
		 * @param outputFile file to write the logs to
		 * @param outputType type of the process output
		 * @param exclusivity task exclusivity
		 */
		public ProcessOutputProcessor(InputStream inputStream, String outputFile,
				OutputType outputType, TaskExclusivity exclusivity) {
			this.inputStream = inputStream;
			this.outputFile = outputFile;
			this.outputType = outputType;
			this.exclusivity = exclusivity;
		}
		
		/**
		 * @see java.lang.Thread#run()
		 */
		public void run() {
			FileOutputStream outputStream = null;
			
			try {
				/* We don't use buffering in the output file beacause we want the file
				 * to be in sync with the process output. 
				 */
				outputStream = new FileOutputStream(outputFile);
			} catch (FileNotFoundException e) {
				System.err.println(e.getMessage());
				return;
			}
			
			try {
				try {
					byte[] buffer = new byte[BUFFER_SIZE];
					int bytesRead;
					while ((bytesRead = inputStream.read(buffer)) != -1) {
						/* Write output to the file. */
						outputStream.write(buffer, 0, bytesRead);
						
						/* Send output to the Host Runtime, if the task is not exclusive nor
						 * context-exclusive.
						 */
						if (exclusivity == TaskExclusivity.NON_EXCLUSIVE) {
							try {
								switch (outputType) {
									case STANDARD:
										hostRuntime.getHostRuntimesPort().addStandardOutput(
												taskDescriptor.getContextId(), 
												taskDescriptor.getTaskTid(),
												new String(buffer, 0, bytesRead)
										);
										break;
									case ERROR:
										hostRuntime.getHostRuntimesPort().addErrorOutput(
												taskDescriptor.getContextId(), 
												taskDescriptor.getTaskTid(),
												new String(buffer, 0, bytesRead)
										);
									default:
										break;
								} 
							} catch (LogStorageException e) {
								System.err.println("Log storage error: " + e.getMessage());
								return;
							}
						}
					}
				} finally {
					outputStream.close();
				}
			} catch (RemoteException e) {
				System.err.println("Error executing remote call from the task to the "
						+ "Task Manager.");
			} catch (IOException e) {
				System.err.println("Error sending task output: " + e.getMessage());
			}
		}
	}
		
	/**
	 * Thread which wais for the task process to end and performs various cleanup
	 * tasks.
	 * 
	 * @author David Majda
	 */
	private class TaskProcessExecutor extends Thread {
		/** Transfer buffer size. */ 
		private static final int BUFFER_SIZE = 4096;

		/** File with standard output of the task process. */
		private String stdoutFile;
		/** File with error output of the task process. */
		private String stderrFile;
		/** Executed command and its parameters. */
		private List<String> commandAndParams;
		/**
		 * Flag indicating if the task process was killed because it run longer
		 * than allowed timeout.
		 */
		private boolean processTimedOut;
		/**
		 * Flag indicating if the process was killed by the <code>kill</code>
		 * method.
		 */
		private boolean processKilledFromOutside;
		
		public void setProcessKilledFromOutside(boolean processKilledFromOutside) {
			this.processKilledFromOutside = processKilledFromOutside;
		}
		
		/**
		 * Allocates a new <code>TaskProcessExecutor</code> object.
		 * 
		 * @param stdoutFile file with standard output of the task process
		 * @param stderrFile file with error output of the task process
		 * @param commandAndParams executed command and its parameters 
		 */
		public TaskProcessExecutor(String stdoutFile, String stderrFile,
				List<String> commandAndParams) {
			this.stdoutFile = stdoutFile;
			this.stderrFile = stderrFile;
			this.commandAndParams = commandAndParams;
		}

		/** Executes the task. */
		private void execute() {
			try {
				/* The process working directory must be set to taskDirectory, so the
				 * classpaths work. More specifically, Benchmark Manager breaks if it
				 * is set to different directory.
				 */
				process = Runtime.getRuntime().exec(
					commandAndParams.toArray(new String[commandAndParams.size()]),
					null,
					new File(taskDirectory) 
				);
			} catch (IOException e) {
				System.err.println("Error executing task process: " + e.getMessage());
				System.exit(1);
			}
		}

		private void maybeStartLoadMonitorDetailedMode(int runCount) {
			if (measureDetailedLoad) {
				LoadMonitorInterface loadMonitor = hostRuntime.getLoadMonitor();
				try {
					if (taskDescriptor.getDetailedLoadInterval() != 0) {
						loadMonitor.startDetailedMode(
							taskDescriptor.getContextId(),
							taskDescriptor.getTaskTid(),
							taskDescriptor.getDetailedLoadInterval()
						);
					} else {
						loadMonitor.startDetailedMode(
							taskDescriptor.getContextId(),
							taskDescriptor.getTaskTid()
						);
					}
				} catch (IllegalOperationException e) {
					throw new AssertionError("Invalid Load Monitor mode.");
				} catch (LoadMonitorException e) {
					System.err.println(e.getMessage());
					System.exit(1);
				} catch (RemoteException e) {
					throw new AssertionError("Local call should not fail.");
				}
			}
		}

		private void maybeStopLoadMonitorDetailedMode() {
			if (measureDetailedLoad) {
				try {
					hostRuntime.getLoadMonitor().stopDetailedMode();
				} catch (IllegalOperationException e) {
					throw new AssertionError("Invalid Load Monitor mode.");
				} catch (LoadMonitorException e) {
					System.err.println(e.getMessage());
					System.exit(1);
				} catch (RemoteException e) {
					throw new AssertionError("Local call should not fail.");
				}
			}
		}
		
		/** Attach processors to the standard and error output. */
		private void attachOutputProcessors() {
			new ProcessOutputProcessor(
				process.getInputStream(),
				stdoutFile,
				OutputType.STANDARD,
				taskDescriptor.getTaskExclusive()
			).start();
			new ProcessOutputProcessor(
				process.getErrorStream(),
				stderrFile,
				OutputType.ERROR,
				taskDescriptor.getTaskExclusive()
			).start();
		}

		/**
		 * Utility thread which is created when a task has specified timeout for its
		 * run. It simply sleeps through the timeout and kills the task process
		 * afterwards.
		 * 
		 * If the task has exited normally before the timeout, this thread is
		 * interrupted by the <code>TaskProcessExecutor</code> thread. This thread
		 * simply ends in this case.
		 * 
		 * @author David Majda
		 */
		private class TaskProcessKiller extends Thread {
			/** Task process timeout. */
			private long timeout;
			
			/**
			 * Allocates a new <code>TaskProcessKiller</code> object.
			 * 
			 * @param timeout task process timeout
			 */
			public TaskProcessKiller(long timeout) {
				this.timeout = timeout; 
			}
			
			/**
			 * @see java.lang.Thread#run()
			 */
			public void run() {
				try {
					sleep(timeout);
					processTimedOut = true;
					process.destroy();
					writeMessage("Task killed after a timeout (" + timeout + " ms).");
				} catch (InterruptedException e) {
					/* This happens when the task has ended before the timeout and the
					 * killer thread is woken up so it can finish.
					 */ 
				}
			}
		}
		
		/** Patiently waits until the process ends. */
		private void waitForProcess() {
			processTimedOut = false;
			processKilledFromOutside = false;
			long timeout = taskDescriptor.getTimeoutRun();
			Thread killer = null;
			if (timeout != 0) {
				killer = new TaskProcessKiller(timeout);
				killer.start();
			}
			
			try {
				process.waitFor();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			
			if (killer != null && !processTimedOut) {
				killer.interrupt();
			}
		}
		
		/** Notifies the Host Runtime about task's finish. */
		private void notifyHostRuntime() {
			try {
				/* Following calls must be in correct order. Inside the call of the
				 * taskReachedState method, the Task Manager may decide to close
				 * context that contains this task and call
				 * HostRuntimeInterface.closeContext method. But if the calls are not
				 * ordered properly, the Host Runtime will not yet be notified about the
				 * task's end (the notification happens in notifyTaskFinished call) and
				 * refuse to close the context (throwing IllegalArgumentException). This
				 * would be a race condition. 
				 */
				hostRuntime.notifyTaskFinished(TaskImplementation.this);
				hostRuntime.getHostRuntimesPort().taskReachedState(
					taskDescriptor.getTaskTid(),
					taskDescriptor.getContextId(),
					processKilledFromOutside ? TaskState.ABORTED : TaskState.FINISHED);
			} catch (RemoteException e) {
				System.err.println("Error executing remote call from the task to the "
					+ "Task Manager.");
				System.exit(1);
			}
		}

		private void sendLogs() {
			for (LogRecord logRecord: logs) {
				try {
					hostRuntime.getHostRuntimesPort().log(
						taskDescriptor.getContextId(),
						taskDescriptor.getTaskTid(),
						logRecord.level,
						logRecord.timestamp,
						logRecord.message
					);
				} catch (RemoteException e) {
					System.err.println("Error executing remote call from the task to the "
							+ "Task Manager.");
					return;
				}
			}
		}

		private void sendOutput(String inputFile, OutputType outputType) {
			InputStream inputStream = null;

			try {
				inputStream = new BufferedInputStream(new FileInputStream(inputFile));
			} catch (FileNotFoundException e) {
				System.err.println(e.getMessage());
				return;
			}
			
			try {
				try {
					byte[] buffer = new byte[BUFFER_SIZE];
					int bytesRead;
					while ((bytesRead = inputStream.read(buffer)) != -1) {
						try {
							switch (outputType) {
								case STANDARD:
									hostRuntime.getHostRuntimesPort().addStandardOutput(
											taskDescriptor.getContextId(), 
											taskDescriptor.getTaskTid(),
											new String(buffer, 0, bytesRead)
									);
									break;
								case ERROR:
									hostRuntime.getHostRuntimesPort().addErrorOutput(
											taskDescriptor.getContextId(), 
											taskDescriptor.getTaskTid(),
											new String(buffer, 0, bytesRead)
									);
								default:
									break;
							} 
						} catch (LogStorageException e) {
							System.err.println("Log storage error: " + e.getMessage());
							return;
						}
					}
				} finally {
					inputStream.close();
				}
			} catch (RemoteException e) {
				System.err.println("Error executing remote call from the task to the "
					+ "Task Manager.");
			} catch (IOException e) {
				System.err.println("Error sending task output: " + e.getMessage());
			}
		}

		/**
		 * Send logs and standard and error output to the Task manager if the task
		 * was exclusive of context-exclusive.
		 */
		private void sendLogAndOutputIfNeeded() {
			if (taskDescriptor.getTaskExclusive() != TaskExclusivity.NON_EXCLUSIVE) {
				sendLogs();
				sendOutput(stdoutFile, OutputType.STANDARD);
				sendOutput(stderrFile, OutputType.STANDARD);
			}
		}

		private void signalCheckPoint(String name, Serializable value) {
			CheckPoint taskStartedCheckPoint = new CheckPoint(
				taskDescriptor.getTaskTid(),
				taskDescriptor.getContextId(),
				name,
				value
			);
			taskStartedCheckPoint.setHostName(taskDescriptor.getHostRuntimesName()[0]);
			try {
				hostRuntime.getHostRuntimesPort().checkPointReached(taskStartedCheckPoint);
			} catch (RemoteException e) {
				System.err.println("Error executing remote call from the task to the "
					+ "Task Manager.");
				System.exit(1);
			}
		}
		
		private void cleanDirectory(String dir) {
			try {
				Delete.deleteDirectory(dir);
			} catch (AntTaskException e) {
				System.err.println(e.getMessage());
				System.exit(1);
			}
			if (!new File(dir).mkdir()) {
				System.err.println("Can not create directory \"" + dir + "\".");
				System.exit(1);
			}
		}
		
		public void run() {
			int runCount = 0;
			boolean taskSuccessfull;
			boolean taskCanBeRestarted;
			do {
				execute();
				if (runCount != 0) {
					maybeStopLoadMonitorDetailedMode();
				}
				maybeStartLoadMonitorDetailedMode(runCount);
				attachOutputProcessors();
				if (runCount == 0) {
					writeMessage("Started.");
					signalCheckPoint(Task.CHECKPOINT_NAME_STARTED, null);
				} else {
					writeMessage("Restarted.");
					try {
						hostRuntime.getHostRuntimesPort().taskRestarted(
							taskDescriptor.getTaskTid(),
							taskDescriptor.getContextId()
						);
					} catch (RemoteException e) {
						writeMessage("Notification of the Task Manager about restart failed.");
					}
				}
				waitForProcess();
				cleanDirectory(temporaryDirectory);
				cleanDirectory(taskDirectory);
				cleanDirectory(serviceDirectory);
				taskSuccessfull = process.exitValue() == 0
					&& !processTimedOut
					&& !processKilledFromOutside;
				taskCanBeRestarted = runCount < taskDescriptor.getRestartMax()
					&& !processKilledFromOutside; 
				runCount++;
			} while (!taskSuccessfull && taskCanBeRestarted);
			
			maybeStopLoadMonitorDetailedMode();
			/* Following two calls should be ordered properly, otherwise the Task
			 * Manager will consider the task finished before sending a checkpoint,
			 * resulting in a exception.
			 */
			if (!processKilledFromOutside) {
				signalCheckPoint(Task.CHECKPOINT_NAME_FINISHED, process.exitValue());
			}
			notifyHostRuntime();
			sendLogAndOutputIfNeeded();
			
			writeMessage("Stopped.");
		}
	}

	/**
	 * Little class which sotres log records on the Host Runtime's side when the
	 * runned task is exclusive or context-exclusive.
	 * 
	 * @author David Majda
	 */
	private static class LogRecord {
		/** Log level of the message. */
		private LogLevel level;
		/** Timestamp of the log event. */
		private Date timestamp;
		/**
		 * Message the log message; can be multiline; cannot be null or contain
		 * "\t\t\t".
		 */
		private String message;
		
		/** @return log level of the message */
		public LogLevel getLevel() {
			return level;
		}
		
		/** @return timestamp of the log event */
		public String getMessage() {
			return message;
		}
		
		/**
		 * @return message the log message; can be multiline; cannot be null or
		 * contain "\t\t\t"
		 */
		public Date getTimestamp() {
			return timestamp;
		}
		
		/**
		 * Allocates a new <code>LogRecord</code> object.
		 * 
		 * @param level log level of the message
		 * @param timestamp timestamp of the log event
		 * @param message message the log message; can be multiline; cannot be null
		 *         or contain "\t\t\t"
		 */
		public LogRecord(LogLevel level, Date timestamp, String message) {
			this.level = level;
			this.timestamp = timestamp;
			this.message = message;
		}
	}
	
	/**
	 * Task logs buffer used to stor logs of exclusive and context-exclusive
	 * tasks.
	 */   
	private List<LogRecord> logs = new LinkedList<LogRecord>();
	
	/** @return task descriptor of this task */
	public TaskDescriptor getTaskDescriptor() {
		return taskDescriptor;
	}
	
	/** @return Host Runtime in which this task is running */
	public HostRuntimeImplementation getHostRuntime() {
		return hostRuntime;
	}
	
	/**
	 * Sends a log message to the log storage or sotres it for later sending if
	 * the task is exclusive or contxt-exclusive.
	 * 
	 * @param level log level of the message
	 * @param timestamp timestamp of the log event
	 * @param message the log message; can be multiline; cannot be null or contain
	 *         "\t\t\t".
	 * @throws RemoteException if something fails during the execution of
	 *          the remote method call.
	 */
	void log(LogLevel level, Date timestamp, String message)
			throws RemoteException {
		if (taskDescriptor.getTaskExclusive() == TaskExclusivity.NON_EXCLUSIVE) {
			hostRuntime.getHostRuntimesPort().log(
				taskDescriptor.getContextId(),
				taskDescriptor.getTaskTid(),
				level,
				timestamp,
				message
			);
		} else {
			logs.add(new LogRecord(level, timestamp, message));
		}
	}

	/**
	 * Creates the task's port implementation class and binds it to the RMI URL.
	 * 
	 * @param tasksPortUrl RMI URL of the task's port
	 */
	private void createAndBindTasksPort(String tasksPortUrl)
			throws RemoteException, TaskException {
		TasksPortImplementation tasksPort = new TasksPortImplementation(this);
		try {
			Naming.rebind(tasksPortUrl, tasksPort);
		} catch (MalformedURLException e) {
			throw new TaskException(e);
		}
	}
	
	/**
	 * Creates specified directory, if it does not already exist, including any
	 * necessary but nonexistent parent directories.
	 * 
	 * @param dir directory to create
	 * @throws IOException if the directory does not exist and can not be created 
	 */
	private void mkdirsIfNotExists(String dir) throws IOException {
		File f = new File(dir);
		if (!f.exists()) {
			if (!f.mkdirs()) {
				throw new IOException("Can not create directory \"" + dir + "\".");
			}
		}
	}
	
	/**
	 * Create all directories of the task.
	 * 
	 * @param rootDirectory root directory of the task's Host Runtime
	 * @throws TaskException if any directory does not exist and can not be
	 *          created
	 */
	private void createTaskDirectories(String rootDirectory) throws TaskException {
		String contextID = taskDescriptor.getContextId();
		String taskID = taskDescriptor.getTaskTid();

		baseDirectory = hostRuntime.getBaseDirectoryForTask(contextID, taskID);
		taskDirectory = hostRuntime.getTaskDirectoryForTask(contextID, taskID);
		workingDirectory = hostRuntime.getWorkingDirectoryForTask(contextID, taskID);
		temporaryDirectory = hostRuntime.getTemporaryDirectoryForTask(contextID, taskID);
		serviceDirectory = hostRuntime.getServiceDirectoryForTask(contextID, taskID);
		
		try {
			mkdirsIfNotExists(baseDirectory);
			mkdirsIfNotExists(taskDirectory);
			mkdirsIfNotExists(workingDirectory);
			mkdirsIfNotExists(temporaryDirectory);
			mkdirsIfNotExists(serviceDirectory);
		} catch (IOException e) {
			throw new TaskException(e);
		}
	}
	
	/**
	 * Extracts the task package.
	 * 
	 * @throws TaskException if extraction of the package fails
	 */
	private void extractPackage() throws TaskException {
		try {
			hostRuntime.getPackageCacheManager().extractPackage(
				taskDescriptor.getPackageName(),
				serviceDirectory
			);
		} catch (HostRuntimeException e) {
			throw new TaskException(e);
		} catch (IOException e) {
			throw new TaskException(e);
		}
	}
	
	/**
	 * Moves files from the package to the task directory and sets up their acces
	 * rights (needed on Linux systems).
	 * 
	 * @throws TaskException if setting of the access rights fails
	 */
	private void moveAndChmodPackageFiles() throws TaskException {
		File[] files = new File(serviceDirectory + File.separator +  "files").listFiles();
		for (File f: files) {
			f.renameTo(new File(taskDirectory + File.separator + f.getName()));
		}

		try {
			Chmod.recursiveDirectoryChmod(taskDirectory, "u+rwx");
		} catch (AntTaskException e) {
			throw new TaskException(e);
		}
	}
	
	/**
	 * Reads the package configuration.
	 * 
	 * @param rootDirectory root directory of the task's Host Runtime
	 * @throws TaskException if the configuration parsing fails
	 */
	private void readConfiguration(String rootDirectory) throws TaskException {
		packageConfiguration = new PackageConfiguration(
			serviceDirectory + File.separator + "config.xml",
			rootDirectory + File.separator + "package-configuration.dtd"
		);
	}
	
	/**
	 * Builds executed command and its parameters for the <code>exec</code> call.
	 * 
	 * @param tasksPortUrl RMI URL of the task's port
	 * @return executed command and its parameters
	 */
	private List<String> buildCommandAndParams(String tasksPortUrl) {
		List<String> result = new LinkedList<String>();
		
		/* "java" command. */
		result.add("java");
		
		/* Java options. */
		String[] javaOptions = taskDescriptor.getJavaOptions();
		for (String javaOption: javaOptions) {
			result.add(javaOption);
		}
		
		/* Directories. */
		result.add("-Dhostruntime.directory.task=" + taskDirectory);
		result.add("-Dhostruntime.directory.working=" + workingDirectory);
		result.add("-Dhostruntime.directory.temporary=" + temporaryDirectory);
		
		/* Task's port URL */
		result.add("-Dhostruntime.tasksport.uri=" + tasksPortUrl);
		
		/* Java' class path. */
		result.add("-cp");
		result.add(
			System.getProperty("java.class.path")
			+ File.pathSeparatorChar + taskDirectory
			+ File.pathSeparatorChar + packageConfiguration.getJavaClassPath()
		);
		
		/* TaskLoader class. */
		result.add(TaskLoader.class.getName());
		/* Task to run. */
		result.add(packageConfiguration.getJavaMainClass());
		
		/* Command-line parameters for the task. */
		for (String argument: taskDescriptor.getTaskArguments()) {
			result.add(argument);
		}
		
		return result;
	}
	
	/**
	 * Executes the task process.
	 * 
	 * @param commandAndParams executed command and its parameters
	 * @throws TaskException if the execution fails 
	 * @throws RemoteException if something failed during the execution of
	 *          the remote method call.
	 */
	private void execute(List<String> commandAndParams) throws TaskException,
			RemoteException {				
		executor = new TaskProcessExecutor(
			baseDirectory + File.separator + STANDARD_OUTPUT_FILE,
			baseDirectory + File.separator + ERROR_OUTPUT_FILE,
			commandAndParams
		);
		executor.start();
	}

	/**
	 * Allocates a new <code>TaskImplementation</code> object and executes
	 * associated task.
	 * 
	 * @param taskDescriptor task descriptor of this task
	 * @param hostRuntime Host Runtime in which this task is running
	 * @param measureDetailedLoad determines whether we want to measure detailed
	 *        load for this task.
	 * @throws TaskException if anything fails when creating the task
	 * @throws RemoteException if something failed during the execution of
	 *          the remote method call.
	 */
	protected TaskImplementation(TaskDescriptor taskDescriptor,
			HostRuntimeImplementation hostRuntime, boolean measureDetailedLoad) throws TaskException,
			RemoteException {
		this.taskDescriptor = taskDescriptor;
		this.hostRuntime = hostRuntime;
		this.measureDetailedLoad = measureDetailedLoad;
		
		String tasksPortUrl = "rmi://localhost/been/hostruntime/tasksport/"
			+ taskDescriptor.getTaskTid();
		String rootDirectory = hostRuntime.getRootDirectory();

		createTaskDirectories(rootDirectory);
		extractPackage();
		moveAndChmodPackageFiles();
		readConfiguration(rootDirectory);
		
		/* Note: We need to bind before we execute, to avoid race condition
		 * (executed task can try to cumminucate with not-yet-bound task's port).
		 */
		createAndBindTasksPort(tasksPortUrl);
		execute(buildCommandAndParams(tasksPortUrl));
	}
	
	/**
	 * Writes a message to the standard output.
	 * 
	 * @param message message text
	 */
	public void writeMessage(String message) {
		System.out.println("[" + taskDescriptor.getContextId() + ":"
			+ taskDescriptor.getTaskTid() + "] " + message);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#getTaskDirectory()
	 */
	public String getTaskDirectory() throws RemoteException {
		return taskDirectory;
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#getWorkingDirectory()
	 */
	public String getWorkingDirectory() throws RuntimeException {
		return workingDirectory;
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#getTemporaryDirectory()
	 */
	public String getTemporaryDirectory() throws RuntimeException {
		return temporaryDirectory;
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#getTaskID()
	 */
	public String getTaskID() throws RemoteException {
		return taskDescriptor.getTaskTid();
	}	

	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#getContextID()
	 */
	public String getContextID() throws RemoteException {
		return taskDescriptor.getContextId();
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#isDetailedLoad()
	 */
	public boolean isDetailedLoad() throws RemoteException {
		return measureDetailedLoad;
	}
	
	private void deleteDirectoryRecursively(String dir) throws IOException {
		File dirFile = new File(dir);
		for (File file: dirFile.listFiles()) {
			if (file.isDirectory()) {
				deleteDirectoryRecursively(file.getPath());
			} else {
				if (!file.delete()) {
					throw new IOException("Can not delete file \"" + file.getPath()
						+ "\".");
				}
			}
		}
		if (!dirFile.delete()) {
			throw new IOException("Can not delete directory \"" + dirFile.getPath()
				+ "\".");
		}
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#destroy()
	 */
	public void destroy() throws TaskException, RemoteException {
		if (isRunning()) {
			throw new IllegalStateException("You must call this method only on non-running tasks.");
		}
		try {
			deleteDirectoryRecursively(baseDirectory);
		} catch (IOException e) {
			throw new TaskException("Error destorying task: " + e.getMessage(), e);
		}
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#kill()
	 */
	public void kill() throws RemoteException {
		executor.setProcessKilledFromOutside(true);
		process.destroy();
		writeMessage("Task killed from outside.");
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#isRunning()
	 */
	public boolean isRunning() throws RemoteException {
		return processTerminationWaiter.isAlive();
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#waitFor()
	 */
	public void waitFor() throws RemoteException {
		try {
			process.waitFor();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TaskInterface#getExitValue()
	 */
	public int getExitValue() throws RemoteException {
		if (isRunning()) {
			throw new IllegalStateException("You must call this method only on non-running tasks.");
		}
		return process.exitValue();
	}
}
