/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.hostruntime;

import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import cz.cuni.mff.been.common.VariableReplacer;
import cz.cuni.mff.been.common.VariableReplacer.ValueProvider;
import cz.cuni.mff.been.logging.LogLevel;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.logging.OutputHandle;
import cz.cuni.mff.been.taskmanager.CheckPoint;
import cz.cuni.mff.been.taskmanager.HostRuntimeRegistrationListener;
import cz.cuni.mff.been.taskmanager.HostRuntimesPortInterface;
import cz.cuni.mff.been.taskmanager.ServiceEntry;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;
import cz.cuni.mff.been.taskmanager.TaskManagerException;
import cz.cuni.mff.been.taskmanager.TaskDescriptor.TaskPropertyObject;
import cz.cuni.mff.been.taskmanager.data.TaskEntry;

/**
 * Implementation of the Host Runtime's port RMI interface provided for the
 * tasks.
 *  
 * @author Antonin Tomecek
 * @author David Majda
 */
public class TasksPortImplementation extends UnicastRemoteObject
		implements TasksPortInterface {
	/** Task this port is associated with.  */
	private TaskImplementation task;
	/** RMI interface to the Task Manager's port to the Host Runtime. */
	private HostRuntimesPortInterface hostRuntimesPort;
	
	/**
	 * Allocates a new <code>TaskImplementation</code> object.
	 * 
	 * @param task this port is associated with
	 * @throws RemoteException if something fails during the execution of
	 *          the remote method call
	 */
	public TasksPortImplementation(TaskImplementation task)
			throws RemoteException {
		this.task = task;
		
		hostRuntimesPort = task.getHostRuntime().getHostRuntimesPort();
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getTaskDirectory(java.lang.String, java.lang.String)
	 */
	public String getTaskDirectory(String contextID, String taskID)
			throws RemoteException {
		return task.getHostRuntime().getTaskDirectoryForTask(contextID, taskID);
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getWorkingDirectory(java.lang.String, java.lang.String)
	 */
	public String getWorkingDirectory(String contextID, String taskID)
			throws RemoteException {
		return task.getHostRuntime().getWorkingDirectoryForTask(contextID, taskID);
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getTemporaryDirectory(java.lang.String, java.lang.String)
	 */
	public String getTemporaryDirectory(String contextID, String taskID)
			throws RemoteException {
		return task.getHostRuntime().getTemporaryDirectoryForTask(contextID, taskID);
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getTaskProperties()
	 */
	public Properties getTaskProperties() throws RemoteException {
		VariableReplacer.ValueProvider valueProvider = new ValueProvider() {
			public String getValue(String variableName) {
				String[] parts = variableName.split(":");
				if (parts.length != 2) {
					return null; 
				}
				
				try {
					if (parts[1].equals("taskDirectory")) {
						return getTaskDirectory(task.getTaskDescriptor().getContextId(), parts[0]);
					} else if (parts[1].equals("temporaryDirectory")) {
						return getTemporaryDirectory(task.getTaskDescriptor().getContextId(), parts[0]);
					} else if (parts[1].equals("workingDirectory")) {
						return getWorkingDirectory(task.getTaskDescriptor().getContextId(), parts[0]);
					} else {
						return null;
					}
				} catch (RemoteException e) {
					return null;
				}
			}
		};
				
		return VariableReplacer.replace(
			task.getTaskDescriptor().getTaskProperties(),
			valueProvider
		);
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getTaskPropertyObjects()
	 */
	public Map<String, Serializable> getTaskPropertyObjects() throws RemoteException {
		TaskPropertyObject[] propertyObjects = task.getTaskDescriptor().getTaskPropertyObjects();
		HashMap<String, Serializable> result = new HashMap<String, Serializable>();
		for (TaskPropertyObject propertyObject : propertyObjects) {
			result.put(propertyObject.getKey(), propertyObject.getValue());
		}
		return result;
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#runTask(cz.cuni.mff.been.taskmanager.TaskDescriptor)
	 */
	public void runTask(TaskDescriptor taskDescriptor) throws RemoteException {
		task.getHostRuntime().getTaskManager().runTask(taskDescriptor);
	}

	/*
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#runTaskImmediately(cz.cuni.mff.been.taskmanager.TaskDescriptor)
	 */
	public void runTaskImmediately(TaskDescriptor taskDescriptor)
		throws RemoteException, IllegalStateException {
		task.getHostRuntime().getTaskManager().runTaskImmediately(taskDescriptor);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#runTask(cz.cuni.mff.been.taskmanager.TaskDescriptor)
	 */
	public void runTasks(TaskDescriptor[] taskDescriptor) throws RemoteException {
		task.getHostRuntime().getTaskManager().runTask(taskDescriptor);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#newContext(java.lang.String, java.lang.String, java.lang.String, java.io.Serializable)
	 */
	public void newContext(String id, String name, String description,
		Serializable data) throws RemoteException {
		task.getHostRuntime().getTaskManager().newContext(id, name, description, data);
	}
		
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#endContext(java.lang.String)
	 */
	public void endContext(String id) throws RemoteException {
		task.getHostRuntime().getTaskManager().closeContext(id);
	}
	
	/** 
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getRegisteredHostRuntimes()
	 */
	public String[] getRegisteredHostRuntimes() throws RemoteException {
		return task.getHostRuntime().getTaskManager().getRegisteredHostRuntimes();
	}

	/** 
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#registerEventListener(cz.cuni.mff.been.taskmanager.HostRuntimeRegistrationListener)
	 */
	public void registerEventListener(HostRuntimeRegistrationListener listener) throws RemoteException {
		task.getHostRuntime().getTaskManager().registerEventListener(listener);
	}

	/** 
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#unregisterEventListener(cz.cuni.mff.been.taskmanager.HostRuntimeRegistrationListener)
	 */
	public void unregisterEventListener(HostRuntimeRegistrationListener listener) throws RemoteException {
		task.getHostRuntime().getTaskManager().unregisterEventListener(listener);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#log(cz.cuni.mff.been.logging.LogLevel, java.util.Date, java.lang.String)
	 */
	public void log(LogLevel level, Date timestamp, String message) throws RemoteException {
		task.log(level, timestamp, message);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getLogsForTask(java.lang.String, java.lang.String)
	 */
	public LogRecord[] getLogsForTask(String context, String taskID)
			throws RemoteException, LogStorageException, IllegalArgumentException,
			NullPointerException {
		return task.getHostRuntime().getTaskManager().getLogsForTask(context, taskID);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getStandardOutput(java.lang.String, java.lang.String)
	 */
	public OutputHandle getStandardOutput(String context, String taskID)
			throws RemoteException, LogStorageException, IllegalArgumentException,
			NullPointerException {
		return task.getHostRuntime().getTaskManager().getStandardOutput(context, taskID);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getErrorOutput(java.lang.String, java.lang.String)
	 */
	public OutputHandle getErrorOutput(String context, String taskID)
			throws RemoteException, LogStorageException, IllegalArgumentException,
			NullPointerException {
		return task.getHostRuntime().getTaskManager().getErrorOutput(context, taskID);
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#checkPointReached(java.lang.String, java.io.Serializable)
	 */
	public void checkPointReached(String name, Serializable value)
			throws RemoteException {
		CheckPoint checkPoint = new CheckPoint(
			task.getTaskDescriptor().getTaskTid(),
			task.getTaskDescriptor().getContextId(),
			name,
			value
		);
		checkPoint.setHostName(task.getTaskDescriptor().getHostRuntimesName()[0]);
		hostRuntimesPort.checkPointReached(checkPoint);

		task.writeMessage("Checkpoint \"" + checkPoint.getName()
			+ "\" set to \"" + checkPoint.getValue() + "\".");
	}
		
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#checkPointWait(java.lang.String, java.lang.String, java.lang.String, long)
	 */
	public Serializable checkPointWait(String contextID, String taskID, String name,
			long timeout) throws RemoteException {
		/* If context is null, use the context of the calling task. */
		String realContextID = contextID != null
			? contextID
			: task.getTaskDescriptor().getContextId();

		try {
			CheckPoint[] checkPoints = hostRuntimesPort.checkPointLook(
					new CheckPoint(taskID, realContextID, name, null),
					timeout
			);
		
			if (checkPoints.length > 0) {
				return checkPoints[0].getValue();
			} else {
				return null;
			}
		} catch (TaskManagerException e) {
			return null;
		}
	}
				
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#serviceRegister(cz.cuni.mff.been.taskmanager.ServiceEntry)
	 */
	public void serviceRegister(ServiceEntry service) throws RemoteException {
		hostRuntimesPort.serviceRegister(service);
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#serviceUnregister(cz.cuni.mff.been.taskmanager.ServiceEntry)
	 */
	public void serviceUnregister(ServiceEntry serviceTemplate)
			throws RemoteException {
		hostRuntimesPort.serviceUnregister(new ServiceEntry(
			serviceTemplate.getServiceName(),
			serviceTemplate.getInterfaceName(),
			serviceTemplate.getRmiAddress(),
			null, null, null)
		);
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#serviceFind(java.lang.String, java.lang.String)
	 */
	public Remote serviceFind(String serviceName, String interfaceName)
			throws RemoteException {
		return hostRuntimesPort.serviceFind(serviceName, interfaceName);
	}
	

	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#serviceFindAll()
	 */
	public ServiceEntry[] serviceFindAll() throws RemoteException {
		return hostRuntimesPort.serviceLook(new ServiceEntry(
			null,
			"main",
			null,
			null,
			null,
			null
		));
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getTaskDescriptor()
	 */
	public TaskDescriptor getTaskDescriptor() throws RemoteException {
		return task.getTaskDescriptor();
	}
	
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.TasksPortInterface#getTaskById(String, String)
	 */
	public TaskEntry getTaskById(String taskId, String contextId) throws RemoteException, IllegalArgumentException {
		return task.getHostRuntime().getTaskManager().getTaskById(taskId, contextId);
	}
	
	
}
