/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.File;
import java.io.IOException;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import cz.cuni.mff.been.benchmarkmanager.AnalysisEntity;
import cz.cuni.mff.been.benchmarkmanager.BenchmarkManagerException;
import cz.cuni.mff.been.benchmarkmanager.EntityStatus;
import cz.cuni.mff.been.benchmarkmanager.gui.BenchmarkManagerGUIInterface;
import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.resultsrepository.api.AnalysisType;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.GraphType;
import cz.cuni.mff.been.resultsrepository.gui.Statistics;


/**
 * Simple class for performing of logs collection from Task Manager and storing it
 * in database
 * 
 * @author Jakub Lehotsky
 */
public final class AnalysisEntityManager {
	
	/** Unable to contact Benchmark manager to get deletion permit error */ 
	private static final String BM_CONTACT_DELETE_PERMIT_ERROR 
		= "Unable to contact benchmark manager to get analysis deletion permit.";

    /** Single instance of load collector */
    private static AnalysisEntityManager analysisEntityManager = null;
	
    /** Databse that is used for results in Results Repository */
    private Database database;
    
    /** Metadata manager - loading and storing */
    private static MetadataManager metaManager;
    
    /** Internal statistics manager */
    private StatisticsManager statisticsManager;
    
    /** Internal graph manager */
    private GraphManager graphManager;
    
    /** Internal scripts runner reference */
    private RScriptsRunner rScriptsRunner;
    
    /**
     *  Construction, notify load collector about Results Repository database when constructing
     */
    private AnalysisEntityManager() {
    	database = Database.getInstance();
    }
    
    /**
     * Returns the single instance of load collector
     * 
     * @return	Reference to LoadCollector instance 
     */
    public static AnalysisEntityManager getInstance() {
    	if (analysisEntityManager == null) {
    		return new AnalysisEntityManager();
    	} else {
    		return analysisEntityManager;
    	}
    }
    
    /**
     * Use to initialize analysis entity manager 
     * 
     * @param database			Instance of Results Repository database
     * @param metaManager		Instance of Results Repository metadata manager
     * @param statisticsManager	Instance of Results Repository statistics manager
     * @param graphManager		Instance of Results Repository graph manager
     * @param scriptsRunner 
     */
    public void initialize(Database database, MetadataManager metaManager, 
    		StatisticsManager statisticsManager, GraphManager graphManager, RScriptsRunner scriptsRunner) {
    	
    	this.database = database;
    	AnalysisEntityManager.metaManager = metaManager;
    	this.statisticsManager = statisticsManager;
    	this.graphManager = graphManager;
    	this.rScriptsRunner = scriptsRunner;
    }
    
    /**
     * Adds new analysis into results repository, so it can be filled with entities and results,
     * it must not exist before.
     * 
     * @param description Description of analysis - all metadata for creation
     * 
     * @throws ResultsRepositoryException in case analysis already exists or some I/O error occurs 
     */
    public void addAnalysis(AnalysisEntity description)
    throws ResultsRepositoryException {
    
    	createAnalysis(description.getId());
    	
    	MetadataRef analysisMetadata = new MetadataRef(new Metadata());
    	
    	analysisMetadata.metadata.addItem("name", description.getName());
    	analysisMetadata.metadata.addItem("type", AnalysisType
    			.toString(description.isRegression() ? AnalysisType.REGRESSION
    					: AnalysisType.COMPARISON));
    	analysisMetadata.metadata.addItem("comment", description
    			.getDescription());
    	analysisMetadata.metadata.addItem("created on", new Date().toString());
    	analysisMetadata.metadata.addItem("experiments", "0");
    	analysisMetadata.metadata.addItem("valid experiments", "0");
    	analysisMetadata.metadata.addItem("complete experiments", "0");
    	analysisMetadata.metadata.addItem("valid", "true");
    	
    	ensureSuccessStoreAnalysisMetadata(analysisMetadata.metadata,
    			description.getId());
    }
    	
    /**
     * Allows adding new custom analysis metadata. 
     * 
     * @param analysisId	Analysis Id
     * @param key			Key of metadata item
     * @param value			Value of metadata item
     * @throws ResultsRepositoryException	In case you attempt to overwrite existing metadata item
     */

    public void addAnalysisMetadata(AID analysisId, String key, String value)
    	throws ResultsRepositoryException {
    	
    	ensureValidAid(analysisId);
    	MetadataRef analysisMetadata = new MetadataRef(new Metadata());
    	ensureSuccessLoadAnalysisMetadata(analysisMetadata, analysisId);
    	
    	if (analysisMetadata.metadata.getItemByKey(key) != null) {
    		throw new ResultsRepositoryException(
    				"Trying to overwrite analysis metadata - "
    						+ Database.getAnalysisReferenceString(analysisId) + ".");
    	}
    	
    	analysisMetadata.metadata.addItem(key, value);
    	ensureSuccessStoreAnalysisMetadata(analysisMetadata.metadata,
    			analysisId);
    	
    	}
    
    /* ************ Analysis ************** */
    
    /**
     * Initializes new analysis (creates it in the database)
     * 
     * @param aid
     *            Analysis ID
     * @throws ResultsRepositoryException
     *             In the case of error (cause IllegalArgumentException if
     *             analysis already exists)
     */
    public void createAnalysis(AID aid) throws ResultsRepositoryException {
    
    	File analysisDirectory = new File(database.getDirectoryInDatabase(aid));
    	if (analysisDirectory.exists() || isValidAid(aid)) {
    		throw new ResultsRepositoryException(new IllegalArgumentException(
    				Database.getAnalysisReferenceString(aid) + " already exists."));
    	}
    	
    	if (!analysisDirectory.mkdir()) {
    		throw new ResultsRepositoryException(
    				"Error creating analysis directory.");
    	}
    	
    	File analysisScriptsDirectory = new File(database.getRScriptsDirectory(aid));
    	if (!analysisScriptsDirectory.mkdir()) {
    		throw new ResultsRepositoryException(
    				"Error creating analysis scritps directory.");
    
    	}
    	
    	File analysisInvalidateScriptsDirectory = new File(database.getInvalidateScriptsDirectory(aid));
    	if (!analysisInvalidateScriptsDirectory.mkdir()) {
    		throw new ResultsRepositoryException(
    				"Error creating analysis invalidate scripts directory.");
    
    	}
    	
    }
    
    /**
	 * Asks benchmark manager, whether analysis is finished
	 * 
	 * @param aid	Analysis Id
	 * 
	 * @return	True if analysis is finished
	 */
	private boolean analysisFinished(AID aid) {
		
		EntityStatus analysisStatus = null;
		
		try {
			
		    BenchmarkManagerGUIInterface bmInstance = database.getBenchmarkManagerInstance();
		    if (bmInstance != null) {
		    	analysisStatus = bmInstance.getAnalysisInterface()
		    		.getEntity(aid).getStatus();
		    	Logger.logDebug(Database.getAnalysisReferenceString(aid) + " status is " + analysisStatus);
		    } else {
		    	Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
		    	return false;
		    }
		    
		} catch (RemoteException e) {
			
		    Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
		    return false;
		    
		} catch (IllegalArgumentException e) {
			
			Logger.logInfo("Test \"analysis finished\": analysis no longer exists in Benchmark Manager.");
			return true;
		}
		
		if (analysisStatus == null) {
			
			Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
			return false;
			
		}
		
		return analysisStatus.equals(EntityStatus.FINISHED); 
	}
    
    /**
     * Returns the list of analyses stored in Results Repository
     * 
     * @return The list
     */    
    @SuppressWarnings("unchecked")
    public List<Entity<AID>> getAnalyses() {
    
    	File[] directoryList;
    	ArrayList<Entity<AID>> result = new ArrayList<Entity<AID>>();
    
    	File path = new File(database.getDatabasePath());
    
    	directoryList = path.listFiles();
    
    	MetadataRef metadataFile = new MetadataRef(new Metadata());
    
    	for (int i = 0; i < directoryList.length; i++) {
    		if (directoryList[i].isDirectory()) {

    			String analDir = directoryList[i].getName();
    
    			AID aid;
    
    			try {
    				aid = new AID(new Long(analDir).longValue());
    			} catch (NumberFormatException e) {
    				Logger.logTrace(
    						"Encountered non-AID item in analyses: " + analDir + ".");
    				continue;
    			}
    
    			ensureSuccessLoadAnalysisMetadata(metadataFile, aid);
    			
    			result.add(new Entity<AID>(aid, metadataFile.metadata.getMetadata(),
    							new Statistics[] { new Statistics(
    									"No all-analyses statistics",
    									new String[] {}, new boolean[] {},
    									new String[] {}, new Double[][] {}) }, 
    									rScriptsRunner.isDeleteAnalysisAllowed(aid) &&
    									analysisFinished(aid)));
    		}
    	}
    
    	Collections.sort((List<Entity<AID>>) result);
    
    	return result;
    }
    
    /**
     * Checks whether AID conforms to some existing analysis in results
     * repository
     * 
     * @param aid
     *            Analysis ID
     * @return True iff analysis does exist in results repository
     */
    public boolean isValidAid(AID aid) {
    	File analysisDir = new File(database.getDirectoryInDatabase(aid));
    
    	return analysisDir.exists() && analysisDir.isDirectory()
    			&& analysisDir.canRead();
    }
    
    /**
     * Ensures that given AID belongs to a valid analysis in results repository,
     * otherwise throws an IllegalArgumentException
     * 
     * @param aid
     *            Analysis ID
     */
    public void ensureValidAid(AID aid) {
    	if (!isValidAid(aid)) {
    		throw new IllegalArgumentException(Database.getAnalysisReferenceString(aid)
    				+ " doesn't exist.");
    	}
    }
    
    /**
     * Returns the Entity&lt;AID&gt; object representing analysis for use in GUI
     * 
     * @param aid	Analysis Id
     * 
     * @return	The analysis entity
     */
    public Entity<AID> getAnalysis(AID aid) {
    
    	File analysisDir;
    
    	ensureValidAid(aid);
    
    	analysisDir = new File(database.getDirectoryInDatabase(aid));
    
    	MetadataRef metadataFile = new MetadataRef(new Metadata());
    	ensureSuccessLoadAnalysisMetadata(metadataFile, aid);
    
    	Statistics[] stat = statisticsManager.loadAnalysisStatistics(aid);
    
    	Entity<AID> result = new Entity<AID>(new AID(new Long(analysisDir
    			.getName()).longValue()), metadataFile.metadata.getMetadata(),
    			stat, rScriptsRunner.isDeleteAnalysisAllowed(aid) && analysisFinished(aid));
    
    	return result;
    }

    /**
     * Tries to load analysis metadata and notify, whether it was successful, eventually
     * log error
     * 
     * @param analysisMetadata	Reference to metadata object, where they will be stored
     * @param aid	Analysis Id
     * @return	True if metadata was loaded successfuly
     */
    public static boolean ensureSuccessLoadAnalysisMetadata(
    		MetadataRef analysisMetadata, AID aid) {
    	try {
    		analysisMetadata.metadata = metaManager.loadAnalysisMetadata(aid);
    	} catch (IOException e) {
    		e.printStackTrace();
    		Logger.logError(Metadata.IO_ERROR_METADATA_READING_MESSAGE
    						+ Database.getAnalysisReferenceString(aid));
    		return false;
    	}
    	return true;
    }

    /**
     * Tries to store analysis metadata and notify, whether it was successful, eventually
     * log error
     * 
     * @param analysisMetadata	Reference to metadata object, where they will be stored
     * @param aid	Analysis Id
     * @return	True if metadata was loaded successfuly
     */
    public static boolean ensureSuccessStoreAnalysisMetadata(
    		Metadata analysisMetadata, AID aid) {
    	try {
    		metaManager.storeAnalysisMetadata(aid, analysisMetadata);
    	} catch (IOException e) {
    		e.printStackTrace();
    		Logger.logError(Metadata.IO_ERROR_METADATA_STORING_MESSAGE
    						+ Database.getAnalysisReferenceString(aid));
    		return false;
    	}
    	return true;
    }
    
    /**
     * Sets the comment metadata of an analysis
     * 
     * @param aid	Analysis Id
     * @param comment	Comment to set
     */
    public void setAnalysisComment(AID aid, String comment) {
    
    	ensureValidAid(aid);
    	metaManager.setAnalysisMetadataItem(aid, "comment", comment);
    }
    
    /**
     * Deletes analysis from database (all metadata, experiments, results, logs,
     * load info ...)
     * 
     * @param aid
     *            Analysis ID
     * @return True iff succesful
     * 
     * @throws DeleteLockedException	If analysis is locked for deleting 
     */
    public boolean eraseAnalysis(AID aid) throws DeleteLockedException {
    
    	ensureValidAid(aid);
    	
    	synchronized (LockManager.eraseLock) {
    	
    		if (!rScriptsRunner.isDeleteAnalysisAllowed(aid) || !analysisFinished(aid)) {
    			Logger.logWarning("Tried to delete locked " 
    					+ Database.getAnalysisReferenceString(aid) + ".");				
    			throw new DeleteLockedException("Analysis is being processed, unable to delete.");
    		}

    		try {
    				
			    BenchmarkManagerGUIInterface bmInstance = database.getBenchmarkManagerInstance();
			    
			    
			    if (bmInstance != null) {
			    	bmInstance.getAnalysisInterface().deleteEntity(aid);
			    } else {
			    	Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
			    	return false;
			    }
			    
			} catch (RemoteException e) {
				
			    Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
			    return false;
			    
			} catch (BenchmarkManagerException e) {
				
				Logger.logError("Benchmark Manager couldn't delete analysis : " + e.getMessage());
				throw new DeleteLockedException("Analysis has active experiments.");
				
			} catch (IllegalArgumentException e) {
				
				// analysis is no more in Benchmark Manager, we can delete it
				Logger.logInfo("Analysis to delete no longer exists in Benchmark Manager.");
				
			}    		
    		
    		File file = new File(database.getDirectoryInDatabase(aid));
    		return Database.eraseDirectoryRecursively(file);
    		
    	}
    	
    }
        
    /**
     * Uploads analysis file (short, passed as an argument)
     * 
     * @see cz.cuni.mff.been.resultsrepository.Database#uploadFile
     * 
     * @param aid
     *            Analysis ID
     * @param name
     *            Name of file to store
     * @param host
     *            Hostname to connect to for data retrieving
     * @param port
     *            Port used in connection with host
     * @param blocking
     *            Determines, whether to use blocking transfer
     * @return Handle of network connection (null for blocking upload)
     * @throws UnknownHostException
     *             In the case host could not be found
     * @throws IOException
     *             In the case of some IO error
     * @throws ResultsRepositoryBusyException
     *             In the case of non-blocking upload when Results Repository is
     *             busy with (down/)uploading files
     */
    public UploadHandle uploadAnalysisFile(AID aid, String name, String host,
    		int port, boolean blocking) throws UnknownHostException,
    		IOException, ResultsRepositoryBusyException {
    
    	ensureValidAid(aid);
    
    	return database.uploadFile(database.getFileInDatabase(aid, name), host, port,
    			blocking, false, null, null);
    }
    
    /**
     * Download analysis file
     * 
     * @see cz.cuni.mff.been.resultsrepository.Database#downloadFile
     * 
     * @param aid
     *            Analysis ID
     * @param name
     *            Name of file to download
     * @param host
     *            Hostname to connect to for data retrieving
     * @param port
     *            Port used in connection with host
     * @param blocking
     *            Determines, whether to use blocking transfer
     * @return Handle of network connection
     * @throws UnknownHostException
     *             In the case host could not be found
     * @throws IOException
     *             In the case of some IO error
     * @throws ResultsRepositoryBusyException
     *             In the case of non-blocking upload when Results Repository is
     *             busy with (down/)uploading files
     */
    public DownloadHandle downloadAnalysisFile(AID aid, String name,
    		String host, int port, boolean blocking)
    		throws UnknownHostException, IOException,
    		ResultsRepositoryBusyException {
    
    	ensureValidAid(aid);
    
    	return database.downloadFile(database.getFileInDatabase(aid, name), host, port,
    			blocking);
    }
    
    /**
     * Returns the analysis graph count of a given graph type
     * 
     * @param graphType	Load or Results graph type
     * @param aid	Analysis Id
     * @return	The count of graphs
     */
    public int getAnalysisGraphCount(GraphType graphType, AID aid) {
    	ensureValidAid(aid);
    	return graphManager.getAnalysisGraphCount(graphType, aid);
    }
    
    /**
     * Returns the analysis graph info for a given graph. GraphInfo object returned contains 
     * width, height and title of the graph
     * 
     * @param graphType	Load or Results graph type
     * @param aid	Analysis Id
     * @param index	Index of graph - can be one of 0..getAnalysisGraphCount()-1
     * @return	The graph info
     * 
     * @throws IOException	In case of error of getting metadata
     */
    public GraphInfo getAnalysisGraphInfo(GraphType graphType, AID aid,
    		int index) throws IOException {
    	ensureValidAid(aid);
    	if (!graphManager.validAnalysisGraph(graphType, aid, index)) {
    		throw new IllegalArgumentException(GraphManager.GRAPH_DOESNT_EXIST_MESSAGE);
    	}
    	return graphManager.getAnalysisGraphInfo(graphType, aid, index);
    }
    
    /**
     * Returns the complete path for a given analysis graph
     * 
     * @param graphType
     * 		Load or Results graph type
     * @param aid
     * 		Analysis Id
     * @param index
     * 		Index of graph - can be one of 0..getAnalysisGraphCount()-1
     * @return
     * 		The path to the file with graph image on disk
     * @throws IOException
     * 		if this info couldn't be retrieved because of I/O problems
     * @throws IllegalArgumentException 
     * 		if graph doesn't exist
     */
    public String getAnalysisGraphPath(GraphType graphType, AID aid, int index)
    		throws IOException {
    	ensureValidAid(aid);
    	if (!graphManager.validAnalysisGraph(graphType, aid, index)) {
    		throw new IllegalArgumentException(GraphManager.GRAPH_DOESNT_EXIST_MESSAGE);
    	}
    	return graphManager.getAnalysisGraphPath(graphType, aid, index);
    }
    
}
