/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import cz.cuni.mff.been.benchmarkmanager.BinaryEntity;
import cz.cuni.mff.been.benchmarkmanager.EntityStatus;
import cz.cuni.mff.been.benchmarkmanager.gui.BenchmarkManagerGUIInterface;
import cz.cuni.mff.been.common.BeenUtils;
import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.Statistics;



/**
 * Simple class for performing of logs collection from Task Manager and storing it
 * in database
 * 
 * @author Jakub Lehotsky
 */
public final class BinaryEntityManager {
	
	/** Unable to contact Benchmark manager to get deletion permit error */ 
	private static final String BM_CONTACT_DELETE_PERMIT_ERROR 
		= "Unable to contact benchmark manager to get binary deletion permit.";

	/** Single instance of load collector */
	private static BinaryEntityManager binaryEntityManager = null;
	
	/** Databse that is used for results in Results Repository */
	private Database database;

	/** Metadata manager - loading and storing */
	private static MetadataManager metaManager;
	
	/** Internal statistics manager */
	private StatisticsManager statisticsManager;

	/** Internal task list manager */
	private TaskListManager taskListManager;
	
	/** Internal scripts runner reference */
	private RScriptsRunner rScriptsRunner;
	
	/** Internal analysis entity manager */
	private AnalysisEntityManager analysisEntityManager;

	/** Internal analysis entity manager */
	private ExperimentEntityManager experimentEntityManager;

	

	/**
	 *  Construction, notify load collector about Results Repository database when constructing
	 */
	private BinaryEntityManager() {
		database = Database.getInstance();
	}
	
	/**
	 * Returns the single instance of load collector
	 * 
	 * @return	Reference to LoadCollector instance 
	 */
	public static BinaryEntityManager getInstance() {
		if (binaryEntityManager == null) {
			return new BinaryEntityManager();
		} else {
			return binaryEntityManager;
		}
	}	
	
	/**
	 * Use to initialize analysis entity manager 
	 * 
	 * @param database			Instance of Results Repository database
	 * @param metaManager		Instance of Results Repository metadata manager
	 * @param statisticsManager	Instance of Results Repository statistics manager
	 * @param taskListManager 	Instance of Results Repository task list manager
	 * @param analysisEntityManager	Instance of Results Repository analysis entity manager
	 * @param scriptsRunner 
	 * @param experimentEntityManager Instance of Results Repository experiment entity manager

	 */
	public void initialize(Database database, MetadataManager metaManager, 
			StatisticsManager statisticsManager, TaskListManager taskListManager,
			RScriptsRunner scriptsRunner, AnalysisEntityManager analysisEntityManager,
			ExperimentEntityManager experimentEntityManager) {
		
		this.database = database;
		BinaryEntityManager.metaManager = metaManager;
		this.statisticsManager = statisticsManager;
		this.taskListManager = taskListManager;
		this.rScriptsRunner = scriptsRunner;
		
		this.analysisEntityManager = analysisEntityManager;		
		this.experimentEntityManager = experimentEntityManager;
	}
	
	
	/**
	 * Adds new binary into given analysis and experiment of results repository,
	 * so it can be filled with entities and results, it must not exist before.
	 * 
	 * @param description
	 *            Object describing binary being entered into Results Repository
	 * 
	 * @throws ResultsRepositoryException
	 * @see cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface#addBinary
	 */
	public void addBinary(BinaryEntity description)
			throws ResultsRepositoryException {
		String runsPerBinary;
		String metaItem;
		Integer binaries;
		Integer validBinaries;

		AID analysisId = description.getAnalysis();
		EID experimentId = description.getExperiment();

		analysisEntityManager.ensureValidAid(analysisId);
		experimentEntityManager.ensureValidEid(analysisId, experimentId);

		createBinary(analysisId, experimentId, description.getId());

		MetadataRef experimentMetadata = new MetadataRef(new Metadata());
		MetadataRef analysisMetadata = new MetadataRef(new Metadata());

		ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(experimentMetadata, analysisId,
				experimentId);
		AnalysisEntityManager.ensureSuccessLoadAnalysisMetadata(analysisMetadata, analysisId);

		runsPerBinary = experimentMetadata.metadata
				.getItemByKey("expected runs per binary");
		if (runsPerBinary == null) {
			throw new ResultsRepositoryException(
					"Experiment metadata error, 'expected runs per binary' "
					+ "item not found when creating binary.");
		}

		MetadataRef binaryMetadata = new MetadataRef(new Metadata());
		binaryMetadata.metadata.addItem("name", description.getName());
		binaryMetadata.metadata.addItem("comment", "");
		binaryMetadata.metadata.addItem("created on", new Date().toString());
		binaryMetadata.metadata.addItem("runs", "0");
		binaryMetadata.metadata.addItem("valid runs", "0");
		binaryMetadata.metadata.addItem("valid", "true");
		binaryMetadata.metadata.addItem("expected runs", runsPerBinary);
		binaryMetadata.metadata.addItem("complete", "false");
		binaryMetadata.metadata.addItem("last valid run idx", "0");
		binaryMetadata.metadata.addItem("last crashed run idx", "0");

		ensureSuccessStoreBinaryMetadata(binaryMetadata.metadata, analysisId,
				experimentId, description.getId());

		/* Now update binaries count in experiment metadata */

		if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
				.getItemByKey("binaries"), "binaries",
				Database.getAnalysisReferenceString(analysisId))) {
			binaries = new Integer(metaItem);
		} else {
			binaries = new Integer(0);
		}

		if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
				.getItemByKey("valid binaries"), "binaries",
				Database.getAnalysisReferenceString(analysisId))) {
			validBinaries = new Integer(metaItem);
		} else {
			validBinaries = new Integer(0);
		}

		binaries++;
		experimentMetadata.metadata.setItemByKey("binaries", binaries
				.toString());

		validBinaries++;
		experimentMetadata.metadata.setItemByKey("valid binaries", binaries
				.toString());

		if (experimentMetadata.metadata.getItemByKey("valid").equals("false")) {
			/* New experiment become valid in analysis */
			int validExps = Integer.parseInt(analysisMetadata.metadata
					.getItemByKey("valid experiments"));
			analysisMetadata.metadata.setItemByKey("valid experiments", String
					.valueOf(validExps + 1));
			analysisMetadata.metadata.setItemByKey("valid", "true");
			if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(analysisMetadata.metadata,
					analysisId)) {
				return;
			}
			experimentMetadata.metadata.setItemByKey("valid", "true");
		}

		if (!ExperimentEntityManager.ensureSuccessStoreExperimentMetadata(experimentMetadata.metadata,
				analysisId, experimentId)) {
			return;
		}
		
		String[] taskNames = null; 
		
		try {
			BenchmarkManagerGUIInterface bmInstance = database.getBenchmarkManagerInstance();
			
			if (bmInstance == null) {
		    	Logger.logError("Error contacting Benchmark Manager.");
		    } else {
		    	taskNames = BeenUtils.extractTaskIDs(bmInstance.getAnalysisInterface()
		    		.getSubentityInterface(analysisId)
		    		.getSubentityInterface(experimentId).getAllTasks(description.getId()));
		    	
				if (taskNames == null) {
					Logger.logError("Task names list of binary is null.");
					return;
				}
		                
				synchronized (LockManager.taskListLock) {
					
					taskListManager.loadBinaryTaskList(analysisId, experimentId, description.getId());
					taskListManager.addNewBinaryTasks(taskNames);
					taskListManager.storeBinaryTaskList(analysisId, experimentId, description.getId());
					
				}

		    }
		} catch (RemoteException e) {
			Logger.logError("Error contacting Benchmark Manager.");
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			Logger.logError("Entities integrity error while creating binary.");
			e.printStackTrace();
		}
                

	}

        /**
         * Allows adding new custom binary metadata. 
         * 
         * @param analysisId	Analysis Id
         * @param experimentId	Experiment Id
         * @param binaryId
         * 
         * @param key			Key of metadata item
         * @param value			Value of metadata item
         * @throws ResultsRepositoryException	In case you attempt to overwrite existing metadata item
         */
	public void addBinaryMetadata(AID analysisId, EID experimentId,
			BID binaryId, String key, String value)
			throws ResultsRepositoryException {

		ensureValidBid(analysisId, experimentId, binaryId);
		MetadataRef binaryMetadata = new MetadataRef(new Metadata());
		ensureSuccessLoadBinaryMetadata(binaryMetadata, analysisId,
				experimentId, binaryId);

		if (binaryMetadata.metadata.getItemByKey(key) != null) {
			throw new ResultsRepositoryException(
					"Trying to overwrite analysis metadata - "
							+ Database.getBinaryReferenceString(analysisId,
									experimentId, binaryId) + ".");
		}

		binaryMetadata.metadata.addItem(key, value);

		ensureSuccessStoreBinaryMetadata(binaryMetadata.metadata, analysisId,
				experimentId, binaryId);

	}

	/* ************* Binary ************** */

	/**
	 * Initializes new binary (creates it in database - for a given experiment
	 * in a given analysis)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID to be created
	 * @throws ResultsRepositoryException
	 */
	public void createBinary(AID aid, EID eid, BID bid)
			throws ResultsRepositoryException {

		if (isValidBid(aid, eid, bid)) {
			throw new ResultsRepositoryException(new IllegalArgumentException(
					Database.getBinaryReferenceString(aid, eid, bid)
							+ " already exists."));
		}

		File binaryDirectory = new File(database.getDirectoryInDatabase(aid, eid, bid));
		if (!binaryDirectory.mkdir()) {
			throw new ResultsRepositoryException(
					"Error creating binary directory...");
		}
		
		File binaryInvalidateScriptsDirectory 
			= new File(database.getInvalidateScriptsDirectory(aid, eid, bid));
		if (!binaryInvalidateScriptsDirectory.mkdir()) {
			throw new ResultsRepositoryException(
					"Error creating binary invalidate scripts directory.");
		}
		
		File loadInfoDirectory
			= new File(database.getLoadDirectory(aid, eid, bid));
		if (!loadInfoDirectory.mkdir()) {
			throw new ResultsRepositoryException(
					"Error creating binary load storage scripts directory.");
		}
	
	}
	
	/**
	 * Asks benchmark manager, whether experiment is finished
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * 
	 * @return	True if experiment is finished
	 */
	public boolean binaryFinished(AID aid, EID eid, BID bid) {
		
		EntityStatus binaryStatus = null;
		
		try {
			
		    BenchmarkManagerGUIInterface bmInstance = database.getBenchmarkManagerInstance();
		    if (bmInstance != null) {
			Logger.logDebug(Database.getBinaryReferenceString(aid, eid, bid) + " status is " + binaryStatus);
		    	binaryStatus = bmInstance.getAnalysisInterface()
		    		.getSubentityInterface(aid).getSubentityInterface(eid)
		    		.getEntity(bid).getStatus();
		    } else {
		    	Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
		    	return false;
		    }
		    
		} catch (RemoteException e) {
			
		    Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
		    return false;
		    
		} catch (IllegalArgumentException e) {
			
			Logger.logInfo("Test \"binary finished\": binary no longer exists in Benchmark Manager.");
			return true;
		}
		
		if (binaryStatus == null) {
			
			Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
			return false;
			
		}
		
		return binaryStatus.equals(EntityStatus.FINISHED); 
	}


    /**
     * Returns the list of binaries in a given experiment stored in Results Repository
     * 
     * @param aid	Analysis Id 
     * @param eid 	Experiment Id
     * 
     * @return The list
     */    
	@SuppressWarnings("unchecked")
	public List<Entity<BID>> getBinariesInExperiment(AID aid, EID eid) {

		experimentEntityManager.ensureValidEid(aid, eid);

		File[] directoryList;
		ArrayList<Entity<BID>> result = new ArrayList<Entity<BID>>();

		File path = new File(database.getDirectoryInDatabase(aid, eid));
		if ((directoryList = path.listFiles()) == null) {
			throw new RuntimeException(new ResultsRepositoryException(
					"Error reading binaries directory ("
							+ Database.getExperimentReferenceString(aid, eid) + ")."));
		}

		MetadataRef metadataFile = new MetadataRef(new Metadata());
		MetadataRef metadataExperimentFile = new MetadataRef(new Metadata());

		for (int i = 0; i < directoryList.length; i++) {
			if (directoryList[i].isDirectory()) {

				BID bid;
				try {
					bid = new BID(new Long(directoryList[i].getName())
							.longValue());
				} catch (NumberFormatException e) {
					Logger.logTrace(
							"Encountered non-BID item in experiment: "
									+ directoryList[i].getName() + ".");
					continue;
				}
				
				ensureSuccessLoadBinaryMetadata(metadataFile, aid, eid, bid);
				ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(
						metadataExperimentFile, aid, eid);
				
				Statistics[] stat = statisticsManager.loadBinaryStatistics(aid,
						eid, bid);
				
				String metaItem;
				boolean completeBinary;
				boolean completeExperiment;
				
				if (Database.ensureMetadataNotNull(metaItem = metadataFile.metadata
						.getItemByKey("complete"), "complete",
						Database.getBinaryReferenceString(aid, eid, bid))) {
					completeBinary = new Boolean(metaItem);
				} else {
					/* invalid metadata shouldn't prevent deleting of entity */
					completeBinary = true; 
				}

				if (Database.ensureMetadataNotNull(metaItem = metadataExperimentFile.metadata
						.getItemByKey("complete"), "complete",
						Database.getExperimentReferenceString(aid, eid))) {
					completeExperiment = new Boolean(metaItem);
				} else {
					/* invalid metadata shouldn't prevent deleting of entity */
					completeExperiment = true; 
				}
				
				result.add(new Entity<BID>(bid, metadataFile.metadata
						.getMetadata(), stat, 
						rScriptsRunner.isDeleteBinaryAllowed(aid, eid, bid) && ( (completeBinary 
								&&  completeExperiment) || 
									experimentEntityManager.experimentFinished(aid, eid) 
								)
				));
			}
		}

		Collections.sort((List<Entity<BID>>) result);

		return result;
	}

	/**
	 * Checks whether BID conforms to some existing binary in results
	 * repository
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @return True iff analysis does exist in results repository
	 */
	public boolean isValidBid(AID aid, EID eid, BID bid) {

		File binaryDir = new File(database.getDirectoryInDatabase(aid, eid, bid));
		return binaryDir.exists() && binaryDir.isDirectory()
				&& binaryDir.canRead();
	}

    /**
     * Ensures that given BID belongs to a valid binary in a given experiment in results repository,
     * otherwise throws an IllegalArgumentException
     * 
     * @param aid
	 *		Analysis ID
     * @param eid
     * 		Experiment Id
     * @param bid
     * 		Binary Id          
     */
	public void ensureValidBid(AID aid, EID eid, BID bid) {
		if (!isValidBid(aid, eid, bid)) {
			throw new IllegalArgumentException(Database.getBinaryReferenceString(aid,
					eid, bid)
					+ " doesn't exist.");
		}
	}
	
    /**
     * Returns the flag, whether binary is deltable. Binary is deletable if it is complete and no script 
     * is running for it or its runs or it has "finished" state in Benchmark manager. 
     * 
     * Synchronization note: binary may come non-deletable between this call and call to eraseBinary()
     * 
     * @param aid	Analysis Id
     * @param eid	Experiment Id 
     * @param bid	Binary Id 
     * @return	True if deletable
     */	
	private boolean isDeletable(AID aid, EID eid, BID bid) {
		
		MetadataRef metadataBinaryFile = new MetadataRef(new Metadata());
		MetadataRef metadataExperimentFile = new MetadataRef(new Metadata());

		ensureSuccessLoadBinaryMetadata(metadataBinaryFile, aid, eid, bid);
		ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(
				metadataExperimentFile, aid, eid);

		String metaItem;
		boolean completeBinary;
		boolean completeExperiment;
		
		if (Database.ensureMetadataNotNull(metaItem = metadataBinaryFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getBinaryReferenceString(aid, eid, bid))) {
			completeBinary = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			completeBinary = true; 
		}

		if (Database.ensureMetadataNotNull(metaItem = metadataExperimentFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getExperimentReferenceString(aid, eid))) {
			completeExperiment = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			completeExperiment = true; 
		}
		
		return completeBinary && completeExperiment;

	}

	/**
	 * Returns the Entity&lt;BID&gt; object representing binary for use in GUI
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id 
	 * 
	 * @return	The binary entity
	 */
	public Entity<BID> getBinary(AID aid, EID eid, BID bid) {

		File binaryDir;
		Entity<BID> result;

		ensureValidBid(aid, eid, bid);

		binaryDir = new File(database.getDirectoryInDatabase(aid, eid, bid));

		MetadataRef metadataFile = new MetadataRef(new Metadata());
		MetadataRef metadataExperimentFile = new MetadataRef(new Metadata());
		
		ensureSuccessLoadBinaryMetadata(metadataFile, aid, eid, bid);
		ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(
				metadataExperimentFile, aid, eid);

		Statistics[] stat = statisticsManager.loadBinaryStatistics(aid, eid,
				bid);
		
		String metaItem;
		boolean completeBinary;
		boolean completeExperiment;
		
		if (Database.ensureMetadataNotNull(metaItem = metadataFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getBinaryReferenceString(aid, eid, bid))) {
			completeBinary = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			completeBinary = true; 
		}

		if (Database.ensureMetadataNotNull(metaItem = metadataExperimentFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getExperimentReferenceString(aid, eid))) {
			completeExperiment = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			completeExperiment = true; 
		}
		
		result = new Entity<BID>(new BID(new Long(binaryDir.getName())
				.longValue()), metadataFile.metadata.getMetadata(), stat, 
				rScriptsRunner.isDeleteBinaryAllowed(aid, eid, bid) && completeBinary 
				&& (completeExperiment || experimentEntityManager.experimentFinished(aid, eid)));

		return result;
	}

	/**
	 * Tries to load binary metadata and notify, whether it was successful, eventually
	 * log error
	 * 
	 * @param binaryMetadata	Reference to metadata object, where they will be stored
	 * @param aid		Analysis Id
	 * @param eid		Experiment Id
	 * @param bid		Binary Id 
	 * 
	 * @return	True if metadata was loaded successfuly
	 */
	public static boolean ensureSuccessLoadBinaryMetadata(
			MetadataRef binaryMetadata, AID aid, EID eid, BID bid) {
		try {
			binaryMetadata.metadata = metaManager.loadBinaryMetadata(aid, eid,
					bid);
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(
					Metadata.IO_ERROR_METADATA_READING_MESSAGE
							+ Database.getBinaryReferenceString(aid, eid, bid));
			return false;
		}
		return true;
	}

	/**
	 * Tries to store binary metadata and notify, whether it was successful, eventually
	 * log error
	 * 
	 * @param binaryMetadata	Reference to metadata object, where they will be stored
	 * @param aid		Analysis Id
	 * @param eid		Experiment Id
	 * @param bid		Binary Id 
	 * 
	 * @return	True if metadata was loaded successfuly
	 */
	public static boolean ensureSuccessStoreBinaryMetadata(
			Metadata binaryMetadata, AID aid, EID eid, BID bid) {
		try {
			metaManager.storeBinaryMetadata(aid, eid, bid, binaryMetadata);
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(
					Metadata.IO_ERROR_METADATA_STORING_MESSAGE
							+ Database.getBinaryReferenceString(aid, eid, bid));
			return false;
		}
		return true;
	}

	/**
	 * Sets the comment metadata of a binary
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param comment	Comment to set
	 */
	public void setBinaryComment(AID aid, EID eid, BID bid, String comment) {

		ensureValidBid(aid, eid, bid);
		metaManager.setBinaryMetadataItem(aid, eid, bid, "comment", comment);
	}

	/**
	 * Deletes given binary
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * 
	 * @return True iff successful
	 * 
	 * @throws DeleteLockedException	In case binary is locked for deleting 
	 */
	public boolean eraseBinary(AID aid, EID eid, BID bid) 
		throws DeleteLockedException {

		ensureValidBid(aid, eid, bid);
		
		synchronized (LockManager.eraseLock) {
			
			/* Note: for further improvement of synchronization, callbacks should be locked too 
			 * (because they handle metadata of existing entities), now it cannot happen. For 
			 * additional measurements, it must be introduced */
			
			if (!(rScriptsRunner.isDeleteBinaryAllowed(aid, eid, bid) && 
					(isDeletable(aid, eid, bid) || experimentEntityManager.experimentFinished(aid, eid))
				)) {
				Logger.logWarning("Tried to delete locked " +
						Database.getBinaryReferenceString(aid, eid, bid) + ".");				
				throw new DeleteLockedException("Binary is being processed, unable to delete.");
			}
	
	
			MetadataRef analysisMetadata = new MetadataRef(new Metadata());
			MetadataRef experimentMetadata = new MetadataRef(new Metadata());
			MetadataRef binaryMetadata = new MetadataRef(new Metadata());
	
			Integer binaries;
			Integer completeBinaries;
			Integer expectedBinaries;
			Integer validBinaries;
			String metaItem;
			boolean valid;
			boolean complete;
	
			/* Update binaries count in experiment metadata */
			if (!AnalysisEntityManager.ensureSuccessLoadAnalysisMetadata(analysisMetadata, aid)
					|| !ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(experimentMetadata,
							aid, eid)
					|| !ensureSuccessLoadBinaryMetadata(binaryMetadata, aid, eid,
							bid)) {
				
				return false;
			}
	
			if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
					.getItemByKey("complete binaries"), "complete binaries",
					Database.getExperimentReferenceString(aid, eid))) {
				completeBinaries = new Integer(metaItem);
			} else {
				completeBinaries = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
					.getItemByKey("binaries"), "binaries",
					Database.getExperimentReferenceString(aid, eid))) {
				binaries = new Integer(metaItem);
			} else {
				binaries = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
					.getItemByKey("expected binaries"), "expected binaries",
					Database.getExperimentReferenceString(aid, eid))) {
				expectedBinaries = new Integer(metaItem);
			} else {
				expectedBinaries = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
					.getItemByKey("valid binaries"), "valid binaries",
					Database.getExperimentReferenceString(aid, eid))) {
				validBinaries = new Integer(metaItem);
			} else {
				validBinaries = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = binaryMetadata.metadata
					.getItemByKey("valid"), "valid", Database.getBinaryReferenceString(aid,
					eid, bid))) {
				valid = metaItem.equals("true");
			} else {
				valid = false;
			}
			
			if (Database.ensureMetadataNotNull(metaItem = binaryMetadata.metadata
					.getItemByKey("complete"), "complete", Database.getBinaryReferenceString(aid,
					eid, bid))) {
				complete = metaItem.equals("true");
			} else {
				complete = false;
			}
	
			if (complete) {
				completeBinaries--; 
			}
			
			expectedBinaries--;
			binaries--;
			experimentMetadata.metadata.setItemByKey("binaries", binaries
					.toString());
			experimentMetadata.metadata.setItemByKey("complete binaries",
					completeBinaries.toString());
			experimentMetadata.metadata.setItemByKey("expected binaries",
					expectedBinaries.toString());
	
			if (valid && binaries > 0) {
				validBinaries--;
				experimentMetadata.metadata.setItemByKey("valid binaries",
						validBinaries.toString());
				if (validBinaries == 0) {
					experimentMetadata.metadata.setItemByKey("valid", "false");
	
					int validExps = Integer.parseInt(analysisMetadata.metadata
							.getItemByKey("valid experiments"));
					analysisMetadata.metadata.setItemByKey("valid experiments",
							String.valueOf(validExps - 1));
	
					if (validExps == 0) {
						analysisMetadata.metadata.setItemByKey("valid", "false");
						analysisMetadata.metadata.setItemByKey("valid experiments",
								"0");
						if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(
								analysisMetadata.metadata, aid)) {
							return false;
						}
	
					} else {
						if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(
								analysisMetadata.metadata, aid)) {
							return false;
						}
					}
				}
			}
	
			if (binaries == 0) {
	
				if (experimentMetadata.metadata.getItemByKey("valid").equals(
						"false")) {
					/* New experiment become valid in analysis */
					int validExps = Integer.parseInt(analysisMetadata.metadata
							.getItemByKey("valid experiments"));
					analysisMetadata.metadata.setItemByKey("valid experiments",
							String.valueOf(validExps + 1));
					analysisMetadata.metadata.setItemByKey("valid", "true");
					if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(
							analysisMetadata.metadata, aid)) {
						return false;
					}
					experimentMetadata.metadata.setItemByKey("valid", "true");
				}
			}
	
			if (!ExperimentEntityManager.ensureSuccessStoreExperimentMetadata(experimentMetadata.metadata,
					aid, eid)) {
				return false;
			}
	
			File file = new File(database.getDirectoryInDatabase(aid, eid, bid));
			boolean result = Database.eraseDirectoryRecursively(file);
			
			/* Now execute invalidation callbacks */
			try {
				
				File scriptsFile = new File(
		        		database.getInvalidateScriptsDirectory(aid, eid) + File.separator +
		        		RManager.INVALIDATE_SCRIPT_FILENAME);
				
				if (scriptsFile.exists()) {
		        		
			        BufferedReader scriptsReader = new BufferedReader(new FileReader(scriptsFile));
			        		        
			        String script;
			        
			        while ((script = scriptsReader.readLine()) != null) {
			        	
			        	if (!script.trim().equals("")) {
			        		rScriptsRunner.enqueueScriptExecution(aid, eid, script);
			        	}			        	
			        }
			        
			        scriptsReader.close();
				}
			        
		    } catch (IOException e) {
		    	Logger.logError("I/O error accessing invalidation script files.");
		    }
			
			return result;
		}
	}

	/**
	 * Returns the context of a given binary
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @return	The context Id 
	 */
	public String getContext(AID aid, EID eid, BID bid) {
		// Note: bid is not used, as contexts are mapped to experiments
		return experimentEntityManager.getExperimentContext(aid, eid);
	}

	/**
	 * Uploads binary file (through TCP)
	 * 
	 * @see cz.cuni.mff.been.resultsrepository.Database#uploadFile
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @param name
	 *            Name of file to store
	 * @param host
	 *            Hostname to connect to for data retrieving
	 * @param port
	 *            Port used in connection with host
	 * @param blocking
	 *            Determines, whether to use blocking transfer
	 * @return Handle of network connection
	 * 
	 * @throws UnknownHostException
	 *             In the case host could not be found
	 * @throws IOException
	 *             In the case of some IO error
	 * @throws ResultsRepositoryBusyException
	 *             In the case of non-blocking upload when Results Repository is
	 *             busy with (down/)uploading files
	 */
	public UploadHandle uploadBinaryFile(AID aid, EID eid, BID bid,
			String name, String host, int port, boolean blocking)
			throws UnknownHostException, IOException,
			ResultsRepositoryBusyException {

		ensureValidBid(aid, eid, bid);

		if (database.isBusy() && !blocking) {
			throw new ResultsRepositoryBusyException(
					Database.TOO_MANY_CONNECTIONS_MESSAGE);
		}

		return database.uploadFile(database.getFileInDatabase(aid, eid, bid, name), host, port,
				blocking, false, null, null);
	}

	/**
	 * Download binary file data from database
	 * 
	 * @see cz.cuni.mff.been.resultsrepository.Database#downloadFile
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @param name
	 *            Name of file to store
	 * @param host
	 *            Hostname to connect to for data retrieving
	 * @param port
	 *            Port used in connection with host
	 * @param blocking
	 *            Determines, whether to use blocking transfer
	 * @return Handle of network connection
	 * 
	 * @throws UnknownHostException
	 *             In the case host could not be found
	 * @throws IOException
	 *             In the case of some IO error
	 * @throws ResultsRepositoryBusyException
	 *             In the case of non-blocking upload when Results Repository is
	 *             busy with (down/)uploading files
	 */
	public DownloadHandle downloadBinaryFile(AID aid, EID eid, BID bid,
			String name, String host, int port, boolean blocking)
			throws UnknownHostException, IOException,
			ResultsRepositoryBusyException {

		ensureValidBid(aid, eid, bid);

		if (database.isBusy() && !blocking) {
			throw new ResultsRepositoryBusyException(
					Database.TOO_MANY_CONNECTIONS_MESSAGE);
		}
		return database.downloadFile(database.getFileInDatabase(aid, eid, bid, name), host, port,
				blocking);
	}

	
}
