/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.Serializable;
import java.net.InetAddress;
import java.rmi.RemoteException;
import java.util.List;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.resultsrepository.gui.DataFormat;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.EntityInterface;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.GraphType;

public class BinaryImplementation extends EntityImplementation<BID>
implements Serializable, EntityInterface<BID>  {
	
	/** Load graphs defined for runs only error message */
	private static final String LOAD_FOR_RUNS_ONLY = "Load graphs defined for runs only.";

	/** Raw data download defined for runs only */ 
	private static final String RAW_DATA_FOR_RUNS_ONLY = "Raw data download defined for runs only.";

	/** Binaries don't support graphs - will be changed maybe ;) */
	private static final String NO_BINARY_GRAPHS = "No graphs for binaries.";
	
	/** Results repository entity manager for analyses */
	private BinaryEntityManager binaryEntityManager;
	
	private AID aid;
	private EID eid;
		
	/** Construction of basic Binary implementation 
	 * @param aid 	AnalysisId
	 * @param eid 	ExperimentId
	 * @throws RemoteException*/
	public BinaryImplementation(AID aid, EID eid) throws RemoteException {
		
		super();
		
		ExperimentEntityManager.getInstance().ensureValidEid(aid, eid); 
		
		this.aid = aid;
		this.eid = eid;
		
		binaryEntityManager = database.getBinaryEntityManager();
	}

	public Entity<BID> getEntity(BID id) throws RemoteException {
		return binaryEntityManager.getBinary(aid, eid, id); 
	}

	public void setEntityComment(BID id, String comment) throws RemoteException {
		binaryEntityManager.setBinaryComment(aid, eid, id, comment);		
	}

	public List<Entity<BID>> getEntities() throws RemoteException {
		return binaryEntityManager.getBinariesInExperiment(aid, eid);
	}

	public EntityInterface getSubentityInterface(BID id) throws RemoteException {
		return new RunImplementation(aid, eid, id);
	}

	public DataFormat[] getExportFormats() throws RemoteException {
		return exporter.getBinaryExportFormats();
	}

	public DownloadHandle beginExport(BID[] ids, String formatId, InetAddress ip, int port) 
		throws RemoteException, ResultsRepositoryBusyException {
		return exporter.beginBinariesExport(aid, eid, ids, formatId, ip, port);
	}

	public DataFormat[] getImportFormats() throws RemoteException {
		return importer.getBinaryImportFormats();
	}

	public UploadHandle beginImport(String formatId, InetAddress ip, int port) 
		throws RemoteException, ResultsRepositoryBusyException {		
		return importer.beginBinariesImport(formatId, ip, port);
	}

	public void deleteEntity(BID id) throws DeleteLockedException, RemoteException {
		binaryEntityManager.eraseBinary(aid, eid, id);
	}

	public int graphCount(GraphType graphType, BID id) throws RemoteException {

		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}

		throw new UnsupportedOperationException(NO_BINARY_GRAPHS);
	}

  public GraphInfo getGraphInfo(GraphType graphType, BID id, int index) throws RemoteException {
	  
		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}
		throw new UnsupportedOperationException(NO_BINARY_GRAPHS);
  }
  
	public DownloadHandle beginGraphDownload(GraphType graphType, BID id, InetAddress ip, int port,
			int index) throws RemoteException, ResultsRepositoryBusyException {
		
  		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}
		
		throw new UnsupportedOperationException(NO_BINARY_GRAPHS);
	}

	public DownloadStatus getGraphDownloadStatus(DownloadHandle handle) throws RemoteException {
		throw new UnsupportedOperationException(NO_BINARY_GRAPHS);
	}

	public void endGraphDownload(DownloadHandle handle) throws RemoteException {		
		throw new UnsupportedOperationException(NO_BINARY_GRAPHS);
	}

	public DownloadHandle beginRawDataDownload(BID id, InetAddress ip, int port) throws ResultsRepositoryBusyException, RemoteException {
		throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
	}

	public void endRawDataDownload(DownloadHandle handle) throws RemoteException {
		throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
	}

	public DownloadStatus getRawDataDownloadStatus(DownloadHandle handle) throws RemoteException {
		throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
	}
	
	public String getContext(BID id) throws RemoteException {
		return binaryEntityManager.getContext(aid, eid, id);
	}

	public String[] getTasks(BID id) throws RemoteException {
		TaskListManager taskListManagerInstance = TaskListManager.getInstance();
		
		synchronized (LockManager.taskListLock) {
			taskListManagerInstance.loadBinaryTaskList(aid, eid, id);
			return taskListManagerInstance.getBinaryTaskList();
		}
	}

	public LogRecord[] getLogsForTask(BID id, String taskID) throws RemoteException,
			LogStorageException, IllegalArgumentException, NullPointerException {
		
	    String contextId = binaryEntityManager.getContext(aid, eid, id);
	    
	    if (!database.getLogStorage().isTaskRegistered(contextId, taskID)) {
		return new LogRecord[] {};
	    }
	    
	    return database.getLogStorage().getLogsForTask(contextId, taskID); 
	}
}
