/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.net.InetAddress;

import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.resultsrepository.gui.DataFormat;

/**
 * This class read the data from user in external form, converts them and send them
 * to Results Repository databse
 * 
 * Exporter and Importer form the main part of user interface for working with data
 * 
 * @author Jakub Lehotsky
 *
 */
public final class Importer {
	
	/** Instance of Results Repository database to use */
	private static Database database;
	
	/** single instance of importer */
	private static Importer importerInstance = null;
	
	/** Table of used data format for import - maybe will be loaded from file in future */
	private static DataFormat[] importFormats = new DataFormat[] { 
		new DataFormat("text/dumb1", "Sample ex/import format 1 (load into temp directory)") 
	};	                                                   	
	
	/**
	 * Singleton constructor - intitalizes database 
	 */
	private Importer() {		
	}
	
	/**
	 * Returns the single instance of importer 
	 *  
	 * @return	Reference to Importer instance 
	 */
	public static Importer getInstance() {
		if (importerInstance == null) {
			importerInstance = new Importer();
		}
		return importerInstance;
	}
	
	/** 
	 * Intialization of importer. Must be called prior to use
	 * 
	 * @param database	Instance of Results Repository database  
	 */
	public static void initialize(Database database) {
		Importer.database = database;
	}
	
	// Temporary:
	private DataFormat[] getImportFormats() {
		return importFormats;
	}
	
	/**
	 * Returns the class with ImportInterface determined by formatId 
	 * @param formatId	String identification (mime type), see DataFormat class 
	 * @return	class instance implementing ImportInterface
	 */
	private ImportInterface getImportInterface(String formatId) {
			
		if (formatId.compareTo(importFormats[0].getId()) == 0) {
			return new ImportDumb();
		}
			
		throw new IllegalArgumentException("Uknown import format " + formatId + ".");		
	}
	
	/* ********* ANALYSIS ********* */ 
	
	
	public DataFormat[] getAnalysisImportFormats() { 
		return getImportFormats();
	}
	
	
	public UploadHandle beginAnalysesImport(String formatId, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException  {
		
		ImportInterface importInterface; 
		
		importInterface = getImportInterface(formatId);
				
		importInterface.setDatabase(database);
		return importInterface.beginAnalysesImport(ip, port);				
	}	
	
	/* ********* EXPERIMENT ********* */ 
		
	public DataFormat[] getExperimentImportFormats() {
		// Return hardcoded list
		return getImportFormats();
	}
	
	
	public UploadHandle beginExperimentsImport(String formatId, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {
		
		ImportInterface importInterface; 		
		
		importInterface = getImportInterface(formatId);
				
		importInterface.setDatabase(database);
		return importInterface.beginExperimentsImport(ip, port);				
	}

	/* ********* BINARY ********* */ 
	
	
	public DataFormat[] getBinaryImportFormats() {
		// Return hardcoded list
		return getImportFormats();
	}
	
	
	public UploadHandle beginBinariesImport(String formatId, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {		
		ImportInterface importInterface; 		
		
		importInterface = getImportInterface(formatId);
		
		importInterface.setDatabase(database);
		return importInterface.beginBinariesImport(ip, port);
	}	
	
	/* ********* RUN ********* */ 
	
	
	public DataFormat[] getRunImportFormats() {
		// Return hardcoded list
		return getImportFormats();
	}
	
	
	public UploadHandle beginRunsImport(String formatId, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {
		ImportInterface importInterface; 		
		
		importInterface = getImportInterface(formatId);
		
		importInterface.setDatabase(database);
		return importInterface.beginRunsImport(ip, port);
	}
	  
	/* ** common ** */
	
	public UploadStatus getImportStatus(UploadHandle handle) {
		return database.getFileUploadStatus(handle);
	}
	
	public void endImport(UploadHandle handle) {
		database.endFileUpload(handle);
	}
}
