/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.File;
import java.io.IOException;
import java.net.UnknownHostException;
import java.rmi.RemoteException;

import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;

/**
 * Simple class for performing of load collection from given Host Runtime and storing results
 * in database when asked
 * 
 * @author Jakub Lehotsky
 */
public final class LoadCollector {

	/** Single instance of load collector */
	private static LoadCollector loadCollectorInstance = null;
	
	/** Databse that is used for results in Results Repository */
	@SuppressWarnings("unused")
	private static Database database;
	
	/** Internal binary entity manager */
	static BinaryEntityManager binaryEntityManager;
	
	/**
	 *  Construction, notify load collector about Results Repository database when constructing
	 */
	private LoadCollector() {	
	}
	
	/**
	 * Returns the single instance of load collector
	 * 
	 * @return	Reference to LoadCollector instance 
	 */
	public static LoadCollector getInstance() {
		if (loadCollectorInstance == null) {
			return new LoadCollector();
		} else {
			return loadCollectorInstance;
		}
	}
	
	/** 
	 * Must be called prior to use
	 * 	 
	 * @param database	An instance of Results Repository database 
	 * @param binaryEntityManager 	Instance of Results Repository binary entity manager
	 */
	public static void initialize(Database database, BinaryEntityManager binaryEntityManager) {
		LoadCollector.database = database;
		
		LoadCollector.binaryEntityManager = binaryEntityManager;
	}

	/**
	 * Intializes load upload for a given run, this must called prior to uploading 
	 * load for a given run
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binaary Id
	 * @param rid	Run Id
	 */
	public void initLoadUpload(AID aid, EID eid, BID bid, RID rid) {
		
		String loadDir = database.getLoadDirectory(aid, eid, bid) + File.separator + rid.toString();
		
		File loadFile = new File(loadDir);

		if (!loadFile.exists()) {
			if (!loadFile.mkdir()) {
				Logger.logError("I/O: Unable to create directory for load information for " 
						+ Database.getRunReferenceString(aid, eid, bid, rid) + ".");
			}
		}
		
	}

	/**
	 * Closes load upload for a given run. Must be called after load is uploaded. Results
	 * Repository updates its metadata about load and eventually cleans data stored if
	 * load uploaded was marked invalid (error occured during upload, etc.) 
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * @param valid	True if uploaded load info is valid, false otherwise
	 */
	public void closeLoadUpload(AID aid, EID eid, BID bid, RID rid, boolean valid) {
		
		if (!valid) {
			String loadDir = database.getLoadDirectory(aid, eid, bid) + File.separator + rid.toString();
			File loadFile = new File(loadDir);
			
			Database.eraseDirectoryRecursively(loadFile);
		} 
		
	}

	/**
	 * Used to start upload of load information file 
	 * The client must open the socket on some port before calling this function. 
	 * Then it calls the <code>uploadFile</code> method.
	 * 
	 * This method work asynchronously and returns immediately.
	 * Before return it starts a new thread that will receive the file contents
	 * from the socket open on given host and port.
	 * 
	 * If there is any error when sending the file contents, the socket
	 * is closed. 
	 * 
	 * Client has an opportunity to check (via the <code>getFileUploadStatus</code> 
	 * method to check the current status of upload (eventually can be error)
	 *  
	 * After the upload finishes, client should close the returned handle
	 * via the <code>endFileUpload</code> method.  
	 *
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 * @param rid	Run ID for upload
	 * @param filename	Filename
	 * @param host	Hostname, from which file is uploaded
	 * @param port	Open port for network transfer
	 * @param blocking 	Determines, whether transfer will be blockin or non-blocking
	 * 
	 * @return 	UploadHandle, which you can use later to refer to transfer
	 * 
	 * @throws UnknownHostException
	 * @throws IOException
	 * @throws RemoteException 
	 * @throws ResultsRepositoryBusyException 
	 */
	public UploadHandle uploadLoadInfo(AID aid, EID eid, BID bid, RID rid, 
			String filename, String host, int port, boolean blocking) 
		throws ResultsRepositoryBusyException, UnknownHostException, IOException {
		
		binaryEntityManager.ensureValidBid(aid, eid, bid);
		
		String dirToStore = database.getLoadDirectory(aid, eid, bid) + File.separator +
			rid.toString() + File.separator + host;
		File dirFile = new File(dirToStore);
		
		if (!dirFile.exists()) {
			if (!dirFile.mkdirs()) {
				throw new IOException("Error creating directory for storing load information - " 
						+ Database.getRunReferenceString(aid, eid, bid, rid));
			}
		}
		
		String fileToStore = dirToStore + File.separator + filename;
		
		return database.uploadFile(fileToStore, host, port, blocking, false, null, null);
	}
}
