/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;
import java.io.File;
import java.io.IOException;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;

/**
 * Class implementing manipulation with metadata and caching their loading (if possible)
 * 
 * @author Jakub Lehotsky
 */
public final class MetadataManager {
	
	/** Analysis metadata filename */
	private static final String ANALYSIS_METADATA_FILENAME = "analysis.meta";

	/** Experiment metadata filename */
	private static final String EXPERIMENT_METADATA_FILENAME = "experiment.meta";

	/** Binary metadata filename */
	private static final String BINARY_METADATA_FILENAME = "binary.meta";
	
	/** Reference to results repository database */
	private static Database database;
	
	/** The single existing intance of RID Manager */	
	private static MetadataManager mMInstance = null;
	
	/** Ascii of tab character */
	protected static final String ASCII_TAB = "\t";
	
	
	/**
	 * Constructor - private, void for singleton use 
	 */
	private MetadataManager() {
	}
	
	/**
	 * Return single instance of Graph manager 
	 * 
	 * @return	reference single running instance of Graph manager
	 */
	public static MetadataManager getInstance() {
		if (mMInstance == null) {
			mMInstance = new MetadataManager();
		}
		return mMInstance;
	}
	
	/**
	 * Initialize Metadata Manager before its first use 
	 * 
	 * @param database	Instance of Results Repository database
	 */
	public static void initialize(Database database) {
		MetadataManager.database = database;
	}
		
	/**
	 * Stores metadata key-value into metadat file, if there is no item with key,
	 * creates a new one.
	 * 
	 * @param metaFile	File with metadata 
	 * @param key	Key of metadata item
	 * @param value	Value of metadata item
	 */
	public void setMetadataItem(String metaFile, String key, String value) {
						
		Metadata metadata;

		File file = new File(metaFile);
		if (file.exists()) {
			try {
				metadata = new Metadata(metaFile);
			} catch (IOException e) {
				throw new RuntimeException(new ResultsRepositoryException(
						Metadata.IO_ERROR_METADATA_READING_MESSAGE));
			}					
			if (metadata.getItemByKey(key) == null) {
				metadata.addItem(key, value);
			} else {
				metadata.setItemByKey(key, value);
			}
		} else {
			metadata = new Metadata();
			metadata.addItem("name", "");
			metadata.addItem(key, value);
		}

		try {
			metadata.store(metaFile);
		} catch (IOException e) {
			throw new RuntimeException(new ResultsRepositoryException(
					Metadata.IO_ERROR_METADATA_STORING_MESSAGE));
		}
	}

	
	/**
	 * Stores metadata for analysis into database
	 * 
	 * @param aid
	 *            Analysis ID to store metadata into
	 * @param metadata
	 *            Metadata to be stored
	 * @throws IOException
	 *             In case of I/O error
	 */
	public void storeAnalysisMetadata(AID aid, Metadata metadata)
			throws IOException {
		String filename;
		filename = database.getFileInDatabase(aid, ANALYSIS_METADATA_FILENAME);
		metadata.store(filename);
	}


	/**
	 * Returns the value of metadata of a given analysis
	 * 
	 * @param aid
	 *            Analysis ID to read metadata from
	 * @return The value of metadata item
	 * @throws IOException
	 */
	public Metadata loadAnalysisMetadata(AID aid)
			throws IOException {
		Metadata metadata = new Metadata(database.getFileInDatabase(aid,
				ANALYSIS_METADATA_FILENAME));
		return metadata;
	}

	/**
	 * Stores metadata key-value into analysis metadata file, if there is no item with key,
	 * creates a new one.
	 * 
	 * @param aid 	Analysis ID 
	 * @param key	Key of metadata item
	 * @param value	Value of metadata item
	 */
	public void setAnalysisMetadataItem(AID aid, String key, String value) {
		setMetadataItem(database.getFileInDatabase(aid, ANALYSIS_METADATA_FILENAME), key, value);
	}

	
	/**
	 * Returns the value of one metadata item of a given analysis
	 * 
	 * @param aid
	 *            Analysis ID to read metadata from
	 * @param key
	 *            Key of item to read
	 * @return The value of metadata item
	 * @throws IOException
	 */
	public String getAnalysisMetadataItem(AID aid, String key)
			throws IOException {
		Metadata metadata = new Metadata(database.getFileInDatabase(aid,
				ANALYSIS_METADATA_FILENAME));
		return metadata.getItemByKey(key);
	}
	

	/**
	 * Stores metadata for experiment into database
	 * 
	 * @param aid
	 *            Analysis ID with given experiment
	 * @param eid
	 *            Experiment ID to store metadata into
	 * @param metadata
	 *            Metadata to be stored
	 * @throws IOException
	 *             In case of I/O error
	 */
	public void storeExperimentMetadata(AID aid, EID eid, Metadata metadata)
			throws IOException {
		String filename;
		filename = database.getFileInDatabase(aid, eid, EXPERIMENT_METADATA_FILENAME);
		metadata.store(filename);
	}

	/**
	 * Returns the value of metadata of a given analysis
	 * 
	 * @param aid
	 *            Analysis ID to read metadata from
	 * @param eid
	 *            Experiment ID to read metadata from            
	 * @return The value of metadata item
	 * @throws IOException
	 */
	public Metadata loadExperimentMetadata(AID aid, EID eid)
			throws IOException {
		Metadata metadata = new Metadata(database.getFileInDatabase(aid, eid,
				EXPERIMENT_METADATA_FILENAME));
		return metadata;
	}
	
	/**
	 * Stores metadata key-value into experiment metadata file, if there is no item with key,
	 * creates a new one.
	 * 
	 * @param aid 	Analysis ID 
	 * @param eid 	Experiment ID
	 * @param key	Key of metadata item
	 * @param value	Value of metadata item
	 */
	public void setExperimentMetadataItem(AID aid, EID eid, String key, String value) {
		setMetadataItem(database.getFileInDatabase(aid, eid, EXPERIMENT_METADATA_FILENAME), key, value);
	}
	
	/**
	 * Returns the value of one metadata item of a given experiment
	 * 
	 * @param aid
	 *            Analysis ID with given experiment
	 * @param eid
	 *            Experiment ID to read metadata from
	 * @param key
	 *            Key of item to read
	 * @return The value of metadata item
	 * @throws IOException
	 */
	public String getExperimentMetadataItem(AID aid, EID eid, String key)
			throws IOException {
		Metadata metadata = new Metadata(database.getFileInDatabase(aid, eid,
				EXPERIMENT_METADATA_FILENAME));
		return metadata.getItemByKey(key);
	}

	/**
	 * Stores metadata for binary into database
	 * 
	 * @param aid
	 *            Analysis ID with given experiment/binary
	 * @param eid
	 *            Experiment ID with given binary
	 * @param bid
	 *            Binary ID to store metadata into
	 * @param metadata
	 *            Metadata to be stored
	 * @throws IOException
	 *             In case of I/O error
	 */
	public void storeBinaryMetadata(AID aid, EID eid, BID bid, Metadata metadata)
			throws IOException {
		String filename;
		filename = database.getFileInDatabase(aid, eid, bid, BINARY_METADATA_FILENAME);
		metadata.store(filename);
	}
	
	/**
	 * Returns the value of metadata of a given binary
	 * 
	 * @param aid
	 *            Analysis ID to read metadata from
	 * @param eid
	 *            Experiment ID to read metadata from
	 * @param bid
	 *            Binary ID to read metadata from            
	 * @return The value of metadata item
	 * @throws IOException
	 */
	public Metadata loadBinaryMetadata(AID aid, EID eid, BID bid)
			throws IOException {
		Metadata metadata = new Metadata(database.getFileInDatabase(aid, eid, bid,
				BINARY_METADATA_FILENAME));
		return metadata;
	}

	/**
	 * Stores metadata key-value into binary metadata file, if there is no item with key,
	 * creates a new one.
	 * 
	 * @param aid 	Analysis ID 
	 * @param eid 	Experiment ID
	 * @param bid 	Binary ID
	 * @param key	Key of metadata item
	 * @param value	Value of metadata item
	 */
	public void setBinaryMetadataItem(AID aid, EID eid, BID bid, String key, String value) {
		setMetadataItem(database.getFileInDatabase(aid, eid, bid, BINARY_METADATA_FILENAME), key, value);
	}
	
	/**
	 * Returns the value of one metadata item of a given binary
	 * 
	 * @param aid
	 *            Analysis ID with given experiment/binary
	 * @param eid
	 *            Experiment ID with given binary
	 * @param bid
	 *            Binary ID to read binary metadata from
	 * @param key
	 *            Key of item to read
	 * @return The value of metadata item
	 * @throws IOException
	 */
	public String getBinaryMetadataItem(AID aid, EID eid, BID bid, String key)
			throws IOException {
		Metadata metadata = new Metadata(database.getFileInDatabase(aid, eid, bid,
				BINARY_METADATA_FILENAME));
		return metadata.getItemByKey(key);
	}
	
}
