/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;


/**
 * Implements management and loading and storing of general numbered tables
 * 
 * @author Jakub Lehotsky
 */
public class NumberedTable {
		
	/** I/O error in validity map reading message */
	public static final String IO_ERROR_LIST_READING_MESSAGE = "I/O error reading metadata of ";
	/** I/O error in validity map reading message */
	public static final String IO_ERROR_LIST_STORING_MESSAGE = "I/O error storing metadata of ";
	
	/** ASCII of delim. character (TAB) */
	private static final String ASCII_TAB = "\t";	
	
	/** Actual metadata */
	private ArrayList<NumberedItem> numberedTable;
	
	/** Last item counter */
	private int lastItem;		
	
	/**
	 * Constructor - creates numbered table, void object 
	 * (meant for creating new validity map tables items, initializing them and storing into file)
	 * 
	 */
	public NumberedTable() {
		numberedTable = new ArrayList<NumberedItem>();
		this.lastItem = 0;
	}
	
	/**
	 * Returns validity map we store
	 * 
	 * @return the metadata
	 */
	
	public NumberedItem[] getNumberedTable() {
		return numberedTable.toArray(new NumberedItem[] {});
	}	
	
	/** 
	 * Constructor - initializes numbered map in memory and read its contents from the file.
	 * 
	 * @param filename	Name of file to read validity table from
	 * @throws IOException 
	 */
	public NumberedTable(String filename) throws IOException {
				
		String[] lineTokens;
		String line;
		BufferedReader inputReader;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {			
			numberedTable = new ArrayList<NumberedItem>();
			Logger.logDebug("File not found - " + filename + ", creating void table");
			return;
		}
				
		numberedTable = new ArrayList<NumberedItem>();
		
		while ((line = inputReader.readLine()) != null) {								
			lineTokens = line.split(ASCII_TAB);
			try {
				if (lineTokens.length == 1) { 
					addItem(new Integer(lineTokens[0]), "");
				} else {
					String finalLine = "";
					/* Following code here is to support multiple column tables - secondary
					 * parsing is upon user */
					for (int i = 1; i < lineTokens.length; i++) {
						finalLine += lineTokens[i];
						if (i < lineTokens.length - 1) {
							finalLine += ASCII_TAB;
						}
					}
					addItem(new Integer(lineTokens[0]), finalLine);
				}
			} catch (IndexOutOfBoundsException e) {
				throw new IOException("Invalid format in table, file " + filename + ".");
			}
		}
				
		inputReader.close();
	}
	
	/**
	 * Stores numbered list we have in our memory to the file.
	 * 
	 * @param filename	Where to store numbered list.
	 * @throws IOException
	 */
	public void store(String filename) throws IOException {
		
		int i;		
		PrintWriter outputWriter;
		
		outputWriter = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
		
		for (i = 0; i < numberedTable.size(); i++) {
			outputWriter.println(getItemKey(i) + ASCII_TAB + getItemValue(i));		
		}
		
		outputWriter.close();
	}
	
	/**
	 * Returns the count of numbered items currently loaded in memory
	 * 
	 * @return	The count of items
	 */
	public int getItemCount() {
		return numberedTable.size();
	}
	
	/**
	 * Adding key-value pair to validity map
	 * 
	 * @param key	Key to added
	 * @param value	Associated value to add
	 * @throws ArrayIndexOutOfBoundsException
	 */
	public void addItem(long key, String value) {		
		numberedTable.add(new NumberedItem(key, value));
		lastItem++;
	}
	
	/**
	 * Deleting item by key
	 * 
	 * @param key	Key of item to be deleted
	 */
	public void deleteItemByKey(long key) {
		
		int i = 0;		
		try {		
			while (getItemKey(i) != key) {
				i++;
				if (i == lastItem) {
					return; // not found
				}
			}
		} catch (IndexOutOfBoundsException e) {
			return; //not found
		}
		
		numberedTable.remove(i);
	}
	
	/**
	 * Returns the key of the item with index.
	 * 
	 * @param index	Index of they key to give.
	 * @return	int - the key
	 */
	public long getItemKey(int index) {		
		if (numberedTable.get(index) == null) {
			Logger.logWarning("Tried to get a key of non-existing item - " + index);
			return -1;
		}
		return numberedTable.get(index).getKey();
	}
	
	/**
	 * Returns the value of the item with index.
	 * 
	 * @param index	Index of the value to give.
	 * @return	String with value.
	 */
	public String getItemValue(int index) {
		return numberedTable.get(index).getValue();
	}
	

	/**
	 * Returns the value of the item, with given int key. 
	 * 
	 * @param key	Key to ask for.
	 * @return		String - patch to associated run, or null if not found. 
	 */
	public String getItemByKey(long key) {
		
		if (numberedTable == null) {
			Logger.logError("While trying to read long " + key + ", encountered " 
					+ "null!");
			assert false : "Table unitialized!";
			return null; 			
		}
		
		int i = 0;
		
		try {		
			while (getItemKey(i) != key) {
				i++;
				if (i == lastItem) {
					return null;
				}
			}
		} catch (IndexOutOfBoundsException e) {
			return null;
		}
		return getItemValue(i);
	}
	
	/**
	 * Sets the value of the table item, with the given long key. 
	 * 
	 * @param key	Key
	 * @param value	New Value
	 * 
	 * @return True iff succesful (false if key doesn't exist)4
	 */
	public boolean setItemByKey(long key, String value) {
		
		if (key == -1) {
			return false;
		}
		
		int i = 0;
		
		try {
			
			while (getItemKey(i) != key) {
				i++;
				if (i == lastItem) {
					return false; 
				}
			}		
		
			numberedTable.set(i, new NumberedItem(key, value));
		} catch (IndexOutOfBoundsException e) {
			return false;
		}
				
		return true; 		
	}
}
