/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;

/**
 * Class that manages callbacks, for a given analysis or experiment, one instance must be running
 * 
 * @author Jakub Lehotsky
 */
public final class RCallbacksManager {

    	/** File to store run scripts selection */
	private static final String RUN_SCRIPTS_FILE = "run-scripts";

	/** File to store binary scripts selection */
	private static final String BINARY_SCRIPTS_FILE = "binary-scripts";

	/** File to store experiment scripts selection */
	private static final String EXPERIMENT_SCRIPTS_FILE = "experiment-scripts";

	/** File to store noncomplete binary scripts selection */
	private static final String BINARY_NONCOMPLETE_SCRIPTS_FILE = "binary-noncomplete-scripts";

	/** File to store noncomplete experiment scripts selection */
	private static final String EXPERIMENT_NONCOMPLETE_SCRIPTS_FILE = "experiment-noncomplete-scripts";

	/** Registered run callbacks for given analysis */
	private static ArrayList<String> registeredRunCallbacks;
	
	/** Registered binary callbacks for given analysis */
	private static ArrayList<String> registeredBinaryCallbacks;
	
	/** Registered binary callbacks for given analysis */
	private static ArrayList<String> registeredNoncompleteBinaryCallbacks;
	
	/** Registered experiment callbacks for given analysis */
	private static ArrayList<String> registeredExperimentCallbacks;
	
	/** Registered experiment callbacks for given analysis */
	private static ArrayList<String> registeredNoncompleteExperimentCallbacks;

	/** Analysis Id, which callbacks manager is tied to */
	private AID aid;
	
	/** Experiment Id, which callbacks manager is tied to */
	private EID eid;
	
	/** Indicates whether we're manager for analysis, or experiment */
	private boolean analysisScript;

	
	/**
	 * Initializes callbacks manager for a given analysis or experiment
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id 
	 * @param analysisScript 	True if script for analysis, false for experiment
	 */
	public RCallbacksManager(AID aid, EID eid, boolean analysisScript) {
	    
	    	this.analysisScript = analysisScript;
	    	
	    	if (analysisScript) {
	    	    Database.getInstance().getAnalysisEntityManager().ensureValidAid(aid);
	    	    this.aid = aid;
	    	} else {
	    	    Database.getInstance().getExperimentEntityManager().ensureValidEid(aid, eid);
	    	    this.aid = aid;
	    	    this.eid = eid;
	    	}		

		registeredRunCallbacks = new ArrayList<String>();
		registeredBinaryCallbacks = new ArrayList<String>();
		registeredNoncompleteBinaryCallbacks = new ArrayList<String>();
		registeredExperimentCallbacks = new ArrayList<String>();
		registeredNoncompleteExperimentCallbacks = new ArrayList<String>();
		
		try {
		    load();
		} catch (IOException e) {
		    clearRegisteredRunCallbacks();
		    clearRegisteredBinaryCallbacks(false);
		    clearRegisteredExperimentCallbacks(false);
		    clearRegisteredBinaryCallbacks(true);
		    clearRegisteredExperimentCallbacks(true);
		}
		
	}
	
	/**
	 * Returns Analysis Id, callback manager is tied to
	 * @return	The AID
	 */
	AID getAID() {
		return aid;
	}
	
	/**
	 * Returns Experiment Id, callback manager is tied to
	 * @return	The EID
	 */
	EID getEID() {
		return eid;
	}
	
	/**
	 * Indication analysis vs. experiment callbacks manager
	 * @return	True if we're analyses manager, false if we're epxeriments manager
	 */
	boolean isAnalysis() {
	    return analysisScript;
	    
	}
	
	/* **************************************************************** */
	/* ***************************** RUNS ***************************** */
	/* **************************************************************** */
	
	/* Methods for callback setup */
	
	/**
	 * Clears all registered run callbacks
	 */	
	void clearRegisteredRunCallbacks() {
		registeredRunCallbacks.clear();
	}
	
	/**
	 * Registers new callback (parameters must be set) for runs 
	 * 
	 * @param runCallback	Callback to register
	 */
	void registerRunCallback(String runCallback) {		
		
		registeredRunCallbacks.add(runCallback);		
	}
	
	/**
	 * Returns the count of registered run callbacks. These are all executed after each run is 
	 * uploaded into results repository
	 * 
	 * @return	The count of run callbacks
	 */
	int getRegisteredRunCallbackCount() {
		return registeredRunCallbacks.size();
	}
	
	/* Methods for run callback execution */
	
	/**
	 * Returns registered run callback function name (based on index)
	 * 
	 * @param functionNumber	The index of callback function in registered run callbacks 
	 * @return			The <code>String</code> containing name of function to use as callback
	 */
	String getRegisteredRunFunction(int functionNumber) {
		return registeredRunCallbacks.get(functionNumber);
	}

	/* ******************************************************************** */	
	/* ***************************** BINARIES ***************************** */
	/* ******************************************************************** */
	
	/* Methods for binary callback setup */
	
	/**
	 * Clears all registered binary callbacks
	 * 
	 * @param completeness If true, add callback to queue to be executed after entity contains all
	 * 						subentities 
	 */	
	void clearRegisteredBinaryCallbacks(boolean completeness) {
		if (completeness) {
			registeredBinaryCallbacks.clear();
		} else {
			registeredNoncompleteBinaryCallbacks.clear();
		}
	}
	
	/**
	 * Registers new callback (parameters must be set) for binaries 
	 * 
	 * @param binaryCallback	Callback to register
	 * @param completeness If true, add callback to queue to be executed after entity contains all
	 * 						subentities	 
	 */
	void registerBinaryCallback(String binaryCallback, boolean completeness) {
		if (completeness) {
			registeredBinaryCallbacks.add(binaryCallback);
		} else {
			registeredNoncompleteBinaryCallbacks.add(binaryCallback);
		}
	}
	
	/**
	 * Returns the count of registered binary callbacks. These are all executed after each binary is 
	 * uploaded into results repository
	 * 
	 * @return	The count of binary callbacks
	 */
	int getRegisteredBinaryCallbackCount(boolean completeness) {
		if (completeness) {
			return registeredBinaryCallbacks.size();
		} else {
			return registeredNoncompleteBinaryCallbacks.size();
		}
	}
	
	/* Methods for binary callback execution */

	/**
	 * Returns registered binary callback function name (based on index)
	 * 
	 * @param functionNumber	The index of callback function in registered binary callbacks
	 * @param completeness If true, add callback to queue to be executed after entity contains all
	 * 						subentities  
	 * @return			The <code>String</code> containing name of function to use as callback
	 */
	String getRegisteredBinaryFunction(int functionNumber, boolean completeness) {
		if (completeness) {
			return registeredBinaryCallbacks.get(functionNumber);
		} else {
			return registeredNoncompleteBinaryCallbacks.get(functionNumber);
		}
	}

	/* ********************************************************************* */
	/* ***************************** EXPERIMENTS *************************** */
	/* ********************************************************************* */

	
	/* Methods for experiment callback setup */
	
	/**
	 * Clears all registered experiment callbacks
	 * 
	 * @param completeness If true, add callback to queue to be executed after entity contains all
	 * 						subentities  	
	 */	
	void clearRegisteredExperimentCallbacks(boolean completeness) {
		if (completeness) { 
			registeredExperimentCallbacks.clear();
		} else {
			registeredNoncompleteExperimentCallbacks.clear();
		}
	}
	
	/**
	 * Registers new callback (parameters must be set) for experiments 
	 * 
	 * @param experimentCallback	Callback to register
	 * @param completeness If true, add callback to queue to be executed after entity contains all
	 * 						subentities  	
	 */
	void registerExperimentCallback(String experimentCallback, boolean completeness) {
		if (completeness) {
			registeredExperimentCallbacks.add(experimentCallback);
		} else {
			registeredNoncompleteExperimentCallbacks.add(experimentCallback);
		}
	}

	/**
	 * Returns the count of registered experiment callbacks. These are all executed after each 
	 * experiment is uploaded into results repository
	 * @param completeness If true, add callback to queue to be executed after entity contains all
	 * 						subentities  	
	 * @return	The count of experiment callbacks
	 */
	int getRegisteredExperimentCallbackCount(boolean completeness) {
		if (completeness) {
			return registeredExperimentCallbacks.size();
		} else {
			return registeredNoncompleteExperimentCallbacks.size();
		}
	}
		
	/* Methods for experiment callback execution */
	
	/**
	 * Returns registered experiment callback function name (based on index)
	 * 
	 * @param functionNumber	The index of callback function in registered experiment callbacks 
	 * @param completeness If true, add callback to queue to be executed after entity contains all
	 * 						subentities  	
	 * @return			The <code>String</code> containing name of function to use as callback
	 */
	String getRegisteredExperimentFunction(int functionNumber, boolean completeness) {
		if (completeness) {
			return registeredExperimentCallbacks.get(functionNumber);
		} else {
			return registeredNoncompleteExperimentCallbacks.get(functionNumber);
		}
	}
	
	/**
	 * Returns callbacks actually loaded in memory
	 * 
	 * @return	CallbackRScripts object describing callbacks
	 */
	public CallbackRScripts getCallbacks() {
	    
	    String runScripts = "";
	    String binaryScripts = "";
	    String experimentScripts = "";
	    String nonCompleteBinaryScripts = "";
	    String nonCompleteExperimentScripts = "";
	    
	    Logger.logTrace("Getting callbacks for " 
	    	+ (analysisScript 
			    ? Database.getAnalysisReferenceString(aid) 
			    : Database.getExperimentReferenceString(aid, eid)));	    
	    
	    for (int i = 0; i < getRegisteredRunCallbackCount(); i++) {
		runScripts += getRegisteredRunFunction(i) + "\n";
	    }
	    for (int i = 0; i < getRegisteredBinaryCallbackCount(true); i++) {
		binaryScripts += getRegisteredBinaryFunction(i, true) + "\n";
	    }
	    for (int i = 0; i < getRegisteredExperimentCallbackCount(true); i++) {
		experimentScripts += getRegisteredExperimentFunction(i, true) + "\n";
	    }
	    for (int i = 0; i < getRegisteredBinaryCallbackCount(false); i++) {
		nonCompleteBinaryScripts += getRegisteredBinaryFunction(i, false) + "\n";
	    }
	    for (int i = 0; i < getRegisteredExperimentCallbackCount(false); i++) {
		nonCompleteExperimentScripts += getRegisteredExperimentFunction(i, false) + "\n";
	    }	    
	    
	    return new CallbackRScripts(runScripts, binaryScripts,
		    experimentScripts, nonCompleteBinaryScripts, nonCompleteExperimentScripts);
	}

	/**
	 * Sets the R script callbacks and store them into database 
	 * 
	 * @param scripts	Scripts to be set
	 * 
	 * @throws IOException	if i/o error occured during storing
	 */
	public void setCallbacks(CallbackRScripts scripts) 
		throws IOException {

	    Logger.logTrace("Setting callbacks for " 
	    	+ (analysisScript 
			    ? Database.getAnalysisReferenceString(aid) 
			    : Database.getExperimentReferenceString(aid, eid)));	    

	    
	    String runScriptString = scripts.getCompleteRunScript();	    
	    String[] runScripts = runScriptString.split("\n");
	    // NOTE: required to create new mutable ArrayList, because Arrays.asList() returns read-only static
	    //       ArrayList. The rest is left up to GC
	    registeredRunCallbacks = new ArrayList<String>(Arrays.asList(runScripts));

	    String binaryScriptString = scripts.getCompleteBinaryScript();	    
	    String[] binaryScripts = binaryScriptString.split("\n");
	    registeredBinaryCallbacks = new ArrayList<String>(Arrays.asList(binaryScripts));

	    String experimentScriptString = scripts.getCompleteExperimentScript();	    
	    String[] experimentScripts = experimentScriptString.split("\n");
	    registeredExperimentCallbacks = new ArrayList<String>(Arrays.asList(experimentScripts));
	    
	    String binaryNCScriptString = scripts.getIncompleteBinaryScript();	    
	    String[] binaryNCScripts = binaryNCScriptString.split("\n");
	    registeredNoncompleteBinaryCallbacks = new ArrayList<String>(Arrays.asList(binaryNCScripts));

	    String experimentNCScriptString = scripts.getIncompleteExperimentScript();	    
	    String[] experimentNCScripts = experimentNCScriptString.split("\n");
	    registeredNoncompleteExperimentCallbacks = new ArrayList<String>(Arrays.asList(experimentNCScripts));
	    
	    store();
	}

	/**
	 * Internal method used to load registered callbacks from file into String array
	 * 
	 * @param scriptsFile		File to load scripts from
	 * @param registeredCallbacks	Class, where to store loaded file
	 */
	private void loadFileIntoArray(File scriptsFile, ArrayList<String> registeredCallbacks) {

		try {
			BufferedReader inf = new BufferedReader(new FileReader(scriptsFile));
			String str;
			while ((str = inf.readLine()) != null) {
				registeredCallbacks.add(str);
			}
			inf.close();
	    } catch (FileNotFoundException e) {
	    	// This is OK - equals void table
	    } catch (IOException e) {
	    	Logger.logError("Error reading analysis R scripts settings : " 
				+ e.getMessage() + ".");
	    }
	}
	
	/**
	 * Internal method used to store registered callbacks from String array into file
	 * 
	 * @param scriptsFile		File to store scripts to
	 * @param registeredCallbacks	Array of Strings to store
	 */	
	private void storeArrayIntoFile(ArrayList<String> registeredCallbacks, String scriptsFile) {
	    try {
	    	BufferedWriter outf = new BufferedWriter(new FileWriter(scriptsFile));
	    	for (String registeredCallback : registeredCallbacks) {
	            outf.write(registeredCallback);
	            outf.newLine();
            }
	    	outf.close();
		
	    } catch (IOException e) {
	    	Logger.logError("Error writing analysis R scripts settings : " 
	    		+ e.getMessage() + ".");
	    }
	}
	
	/**
	 * Loads registered R scripts callbacks from database into memory 
	 * 
	 * @throws IOException
	 */
	private void load() 
		throws IOException {

	    String scriptsDirStr = 
		(analysisScript 
			? Database.getInstance().getRScriptsDirectory(aid) 
			: Database.getInstance().getRScriptsDirectory(aid, eid));
		
	    File scriptsDir = new File(scriptsDirStr);
	    
	    if (!(scriptsDir.exists() && scriptsDir.canRead() && scriptsDir.isDirectory())) {
		throw new IOException("Error accessing R Scrips directory for " 
			+ (analysisScript ? Database.getAnalysisReferenceString(aid) + "."
				: Database.getExperimentReferenceString(aid, eid)) + ".");
	    }	    
	    
	    File runScriptsFile = new File(scriptsDirStr 
		    + File.separator + RUN_SCRIPTS_FILE);
	    clearRegisteredRunCallbacks();
	    loadFileIntoArray(runScriptsFile, registeredRunCallbacks);	    
	    File binaryScriptsFile = new File(scriptsDirStr
		    + File.separator + BINARY_SCRIPTS_FILE);
	    clearRegisteredBinaryCallbacks(true);
	    loadFileIntoArray(binaryScriptsFile, registeredBinaryCallbacks);
	    File experimentScriptsFile = new File(scriptsDirStr 
		    + File.separator + EXPERIMENT_SCRIPTS_FILE);
	    clearRegisteredExperimentCallbacks(true);
	    loadFileIntoArray(experimentScriptsFile, registeredExperimentCallbacks);
	    File binaryNonCompleteScriptsFile = new File(scriptsDirStr
		    + File.separator + BINARY_NONCOMPLETE_SCRIPTS_FILE);
	    clearRegisteredBinaryCallbacks(false);
	    loadFileIntoArray(binaryNonCompleteScriptsFile, registeredNoncompleteBinaryCallbacks);
	    File experimentNonCompleteScriptsFile = new File(scriptsDirStr 
		    + File.separator + EXPERIMENT_NONCOMPLETE_SCRIPTS_FILE);
	    clearRegisteredExperimentCallbacks(false);
	    loadFileIntoArray(experimentNonCompleteScriptsFile, registeredNoncompleteExperimentCallbacks);
	}
	
	/**
	 * Stores registered callback scripts into database
	 * 
	 * @throws IOException
	 */
	private void store() 
		throws IOException {
	    String scriptsDirStr = 
		(analysisScript 
			? Database.getInstance().getRScriptsDirectory(aid) 
			: Database.getInstance().getRScriptsDirectory(aid, eid));
		
	    File scriptsDir = new File(scriptsDirStr);	    
	    
	    if (!(scriptsDir.exists() && scriptsDir.canRead() && scriptsDir.isDirectory())) {
		throw new IOException("Error accessing R Scrips directory for " 
			+ Database.getAnalysisReferenceString(aid) + ".");
	    }
	    storeArrayIntoFile(registeredRunCallbacks, 
		    scriptsDirStr + File.separator + RUN_SCRIPTS_FILE);
	    storeArrayIntoFile(registeredBinaryCallbacks, 
		    scriptsDirStr  + File.separator + BINARY_SCRIPTS_FILE);
	    storeArrayIntoFile(registeredExperimentCallbacks, 
		    scriptsDirStr  + File.separator + EXPERIMENT_SCRIPTS_FILE);
	    storeArrayIntoFile(registeredNoncompleteBinaryCallbacks, 
		    scriptsDirStr  + File.separator + BINARY_NONCOMPLETE_SCRIPTS_FILE);
	    storeArrayIntoFile(registeredNoncompleteExperimentCallbacks, 
		    scriptsDirStr  + File.separator + EXPERIMENT_NONCOMPLETE_SCRIPTS_FILE);
	    
	}
}
