/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;

import org.omegahat.R.Java.REvaluator;
import org.omegahat.R.Java.ROmegahatInterpreter;

import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;
import cz.cuni.mff.been.resultsrepository.gui.RFunctionHelp;
import cz.cuni.mff.been.resultsrepository.gui.RScriptPackage;


/**
 * Class that manages all that can be done with R in Results Repository. It manages handling with
 * R packages, initialization of R and low-level execution of R scripts. R scripts background thread,
 * management of R help functionalities and callbacks registering are delegated to the respective 
 * classes - in order RScriptsRunner, RHelpManager, RCallbacksManagerFactory 
 * 
 * @author Jakub Lehotsky
 */
public final class RManager {
	
	/** Name of the file, where invalidate scripts are stored */
	public static final String INVALIDATE_SCRIPT_FILENAME = "scripts";

	/** Single instance of R Package Manager */
	private static RManager rpMInstance;
	
	/** Internal reference to results repository database */
	private static Database database;
	
	/** R scripting functionality */
	private static boolean rScripts;
	
	/** Internal Callbacks Manager Factory and holder */
	private static RCallbacksManagerFactory rCallbacksManagerFactory;
	
	/** R help manager */
	private static RHelpManager rHelpManager;
	
	/** R scripts runner - manages background thread for running scripts */
	private static RScriptsRunner rScriptsRunner;
	
	/** R evaluator */
	private static REvaluator rEvaluator;

	/** R interpreter */
	@SuppressWarnings("unused")
	private static ROmegahatInterpreter rInterpreter;

	/**
	 * Constructor - private, void for singleton use 
	 */
	private RManager() {
	}
	
	/**
	 * Return single instance of R manager 
	 * 
	 * @return	reference single running instance of Graph manager
	 */
	public static RManager getInstance() {
		if (rpMInstance == null) {
			rpMInstance = new RManager();
		}
		return rpMInstance;
	}
	
	/**
	 * Initialize R Manager before its first use 
	 * 
	 * @param database	Instance of database we use
	 */
	public static void initialize(Database database) {
		
		RManager.database = database;
		
		/* Initialize R help manager */
		rHelpManager = RHelpManager.getInstance();
		rHelpManager.initialize();
		try {
			rHelpManager.reloadRFunctionsHelp();
		} catch (IOException e) {
			Logger.logError(
					"Error reading help files " + e.getMessage() + ".");
		}
		
		/* Intialize callback managers */
		rCallbacksManagerFactory = RCallbacksManagerFactory.getInstance();
		rCallbacksManagerFactory.initialize();
		
		/* Try to run and int R with SJava */
		initR();
		
		/* Intialize R background script runner */
		rScriptsRunner = RScriptsRunner.getInstance();
		rScriptsRunner.initialize();
		rScriptsRunner.startBackgroundThread();
	}
	
	/**
	 * Shutdown of R sub-system. Stops R scripts execution
	 * background thread
	 */
	public void shutdown() {
		rScriptsRunner.stopBackgroundThread();
	}
	
	/**
	 * Informs about status of R scripts execution
	 * 
	 * @return	True if scripts running is OK
	 */
	public boolean rScriptingEnabled() {
		return rScripts;
	}
	
	/**
	 * Returns internal scripts runner (for enqueuing scripts and callbacks)
	 * 
	 * @return	The instance of RScriptsRunner
	 */
	public static RScriptsRunner getRScriptsRunner() {
		return rScriptsRunner;
	}
	
	/**
	 * Executes one R command
	 * 
	 * @param command	String containing command to be executed in R
	 */
	public static void executeRCommand(String command) {
		try {
			rEvaluator.voidEval(command.trim());
		} catch (Exception e) {
			Logger.logError("Exception caught when executing R: " + e.getMessage() 
					+ ", dumping stack trace.");
			e.printStackTrace();
		}
	}

	/**
	 * Initializes R: load RNetCDF and SJava library in R, set the source for
	 * used R scripts and setup R scripts environment to Results Repository
	 * database.
	 */
	public static void initR() {

		String[] args = new String[1];
		args[0] = "--no-save";

		rScripts = true;

		try {
			rInterpreter = new ROmegahatInterpreter(ROmegahatInterpreter
					.fixArgs(args), false);
			rEvaluator = new REvaluator();
		} catch (NoClassDefFoundError e) {
			Logger.logError(
					"R not initialized, statistics and graph counting disabled.");
			rScripts = false;
		} catch (UnsatisfiedLinkError e) {
			Logger.logError(
					"R not initialized (native R libraries could not be loaded), "
							+ "statistics and graph counting disabled.");
			rScripts = false;
		}

		if (rScripts) {
			Object[] funArgs;

			funArgs = new Object[1];
			funArgs[0] = "RNetCDF";
			rEvaluator.call("library", funArgs);

			try {
				sourceRegisteredScripts();
			} catch (ResultsRepositoryException e) {
				/*
				 * Do nothing here - exception is thrown onl if rScripts ==
				 * false;
				 */
				assert false : "rScripting is off, so it can't be on";
			}

			Logger.logInfo("R libraries initialized.");
		}

	}
	
	/**
	 * Initialize all registered R scripts (sources them, so that functions are stored inside R)
	 * 
	 * @throws ResultsRepositoryException
	 */
	public static void sourceRegisteredScripts() throws ResultsRepositoryException {

		File[] directoryList;

		File path = new File(database.getDatabasePath()
				+ File.separator + "rscripts");

		directoryList = path.listFiles();

		for (int i = 0; i < directoryList.length; i++) {
			Logger.logInfo(
					"R Init: Checking package: " + directoryList[i].getName() + ".");
			if (directoryList[i].isDirectory()) {

				File packagePath = directoryList[i];

				FilenameFilter fileFilter = new FilenameFilter() {
					public boolean accept(File dir, String filename) {
						return dir.isDirectory()
								&& (filename.matches(".*\\.r"));
					}
				};

				File[] packageRList = packagePath.listFiles(fileFilter);

				for (int j = 0; j < packageRList.length; j++) {
					Logger.logDebug(
							"R Init: Sourcing script: "
									+ packageRList[j].getAbsolutePath() + ".");
					initRScript(packageRList[j].getAbsolutePath());
				}

			}
		}
		
		Logger.logTrace("Database path is : " + database.getDatabasePath() + ".");
		
		executeRCommand("base.initScripts('" + database.getDatabasePath() + "')");

	}
	
	/**
	 * Sources one R package, this method should be called after all initialization
	 * 
	 * @param packageName - the name of the package to source
	 * @throws ResultsRepositoryException	in case something goes wrong
	 */
	public void sourceOnePackageScript(String packageName) 
		throws ResultsRepositoryException {

	    File packagePath = new File(database.getDatabasePath()
			+ File.separator + "rscripts" + File.separator + packageName);

	    if (!(packagePath.exists() && packagePath.canRead() && packagePath.isDirectory())) { 
		throw new IllegalArgumentException ("Package \"" + packageName + "\" not registered.");
	    }
            
	    FilenameFilter fileFilter = new FilenameFilter() {
		public boolean accept(File dir, String filename) {
		    return dir.isDirectory()
		    && (filename.matches(".*\\.r"));
		}
	    };
            
	    File[] packageRList = packagePath.listFiles(fileFilter);
            
	    for (int j = 0; j < packageRList.length; j++) {
		Logger.logTrace(
			"R Init: Sourcing script: "
			+ packageRList[j].getAbsolutePath() + ".");
		initRScript(packageRList[j].getAbsolutePath());
	    }
	}

	
	/**
	 * Source one script - i.e. load it from file system to be used inside R
	 * 
	 * @param pathToScript
	 *            path to R scripts
	 * @throws ResultsRepositoryException
	 *             when R scripts are not turned on
	 */
	private static void initRScript(String pathToScript)
			throws ResultsRepositoryException {

		if (!rScripts) {
			throw new ResultsRepositoryException(
					"R scripts are not turned on - R not initialized?");
		}

		executeRCommand("source('" + pathToScript + "')");
	}

	/**
	 * Uploads R package and install it inside results repository (through TCP)
	 * 
	 * 
	 * @see cz.cuni.mff.been.resultsrepository.Database#uploadFile
	 * 
	 * @param name
	 *            Name of the package to be uploaded
	 * @param host
	 *            Hostname to connect to for data retrieving
	 * @param port
	 *            Port used in connection with host
	 * @return Handle of network connection
	 * 
	 * @throws UnknownHostException
	 *             In the case host could not be found
	 * @throws IOException
	 *             In the case of some IO error
	 * @throws ResultsRepositoryBusyException
	 *             In the case of non-blocking upload when Results Repository is
	 *             busy with (down/)uploading files
	 */
	public UploadHandle uploadAndRegisterRPackge(String name, String host,
			int port) throws UnknownHostException, IOException,
			ResultsRepositoryBusyException {

		ensureRPackageNotRegistered(name);

		if (database.isBusy()) {
			Logger.logWarning(
					"Refusing upload - too many connections (registering"
							+ "R packge: " + name + ".");

			throw new ResultsRepositoryBusyException(
					Database.TOO_MANY_CONNECTIONS_MESSAGE);
		}

		return database.uploadFile(name, host, port, false, true, null, null);
	}
	
	
	/**
	 * Checks whether given R scripts package is registered in results
	 * repository
	 * 
	 * @param name
	 *            Name of the package to check
	 * @return True iff package is registered
	 */
	public boolean isRPackageRegistered(String name) {
		File packageDir = new File(database.getDatabasePath() + File.separator
				+ Database.RSCRIPTS_DIRECTORY + File.separator + name);
		return packageDir.exists() && packageDir.canRead()
				&& packageDir.isDirectory();
	}

	/**
	 * Checks that R package is really registered, otherwise throw an
	 * IllegalArgumentException
	 * 
	 * @param name
	 *            The name of the package
	 */
	public void ensureRPackageRegistered(String name) {
		if (!isRPackageRegistered(name)) {
			throw new IllegalArgumentException("R package " + name
					+ " is not registered.");
		}
	}

	/**
	 * Checks that R package is not yet registered, otherwsie throw an
	 * IllegalArgumentException
	 * 
	 * @param name
	 *            The name of the package
	 */
	public void ensureRPackageNotRegistered(String name) {
		if (isRPackageRegistered(name)) {
			throw new IllegalArgumentException("R package " + name
					+ " is already registered.");
		}
	}
	
	/**
	 * Deletes (unregister) R package from Results Repository
	 *  
	 * @param name	Name of the package to be deleted
	 * 
	 * @return	True iff successful
	 */
	public boolean deleteRPackage(String name) {

		ensureRPackageRegistered(name);

		boolean result = Database.eraseDirectoryRecursively(new File(database.getDatabasePath()
				+ File.separator + Database.RSCRIPTS_DIRECTORY + File.separator + name));

		if (result) {
			try {
				rHelpManager.reloadRFunctionsHelp();
			} catch (IOException e) {
				Logger.logError(
					"Error reloading help for R scripts after R packing deletion.");
			}
		}

		return result;
	}
	
	/**
	 * Returns all errors that occured in the case R package was rejected for some reason
	 * 
	 * @param handle	Handle of upload operation
	 * @return	Array of strings containing error messages
	 */
	public String[] getRejectedRPackageUploadErrorMessages(UploadHandle handle) {
		return database.getRejectedUploadErrorMessages(handle);
	}
		

	/**
	 * Gives list of all registered R Script packages 
	 * 
	 * @return	Array of RScriptPackage, each item contains info about one registered package
	 */
	public RScriptPackage[] getRScriptPackages() {

		ArrayList<RScriptPackage> scriptPackages = new ArrayList<RScriptPackage>();

		File[] directoryList;

		File path = new File(database.getDatabasePath()
				+ File.separator + "rscripts");

		directoryList = path.listFiles();

		for (int i = 0; i < directoryList.length; i++) {
			if (directoryList[i].isDirectory()) {

				File packagePath = directoryList[i];

				FilenameFilter fileFilter = new FilenameFilter() {
					public boolean accept(File dir, String filename) {
						return filename.matches("description");
					}
				};

				File[] packageRList = packagePath.listFiles(fileFilter);

				String description = "";

				try {
					
					BufferedReader inf = new BufferedReader(new FileReader(
							packageRList[0].getAbsolutePath()));
					String str;
					while ((str = inf.readLine()) != null) {
						description += str + "\n";
					}
					inf.close();
				} catch (IOException e) {
					Logger.logError(
							"Error reading description of R script "
									+ e.getMessage());
					description = "<couldn't read>";
				}

				RScriptPackage rPackage = new RScriptPackage(directoryList[i]
						.getName(), description);
				scriptPackages.add(rPackage);
			}
		}

		return scriptPackages.toArray(new RScriptPackage[] {});
	}

	/**
	 * Returns list of all set anaysis callback R scripts . Analysis scripts serve just as 
	 * a template for experiment scripts, which are really used at finish of run upload
	 * 
	 * @param aid	Analysis Id
	 * @return		CallbackRScripts object containing callback scripts to be executed 
	 * 				on each level of database after upload is done. 
	 */
	public CallbackRScripts getAnalysisCallbackRScripts(AID aid) {
		return RCallbacksManagerFactory.getRCallbacksManager(aid)
				.getCallbacks();
	}

	/**
	 * Sets the list of all callback R scripts for a given analysis. Analysis scripts serve just as 
	 * a template for experiment scripts, which are really used at finish of run upload
	 * 
	 * @param aid	Analysis Id
	 * @param callbackRScripts		CallbackRScripts object containing callback scripts to 
	 * 	be executed on each level of database after upload is done. 
	 */
	public void setAnalysisCallbackRScripts(AID aid,
			CallbackRScripts callbackRScripts) {
		database.getAnalysisEntityManager().ensureValidAid(aid);
		try {
			RCallbacksManagerFactory.getRCallbacksManager(aid).setCallbacks(
					callbackRScripts);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Sets the list of all callback R scripts to be executed for a given experiment. 
	 * 
	 * @param aid	Analysis Id
	 * @param eid 	Experiment Id
	 * @param callbackRScripts		CallbackRScripts object containing callback scripts to be executed 
	 * 								on each level of database after upload is done. 
	 */
	public void setExperimentCallbackRScripts(AID aid, EID eid,
			CallbackRScripts callbackRScripts) {
		database.getExperimentEntityManager().ensureValidEid(aid, eid);
		try {
			RCallbacksManagerFactory.getRCallbacksManager(aid, eid)
					.setCallbacks(callbackRScripts);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Returns help for R Scripts. Used by GUI
	 * 
	 * @return	Array of RFunctionHelp containing help for all R functions in all packages
	 */
	public RFunctionHelp[] getRFunctionsHelp() {
		return RHelpManager.getInstance().getRFunctionsHelp();
	}

	/**
	 * Causes R functions help to be reloaded from packages, useful on startup and after package 
	 * deletion, package registration
	 * 
	 * @throws IOException	In case something goes wrong when loading
	 */
	public void reloadRFunctionsHelp() 
		throws IOException {
		rHelpManager.reloadRFunctionsHelp();		
	}

}
