/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import cz.cuni.mff.been.common.id.RID;


/**
 * Implements management and loading and storing of validity maps to the file
 * 
 * @author Jakub Lehotsky
 */
public class RRIDTable {
		
	/** I/O error in validity map reading message */
	public static final String IO_ERROR_VALIDITY_READING_MESSAGE = "I/O error reading validity map of ";
	/** I/O error in validity map reading message */
	public static final String IO_ERROR_VALIDITY_STORING_MESSAGE = "I/O error storing validity map of ";
	
	/** ASCII of delim. character (TAB) */
	private static final String ASCII_TAB = "\t";	
	
	/** Actual metadata */
	private ArrayList<RRIDItem> rridTable;
	
	/** Last item counter */
	private int lastItem;		
	
	/**
	 * Constructor - creates rrid Table, void object 
	 * (meant for creating new validity map tables items, initializing them and storing into file)
	 * 
	 */
	public RRIDTable() {
		rridTable = new ArrayList<RRIDItem>();
		this.lastItem = 0;
	}
	
	/**
	 * Returns validity map we store
	 * 
	 * @return the metadata
	 */
	
	public RRIDItem[] getRridTable() {
		return rridTable.toArray(new RRIDItem[] {});
	}
	
	/** 
	 * Constructor - initializes vliadity map in memory and read its contents from the file.
	 * 
	 * @param filename	Name of file to read validity table from
	 * @throws IOException 
	 */
	public RRIDTable(String filename) throws IOException {

		String[] lineTokens;
		String line;
		BufferedReader inputReader;

		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {			
			rridTable = new ArrayList<RRIDItem>();
			return;
		}

		rridTable = new ArrayList<RRIDItem>();
		
		while ((line = inputReader.readLine()) != null) {								
			lineTokens = line.split(ASCII_TAB);
			try {
			    	RRIDItem item = new RRIDItem(new RID(Integer.valueOf(lineTokens[0])), 
			    		Boolean.valueOf(lineTokens[1]),  
			    		lineTokens[2], Integer.valueOf(lineTokens[3]));			    	
			    	
			    	addItem(item);
			} catch (IndexOutOfBoundsException e) {
				throw new IOException("Invalid format in validity maps file " + filename + ".");
			}
		}
				
		inputReader.close();
	}

	
	/**
	 * Stores validity map we have in our memory to the file.
	 * 
	 * @param filename	Where to store validity map.
	 * @throws IOException
	 */
	public void store(String filename) throws IOException {
		
		int i;		
		PrintWriter outputWriter;
		
		outputWriter = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
		
		for (i = 0; i < rridTable.size(); i++) {
		    String lineBuffer;
		    lineBuffer = getItemKey(i) + ASCII_TAB + getItemValidity(i) 
		    	+ ASCII_TAB + rridTable.get(i).getTableName() 
		    	+ ASCII_TAB + rridTable.get(i).getHostIndex();
		    outputWriter.println(lineBuffer);		
		}
		
		outputWriter.close();
	}
	
	/**
	 * Returns the count of items currently loaded in memory
	 * 
	 * @return	The count of items
	 */
	public int getItemCount() {
		return rridTable.size();
	}
	
	/**
	 * Adding key-value pair to validity map
	 *
	 * @param anItem	an item to be inserted to the array
	 * @throws ArrayIndexOutOfBoundsException
	 */
	public void addItem(RRIDItem anItem) {		
		rridTable.add(anItem);
		lastItem++;
	}
	
	/**
	 * Deleting item by key
	 * 
	 * @param key	Key of item to be deleted
	 */
	public void deleteItemByKey(RID key) {
		
		int i = 0;		
		try {		
			while (getItemKey(i).compareTo(key) != 0) {
				i++;
				if (i == lastItem) {
					return; // not found
				}
			}
		} catch (ArrayIndexOutOfBoundsException e) {
			return; //not found
		}		
		
		rridTable.remove(i);
	}
	
	/**
	 * Returns the key of the item with index.
	 * 
	 * @param index	Index of they key to give.
	 * @return	RID - the key
	 */
	public RID getItemKey(int index) {
		return rridTable.get(index).getKey();
	}
	
	/**
	 * Returns the value of the item with index.
	 * 
	 * @param index	Index of the value to give.
	 * @return	String with value.
	 */
	/*public String getItemValue(int index) {
		return rridTable.get(index).getFileName();
	}*/
	
	/**
	 * Returns the value of the item with index.
	 * 
	 * @param index	Index of the value to give.
	 * @return	String with value.
	 */
	public boolean getItemValidity(int index) {
		return rridTable.get(index).getValid();
	}

	/**
	 * Returns the value of the item, with given RID key. 
	 * 
	 * @param key	Key to ask for.
	 * @return		String - patch to associated run, or null if not found. 
	 */
	public RRIDItem getItemByKey(RID key) {
		
		if (rridTable == null) {
			Logger.logError("While trying to read RID " + key + ", encountered " 
					+ "null validity map.");
			assert false : "Validity map table unitialized!";
			return null; 
		}
		
		int i = 0;
		
		try {		
			while (getItemKey(i).compareTo(key) != 0) {
				i++;
				if (i == lastItem) {
					return null;
				}
			}
		} catch (IndexOutOfBoundsException e) { 
			return null;
		}
		return rridTable.get(i);		
	}

	/**
	 * Returns the index in array of the item, with given RID key. 
	 * 
	 * @param key	Key to ask for.
	 * @return		index in the array 
	 */
	public int getItemIndexByKey(RID key) {
		
		if (rridTable == null) {
			Logger.logError("While trying to read RID " + key + ", encountered " 
					+ "null validity map.");
			assert false : "Validity map table unitialized!";
			return -1; 
		}
		
		int i = 0;
		
		try {		
			while (getItemKey(i).compareTo(key) != 0) {
				i++;
				if (i == lastItem) {
					return -1;
				}
			}
		} catch (IndexOutOfBoundsException e) { 
			return -1;
		}
		return i;		
	}

	
	/**
	 * Check wether validity map contains given run.
	 * This function serves as support for duplicities testing
	 * 
	 * @param value	Key to ask for.
	 * @return		String - patch to associated run, or null if not found. 
	 */
	public boolean isValueInTable(String value) {
		
		if (rridTable == null) {
			Logger.logError("While checking duplicities of " + value + ", encountered " 
					+ "null validity map.");
			assert false : "Validity map table unitialized!";
			return false; 
		}
		
		try {		
		    for (RRIDItem rridItem : rridTable) {
	    		if (rridItem.getTableName().compareTo(value) == 0) {
	    			return true;
	    		}		    	
		    }
		} catch (IndexOutOfBoundsException e) { 
			return false;
		}
		return false;
	}

	
	/**
	 * Returns the value of the item, with given RID key. 
	 * 
	 * @param key	Key to ask for.
	 * @return		Validity of given run 
	 */
	public boolean getValidityByKey(RID key) {
		
		if (rridTable == null) {
			Logger.logError("While trying to read RID " + key + ", encountered " 
					+ "null validity map.");
			assert false : "Validity map table unitialized!";
			return false; 
		}
		
		int i = 0;
		
		try {		
			while (getItemKey(i).compareTo(key) != 0) {
				i++;
				if (i == lastItem) {
					return false;
				}
			}
		} catch (IndexOutOfBoundsException e) {
			return false;
		}
		return getItemValidity(i);		
	}
	
	/**
	 * Returns the table name of a given item
	 * 
	 * @param index	Index of the item
	 * @return	A table name
	 */
	public String getItemTablename(int index) {
		return rridTable.get(index).getTableName();
	}
	
	/**
	 * Splits the first number from runs data filename (denoted by either '-' or '.'), returns
	 * array of two strings - first containing the beginning number, second the rest of the
	 * filename
	 * 
	 * @param input	The filename
	 * @return	Array of two strings containing the info mentioned above
	 */
	private String[] splitFirstNumber(String input) {
		
		int firstMinus = input.indexOf("-");
		int firstDot = input.indexOf(".");
		int catValue;
		
		if (firstMinus == -1 && firstDot == -1) {
			return new String[] { "", input };
		}
		
		if (firstMinus == -1) {
			catValue = firstDot; 
		} else if (firstDot == -1) {
			catValue = firstMinus;
		} else {
			catValue = Math.min(firstDot, firstMinus);
		}		
		
		return new String[] { input.substring(0, catValue), input.substring(catValue) };
	}
	
	/**
	 * Returns list of all integer indexes, which items number is equal to a given key 
	 * 
	 * @param key	The key
	 * @return	The list
	 */
	public Integer[] getAllIndexesEqualTo(long key) {
		
		ArrayList<Integer> returnValue = new ArrayList<Integer>() {};
		
		for (int i = 0; i < rridTable.size(); i++) {
			RRIDItem rridItem = rridTable.get(i);
			String indexStr = splitFirstNumber(rridItem.getTableName())[0];
			if (Integer.valueOf(indexStr) == key) {
				returnValue.add(i);
			}
		}
		
		return returnValue.toArray(new Integer[] {});
	}
	
	/**
	 * Returns the real index of an item
	 * 
	 * @param index	The index
	 * 
	 * @return 	The real index (internal)
	 */
	public long getRealRunIndex(int index) {
		
		RRIDItem rridItem = rridTable.get(index);
		
		String[] parsed = splitFirstNumber(rridItem.getTableName());
		
		try {
			return Long.valueOf(parsed[0]);
		} catch (NumberFormatException e) {
			return -1;
		}
		
	}
	
	/**
	 * In table, decrease the true index of run file by one
	 * 
	 * @param index	The index
	 */
	public void decreaseIndexByOne(int index) {
		
		RRIDItem rridItem = rridTable.get(index);
		
		String[] parsed = splitFirstNumber(rridItem.getTableName());
		rridItem.setTableName(String.valueOf(Long.valueOf(parsed[0]) - 1) + parsed[1]);
		
	}

	/**
	 * Returns the table name of a given item
	 * 
	 * @param index	Index of the item
	 * @return	A table name
	 */
	public String getItemDecreasedTablename(int index) {
		
		String[] parsed = splitFirstNumber(rridTable.get(index).getTableName());		
		return String.valueOf(Long.valueOf(parsed[0]) - 1) + parsed[1];
	}
	
	
}
