/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;

import cz.cuni.mff.been.benchmarkmanager.AnalysisEntity;
import cz.cuni.mff.been.benchmarkmanager.BinaryEntity;
import cz.cuni.mff.been.benchmarkmanager.ExperimentEntity;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;
import cz.cuni.mff.been.resultsrepository.gui.EntityInterface;
import cz.cuni.mff.been.resultsrepository.gui.RFunctionHelp;
import cz.cuni.mff.been.resultsrepository.gui.RScriptPackage;


/**
 * Implementation of Results Repository. It implements two RMI interfaces:
 *   <code>ResultsRepositoryInterface</code> - communication with BEEN, results collection
 *   <code>gui.ResultsRepositoryInterface</code> - communication with user, data browsing,
 *  			processing, etc. everything needed for GUI 
 *    
 * This class function (single instance allowed) is an encapsulation for all the other 
 * parts of Results Repository, through implemented interfaces it passes all
 * work to the respective parts (to LoadCollector, LogCollector, Database, RManager, etc.) 
 * It acts as wrapper for all other parts of the system, it is first the main class that delegates
 * execution of all calls from RMI interfaces into other classes that form ResultsRepository. 
 * 
 * @author Jakub Lehotsky
 */
public final class ResultsRepositoryImplementation extends UnicastRemoteObject implements 
	ResultsRepositoryInterface, cz.cuni.mff.been.resultsrepository.gui.ResultsRepositoryInterface,
	ResultsRepositoryDebugInterface {

	/** Single instance of Results Repository implementation */
	private static ResultsRepositoryImplementation resultsRepositoryInstance = null;
	
	/** Internal Database instance to work with */
	private static Database database;
	
	/** Internal load collecor of database */
	@SuppressWarnings("unused")
	private static LoadCollector loadCollector;	
	
	/** Internal log collector of database */
	private static LogCollector logCollector; 
	
	/** Internal R manager (packages, scripts, help, etc.) */
	private static RManager rManager;
	
	/** Internal analysis entity manager */
	AnalysisEntityManager analysisEntityManager; 

	/** Internal experiment entity manager */
	ExperimentEntityManager experimentEntityManager; 

	/** Internal experiment entity manager */
	BinaryEntityManager binaryEntityManager; 

	/** Internal run entity manager */
	RunEntityManager runEntityManager; 
	
	
	/**
	 * Empty singleton constructor 
	 * 
	 * @throws RemoteException
	 */	
	private ResultsRepositoryImplementation() 
		throws RemoteException {
	}
	
	/** 
	 * Initialization of Results Repository - loads configuration, sets up all parts of res. 
	 * rep - database at first for data storage and retrieval, then setup all other necessary 
	 * parts and connect them with database 
	 * 
	 * @param dbaseDir	Directory with database
	 * @param tempDir	Temporary directory to use
	 */	
	public void initialize(String dbaseDir, String tempDir) {
		/* Very important to initialize before other components, which use it */
		Logger.initialize();
		database = Database.getInstance();
		database.initialize(dbaseDir, tempDir, 0);		
		analysisEntityManager = database.getAnalysisEntityManager();
		experimentEntityManager = database.getExperimentEntityManager();
		binaryEntityManager = database.getBinaryEntityManager();
		runEntityManager = database.getRunEntityManager();
		loadCollector = database.getLoadCollector();
		logCollector = database.getLogCollector();		
		rManager = database.getRManager();
		
	}
	
	/**
	 * Shutdown of results repositoy - database shutdown
	 */
	public static void shutdown() {
	    database.shutdown();	    
    }
	
	/**
	 * Returns the single instance of Results Repository implementation
	 * 
	 * @return	reference to working instance of Results Repository implementation
	 * @throws RemoteException 
	 */
	public static ResultsRepositoryImplementation getInstance() 
		throws RemoteException {
		if (resultsRepositoryInstance == null) {
			return new ResultsRepositoryImplementation();
		} else {
			return resultsRepositoryInstance;
		}
	}

	/**
	 * Returns notification about whether R scripts are enabled
	 * 
	 * @return true, if scripts are enabled
	 *  
	 * @throws RemoteException	In case that something goes wrong in RMI comunication 
	 */
	public boolean rScriptingEnabled() throws RemoteException {
	    return rManager.rScriptingEnabled();
    }	

	
	/* -----------------------------------------
	 * ResultsRepositoryInterface implementation
	 * -----------------------------------------*/
	
	/**
	 * Adds new analysis into results repository, so it can be filled with entities and results,
	 * it must not exist before.
	 * 
	 * @param description Description of analysis - all metadata for creation
	 * 
	 * @throws ResultsRepositoryException	In case something went wrong (analysis already exists,
	 * 										 i/o error)
	 * @throws RemoteException	In case of RMI network connection malfunction
	 */
	public void addAnalysis(AnalysisEntity description) 
			throws ResultsRepositoryException, RemoteException {

		analysisEntityManager.addAnalysis(description);
	}

	/**
	 * Allows adding new custom analysis metadata. 
	 * 
	 * @param analysisId	Analysis Id
	 * @param key			Key of metadata item
	 * @param value			Value of metadata item
	 * @throws ResultsRepositoryException	In case you attempt to overwrite existing metadata item
	 * @throws RemoteException 
	 */
	public void addAnalysisMetadata(AID analysisId, String key, String value)
		throws ResultsRepositoryException, RemoteException {

		analysisEntityManager.addAnalysisMetadata(analysisId, key, value);
	}

	/**
	 * Adds new experiment into given analysis of results repository, so it can be filled 
	 * with entities and results, it must not exist before. 
	 *  
	 * @param description 	Object describing the experiment
	 * @throws ResultsRepositoryException 
	 * @throws RemoteException 
	 * @see cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface#addExperiment
	 */
	public void addExperiment(ExperimentEntity description) 
			throws ResultsRepositoryException, RemoteException {
		
		experimentEntityManager.addExperiment(description);				
	}
	
	/**
	 * Allows adding new custom experiment metadata. 
	 * 
	 * @param analysisId	Analysis Id
	 * @param experimentId	Experiment Id
	 * @param key			Key of metadata item
	 * @param value			Value of metadata item
	 * @throws RemoteException
	 * @throws ResultsRepositoryException	In case you attempt to overwrite existing metadata item
	 */	
	public void addExperimentMetadata(AID analysisId, EID experimentId, String key, String value) 
		throws ResultsRepositoryException, RemoteException {
		
		experimentEntityManager.addExperimentMetadata(analysisId, experimentId, key, value);		
	}

	/**
	 * Adds new binary into given analysis and experiment of results repository, so it can 
	 * be filled with entities and results, it must not exist before. 
	 *  
	 * @param description 	Object describing binary being entered into Results Repository
	 * 
	 * @throws ResultsRepositoryException 
	 * @throws RemoteException
	 * @see cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface#addBinary
	 */
	public void addBinary(BinaryEntity description) 
			throws ResultsRepositoryException, RemoteException {
	
		binaryEntityManager.addBinary(description);		
	}

	/**
	 * Allows adding new custom binarydata. 
	 * 
	 * @param analysisId	Analysis Id
	 * @param experimentId	Experiment Id
	 * @param binaryId		Binary Id
	 * @param key			Key of metadata item
	 * @param value			Value of metadata item
	 * @throws RemoteException
	 * @throws ResultsRepositoryException	In case you attempt to overwrite existing metadata item
	 */	

	public void addBinaryMetadata(AID analysisId, EID experimentId, BID binaryId, String key, String value) 
	throws ResultsRepositoryException, RemoteException {
	
		binaryEntityManager.addBinaryMetadata(analysisId, experimentId, binaryId, key, value);		
	}

	public void initLoadUpload(AID aid, EID eid, BID bid, RID rid) throws RemoteException {
		loadCollector.initLoadUpload(aid, eid, bid, rid);
	}
	
	public void closeLoadUpload(AID aid, EID eid, BID bid, RID rid, boolean valid) throws RemoteException {
		loadCollector.closeLoadUpload(aid, eid, bid, rid, valid);
	};
	
	public UploadHandle uploadLoadInfo(AID aid, EID eid, BID bid, RID rid, String filename, 
			String host, int port, boolean blocking) 
		throws UnknownHostException, IOException, ResultsRepositoryBusyException, RemoteException { 
		
		return loadCollector.uploadLoadInfo(aid, eid, bid, rid, filename, host, port, blocking);
	};	


	
	/*
	 * @see cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface#uploadRunResults
	 */
	public UploadHandle uploadRunResults(AID aid, EID eid, BID bid, RID rid,
			String roleName, String host, int port, boolean runValid, boolean blocking,
			int tableIndex, int totalTables) 
		throws UnknownHostException, IOException, ResultsRepositoryBusyException, RemoteException {		
		
		return runEntityManager.uploadRunResults(aid, eid, bid, rid, roleName,
				host, port, runValid, blocking, tableIndex, totalTables);
	}
	
	/*
	 * @see cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface#getFileUploadStatus
	 */
	public UploadStatus getFileUploadStatus(UploadHandle handle) 
		throws RemoteException {
		return database.getFileUploadStatus(handle);
	}
	
	/*
	 * @see cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface#endFileUpload
	 */
	public void endRunUpload(AID aid, EID eid, BID bid, RID rid,
			String host, UploadHandle handle) 
		throws RemoteException {
		runEntityManager.endRunUpload(aid, eid, bid, rid, host, handle);
	}
	
	/*
	 * @see cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface#endFileUpload
	 */
	public void endFileUpload(UploadHandle handle) 
		throws RemoteException {
		database.endFileUpload(handle);
	}
	
	/* ---------------------------------------------
	 * gui.ResultsRepositoryInterface implementation
	 * ---------------------------------------------
	 * (see cz.cuni.mff.been.resultsrepository.ResultsRepositoryUserInterface for
	 * comments upon methods) 
	 */

	public EntityInterface<AID> getAnalysisInterface() throws RemoteException {
		return new AnalysisImplementation();
	}
	
	public UploadHandle beginRScriptPackageUpload(String name, InetAddress ip, int port) 
		throws RemoteException {
		
		try {
		    	return rManager.uploadAndRegisterRPackge(name, ip.getCanonicalHostName(), port);
		} catch (ResultsRepositoryBusyException e) {
			Logger.logError("Unable to upload script package - Results Repository is busy");
		} catch (UnknownHostException e) {
			Logger.logError("Unable to upload script package - Host not found");
		} catch (IOException e) {
			Logger.logError("Unable to upload script package - I/O error " + e.getMessage());
		}
		return null;
	}

	/**
	 * Deletes (unregister) R package from Results Repository
	 *  
	 * @param name	Name of the package to be deleted
	 * 
	 * @return	True iff successful
	 * @throws RemoteException 
	 */
	public boolean deleteRScriptPackage(String name) throws RemoteException {
		return rManager.deleteRPackage(name);
	}

	/**
	 * Closes the upload handle. Client should call this method after the upload
	 * status returned by <code>getFileUploadStatus</code> has value
	 * <code>ACCEPTED</code>, or <code>ERROR</code>.
	 * 
	 * If the client doesn't close the upload handle, Results Repository will leak
	 * a little amount of memory. However, no other resources than memory
	 * (such as open files, locks...) are associated with the upload handle. 
	 * 
	 * @param handle upload handle
	 * @throws RemoteException when something in RMI goes bad
	 */	
	public void endRScriptPackageUpload(UploadHandle handle) throws RemoteException {
		database.endFileUpload(handle);		
	}

	/**
	 * Returns list of all set anaysis callback R scripts . Analysis scripts serve just as 
	 * a template for experiment scripts, which are really used at finish of run upload
	 * 
	 * @param aid	Analysis Id
	 * @return		CallbackRScripts object containing callback scripts to be executed 
	 * 				on each level of database after upload is done. 
	 * @throws RemoteException 
	 */
	public CallbackRScripts getAnalysisCallbackRScripts(AID aid) throws RemoteException {
		return rManager.getAnalysisCallbackRScripts(aid); 
	}

	/**
	 * Returns help for R Scripts. Used by GUI
	 * 
	 * @return	Array of RFunctionHelp containing help for all R functions in all packages
	 * @throws RemoteException 
	 */
	public RFunctionHelp[] getRFunctionsHelp() throws RemoteException {
	    return rManager.getRFunctionsHelp();
	}

	/**
	 * Returns the status of the file upload. Result is one
	 * of the values listed in the <code>UploadStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>UPLOADING</code> mean that
	 * Results Repository still works on the file upload. Values
	 * <code>ACCEPTED</code>, <code>REJECTED</code> and <code>ERROR</code> mean that 
	 * Results Repository has finished its work (either successfully or unsuccessfully) 
	 * and the download handle is ready to be closed by the <code>endRScriptPackageUpload</code> method.   
	 * 
	 * @param handle file upload handle
	 * @return upload status
	 * @throws RemoteException when something in RMI goes bad
	 */

	public UploadStatus getRScriptPackageUploadStatus(UploadHandle handle) throws RemoteException {
		return database.getFileUploadStatus(handle);
	}

	/**
	 * Gives list of all registered R Script packages 
	 * 
	 * @return	Array of RScriptPackage, each item contains info about one registered package
	 * @throws RemoteException 
	 */
	public RScriptPackage[] getRScriptPackages() throws RemoteException {
	    return rManager.getRScriptPackages();		
	}

	/**
	 * Returns all errors that occured in the case R package was rejected for some reason
	 * 
	 * @param handle	Handle of upload operation
	 * @return	Array of strings containing error messages
	 * @throws RemoteException 
	 */
	public String[] getRejectedRScriptPackageUploadErrorMessages(UploadHandle handle) throws RemoteException {
		return rManager.getRejectedRPackageUploadErrorMessages(handle);
	}

	/**
	 * Sets the list of all callback R scripts for a given analysis. Analysis scripts serve just as 
	 * a template for experiment scripts, which are really used at finish of run upload
	 * 
	 * @param aid	Analysis Id
	 * @param callbackRScripts		CallbackRScripts object containing callback scripts to 
	 * 	be executed on each level of database after upload is done. 
	 * @throws RemoteException 
	 */
	public void setAnalysisCallbackRScripts(AID aid, CallbackRScripts callbackRScripts) throws RemoteException {
		rManager.setAnalysisCallbackRScripts(aid, callbackRScripts);
	}

	/**
	 * Sets the list of all callback R scripts to be executed for a given experiment. 
	 * 
	 * @param aid	Analysis Id
	 * @param eid 	Experiment Id
	 * @param callbackRScripts		CallbackRScripts object containing callback scripts to be executed 
	 * 								on each level of database after upload is done. 
	 * @throws RemoteException 
	 */
	public void setExperimentCallbackRScripts(AID aid, EID eid, CallbackRScripts callbackRScripts) throws RemoteException {
	    rManager.setExperimentCallbackRScripts(aid, eid, callbackRScripts);
	}

	/**
	 * Finishes logs upload. This method should be called on the end of the experiments, as it
	 * reads through the list of tasks and uploads logs of all tasks, which weren't uploaded yet.
	 * 
	 * @param analysisId	Analysis Id
	 * @param experimentId	Experiment Id
	 * @throws ResultsRepositoryException
	 * @throws RemoteException 
	 */	
	public void finishLogUpload(AID analysisId, EID experimentId) 
		throws ResultsRepositoryException, RemoteException {
		logCollector.finishLogUpload(analysisId, experimentId);
	}

	/**
	 * This method causes log upload to start. It is syncrhonnous and blocking to ensure that all
	 * logs are uploaded, before work continues on the other side (so that they are not cleaned up) 
	 * 
	 * @param aid		AnalysisId
	 * @param eid		ExperimentId
	 * @param contextName	Context of task to upload logs from
	 * @param taskName		The name of the task to upload logs from
	 * @throws ResultsRepositoryException
	 * @throws RemoteException 
	 */
	public void performLogUpload(AID aid, EID eid, String contextName, String taskName) 
		throws ResultsRepositoryException, RemoteException {
		logCollector.performLogUpload(aid, eid, contextName, taskName);
		
	}
	
	/* ********************************************** */
	/* ResultsRepositoryDebugInterface implementation */
	/* ********************************************** */

	/**
	 * Adds new function execution into the background queue of R scripts execution
	 * 
	 * @param 	aid	Analysis ID
	 * @param	eid Experiment ID
	 * @param	functionToRun	the string containing function to be executed in R
	 * @throws RemoteException 
	 */
	public void enqueueScriptExecution(AID aid, EID eid, String functionToRun) 
		throws RemoteException {
	    RScriptsRunner.getInstance().enqueueScriptExecution(aid, eid, functionToRun);
	}

	/**
	 * Adds new function execution into the background queue of R scripts execution
	 * 
	 * @param 	aid	Analysis ID
	 * @param	eid Experiment ID
	 * @param	bid	Binary ID
	 * @param	functionToRun	the string containing function to be executed in R
	 * @throws RemoteException 
	 */
	public void enqueueScriptExecution(AID aid, EID eid, BID bid, String functionToRun) 
		throws RemoteException {
		RScriptsRunner.getInstance().enqueueScriptExecution(aid, eid, functionToRun);
	}

	/**
	 * Adds new function execution into the background queue of R scripts execution
	 * 
	 * @param 	aid	Analysis ID
	 * @param	eid Experiment ID
	 * @param	bid Binary ID
	 * @param rid 
	 * @param	functionToRun	the string containing function to be executed in R
	 * @throws RemoteException 
	 */
	public void enqueueScriptExecution(AID aid, EID eid, BID bid, RID rid, String functionToRun) 
		throws RemoteException {
		RScriptsRunner.getInstance().enqueueScriptExecution(aid, eid, functionToRun);
	}
	
	/**
	 * Causes R scripts (in working directory) to be reloaded into R 
	 * 
	 * @throws RemoteException 
	 * @throws ResultsRepositoryException 
	 */
	public void resourceRScripts() 
		throws RemoteException, ResultsRepositoryException {
			RManager.sourceRegisteredScripts();
	}

	/**
	 * Reports column names for experiment
	 * 
	 * @param analysisId	Analysis Id
	 * @param experimentId	Experiment Id
	 * @param columnNames	<code>ArrayList&lt;String&gt;</code> containing list of column names 
	 * @throws RemoteException
	 */
	public void reportExperimentColumnNames(AID analysisId, EID experimentId, ArrayList<String> columnNames) 
		throws RemoteException {
		
		synchronized (LockManager.columnsLock) {
			experimentEntityManager.reportExperimentColumnNames(analysisId, experimentId, columnNames);
		}
		
	}
}
