/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.File;
import java.io.IOException;

import cz.cuni.mff.been.task.Service;
import cz.cuni.mff.been.task.Task;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

public class ResultsRepositoryService extends Service {
	/** Name of the service. */
	public static final String SERVICE_NAME = "resultsrepository";
	/** Human-readable name of the service (displayed in the web interface). */
	public static final String SERVICE_HUMAN_NAME = "Results Repository";

	/** Main ResultsRepository instance, which handles all RMI interfaces */
	private static ResultsRepositoryImplementation resultsRepositoryInstance;
	
	/** Command line argument name for the file to be copied */
	public static final String DBASE_DIRECTORY = "dbase_dir";
	
	/** The path to database of results */
	private static String dbaseDirectory;	
	
	/** The path to the task temporary working directory */
	private static String tempDirectory;
	
	
	/**
	 * Allocates a new <code>ResultsRepositoryService</code> object. It is here
	 * only to shut up the compiler.
	 * 
	 * @throws TaskInitializationException if something fails during the task
	 *          creation
	 */
	public ResultsRepositoryService() throws TaskInitializationException {
		/* Check all necessary running directories */
		getAndCheckDirectories();
		
		/* Create the results repository instance. */
		createInstance();
		
		/* Bind the results repository interfaces to RMI URLs*/
		addRemoteInterface(cz.cuni.mff.been.resultsrepository.gui.ResultsRepositoryInterface.RMI_NAME, 
				(cz.cuni.mff.been.resultsrepository.gui.ResultsRepositoryInterface) resultsRepositoryInstance);
		addRemoteInterface(ResultsRepositoryInterface.RMI_NAME, 
				(ResultsRepositoryInterface) resultsRepositoryInstance);
		addRemoteInterface(cz.cuni.mff.been.resultsrepository.ResultsRepositoryDebugInterface.RMI_NAME, 
				(cz.cuni.mff.been.resultsrepository.ResultsRepositoryDebugInterface) resultsRepositoryInstance);
		
		/* Everything is OK */
		logInfo("Results repository is up and running.");
	
	}	
	
	/**
	 * Check whether valid directory directoryName exists. In case of failure, exit task 
	 * (this is method intended for necessary directories testing)
	 * 
	 * @param directoryName	Name of directory to test
	 */	 
	private void checkDirectory(String directoryName) {
		
		File directory = new File(directoryName);
		
		if (!directory.exists()) {
			logFatal(directory + " does not exist.");
			System.exit(Task.EXIT_CODE_ERROR);
		}
		if (!directory.isDirectory()) {
			logFatal(directory + " is not directory.");
			System.exit(Task.EXIT_CODE_ERROR);
		}
		if (!directory.canWrite()) {
			logFatal(directory + " is not writable.");
			System.exit(Task.EXIT_CODE_ERROR);
		}
		
	}	
	
	/**
	 * Get parameters of task (directories) and check their existence
	 * 
	 * @param args	Arguments from command line
	 */
	private void getAndCheckDirectories() {
		
		String beenHome;
		
		/* Ensure BEEN_HOME environment variable exists */
		if ((beenHome = System.getenv("BEEN_HOME")) == null) {
			logFatal("Environment variable BEEN_HOME not defined.");
			System.exit(EXIT_CODE_ERROR);
		}
		
		/* Check whether this directory exists */
		checkDirectory(beenHome);
				
		/* Set up and check results repository database directory */
		if (getTaskProperty(DBASE_DIRECTORY) == null) {
			dbaseDirectory = getWorkingDirectory();
		} else {
			dbaseDirectory = beenHome + File.separatorChar 
				+ getTaskProperty(DBASE_DIRECTORY);
		}
				
		checkDirectory(dbaseDirectory);
		
		tempDirectory = getTempDirectory();
		checkDirectory(tempDirectory);
		
	}

	/**
	 * Creates instance of results repository implementation, which are initialized with 
	 * all directories
	 * 
	 * @throws TaskInitializationException
	 */
	private void createInstance() throws TaskInitializationException {		
		resultsRepositoryInstance = null;
		try {
			resultsRepositoryInstance = ResultsRepositoryImplementation.getInstance();
		} catch (IOException e) {
			logFatal("I/O error occuerd: " + e.getMessage());
			throw new TaskInitializationException(e);
		}
	}
	
	/**
	 * @see cz.cuni.mff.been.task.Service#getName()
	 */
	public String getName() {
		return SERVICE_NAME;
	}
	
	/**
	 * @see cz.cuni.mff.been.task.Service#start()
	 */
	protected void start() throws TaskException {

		try {
			tryCopyExampleData();
			/* NOTE: copies all package data, not just exampleData as name denotes */
		} catch (IOException e) {
			throw new TaskException(e);
		}		
		
		resultsRepositoryInstance.initialize(dbaseDirectory, tempDirectory);
	}

	/**
	 * @see cz.cuni.mff.been.task.Service#stop()
	 */
	protected void stop() throws TaskException {
	    ResultsRepositoryImplementation.shutdown();
	}
}
