/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.IOException;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.EID;


/**
 * This class is used for managing roles and hosts for each experiment
 * 
 * @author Jakub Lehotsky
 */
public final class RolesManager {

	/** Extensions of temporary directory for R scripts unpacking */
	private static final String ROLES_LIST_FILENAME = "roles";
	
	/** Single instance of roles manager */
	private static RolesManager rMInstance = null;
	
	/** Databse that is used for results in Results Repository */
	@SuppressWarnings("unused")
	private static Database database;
	
	/** Roles table */
	private static RoleList rolesTable;
	
	/**
	 *  Construction, notify load collector about Results Repository database when constructing
	 */
	private RolesManager() {	
	}
	
	/**
	 * Returns the single instance of roles manager
	 * 
	 * @return	Reference to RolesManager instance 
	 */
	public static RolesManager getInstance() {
		if (rMInstance == null) {
			return new RolesManager();
		} else {
			return rMInstance;
		}
	}
	
	/** 
	 * Must be called prior to use
	 * 	 
	 * @param database	An instance of Results Repository database 
	 */
	public static void initialize(Database database) {
		RolesManager.database = database;
	}
	
	
	/**
	 * Loads experiment table of roles on hosts mapping into memory 
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 */
	public void loadRoleList(AID aid, EID eid) {
		try {
	    	rolesTable = new RoleList(database.getFileInDatabase(aid, eid, ROLES_LIST_FILENAME));
	    } catch (IOException e) {
	    	e.printStackTrace();
	    	Logger.logError(NumberedTable.IO_ERROR_LIST_READING_MESSAGE 
			+ Database.getExperimentReferenceString(aid, eid));
		
	    	rolesTable = new RoleList();
	    	return;	
	    }	
	}
	
	/**
	 * Stores role list on disk for a given experiment
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 */
	void storeRoleList(AID aid, EID eid) {
	    try {
	        	rolesTable.store(database.getFileInDatabase(aid, eid, ROLES_LIST_FILENAME));
            } catch (IOException e) {
            	e.printStackTrace();
            	Logger.logError(NumberedTable.IO_ERROR_LIST_STORING_MESSAGE
            			+ Database.getExperimentReferenceString(aid, eid));
            }
	}

	/**
	 * Creates the experiment roles vs. hosts table when creating new experiment
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param roleItems	Array with list of items to be put into array
	 */
	void makeExperimentTable(AID aid, EID eid, RoleItem[] roleItems) {
		 
		rolesTable = new RoleList();
		
		for (RoleItem roleItem : roleItems) {
			rolesTable.addItem(roleItem);
		}
		
		storeRoleList(aid, eid);
		
	}
	
	/**
	 * Returns the key associated with a given host name for a given experiment (must 
	 * be loaded before use of this merhod (see <code>loadRoleList()</code>)
	 * 
	 * @param hostname		Hostname to be querried
	 * @return	The key		The key which was associated with him, -1 is it doesn't exist
	 */
	long getKeyOfHostname(String hostname) {	
		
		int itemIndex = rolesTable.getItemIndexByHostname(hostname);
		return (itemIndex > 0) ? rolesTable.getItemKey(itemIndex) : itemIndex;
		
	}
	
	/**
	 * Returns one role item identified by index
	 * 
	 * @param index	The index of the item
	 * 
	 * @return	The item from role list
	 */
	RoleItem getRoleItem(int index) {
		return rolesTable.getItem(index);
	}
	
	/**
	 * Returns the index the item, with given key. 
	 * 
	 * @param key	Key to ask for.
	 * @return		Index of associated item in the table, -1 if it doesn't exist 
	 */
	public int getItemIndex(long key) {
		return rolesTable.getItemIndex(key);
	}
	
	/**
	 * Returns role count in a table
	 * 
	 * @return	Role count
	 */
	public int getRoleCount() {
		return rolesTable.getItemCount();
	}

}
