/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

import cz.cuni.mff.been.benchmarkmanager.gui.BenchmarkManagerGUIInterface;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.Statistics;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;




/**
 * Simple class for performing of logs collection from Task Manager and storing it
 * in database
 * 
 * @author Jakub Lehotsky
 */
public final class RunEntityManager {

	/** Single instance of load collector */
	private static RunEntityManager runEntityManager = null;
	
	/** Databse that is used for results in Results Repository */
	private Database database;
	
	/** Internal statistics manager */
	private StatisticsManager statisticsManager;

	/** Internal task list manager */
	private TaskListManager taskListManager;

	/** Internal graph manager */
	private GraphManager graphManager;

	/** Internal RID Manager */
	private static  RunValidityMapsManager ridManager;

	/** Internal binary entity manager */
	private BinaryEntityManager binaryEntityManager;
	
	/** Internal binary entity manager */
	private ExperimentEntityManager experimentEntityManager;


	/** Internal lock manager */
	private LockManager lockManager;

	/** Internal scripts runner reference */
	private RScriptsRunner rScriptsRunner;

	
	/**
	 *  Construction, notify load collector about Results Repository database when constructing
	 */
	private RunEntityManager() {
		database = Database.getInstance();
	}
	
	/**
	 * Returns the single instance of load collector
	 * 
	 * @return	Reference to LoadCollector instance 
	 */
	public static RunEntityManager getInstance() {
		if (runEntityManager == null) {
			return new RunEntityManager();
		} else {
			return runEntityManager;
		}
	}	
	
	/**
	 * Use to initialize analysis entity manager 
	 * 
	 * @param database			Instance of Results Repository database
	 * @param statisticsManager	Instance of Results Repository statistics manager
	 * @param taskListManager 	Instance of Results Repository task list manager
	 * @param graphManager 		Instance of Results Repository graph manager
	 * @param ridManager 		Instance of Results Repository run validity maps manager
	 * @param lockManager		Instance of Results Repository lock manager 
	 * @param scriptsRunner 	Instance of Results Repository scripts runner
	 * @param binaryEntityManager Instance of Results Repository binary entity manager
	 * @param experimentEntityManager	Instance of Results Repository experiment entity manager 

	 */
	public void initialize(Database database,  
			StatisticsManager statisticsManager, TaskListManager taskListManager,
			GraphManager graphManager, RunValidityMapsManager ridManager,
			LockManager lockManager,
			RScriptsRunner scriptsRunner, BinaryEntityManager binaryEntityManager,
			ExperimentEntityManager experimentEntityManager) {
		
		this.database = database;
		this.statisticsManager = statisticsManager;
		this.taskListManager = taskListManager;
		this.graphManager = graphManager;
		RunEntityManager.ridManager = ridManager;
		this.lockManager = lockManager;
		this.rScriptsRunner = scriptsRunner;
		
		this.binaryEntityManager = binaryEntityManager;
		this.experimentEntityManager = experimentEntityManager;

	}
	

	/**
	 * Returns position of actual physical file with run data, given aid, eid,
	 * bid and rid, null in the case it doesn't exist. Path is relative to
	 * binary directory.
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 * @param rid
	 *            RunID
	 * @return Position of run data in binary directory
	 */
	public String[] getRunPaths(AID aid, EID eid, BID bid, RID rid) {
		return ridManager.getRunPaths(aid, eid, bid, rid);
	}	
	
    /**
     * Returns the list of runs in a given binary stored in Results Repository
     * 
     * @param aid	Analysis Id 
     * @param eid 	Experiment Id
     * @param bid 	Binary Id
     * 
     * @return The list
     */
	@SuppressWarnings("unchecked")
	public List<Entity<RID>> getRunsInBinary(AID aid, EID eid, BID bid) {

		binaryEntityManager.ensureValidBid(aid, eid, bid);

		RRIDTable runList;
		ArrayList<Entity<RID>> result;

		runList = ridManager.getRunTable(aid, eid, bid);
		result = new ArrayList<Entity<RID>>(runList.getItemCount());

		if (runList == null) {
			throw new RuntimeException(new ResultsRepositoryException(
					"Error reading run's files ("
							+ Database.getBinaryReferenceString(aid, eid, bid) + ")."));
		}

		MetadataRef metadataBinaryFile = new MetadataRef(new Metadata());
		MetadataRef metadataExperimentFile = new MetadataRef(new Metadata());
		Metadata metadataFile;

		HashSet<RID> ridSet = new HashSet();
		
		for (int i = 0; i < runList.getItemCount(); i++) {

			RID rid = runList.getItemKey(i);
			if (ridSet.contains(rid)) {
				continue;
			}
			
			ridSet.add(rid);						

			/*
			 * optimize these? event. create and use getRun<whatever>Path(aid,
			 * eid, bid, index)
			 */
			String metadataPath = ridManager.getRunMetadataPath(aid, eid, bid,
					rid);

			try {
				metadataFile = new Metadata(metadataPath);
			} catch (IOException e) {
				throw new RuntimeException(new ResultsRepositoryException(
						Metadata.IO_ERROR_METADATA_READING_MESSAGE + "runs in "
								+ Database.getBinaryReferenceString(aid, eid, bid)) + ".");
			}

			Statistics[] stat;

			stat = statisticsManager.loadRunStatistics(aid, eid, bid, rid);

			metadataFile.addItem("valid", new Boolean(ridManager.isValidRun(
					aid, eid, bid, rid)).toString());

			/*
			 * There is no "partially uploaded run" - this operation is (hope,
			 * will it will be true :-P) atomic
			 */
			metadataFile.addItem("complete", "true");
			
			BinaryEntityManager.ensureSuccessLoadBinaryMetadata(metadataBinaryFile, aid, eid, bid);
			ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(
					metadataExperimentFile, aid, eid);
			
			String metaItem;
			boolean completeBinary;
			boolean completeExperiment;
			
			if (Database.ensureMetadataNotNull(metaItem = metadataBinaryFile.metadata
					.getItemByKey("complete"), "complete",
					Database.getBinaryReferenceString(aid, eid, bid))) {
				completeBinary = new Boolean(metaItem);
			} else {
				/* invalid metadata shouldn't prevent deleting of entity */
				completeBinary = true; 
			}

			if (Database.ensureMetadataNotNull(metaItem = metadataExperimentFile.metadata
					.getItemByKey("complete"), "complete",
					Database.getExperimentReferenceString(aid, eid))) {
				completeExperiment = new Boolean(metaItem);
			} else {
				/* invalid metadata shouldn't prevent deleting of entity */
				completeExperiment = true; 
			}

			result.add(new Entity<RID>(rid, metadataFile.getMetadata(), stat, 
					rScriptsRunner.isDeleteRunAllowed(aid, eid, bid, rid) 
					&& ((completeBinary && completeExperiment) 
							||	(binaryEntityManager.binaryFinished(aid, eid, bid) 
								&& experimentEntityManager.experimentFinished(aid, eid))
						)));
		}

		Collections.sort((List<Entity<RID>>) result);

		return result;
	}

	/**
	 * Checks whether RID conforms to some existing run in results
	 * repository
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @param rid
	 *            Run ID
	 * @return True iff analysis does exist in results repository
	 */
	public boolean isValidRid(AID aid, EID eid, BID bid, RID rid) {

		/*
		 * Reads validity map table and ensure, that file it points to is
		 * readable
		 */
		String[] runPaths = getRunPaths(aid, eid, bid, rid);

		if (runPaths == null || runPaths.length == 0) {
			return false;
		}

		for (String runPath : runPaths) {
			String fullRunPath = database.getFileInDatabase(aid, eid, bid, runPath);
			File runFile = new File(fullRunPath);
			
			Logger.logTrace("Check path to " + Database.getRunReferenceString(aid, eid, bid, rid) 
					+ " - it is " + fullRunPath);

			if (!(runFile.exists() && runFile.isFile() && runFile.canRead())) {
				Logger.logTrace("Check failed...");
				return false;
			}
			Logger.logTrace("Check passed...");
		}

		return true;
	}

    /**
     * Ensures that given RID belongs to a valid run in a given binary in results repository,
     * otherwise throws an IllegalArgumentException
     * 
     * @param aid
	 *		Analysis ID
     * @param eid
     * 		Experiment Id
     * @param bid
     * 		Binary Id
     * @param rid
     * 		Run Id
     */
	public void ensureValidRid(AID aid, EID eid, BID bid, RID rid) {
		if (!isValidRid(aid, eid, bid, rid)) {
			throw new IllegalArgumentException(Database.getRunReferenceString(aid, eid,
					bid, rid)
					+ " doesn't exist.");
		}
	}
	
	/**
	 * Checks whether run is deletable - i.e. no event is scheduled to perform upon it that
	 * would prevent deletion
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * @return	True, if no event is blocking run delete
	 */
	private boolean isDeletable(AID aid, EID eid, BID bid, RID rid) {
		
		MetadataRef metadataBinaryFile = new MetadataRef(new Metadata());
		MetadataRef metadataExperimentFile = new MetadataRef(new Metadata());
		
		BinaryEntityManager.ensureSuccessLoadBinaryMetadata(metadataBinaryFile, aid, eid, bid);
		ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(
				metadataExperimentFile, aid, eid);
		
		String metaItem;
		boolean completeBinary;
		boolean completeExperiment;
		
		if (Database.ensureMetadataNotNull(metaItem = metadataBinaryFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getBinaryReferenceString(aid, eid, bid))) {
			completeBinary = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			completeBinary = true; 
		}

		if (Database.ensureMetadataNotNull(metaItem = metadataExperimentFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getExperimentReferenceString(aid, eid))) {
			completeExperiment = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			completeExperiment = true; 
		}
		
		return completeBinary && completeExperiment;
		
	}

	/**
	 * Returns the Entity&lt;RID&gt; object representing run for use in GUI
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id 
	 * @param rid 	Run Id
	 * 
	 * @return	The binary entity
	 */
	public Entity<RID> getRun(AID aid, EID eid, BID bid, RID rid) {

		ensureValidRid(aid, eid, bid, rid);

		Entity<RID> result;

		MetadataRef metadataBinaryFile = new MetadataRef(new Metadata());
		MetadataRef metadataExperimentFile = new MetadataRef(new Metadata());
		Metadata metadataFile;

		try {
			metadataFile = new Metadata(ridManager.getRunMetadataPath(aid, eid,
					bid, rid));
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Unable to read run data.");
			return null;
		}

		metadataFile.addItem("valid", new Boolean(ridManager.isValidRun(aid,
				eid, bid, rid)).toString());
		
		/* There is no "partially uploaded run" - this operation is atomic */
		metadataFile.addItem("complete", "true");
		
		BinaryEntityManager.ensureSuccessLoadBinaryMetadata(metadataBinaryFile, aid, eid, bid);
		ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(
				metadataExperimentFile, aid, eid);
		
		String metaItem;
		boolean completeBinary;
		boolean completeExperiment;
		
		if (Database.ensureMetadataNotNull(metaItem = metadataBinaryFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getBinaryReferenceString(aid, eid, bid))) {
			completeBinary = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			completeBinary = true; 
		}

		if (Database.ensureMetadataNotNull(metaItem = metadataExperimentFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getExperimentReferenceString(aid, eid))) {
			completeExperiment = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			completeExperiment = true; 
		}

		result = new Entity<RID>(rid, metadataFile.getMetadata(),
				statisticsManager.loadRunStatistics(aid, eid, bid, rid), 
		rScriptsRunner.isDeleteRunAllowed(aid, eid, bid, rid) 
			&& ((completeBinary && completeExperiment) 
				|| (binaryEntityManager.binaryFinished(aid, eid, bid) 
					&& experimentEntityManager.experimentFinished(aid, eid))
			));

		return result;
	}

	/**
	 * Sets the comment metadata of a run
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * @param comment	Comment to set
	 */
	public void setRunComment(AID aid, EID eid, BID bid, RID rid, String comment) {

		ensureValidRid(aid, eid, bid, rid);

		String metaPath = ridManager.getRunMetadataPath(aid, eid, bid, rid);

		File file = new File(metaPath);
		Metadata metadata;

		if (file.exists()) {
			try {
				metadata = new Metadata(metaPath);
			} catch (IOException e) {
				throw new RuntimeException(new ResultsRepositoryException(
						Metadata.IO_ERROR_METADATA_READING_MESSAGE
								+ Database.getRunReferenceString(aid, eid, bid, rid)));
			}
			metadata.setItemByKey("comment", comment);

		} else {
			metadata = new Metadata();
			metadata.addItem("name", "");
			metadata.addItem("comment", comment);
		}

		try {
			metadata.store(metaPath);
		} catch (IOException e) {
			throw new RuntimeException(new ResultsRepositoryException(
					Metadata.IO_ERROR_METADATA_READING_MESSAGE
							+ Database.getRunReferenceString(aid, eid, bid, rid)));
		}
	}
	

	/**
	 * Deletes given run
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @param rid
	 *            Run ID
	 * 
	 * @return True iff successful
	 * 
	 * @throws DeleteLockedException	If run is locked for deleting 
	 */
	public boolean eraseRun(AID aid, EID eid, BID bid, RID rid) 
		throws DeleteLockedException {

		ensureValidRid(aid, eid, bid, rid);
		
		synchronized (LockManager.eraseLock) {
				
			/* Note: for further improvement of synchronization, callbacks should be locked too 
			 * (because they handle metadata of existing entities), now it cannot happen. For 
			 * additional measurements, it must be introduced */
			
			if (!rScriptsRunner.isDeleteRunAllowed(aid, eid, bid, rid) 
					&& (isDeletable(aid, eid, bid, rid)
							|| (binaryEntityManager.binaryFinished(aid, eid, bid) 
									&& experimentEntityManager.experimentFinished(aid, eid)								
								) 
						)
				) {
				Logger.logWarning("Tried to delete locked " 
						+ Database.getRunReferenceString(aid, eid, bid, rid) + ".");
				throw new DeleteLockedException("Run is being processed, unable to delete.");

			}
					
			MetadataRef binaryMetadata = new MetadataRef(new Metadata());
			MetadataRef experimentMetadata = new MetadataRef(new Metadata());
			MetadataRef analysisMetadata = new MetadataRef(new Metadata());
			Integer runs;
			Integer validRuns;
			Integer expectedRuns;
			String metaItem;
			boolean validRun;
			
			validRun = ridManager.isValidRun(aid, eid, bid, rid); 
	
			/* Update run count in binary metadata */
			if (!BinaryEntityManager.ensureSuccessLoadBinaryMetadata(binaryMetadata, aid, eid, bid)) {			
				return false;
			}
	
			if (Database.ensureMetadataNotNull(metaItem = binaryMetadata.metadata
					.getItemByKey("valid runs"), "valid runs",
					Database.getBinaryReferenceString(aid, eid, bid))) {
				validRuns = new Integer(metaItem);
			} else {
				validRuns = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = binaryMetadata.metadata
					.getItemByKey("runs"), "runs", Database.getBinaryReferenceString(aid,
					eid, bid))) {
				runs = new Integer(metaItem);
			} else {
				runs = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = binaryMetadata.metadata
					.getItemByKey("expected runs"), "expected runs",
					Database.getBinaryReferenceString(aid, eid, bid))) {
				expectedRuns = new Integer(metaItem);
			} else {
				expectedRuns = new Integer(0);
			}
	
			expectedRuns--;
			runs--;
			binaryMetadata.metadata.setItemByKey("runs", runs.toString());
			binaryMetadata.metadata.setItemByKey("expected runs", expectedRuns
					.toString());
	
			if (validRun) {
	
				validRuns--;
				binaryMetadata.metadata.setItemByKey("valid runs", validRuns
						.toString());
				
				if (Database.ensureMetadataNotNull(metaItem = binaryMetadata.metadata
						.getItemByKey("last valid run idx"), "last valid run idx",
						Database.getBinaryReferenceString(aid, eid, bid))) {
					Integer lastValidRunIndex = Integer.valueOf(metaItem);
					lastValidRunIndex--;
					binaryMetadata.metadata.setItemByKey("last valid run idx", 
							lastValidRunIndex.toString());
				}
				
				if (runs > 0) {
	
					if (validRuns == 0) {
						binaryMetadata.metadata.setItemByKey("valid", "false");
						if (!ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(
								experimentMetadata, aid, eid)) {
							return false;
						}
	
						Integer validBins;
						
						if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
								.getItemByKey("valid binaries"), "valid binaries",
								Database.getExperimentReferenceString(aid, eid))) {
	
							validBins = Integer.valueOf(metaItem);
							experimentMetadata.metadata.setItemByKey("valid binaries",
									String.valueOf(validBins - 1));
						} else {
							validBins = new Integer(0);
						}
	
						/*
						 * Now check, whether experiment didn't become invalid, ie
						 * this was last valid binary (although rare and prob.
						 * temporary case )
						 */
						if (validBins.equals(new Integer(0))) {
							experimentMetadata.metadata.setItemByKey("valid",
									"false");
							experimentMetadata.metadata.setItemByKey(
									"valid binaries", "0");
							if (!ExperimentEntityManager.ensureSuccessStoreExperimentMetadata(
									experimentMetadata.metadata, aid, eid)) {
								return false;
							}
	
							if (!AnalysisEntityManager.ensureSuccessLoadAnalysisMetadata(
									analysisMetadata, aid)) {
								return false;
							}
	
							Integer validExps;
							
							if (Database.ensureMetadataNotNull(metaItem = analysisMetadata.metadata
									.getItemByKey("valid experiments"), "valid experiments",
									Database.getAnalysisReferenceString(aid))) {
								
								validExps = Integer.valueOf(metaItem);
								analysisMetadata.metadata.setItemByKey(
										"valid experiments", String
												.valueOf(validExps - 1));
							} else {
								validExps = new Integer(0);
							}						
	
							/* Now the same with analysis level */
							if (validExps == 0) {
								analysisMetadata.metadata.setItemByKey("valid",
										"false");
								analysisMetadata.metadata.setItemByKey(
										"valid experiments", "0");
								if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(
										analysisMetadata.metadata, aid)) {
									return false;
								}
							} else {
								if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(
										analysisMetadata.metadata, aid)) {
									return false;
								}
							}
						} else {
							if (!ExperimentEntityManager.ensureSuccessStoreExperimentMetadata(
									experimentMetadata.metadata, aid, eid)) {
								return false;
							}
						}
					}
				} else /* runs == 0 */ {
	
					String binaryValid = binaryMetadata.metadata
							.getItemByKey("valid");
	
					if (binaryValid.equals("false")) {
						/* One valid binary has been added */
						if (!ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(
								experimentMetadata, aid, eid)) {
							return false;
						}
						
						boolean expValid = experimentMetadata.metadata
								.getItemByKey("valid").equals("true");
						
						Integer validBins;
						
						if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
								.getItemByKey("valid binaries"), "valid binaries", 
								Database.getExperimentReferenceString(aid, eid))) {
							validBins = Integer.valueOf(metaItem);
							experimentMetadata.metadata.setItemByKey("valid binaries",
									String.valueOf(validBins + 1));
						} else {
							validBins = new Integer(0);
						}						
						
						experimentMetadata.metadata.setItemByKey("valid", "true");

						if (!ExperimentEntityManager.ensureSuccessStoreExperimentMetadata(
								experimentMetadata.metadata, aid, eid)) {
							return false;
						}
						if (!expValid) {
							/* One valid experiment has been added */
							if (!AnalysisEntityManager.ensureSuccessLoadAnalysisMetadata(
									analysisMetadata, aid)) {
								return false;
							}
							
							Integer validExps;
	
							if (Database.ensureMetadataNotNull(metaItem = analysisMetadata.metadata
									.getItemByKey("valid experiments"), "valid experiments", 
									Database.getAnalysisReferenceString(aid))) {
								validExps = Integer.valueOf(metaItem);
								analysisMetadata.metadata.setItemByKey(
										"valid experiments", String
												.valueOf(validExps + 1));
							} else {
								validExps = new Integer(0);
							}						
							
							analysisMetadata.metadata.setItemByKey("valid", "true");

							if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(
									analysisMetadata.metadata, aid)) {
								return false;
							}
						}
					}
	
					binaryMetadata.metadata.setItemByKey("valid", "true");
				}
			}
	
			if (!BinaryEntityManager.ensureSuccessStoreBinaryMetadata(binaryMetadata.metadata, aid,
					eid, bid)) {
				return false;
			}
	
			boolean result;
	
			result = true;
			
			String[] runPaths = getRunPaths(aid, eid, bid, rid);
	
			for (String runPath : runPaths) {
				File file = new File(database.getFileInDatabase(aid, eid, bid, runPath));
	
				if (!file.delete()) {
					return false;
				}
	
				file = new File(database.getFileInDatabase(aid, eid, bid, 
						ridManager.getRunStatisticsPath(aid, eid, bid, rid)));
				if (file.exists()) {
					result = result && file.delete();
				}
	
				file = new File(database.getFileInDatabase(aid, eid, bid, 
						ridManager.getRunMetadataPath(aid, eid, bid, rid)));
				if (file.exists()) {
					result = result && file.delete();
				}
			}
	
			long runIndex = ridManager.getRealRunIndex(aid, eid, bid, rid);
			
			if (runIndex == -1) {
				Logger.logError("Couldn't get internal runs index of " 
						+ Database.getRunReferenceString(aid, eid, bid, rid) +
						" reindexing not performed, database is probably inconsistent for this binary " +
						"- delete recommended.");
				
				return false;
			}
			
			ridManager.deleteItem(aid, eid, bid, rid);
			
			if (validRun) {
				RRIDTable ridTable = ridManager.getRunTable(aid, eid, bid);
				
				Integer[] indexes;
				while ((indexes = ridTable.getAllIndexesEqualTo(++runIndex)).length > 0) {
					for (int i : indexes ) {
						(ridTable.getRridTable())[i].getTableName();
		
						File tableFile = new File(database.getFileInDatabase(aid, eid, 
								bid, ridTable.getItemTablename(i)));
						
						File tableNewFile = new File(database.getFileInDatabase(aid, eid, 
								bid, ridTable.getItemDecreasedTablename(i)));				
					
						tableFile.renameTo(tableNewFile);
						
						ridManager.decreaseIndexByOne(i);
					}
				}
			}
	
			Logger.logDebug("Success of run erasing = " + result);

			/* Now execute invalidation callbacks */
			try {
				
				File scriptsFile = new File(
		        		database.getInvalidateScriptsDirectory(aid, eid, bid) + File.separator +
		        		RManager.INVALIDATE_SCRIPT_FILENAME);
				
				if (scriptsFile.exists()) {
		        		
			        BufferedReader scriptsReader = new BufferedReader(new FileReader(scriptsFile));
			        		        
			        String script;
			        
			        while ((script = scriptsReader.readLine()) != null) {
			        	
			        	if (!script.trim().equals("")) {
			        		rScriptsRunner.enqueueScriptExecution(aid, eid, bid, script);
			        	}			        	
			        }
			        
			        scriptsReader.close();
				}
			        
		    } catch (IOException e) {
		    	Logger.logError("I/O error accessing invalidation script files.");
		    }
			
			return result;
		}
	}

	/**
	 * Returns the run load graph count of a given graph type
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id 
	 * @param bid	Binary Id 
	 * @param rid 	Run Id
	 * 
	 * @return	The count of graphs
	 */
	public int getRunLoadGraphCount(AID aid, EID eid, BID bid, RID rid) {
		
		ensureValidRid(aid, eid, bid, rid);
		return graphManager.getRunLoadGraphCount(aid, eid, bid, rid);
	}

	/**
	 * Returns the run load graph info for a given graph. GraphInfo object returned contains 
	 * width, height and title of the graph
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id 
	 * @param bid	Binary Id 
	 * @param rid 	Run Id
	 * 
	 * @param index	Index of graph - can be one of 0..getExperimentGraphCount()-1
	 * @return	The graph info
	 * 
	 * @throws IOException	In case of error of getting metadata
	 */
	public GraphInfo getRunLoadGraphInfo(AID aid, EID eid,
			BID bid, RID rid, int index) throws IOException {
		
		ensureValidRid(aid, eid, bid, rid);
		
		if (!graphManager.validRunLoadGraph(aid, eid, bid, rid, index)) {
			throw new IllegalArgumentException(GraphManager.LOAD_GRAPH_DOESNT_EXIST_MESSAGE);
		}
		return graphManager.getRunLoadGraphInfo(aid, eid, bid, rid, index);
	}

    /**
     * Returns the complete path for a given run load graph
     * 
     * @param aid
     * 		Analysis Id
     * @param eid
     * 		Experiment Id 
     * @param bid 
     * 		Binary Id
     * @param rid 
     * 		Run Id
     * @param index
     * 		Index of graph - can be one of 0..getExperimentGraphCount()-1
     * 
     * @return
     * 		The path to the file with graph image on disk
     * 
     * @throws IOException
     * 		if this info couldn't be retrieved because of I/O problems
     * @throws IllegalArgumentException 
     * 		if graph doesn't exist
     */

	public String getRunLoadGraphPath(AID aid, EID eid, BID bid, RID rid,
			int index) throws IOException {
		if (!graphManager.validRunLoadGraph(aid, eid, bid, rid, index)) {
			throw new IllegalArgumentException(GraphManager.LOAD_GRAPH_DOESNT_EXIST_MESSAGE);
		}
		return graphManager.getRunLoadGraphPath(aid, eid, bid, rid, index);
	}

	
	/**
	 * Returns the context of a given binary
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid   Run Id
	 * @return	The context Id 
	 */
	public String getContext(AID aid, EID eid, BID bid, RID rid) {
		// Note: bid is not used, as contexts are mapped to experiments
		return binaryEntityManager.getContext(aid, eid, bid);
	}


	/**
	 * Uploads binary file containing Run Results (through TCP)
	 * 
	 * @see cz.cuni.mff.been.resultsrepository.Database#uploadFile
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @param rid
	 *            Run ID
	 * @param roleName
	 * 			  String identifying the results role, from which results arrive
	 * @param host
	 *            Hostname to connect to for data retrieving
	 * @param port
	 *            Port used in connection with host
	 * @param validRun
	 *            Determine, whether run is valid or crashed
	 * @param blocking
	 *            Determine, whether to use blocking transfer
	 * @param tableIndex 
	 * 			  Index of the table we're uploading
	 * @param totalTables
	 * 			  Total number of tables to be uploaded
	 *  
	 * @return Handle of network connection
	 * 
	 * @throws UnknownHostException
	 *             In the case host could not be found
	 * @throws IOException
	 *             In the case of some IO error
	 * @throws ResultsRepositoryBusyException
	 *             In the case of non-blocking upload when Results Repository is
	 *             busy with (down/)uploading files
	 */
	public UploadHandle uploadRunResults(AID aid, EID eid, BID bid, RID rid,
			String roleName, String host, int port, boolean validRun, boolean blocking,
			int tableIndex, int totalTables)
			throws UnknownHostException, IOException,
			ResultsRepositoryBusyException {

		binaryEntityManager.ensureValidBid(aid, eid, bid);

		Logger.logInfo(
				"Results upload intialized for "
						+ Database.getRunReferenceString(aid, eid, bid, rid) + " - table " 
						+ tableIndex + "/" + totalTables + " from " + host + ".");

		if (database.isBusy() && !blocking) {
			Logger.logWarning(
					"Refusing upload - too many connections ("
							+ Database.getRunReferenceString(aid, eid, bid, rid) + ").");
			throw new ResultsRepositoryBusyException(
					Database.TOO_MANY_CONNECTIONS_MESSAGE);
		}

		String[] taskNames = null; 
		
		try {
			BenchmarkManagerGUIInterface bmInstance = database.getBenchmarkManagerInstance();

			if (bmInstance == null) {
		    	Logger.logError("Error contacting Benchmark Manager.");
		    } else {

		    	TaskDescriptor[] descs = bmInstance.getAnalysisInterface()
		    		.getSubentityInterface(aid).getSubentityInterface(eid).getSubentityInterface(bid).getTasks(rid);
		    	taskNames = new String[descs.length];
		    	for (int i = 0; i < descs.length; i++) {
		    		taskNames[i] = descs[i].getTaskTid();
				}
		    	if (descs.length == 0) {
		    		Logger.logWarning("Zero tasks registered for upload of the log" 
		    				+ Database.getRunReferenceString(aid, eid, bid, rid) + ".");
		    	}
		    	for (String taskName : taskNames) {
					Logger.logTrace("Task to be registered under given run : " + taskName);
				}
		                
		    	synchronized (LockManager.taskListLock) {
		    		
		    		taskListManager.loadRunsTaskList(aid, eid, bid);
		    		taskListManager.addNewRunsTasks(rid, taskNames);
		    		taskListManager.storeRunsTaskList(aid, eid, bid);
		    		
		    	}
		    }
		} catch (RemoteException e) {
			Logger.logError("Error contacting Benchmark Manager.");
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			Logger.logError("Entities integrity error while creating binary.");
			e.printStackTrace();
		}				
		
		String name;
		
		synchronized (LockManager.validityMapLock) {
			
			name = ridManager.generateNewFilename(aid, eid, bid, rid, tableIndex, totalTables,
					host, roleName, validRun);
			
		}
		
		lockManager.getLock(aid, eid, bid, "uploading-" + rid + "-" + host);

		return database.uploadFile(database.getFileInDatabase(aid, eid, bid, name), host, port,
				blocking, false, new RunUploadCallback(database, 
						aid, eid, bid, rid, validRun, totalTables), null);
	}

	/**
	 * Closes the upload handle after uploading the run. Client should call 
	 * this method after the upload status returned by <code>getFileUploadStatus</code> 
	 * has value <code>ACCEPTED</code>, or <code>ERROR</code>.
	 * 
	 * This method is crucial, so that synchronization and upload callbacks work correctly 
	 * 
	 * If the client doesn't close the upload handle, Results Repository will leak
	 * a little amount of memory. However, no other resources than memory
	 * (such as open files, locks...) are associated with the upload handle.
	 *  
	 * @param aid	Analysis Id 
	 * @param eid 	Experiment Id
	 * @param bid 	Binary Id
	 * @param rid 	Run Id
	 * @param host 	Host, from which results were uploaded
	 * 
	 * @param handle upload handle
	 */
	public void endRunUpload(AID aid, EID eid, BID bid, RID rid,
			String host, UploadHandle handle) {

		lockManager.clearLock(aid, eid, bid, "uploading-" + rid + "-" + host);
		database.endFileUpload(handle);
		
	}

}
