/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.net.InetAddress;
import java.rmi.RemoteException;
import java.util.List;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.resultsrepository.gui.DataFormat;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.EntityInterface;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.GraphType;

public class RunImplementation extends EntityImplementation<RID>
	implements Serializable, EntityInterface<RID>  {

	private AID aid;
	private EID eid;
	private BID bid;
	
	/** No graphs suppported for binaries */
	private static final String NO_RUN_GRAPHS = "No results graphs for runs.";
	
	/** Results repository entity manager for analyses */
	private BinaryEntityManager binaryEntityManager;
	
	/** Results repository entity manager for analyses */
	private RunEntityManager runEntityManager;

		
	/** Construction of basic run implementation 
	 * @param aid	AnalysisID
	 * @param eid 	ExperimentID
	 * @param bid 	BinaryID
	 * @throws RemoteException */
	public RunImplementation(AID aid, EID eid, BID bid) throws RemoteException {
		
		super();

		binaryEntityManager = BinaryEntityManager.getInstance();
		BinaryEntityManager.getInstance().ensureValidBid(aid, eid, bid); 
		
		this.aid = aid;
		this.eid = eid;
		this.bid = bid;

		runEntityManager = database.getRunEntityManager();
	}

	public Entity<RID> getEntity(RID id) throws RemoteException {
		return runEntityManager.getRun(aid, eid, bid, id); 
	}

	public void setEntityComment(RID id, String comment) throws RemoteException {
		runEntityManager.setRunComment(aid, eid, bid, id, comment);
		
	}

	public List<Entity<RID>> getEntities() throws RemoteException {
		return runEntityManager.getRunsInBinary(aid, eid, bid);
	}

	public EntityInterface getSubentityInterface(RID id) throws RemoteException {		
		throw new UnsupportedOperationException("No runs subentity defined.");
	}

	public DataFormat[] getExportFormats() throws RemoteException {
		return exporter.getRunExportFormats();
	}

	public DownloadHandle beginExport(RID[] ids, String formatId, InetAddress ip, int port) 
		throws RemoteException, ResultsRepositoryBusyException {
		return exporter.beginRunsExport(aid, eid, bid, ids, formatId, ip, port);
	}

	public DataFormat[] getImportFormats() throws RemoteException {
		return importer.getRunImportFormats();
	}

	public UploadHandle beginImport(String formatId, InetAddress ip, int port) 
		throws RemoteException, ResultsRepositoryBusyException {		
		return importer.beginRunsImport(formatId, ip, port);
	}

	public void deleteEntity(RID id) throws DeleteLockedException, RemoteException {
		runEntityManager.eraseRun(aid, eid, bid, id);
	}

	public int graphCount(GraphType graphType, RID id) throws RemoteException {
		if (graphType == GraphType.RESULTS) {
			throw new UnsupportedOperationException(NO_RUN_GRAPHS);
		} else {
	  		return runEntityManager.getRunLoadGraphCount(aid, eid, bid, id);
		}
	}

	public GraphInfo getGraphInfo(GraphType graphType, RID id, int index) 
		throws RemoteException, ResultsRepositoryException {
		if (graphType == GraphType.RESULTS) {
			throw new UnsupportedOperationException(NO_RUN_GRAPHS);
		} else { 
	  		try {
				return runEntityManager.getRunLoadGraphInfo(aid, eid, bid, id, index);
			} catch (IOException e) {
				Logger.logError("I/O error reading graph info for " 
						+ Database.getRunReferenceString(aid, eid, bid, id) + ".");
				return null;
			}
		}

	}

	public DownloadHandle beginGraphDownload(GraphType graphType, RID id, InetAddress ip, int port,
			int index) throws RemoteException, ResultsRepositoryBusyException, ResultsRepositoryException {
		if (graphType == GraphType.RESULTS) {
			throw new UnsupportedOperationException(NO_RUN_GRAPHS);
		}
		
		if (graphCount(graphType, id) == 0) {
			throw new IllegalArgumentException("Load graph doesn't exist.");
		}
			
		try {
			runEntityManager.ensureValidRid(aid, eid, bid, id); // Do it here, because database doesn't know
			// what run are we handling
			return binaryEntityManager.downloadBinaryFile(aid, eid, bid, 
					"load" + File.separator + id + File.separator 
					+ runEntityManager.getRunLoadGraphPath(aid, eid, bid, id, index), 
					ip.getCanonicalHostName(), port, false);
		} catch (IOException e) {
			Logger.logError("I/O error reading load graph for " 
					+ Database.getRunReferenceString(aid, eid, bid, id) + ".");
			throw new ResultsRepositoryException(e);
		}

	}

	public DownloadHandle beginRawDataDownload(RID id, InetAddress ip, int port) 
		throws ResultsRepositoryBusyException, RemoteException, ResultsRepositoryException {
		try {
			runEntityManager.ensureValidRid(aid, eid, bid, id);
			// FIXME: pack all table files for run for download 
			return binaryEntityManager.downloadBinaryFile(aid, eid, bid,  
					runEntityManager.getRunPaths(aid, eid, bid, id)[0], 
					ip.getCanonicalHostName(), port, false);
		} catch (IOException e) {
			Logger.logError("I/O error reading load graph for " 
					+ Database.getRunReferenceString(aid, eid, bid, id) + ".");
			throw new ResultsRepositoryException(e);
		}		
	}

	public void endRawDataDownload(DownloadHandle handle) throws RemoteException {
		database.endFileDownload(handle);
	}

	public DownloadStatus getRawDataDownloadStatus(DownloadHandle handle) throws RemoteException {
		return database.getFileDownloadStatus(handle);
	}
	
	public String getContext(RID id) throws RemoteException {
		return runEntityManager.getContext(aid, eid, bid, id);
	}

	public String[] getTasks(RID id) throws RemoteException {
		TaskListManager taskListManagerInstance = TaskListManager.getInstance();
		
		taskListManagerInstance.loadRunsTaskList(aid, eid, bid);
		return taskListManagerInstance.getRunTaskList(id);
		
	}

	public LogRecord[] getLogsForTask(RID id, String taskID) throws RemoteException,
			LogStorageException, IllegalArgumentException, NullPointerException {
		
	    String contextId = runEntityManager.getContext(aid, eid, bid, id);
	    
	    if (!database.getLogStorage().isTaskRegistered(contextId, taskID)) {
	    	return new LogRecord[] {};
	    }
	    
	    return database.getLogStorage().getLogsForTask(contextId, taskID); 
	}
	
}
