/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;

/**
 * Class managing translations of BEEN RID to Results repository internal RRID
 * 
 * @author Jakub Lehotsky
 */
public final class RunValidityMapsManager {

	/** Validity map filename */
	public static final String BINARY_VALIDITYMAP_FILENAME = "validity.map";

	/** Metadata file extension */
	private static final String META_TYPE = ".meta";

	/** Statistics file extension */
	private static final String STATISTIC_TYPE = ".stat";

	/** Reference to Results Repository database */
	private static Database database;
	
	/** Reference to Results Repository roles manager */
	private static RolesManager rolesManager;

	/** The single existing intance of RID Manager */
	private static RunValidityMapsManager rvmInstance = null;

	/** AID of table in memory */
	private static AID tableAID = null;

	/** EID of table in memory */
	private static EID tableEID = null;

	/** BID of table in memory */
	private static BID tableBID = null;

	/** Validity table data */
	private static RRIDTable table = null;

	/**
	 * Constructor - private, void for singleton use
	 * 
	 */
	private RunValidityMapsManager() {
	}

	/**
	 * Return single instance of RID manager
	 * 
	 * @return reference single running instance of graph manager
	 */
	public static RunValidityMapsManager getInstance() {
		if (rvmInstance == null) {
			rvmInstance = new RunValidityMapsManager();
		}
		return rvmInstance;
	}

	/**
	 * Initialize RID Manager before its first use
	 * 
	 * @param database
	 *            Instance of database we use
	 * @param rolesManager 
	 * 			  Instance of roles manager
	 */
	public static void initialize(Database database, RolesManager rolesManager) {
		RunValidityMapsManager.database = database;
		RunValidityMapsManager.rolesManager = rolesManager;
	}

	/**
	 * Private method, used to load validity map for binary aid/eid/bid. If it
	 * was called with previous parameters, it doesn't reload it. (warning!
	 * unless you don't call storeRRIDTable(), changes AREN'T written back to
	 * table on the disk)
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 */
	private void openRRIDTable(AID aid, EID eid, BID bid) {
		
		/* Test, if we haven't validity map in the memory */
		// note: comment this if out, if you want to disable caching of validity
		// maps
		
		// NOTE: synchronization for reading
		
		/*if (tableAID != null && tableEID != null && tableBID != null 
				&& aid.equals(tableAID) && (eid.equals(tableEID)) && (bid.equals(tableBID))) {
			return;
		}*/
		
		/* Load it */
		try {
			table = new RRIDTable(database.getFileInDatabase(aid, eid, bid, 
					BINARY_VALIDITYMAP_FILENAME));
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(RRIDTable.IO_ERROR_VALIDITY_READING_MESSAGE 
					+ Database.getBinaryReferenceString(aid, eid, bid));
			
			table = new RRIDTable();
			tableAID = new AID(-1);
			tableEID = new EID(-1);
			tableBID = new BID(-1);
			
			return;
		}
		
		/* Remember actual validity map position, we are working with */
		tableAID = aid;
		tableEID = eid;
		tableBID = bid;		
	
	}

	/**
	 * Private method, used to store whole validity table currently cached back
	 * to hard disk.
	 */
	private void storeRRIDTable() 
		throws IOException {
			table.store(database.getFileInDatabase(tableAID, tableEID, tableBID, 
					BINARY_VALIDITYMAP_FILENAME));
	}

	/**
	 * Call this method to generate new filename for run to be stored (relative
	 * to binary core directory), it updates index of valid runs in given
	 * analysis
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 * @param rid
	 *            RunID
	 * @param tableIndex 
	 * 			  Index of the table
	 * @param totalTables 
	 * 			  Total count of tables
	 * @param hostName
	 * 			  The name of hosts from which results arrive
	 * @param roleName 
	 * 			  The name of the role
	 * @param runValid
	 * 
	 * @return null in case of error
	 * @throws IOException 
	 */
	public String generateNewFilename(AID aid, EID eid, BID bid, RID rid,
			int tableIndex,	int totalTables, String hostName, String roleName, boolean runValid) 
		throws IOException {

		MetadataRef binaryMetadata = new MetadataRef(new Metadata());
		Integer lastValidRunIndex;
		Integer lastCrashedRunIndex;
		String returnValue;
		
		rolesManager.loadRoleList(aid, eid);
		long roleHostKey = rolesManager.getKeyOfHostname(hostName);
		
		if (roleHostKey == -1) {
			Logger.logError("Host " + hostName + " not registered for results upload for a given experiment!");
			
			return null;
		}
		
		Logger.logTrace("The key of item read is : " + roleHostKey);
		
		int itemIndex = rolesManager.getItemIndex(roleHostKey);
		
		Logger.logTrace("The index of item read is : " + roleHostKey);
		RoleItem roleItem = rolesManager.getRoleItem(itemIndex);
		
		Logger.logTrace("The role name to be checked is " + roleItem.getRoleName());
		Logger.logTrace("The host name to be checked is " + roleItem.getHostName());
		
		if (!roleItem.getRoleName().equals(roleName) ||
				!roleItem.getHostName().equals(hostName)) {
			Logger.logError("Role for " + hostName + " should be \"" + roleItem.getRoleName() +
					"\", but is \"" + roleName + "\".");
			return null;
		}

		/* Read metadata from binary */
		if (!BinaryEntityManager.ensureSuccessLoadBinaryMetadata(binaryMetadata, 
				aid, eid, bid)) {
			throw new IOException("Metadata reading error - see logs for details.");
		}		

		/* Now read info about indexes */
		String metaItem = binaryMetadata.metadata.getItemByKey("last valid run idx");
		if (metaItem == null) {
			Logger.logError(
					"Error reading binary metadata item"
							+ "'last valid run idx' - null ("
							+ Database.getBinaryReferenceString(aid, eid, bid) + ").");
			lastValidRunIndex = new Integer(0);
		} else {
			lastValidRunIndex = new Integer(metaItem);
		}

		metaItem = binaryMetadata.metadata.getItemByKey("last crashed run idx");
		if (metaItem == null) {
			Logger.logError(
					"Error reading binary metadata item"
							+ "'last crashed run idx' - null ("
							+ Database.getBinaryReferenceString(aid, eid, bid) + ").");
			lastCrashedRunIndex = new Integer(0);
		} else {
			lastCrashedRunIndex = new Integer(metaItem);
		}

		if (runValid) {
			
			openRRIDTable(aid, eid, bid);
			
			RRIDItem item;
			
			item = table.getItemByKey(rid);
						
			if (item != null) {
				
				returnValue = item.getIndexOfTable() + "-" + tableIndex + "-" + roleHostKey;
				Logger.logDebug("Adding new file to already existing run: " + returnValue);
				
			} else {
				
				returnValue = lastValidRunIndex.toString() + "-" + tableIndex + "-" + roleHostKey;
				
				Logger.logDebug("VALIDITY MAP UPDATE - Increasing index of valid runs");
				
				lastValidRunIndex++;
				lastCrashedRunIndex = 0;
				
			}
			
			if (table.isValueInTable(returnValue)) {
				throw new IOException("Item already added into validity map, RID(" + rid 
						+ "), table(" + tableIndex + "), role(" + roleHostKey + ", value " 
						+ returnValue + " for " + Database.getBinaryReferenceString(aid, eid, bid) 
						+ ".");
			}

			table.addItem(new RRIDItem(rid, runValid, returnValue, 
					Long.valueOf(roleHostKey)));
			try {
				storeRRIDTable();
			} catch (IOException e) {
				e.printStackTrace();
				Logger.logError(
						RRIDTable.IO_ERROR_VALIDITY_STORING_MESSAGE
								+ "of " + Database.getBinaryReferenceString(aid, eid, bid) + ".");
				throw e;
			}

			binaryMetadata.metadata.setItemByKey("last valid run idx", lastValidRunIndex
					.toString());
			binaryMetadata.metadata.setItemByKey("last crashed run idx",
					lastCrashedRunIndex.toString());
			
			if (!BinaryEntityManager.ensureSuccessStoreBinaryMetadata(binaryMetadata.metadata, 
					aid, eid, bid)) {
				throw new IOException("Metadata storing error - see logs for details.");
			}

			return returnValue; 

		} else {

			openRRIDTable(aid, eid, bid);
			
			RRIDItem item;
			
			item = table.getItemByKey(rid);
			
			if (item != null) {
				
				returnValue = item.getIndexOfTable() + "-" + tableIndex + "-" + roleHostKey;
				Logger.logDebug("Adding new file to already existing invalid run: " + returnValue);
				
			} else {
						
				returnValue = lastValidRunIndex.toString() + "."
				+ lastCrashedRunIndex.toString() + "-" + tableIndex + "-" + roleHostKey;
				
				Logger.logDebug("VALIDITY MAP UPDATE - Increasing index of crashed runs");
				
				lastValidRunIndex++;
				lastCrashedRunIndex = 0;
				
			}
			
			if (table.getItemByKey(rid) != null || table.isValueInTable(returnValue)) {
				throw new IOException("Item already added into validity map, RID " + rid 
						+ ", value " + returnValue + " for " 
						+ Database.getBinaryReferenceString(aid, eid, bid) + "."); 
			}
			table.addItem(new RRIDItem(rid, runValid, returnValue,
					Long.valueOf(roleHostKey)));
			try {
				storeRRIDTable();
			} catch (IOException e) {
				e.printStackTrace();
				Logger.logError(
						RRIDTable.IO_ERROR_VALIDITY_STORING_MESSAGE
								+ "of " + Database.getBinaryReferenceString(aid, eid, bid) + ".");
				throw e;
			}

			binaryMetadata.metadata.setItemByKey("last valid run idx", lastValidRunIndex
					.toString());
			binaryMetadata.metadata.setItemByKey("last crashed run idx",
					lastCrashedRunIndex.toString());
			
			if (!BinaryEntityManager.ensureSuccessStoreBinaryMetadata(binaryMetadata.metadata, 
					aid, eid, bid)) {
				throw new IOException("Metadata storing error - see logs for details.");
			}			

			return returnValue; 

		}
	}

	/**
	 * Returns valid run count of given analysis (not read from metadata, but
	 * from table)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @return Actual run count
	 */
	public int getRunCount(AID aid, EID eid, BID bid) {
		openRRIDTable(aid, eid, bid);
		return table.getItemCount();
	}

	/**
	 * Returns true if run is in table and is not crashed, don't exchange with
	 * Database.s isValidRunIndex! It is something different
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 * @param rid
	 *            RunID
	 * @return True iff run correctly exists
	 */
	public boolean isValidRun(AID aid, EID eid, BID bid, RID rid) {
		openRRIDTable(aid, eid, bid);
		return table.getValidityByKey(rid);
	}

	/**
	 * Returns position of actual physical file with run data, given aid, eid,
	 * bid and rid, null in the case it doesn't exist. Path is relative to
	 * binary directory.
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 * @param rid
	 *            RunID
	 * @return Position of run data in binary directory
	 */
	public String[] getRunPaths(AID aid, EID eid, BID bid, RID rid) {

		File runDir = new File(database.getDirectoryInDatabase(aid, eid, bid));
		if (!runDir.exists() || !runDir.isDirectory()) {
			return null;
		}

		openRRIDTable(aid, eid, bid);		

		if ((table == null) || (table.getItemByKey(rid) == null)) {
		    return null;		    
		}
				
		return new String[] { table.getItemByKey(rid).getTableName() };

	}

	/**
	 * Returns position of actual physical file with run metadata, given aid,
	 * eid, bid and rid, null in the case it doesn't exist. Path is relative to
	 * binary directory.
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 * @param rid
	 *            RunID
	 * @return Position of run data in binary directory
	 */
	public String getRunMetadataPath(AID aid, EID eid, BID bid, RID rid) {

		String name;

		File runDir = new File(database.getDirectoryInDatabase(aid, eid, bid));
		if (!runDir.exists() || !runDir.isDirectory()) {
			return null;
		}

		openRRIDTable(aid, eid, bid);
		name = rid.toString();

		return name + META_TYPE;
	}

	/**
	 * Returns position of directory with run load graph table, if
	 * this does not exist, create one, write it to file and call R functions to
	 * generate all the graphs (if possible, i.e. if data are present)
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 * @param rid
	 *            RunID
	 *            
	 * @return Position of run data in binary directory
	 * 
	 * @throws IOException In case of invalid structure
	 */
	public String getRunLoadGraphsPath(AID aid, EID eid, BID bid, RID rid) 
		throws IOException {

		String loadDir = database.getLoadDirectory(aid, eid, bid) + File.separator + rid;

		File loadDirFile = new File(loadDir);

		if (!loadDirFile.exists() || !loadDirFile.isDirectory()) {
			return null;
		}

		String loadGraphTable = loadDir + File.separator + "load.graphs";
		File loadGraphFile = new File(loadGraphTable);

		if (loadGraphFile.exists()) {
			Logger.logDebug("Graph loads already generated...");
			return loadDir;			
		} else {
			Logger.logDebug("Generating graphs for load...");
			if (RManager.getInstance().rScriptingEnabled()) {
				RManager.executeRCommand("load.generateGraphs("
					+ aid + "," + eid + "," + bid + "," + rid + ");");
			}
			return loadDir;
		}

	}

	/**
	 * Returns position of actual physical file with run statistics, given aid,
	 * eid, bid and rid, null in the case it doesn't exist. Path is relative to
	 * binary directory.
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 * @param rid
	 *            RunID
	 * @return Position of run data in binary directory
	 */
	public String getRunStatisticsPath(AID aid, EID eid, BID bid, RID rid) {

		String name;

		File runDir = new File(database.getDirectoryInDatabase(aid, eid, bid));
		if (!runDir.exists() || !runDir.isDirectory()) {
			return null;
		}

		openRRIDTable(aid, eid, bid);
		name = rid.toString();

		return name + STATISTIC_TYPE;
	}

	/**
	 * Returns the whole table - for optimization and internal database use
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 *            ExperimentID
	 * @param bid
	 *            BinaryID
	 * @return The table containing whole validity map
	 */
	public RRIDTable getRunTable(AID aid, EID eid, BID bid) {
		openRRIDTable(aid, eid, bid);
		return table;
	}

	/**
	 * Returns list of files with runs data
	 * 
	 * @param aid
	 *            AnalysisID
	 * @param eid
	 * @param bid
	 * @return List&lt;File&gt; containg list of files representing results of
	 *         run
	 */
	public ArrayList<File> readFileList(AID aid, EID eid, BID bid) {

		ArrayList<File> returnValue;
		returnValue = new ArrayList<File>();

		openRRIDTable(aid, eid, bid);
		String pathToBin = database.getDirectoryInDatabase(aid, eid, bid);

		
		for (RRIDItem rridItem : table.getRridTable()) {
			returnValue.add(new File(pathToBin + File.separator + rridItem.getTableName()));
		}


		return returnValue;
	}

	/**
	 * Deletes item from the validity map
	 * 
	 * @param aid
	 *            AnalysisId
	 * @param eid
	 *            ExperimentId
	 * @param bid
	 *            BinaryId
	 * @param rid
	 *            RunId
	 */
	public void deleteItem(AID aid, EID eid, BID bid, RID rid) {		
		openRRIDTable(aid, eid, bid);
		table.deleteItemByKey(rid);
		try {
			storeRRIDTable();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(
					RRIDTable.IO_ERROR_VALIDITY_STORING_MESSAGE
							+ "runs of " + Database.getBinaryReferenceString(aid, eid, bid) 
							+ "when deleting run " + rid + ".");			
		}
	}
	
	/**
	 * Checks whether all items for given host are in validity table   
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * @param totalTables	Total count of tables;
	 * 
	 * @return	True, if all runs are uploaded, false otherwise
	 */
	public synchronized boolean allHostsUploaded(AID aid, EID eid, BID bid, RID rid, int totalTables) {
		
		int tableCount, expectedHostCount;
		
		rolesManager.loadRoleList(aid, eid);
		expectedHostCount = rolesManager.getRoleCount();
		Logger.logTrace("Validity Maps: Expected hosts count is : " + rolesManager.getRoleCount());
		Logger.logTrace("Validity Maps: Reported total table count for this run is : " + totalTables);
		tableCount = 0;
		
		openRRIDTable(aid, eid, bid);
		for (RRIDItem rridItem : table.getRridTable()) {
			if (rridItem.getKey().equals(rid)) {
				tableCount++;
			}
		}
		
		Logger.logTrace("Validity Maps: Table count is now " + tableCount);

		if (tableCount > expectedHostCount * totalTables)
			Logger.logWarning("Inconsistent Results Repository: real tables count (" + tableCount 
					+") is greater than maximal expected host count (" + expectedHostCount + ")");
		
		return (tableCount == expectedHostCount * totalTables);
	}
	
	/**
	 * Returns the real index of an item
	 * 
	 * @param aid	Analysis Id 
	 * @param eid 	Experiment Id
	 * @param bid 	Binary Id
	 * @param rid 	Run Id
	 * 
	 * @return 	The real index (internal)
	 */
	public long getRealRunIndex(AID aid, EID eid, BID bid, RID rid) {
		
		openRRIDTable(aid, eid, bid);
		
		return table.getRealRunIndex(table.getItemIndexByKey(rid));		
	}

	/**
	 * In table, decrease the true index of run file by one, used for run reindexing
	 * after delete
	 * 
	 * @param index	The index
	 */
	public void decreaseIndexByOne(int index) {
		
		table.decreaseIndexByOne(index);
		
		try {
			storeRRIDTable();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(
					RRIDTable.IO_ERROR_VALIDITY_STORING_MESSAGE
							+ "run when reindexing after delete.");			
		}		
	}
}
