/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository.gui;

import java.io.Serializable;

/**
 * Class containing information about entity (analysis, experiment, binary or
 * run). Used to exchange this information with the web interface.
 * 
 * The class is typed by identifier of the entity.
 *
 *
 * @param <ID>	ID Type describing entity (AID, EID, BIR or RID)
 * @author David Majda
 * 
 */
public class Entity<ID extends cz.cuni.mff.been.common.id.ID> 
		implements Serializable, Comparable<Entity<ID>> {
	/** Entity identifier. */
	private ID id;
	/** Entity metadata. */
	private MetadataItem[] metadata;
	/** Flag determining whether delete is allowed */
	private boolean isDeletable;
	
	/** Entity statistics. */
	private Statistics[] statistics;

	/** @return the entity identifier */
	public ID getId() {
		return id;
	}

	/** @return the entity metadata */
	public MetadataItem[] getMetadata() {
		return metadata;
	}

	/** @return the entity statistics */
	public Statistics[] getStatistics() {
		return statistics;
	}
	
	public boolean isDeletable() {
		return isDeletable;
	}
	
	/**
	 * Returns the entity name from the metadata.
	 * 
	 * @return the entity name
	 */
	public String getName() {
		for (MetadataItem item: metadata) {
			if (item.getKey().equals(MetadataItem.NAME_KEY)) {
				return item.getValue();
			}
		}
		assert false: "Entity doesn't have \"" + MetadataItem.NAME_KEY
			+ "\" key in its metadata.";
		return null; // to shut up the compiler
	}

	/**
	 * Returns the entity comment from the metadata.
	 * 
	 * @return the entity comment
	 */
	public String getComment() {
		for (MetadataItem item: metadata) {
			if (item.getKey().equals(MetadataItem.COMMENT_KEY)) {
				return item.getValue();
			}
		}
		assert false: "Entity doesn't have \"" + MetadataItem.COMMENT_KEY
			+ "\" key in its metadata.";
		return null; // to shut up the compiler
	}

	/**
	 * Returns the entity validity flag from the metadata.
	 * 
	 * @return the entity validity flag
	 */
	public boolean isValid() {
		for (MetadataItem item: metadata) {
			if (item.getKey().equals(MetadataItem.VALID_KEY)) {
				if (item.getValue().equals("true")) {
					return true;
				}
				if (item.getValue().equals("false")) {
					return false;
				}
				assert false: "Entity does have invalid \"" + MetadataItem.VALID_KEY
					+ "\" value in its metadata.";
			}
		}
		assert false: "Entity doesn't have \"" + MetadataItem.VALID_KEY
			+ "\" key in its metadata.";
		return false; // to shut up the compiler
	}

	/**
	 * Returns the entity completness flag from the metadata.
	 * 
	 * @return the entity completness flag
	 */
	public boolean isComplete() {
		for (MetadataItem item: metadata) {
			if (item.getKey().equals(MetadataItem.COMPLETE_KEY)) {
				if (item.getValue().equals("true")) {
					return true;
				}
				if (item.getValue().equals("false")) {
					return false;
				}
				assert false: "Entity does have invalid \"" + MetadataItem.COMPLETE_KEY
					+ "\" value in its metadata.";
			}
		}
		assert false: "Entity doesn't have \"" + MetadataItem.COMPLETE_KEY
			+ "\" key in its metadata.";
		return false; // to shut up the compiler
	}

	/**
	 * Allocates a new <code>Entity</code> object.
	 *
	 * @param id entity identifier
	 * @param metadata entity metadata
	 * @param statistics entity statistics
	 * @param isDeletable flag determining whether deletion is allowed
	 */
	public Entity(ID id, MetadataItem[] metadata, Statistics[] statistics, boolean isDeletable) {
		this.id = id;
		this.metadata = metadata;
		this.statistics = statistics;
		this.isDeletable = isDeletable;
	}

	/**
	 * Compares this entity with the another entity for order. Returns a negative 
	 * integer, zero, or a positive integer as this entity is less than, equal
	 * to, or greater than the specified ID.
	 *       
	 * @param anotherEntity the <code>Entity&lt;ID&gt;</code> to be compared
	 * @return a negative integer, zero, or a positive integer as this entity is
	 *          less than, equal to, or greater than the specified object
	 */
	public int compareTo(Entity<ID> anotherEntity) {
		return (int) (
			(Long) this.getId().value() - (Long) anotherEntity.getId().value()
		); 
	}
}
