/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository.gui;

import java.io.Serializable;

/**
 * Class containing information about one table of entity's statistics,
 * computed from the results. Used to exchange this information with the web
 * interface.  
 *
 * @author Jakub Lehotsky
 * @author David Majda
 */
public class Statistics implements Serializable {
	/** Table label. */
	private String label;
	/** Column names. */
	private String[] columnNames;	
	/**
	 * Column highlight flags (if the flags is set in particular column, changes
	 * in that column will be highlighted in the web interface). 
	 */
	private boolean[] columnHighlightFlags;	
	/** Row names. */
	private String[] rowNames;	
	/** Cell values. */
	private Double[][] values;

	/** @return the table label */
	public String getLabel() {
		return label;
	}

	/** @return the column names */
	public String[] getColumnNames() {
		return columnNames;
	}

	/**
	 * @return the column highlight flags (if the flags is set in particular
	 * column, changes in that column will be highlighted in the web interface)
	 */
	public boolean[] getColumnHighlightFlags() {
		return columnHighlightFlags;
	}

	/** @return the row names */
	public String[] getRowNames() {
		return rowNames;
	}
	
	/** @return the cell values */
	public Double[][] getValues() {
		return values;
	}

	/**
	 * Allocates a new <code>Statistics</code> object.
	 * 
	 * @param label table label
	 * @param columnNames column names
	 * @param columnHighlightFlags column highlight flags (if the flags is set in
	 *         particular column, changes in that column will be highlighted in
	 *         the web interface)
	 * @param rowNames row names
	 * @param values cell values
	 * @throws IllegalArgumentException if the dimensions of the
	 *          <code>values</code> array don't match size of the
	 *          <code>columnNames</code> and <code>rowNames</code> arrays 
	 */
	public Statistics(String label, String[] columnNames,
			boolean[] columnHighlightFlags, String[] rowNames, Double[][] values) {
		
		/*Task.getTaskHandle().logDebug("values.length = " + values.length);
		Task.getTaskHandle().logDebug("rowNames.length = " + rowNames.length);*/
			
		if (values.length != rowNames.length) {
			throw new IllegalArgumentException("Dimensions of the values array does "
				+ "not match size of the rowNames array.");
		}		
		
		if (columnNames.length != columnHighlightFlags.length) {
			throw new IllegalArgumentException("Size of the columnNames array does "
				+ "not match size of the columnHighlightFlags array.");
		}
		for (Double[] valueRow: values) {
			if (valueRow.length != columnNames.length) {
				throw new IllegalArgumentException("Dimensions of the values array does "
					+ "not match size of the columnNames array.");
			}
		}
		
		this.label = label;
		this.columnNames = columnNames;
		this.columnHighlightFlags = columnHighlightFlags;
		this.rowNames = rowNames;
		this.values = values;
	}

}
