/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.softwarerepository;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.Version;
import cz.cuni.mff.been.task.Task;

/**
 * Implementation of the main Software Repository interface.
 *  
 * @author David Majda
 */
public class SoftwareRepositoryImplementation extends UnicastRemoteObject implements SoftwareRepositoryInterface  {
	/** Class instance (singleton pattern). */
	private static SoftwareRepositoryImplementation instance;

	/** Name of the directory with real content in packages. */
	public static final String PACKAGE_FILES_DIR = "files";
	/** Name of the file with metadata in packages. */
	public static final String PACKAGE_METADATA_FILE = "metadata.xml";
	/** Name of the file with configuration information in packages. */
	public static final String PACKAGE_CONFIG_FILE = "config.xml";
	/** Suffix of the package files. Case sensitive, with leading ".". */
	private static final String PACKAGE_FILE_SUFFIX = ".bpk";

	/** Prefix for temporary files used by Software Repository. */
	private static final String TEMPFILE_PREFIX = "been-software-repository"; 
	/** Suffix for temporary files used by Software Repository. */
	private static final String TEMPFILE_SUFFIX = null; // null = use default 
	
	/** Name of the file where the autoincremented counter value is stored. */
	private static final String COUNTER_FILE = "counter.dat";
	
	/** Name of the directory with data files. */ 
	private String dataDir;
	/** Name of the directory with temporary files. */ 
	private String tempDir;
	/** Software Repository's task. */
	private Task task;
	
	/** Package metadata. */
	private List<PackageMetadata> packageMetadata = new LinkedList<PackageMetadata>();
	/**
	 * Counter used to generate unique file names for packages in Software
	 * Repository. It is incremented automatically when needed.
	 */
	private long autoIncrementedCounter;
    
	/** Download statuses. */
	private Map<DownloadHandle, DownloadStatus> downloadStatuses
		= Collections.synchronizedMap(new HashMap<DownloadHandle, DownloadStatus>());
	/** Upload statuses. */
	private Map<UploadHandle, UploadStatus> uploadStatuses
		= Collections.synchronizedMap(new HashMap<UploadHandle, UploadStatus>());
	/** Upload error messages. */
	private Map<UploadHandle, String[]> uploadErrorMessages
		= Collections.synchronizedMap(new HashMap<UploadHandle, String[]>());
	
	/**
	 * Allocates a new <code>SoftwareRepositoryImplementation</code> object.
	 * Construcor is private so only instance in <code>instance</code> field can
	 * be constructed (singleton pattern).
	 *
	 * @throws RemoteException if failed to export object
	 */
	private SoftwareRepositoryImplementation() throws RemoteException {
		super();
		this.task = Task.getTaskHandle();
	}
	
	/**
	 * @return the dataDir
	 */
	public String getDataDir() {
		return dataDir;
	}
		
	/**
	 * @return the tempDir
	 */
	public String getTempDir() {
		return tempDir;
	}
	
	/**
	 * Returns the only class instance (singleton pattern).
	 * 
	 * @return class instance
	 * @throws RemoteException if failed to export object
s	 */
	public static SoftwareRepositoryImplementation getInstance() throws RemoteException {
		if (instance == null) {
			 instance = new SoftwareRepositoryImplementation();
		}
		return instance;
	}

	/**
	 * Initializes the service, i.e. reads the metadata form the index file or
	 * tries to extract it form the packages.
	 * 
	 * @param dataDir name of the directory with data files
	 * @param tempDir name of the directory with temporary files
	 * @throws IOException if some I/O error occurs
	 * @throws ClassNotFoundException if error during loading metadata occurs
	 */
	public void initialize(String dataDir, String tempDir) throws IOException, ClassNotFoundException {
		this.dataDir = dataDir;
		this.tempDir = tempDir;
		if (counterFileExists()) {
			logInfo("Counter file found - loading...");
			loadCounter();
			logInfo("Counter loaded OK.");
		}
		logInfo("Extracting metadata from packages...");
		extractMetadataFromPackages();
		logInfo("Metadata extracted and saved OK.");
	}
	
	/**
	 * Logs an information message. If the Software Repository is run as a task
	 * (i.e. <code>task</code> field is not <code>null</code>), it uses task
	 * logging facility to write the message, otherwise it prints it to the error
	 * output.
	 * 
	 * @param message message text
	 */
	private void logInfo(String message) {
		if (task != null) {
			task.logInfo(message);
		} else {
			System.out.println(message);
		}
	}
	
	/**
	 * Logs an error message. If the Software Repository is run as a task
	 * (i.e. <code>task</code> field is not <code>null</code>), it uses task
	 * logging facility to write the message, otherwise it prints it to the error
	 * output.
	 * 
	 * @param message message text
	 */
	private void logError(String message) {
		if (task != null) {
			task.logError(message);
		} else {
			System.err.println(message);
		}
	}

	/**
	 * Reads XML document from file in a ZIP archive.   
	 * 
	 * @param zipFile the archive file
	 * @param xmlFilename name of the file in the archive with the XML document
	 * @return XML document if the ZIP archive is succesfully opened, file
	 *          <code>xmlFilename</code> found and read and it contains valid
	 *          XML document; <code>null</code> otherwise 
	 * @throws IOException if some I/O error occurs
	 * @throws ParserConfigurationException if serious parser configuration error occurs   
	 * @throws FactoryConfigurationError if serious parser factory configuration error occurs
	 */
	private Document readXMLDocumentFromZippedFile(ZipFile zipFile,
			String xmlFilename) throws IOException, ParserConfigurationException {
		
		Document result = null;
		InputStream inputStream = null;
		try {
			inputStream = zipFile.getInputStream(zipFile.getEntry(xmlFilename));
			DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			try {
				result = builder.parse(inputStream);
			} catch (SAXException e) {
				result = null;   
			}
		} finally {
			inputStream.close();
		}

		return result;
	}
	
	/**
	 * Determines, what package type is given XML document describing.
	 * 
	 * @param document document to examine
	 * @return resulting package type
	 */
	private PackageType getDocumentPackageType(Document document) {
		AttributeHelper helper = PackageTypeAttributeHelper.getInstance();
		Element typeElement = getAttributeElementByName(document, "type");
		if (typeElement == null) {
			return null;
		}
		if (helper.validateInXML(typeElement) != null) {
			return null;
		}
		return ((PackageType) helper.readValueFromElement(typeElement));
	}
	
	/**
	 * Retreives an element associated with given metadata attribute name
	 * from given XML document. 
	 * 
	 * @param document document to examine
	 * @param name attribute name 
	 * @return associated element if exactly one element associated with 
	 *          given metadata attribute is found;
	 *          <code>null</code> otherwise 
	 */
	private Element getAttributeElementByName(Document document, String name) {
		NodeList list = document.getElementsByTagName(name);
		if (list.getLength() == 1) {
			return (Element) list.item(0);   
		} else {
			return null;
		}
	}
	
	/**
	 * Determines, whether metadata attribute exists in given XML document
	 * and has valid value. 
	 * 
	 * @param document document to examine
	 * @param info information about the attribute
	 * @return empty <code>String</code> array if metadata attribute exists in
	 *          given XML document and has valid value for given attribute type;
	 *          list of found errors in human-readable form otherwise
	 */
	private String validateAttributeExiststenceAndValue(Document document, AttributeInfo info) {
		Element attributeElement = getAttributeElementByName(document, info.getName());
		if (attributeElement == null) {
			return PACKAGE_METADATA_FILE + ": Missing <" + info.getName()
			  + "> element.";
		}
		String validationResult = info.getHelper().validateInXML(attributeElement);
		return validationResult != null
			? PACKAGE_METADATA_FILE + ": " + validationResult 
			: null;
	}
	
	
	/**
	 * Determines, whether document contains all required metadata attributes
	 * for its package type and those attributes contain valid values.
	 * 
	 * @param document document to examine
	 * @param packageType package type 
	 * @return empty <code>String</code> array if if all metadata attributes
	 *          required for given package type exist in given XML document and
	 *          have valid values for their attribute types;
	 *          list of found errors in human-readable form otherwise
	 */
	private String[] validateRequiredAttributesAndValues(
			Document document, PackageType packageType) {
		/* Go through all metadata attributes and if the attribute is
		 * required for given package type, check its presence and validity
		 * of contained information.
		 */
		List<String> result = new LinkedList<String>();
		for (int i = 0; i < PackageMetadata.ATTRIBUTE_INFO.length; i++) {
			AttributeInfo attributeInfo = PackageMetadata.ATTRIBUTE_INFO[i];
			if (attributeInfo.getRequired().contains(packageType)) {
				String errorMessage = validateAttributeExiststenceAndValue(document, attributeInfo);
				if (errorMessage != null) {
					result.add(errorMessage);
				}
			}
		}
		return result.toArray(new String[result.size()]);
	}
	
	/**
	 * Determines, whether given package contains valid package metadata file and
	 * reports errors in human-readable form.
	 * 
	 * @param zipFile package file
	 * @return empty <code>String</code> array if the package contains valid
	 *          metadata file;
	 *          list of found errors in human-readable form otherwise
	 * @throws IOException if some I/O error occurs
	 * @throws ParserConfigurationException if serious parser configuration error occurs   
	 * @throws FactoryConfigurationError if serious parser factory configuration error occurs
	 */
	private String[] validatePackageMetadataFile(ZipFile zipFile)
			throws IOException, ParserConfigurationException {
		Document document = readXMLDocumentFromZippedFile(zipFile, PACKAGE_METADATA_FILE);
		
		/* Check that document is not null. If it is, it means it isn't valid XML
		 * file, so report error.
		 */
		if (document == null) {
			return new String[] { PACKAGE_METADATA_FILE + ": Not valid XML file." };
		}
		
		Element documentElement = document.getDocumentElement();
		
		/* Check that root element is <package>. */
		if (!documentElement.getNodeName().equals("package")) {
			return new String[] { PACKAGE_METADATA_FILE + ": Root element must be <package>." };
		}
		
		/* Determine package type. */
		PackageType packageType = getDocumentPackageType(document);
		if (packageType == null) {
			return new String[] { PACKAGE_METADATA_FILE + ": Missing <type> element or invalid value of element <type>." };
		}

		/* Check attributes for given package type. */ 
		return validateRequiredAttributesAndValues(document, packageType);
	}
	
	/**
	 * Determines, whether given package contains valid package configuration file
	 * and reports errors in human-readable form.  
	 * 
	 * @param zipFile package file
	 * @return empty <code>String</code> array if the package contains valid
	 *          configuration file;
	 *          list of found errors in human-readable form otherwise
	 * @throws IOException if some I/O error occurs
	 * @throws ParserConfigurationException if serious parser configuration error occurs   
	 */
	private String[] validatePackageConfigFile(ZipFile zipFile)
			throws IOException, ParserConfigurationException {
		Document document = readXMLDocumentFromZippedFile(zipFile, PACKAGE_CONFIG_FILE);

		/* Check that document is not null. If it is, it means it isn't valid XML
		 * file, so report error.
		 */
		if (document == null) {
			return new String[] { PACKAGE_CONFIG_FILE + ": Not valid XML file." };
		}
		
		Element documentElement = document.getDocumentElement();
		
		/* Check that root element is <packageConfiguration>. */
		if (!documentElement.getNodeName().equals("packageConfiguration")) {
			return new String[] { PACKAGE_CONFIG_FILE + ": Root element must be <packageConfiguration>." };
		}

		/* Check presence of the <java> element. */ 
		NodeList javaElements = documentElement.getElementsByTagName("java");
		Element javaElement;
		if (javaElements.getLength() == 1) {
			javaElement = (Element) javaElements.item(0);   
		} else {
			return new String[] { PACKAGE_CONFIG_FILE + ": There must be one <java> element present." }; 
		}	

		/* Check presence of the <java> element's attributes. */
		if (!javaElement.hasAttribute("classPath")) {
			return new String[] { PACKAGE_CONFIG_FILE + ": Missing \"classPath\" attribute of the <java> element." };
		}
		if (!javaElement.hasAttribute("mainClass")) {
			return new String[] { PACKAGE_CONFIG_FILE + ": Missing \"mainClass\" attribute of the <java> element." };
		}
		
		/* We've passed all the tests now. */
		return new String[0];
	}
	
	/**
	 * Determines, whether given file contains valid package and reports errors in
	 * human-readable form.  
	 *
	 * @param filename file to examine
	 * @return empty <code>String</code> array if given file contains valid package;
	 *          list of found errors in human-readable form otherwise
	 * @throws IOException if some I/O error occurs
	 * @throws ParserConfigurationException if serious parser configuration error occurs   
	 * @throws FactoryConfigurationError if serious parser factory configuration error occurs
	 */
	private String[] validatePackage(String filename) throws IOException,
			ParserConfigurationException {		
		/* Open the package file. In fact, it is a ZIP file. */
		ZipFile zipFile; 
		try {
			zipFile = new ZipFile(filename);
		} catch (ZipException e) {
			return new String[] { "Error reading package file." };
		}
        
		try {
			List<String> result = new LinkedList<String>();
			
			/* Check, if directory "files" and file "metadata.xml" are present. */
			ZipEntry filesDirEntry = zipFile.getEntry(PACKAGE_FILES_DIR + "/");
			if (filesDirEntry == null) {
				result.add("Missing \"" + PACKAGE_FILES_DIR + "\" directory.");
			}
			ZipEntry metadataFileEntry = zipFile.getEntry(PACKAGE_METADATA_FILE);
			if (metadataFileEntry == null) {
				result.add("Missing \"" + PACKAGE_METADATA_FILE + "\" file.");
			}
			if (!result.isEmpty()) {
				return result.toArray(new String[result.size()]);
			}
			
			/* Check the validity of "metadata.xml". */
			result = Arrays.asList(validatePackageMetadataFile(zipFile));
			if (!result.isEmpty()) {
				return result.toArray(new String[result.size()]);
			}
			
			/* Now we know he have valid metadata. We read it and determine the
			 * package type. If the package type is "task", we also check if the
			 * "config.xml" exists and is valid.
			 * 
			 * Note that reading the whole metadata at this point is suboptimal,
			 * but this is no big deal, because the file is small. We may optimize
			 * this later, if necessary.
			 */
			PackageMetadata metadata = readPackageMetadata(filename);
			if (metadata.getType().equals(PackageType.TASK)) {
				ZipEntry configFileEntry = zipFile.getEntry(PACKAGE_CONFIG_FILE);
				if (configFileEntry == null) {
					return new String[] { "Missing \"" + PACKAGE_CONFIG_FILE + "\" file." };
				}
				return validatePackageConfigFile(zipFile);
			}
		} finally {
			zipFile.close();
		}
		
		/* We've passed all the tests now. */
		return new String[0];
	}
	
	/**
	 * Reads metadata form the package. Assumes the package format is correct (pre-validated).
	 * 
	 * @param filename package file name
	 * @return package metadata
	 * @throws IOException if some I/O error occurs
	 * @throws ParserConfigurationException if serious parser configuration error occurs   
	 * @throws FactoryConfigurationError if serious parser factory configuration error occurs
	 */
	private PackageMetadata readPackageMetadata(String filename)
			throws IOException, ParserConfigurationException { 
		/* Open the package file. In fact, it is a ZIP file. */
		ZipFile zipFile = new ZipFile(filename);
		try {
			/* Extract only file name (without path). */
			int separatorPos = filename.lastIndexOf(File.separator);
			filename = filename.substring(separatorPos + 1);
			
			/* Read XML document from the "metadata.xml" and extract metadata attributes. */
			Document document = readXMLDocumentFromZippedFile(zipFile, PACKAGE_METADATA_FILE);
			
			Element nameElement = getAttributeElementByName(document, "name"); 
			Element versionElement = getAttributeElementByName(document, "version"); 
			Element hardwarePlatformsElement = getAttributeElementByName(document, "hardwarePlatforms"); 
			Element softwarePlatformsElement = getAttributeElementByName(document, "softwarePlatforms"); 
			Element typeElement = getAttributeElementByName(document, "type"); 
			Element humanNameElement = getAttributeElementByName(document, "humanName"); 
			Element downloadURLElement = getAttributeElementByName(document, "downloadURL"); 
			Element downloadDateElement = getAttributeElementByName(document, "downloadDate"); 
			Element sourcePackageFilenameElement = getAttributeElementByName(document, "sourcePackageFilename");
			Element binaryIdentifierElement = getAttributeElementByName(document, "binaryIdentifier");
			Element buildConfigurationElement = getAttributeElementByName(document, "buildConfiguration");
            
			return new PackageMetadata(
					filename,
					new File(filename).length(),
					(String) StringAttributeHelper.getInstance().readValueFromElement(
							nameElement
					),
					(Version) VersionAttributeHelper.getInstance().readValueFromElement(
							versionElement
					),
					hardwarePlatformsElement != null
					? (ArrayList) ArrayListAttributeHelper.getInstance().readValueFromElement(
							hardwarePlatformsElement
					)
					: new ArrayList(),
					softwarePlatformsElement != null
					? (ArrayList) ArrayListAttributeHelper.getInstance().readValueFromElement(
							softwarePlatformsElement
					)
					: new ArrayList(),
					(PackageType) PackageTypeAttributeHelper.getInstance().readValueFromElement(
							typeElement
					),
					(String) StringAttributeHelper.getInstance().readValueFromElement(
							humanNameElement
					),
					downloadURLElement != null
					? (String) StringAttributeHelper.getInstance().readValueFromElement(
							downloadURLElement
					)
					: null,
					downloadDateElement != null
					? (Date) DateAttributeHelper.getInstance().readValueFromElement(
							downloadDateElement
					)
					: new Date(0),
					sourcePackageFilenameElement != null
					? (String) StringAttributeHelper.getInstance().readValueFromElement(
							sourcePackageFilenameElement
					)
					: null,
					binaryIdentifierElement != null
					? (String) StringAttributeHelper.getInstance().readValueFromElement(
							binaryIdentifierElement
					)
					: null,
					buildConfigurationElement != null
					? (String) StringAttributeHelper.getInstance().readValueFromElement(
							buildConfigurationElement
					)
					: null
			);       
		} finally {
			zipFile.close();
		}
	}
	
	/**
	 * Adds new metadata to the <code>packageMetadata</code> list. 
	 * 
	 * @param metadata metadata to add
	 * @throws IOException if some I/O error occurs
	 */
	private void addMetadata(PackageMetadata metadata) throws IOException {
		synchronized (packageMetadata) {
			packageMetadata.add(metadata);
			saveCounter();
		}
	}
	
	/**
	 * Deletes the metadata for package specified by it's filename. 
	 * 
	 * @param filename package do delete
	 */
	private void deleteMetadataForFilename(String filename) {
		synchronized (packageMetadata) {
			/* Firstly go through the metadata and determine if there is anything
			 * to delete.
			 */
			int index = -1;
			for (int i = 0; i < packageMetadata.size(); i++) {
				if (packageMetadata.get(i).getFilename().equals(filename)) {
					index = i;
					break;
				}
			}
			
			/* If no item to delete was found, blame the caller. */
			if (index == -1) {
				throw new IllegalArgumentException("Package \"" + "\" doesn't exist.");   
			}
			
			/* Do the actual deletion. */
			packageMetadata.remove(index);
		}
	}
	
	/**
	 * Creates unique package file name from the package metadata and increases
	 * internal counter, which guarantees that the name is unique. This increment
	 * operation must be synchronized.
	 * 
	 * @param metadata package metadata
	 * @return name of the package
	 */
	private String createPackageFilename(PackageMetadata metadata) {
		/* Concatenate all hardware platforms into "+"-separated string. */
		String hardwarePlatforms = "";
		ListIterator hardwarePlatformsIterator = metadata.getHardwarePlatforms().listIterator();
		while (hardwarePlatformsIterator.hasNext()) {
			if (hardwarePlatformsIterator.nextIndex() > 0) {
				hardwarePlatforms += "+";
			}
			hardwarePlatforms += hardwarePlatformsIterator.next();
		}
		
		/* Concatenate all software platforms into "+"-separated string. */
		String softwarePlatforms = "";
		ListIterator softwarePlatformsIterator = metadata.getSoftwarePlatforms().listIterator();
		while (softwarePlatformsIterator.hasNext()) {
			if (softwarePlatformsIterator.nextIndex() > 0) {
				softwarePlatforms += "+";
			}
			softwarePlatforms += softwarePlatformsIterator.next();
		}
		
		/* Create the package name. */
		synchronized (this) {
			String result = metadata.getName() + "-"
				+ hardwarePlatforms + "-" 
				+ softwarePlatforms + "-"
				+ metadata.getType().getSuffix() + "."
				+ autoIncrementedCounter + PACKAGE_FILE_SUFFIX;
			autoIncrementedCounter++;
			return result;
		}
	}
	
	/**
	 * Walks through packages in the data directory and extracts metadata from them.
	 * 
	 * @throws IOException if some I/O error occurs
	 */
	private void extractMetadataFromPackages() throws IOException {
		synchronized (packageMetadata) {
			packageMetadata.clear();
			
			/* Java's ugly and verbose way to say "get me files like *.bpk"... */  	
			File[] packages = new File(dataDir).listFiles(new FilenameFilter() {
				public boolean accept(File dir, String name) {
					return (name.endsWith(PACKAGE_FILE_SUFFIX));
				}
			});
		
			for (File packageFile: packages) {
				try {
					if (validatePackage(packageFile.getPath()).length == 0) {
						packageMetadata.add(readPackageMetadata(packageFile.getPath()));
						logInfo(packageFile.getName() + ": metadata indexed.");
					} else {
						logError(packageFile.getName() + ": invalid package.");
					}
				} catch (IOException e) {
					/* Ignore - if the package reading fails, we just don't care about that file. */ 
					logError(packageFile.getName() + ": exception while reading: " + e.toString());
				} catch (ParserConfigurationException e) {
					/* Ignore - if the package reading fails, we just don't care about that file. */ 
					logError(packageFile.getName() + ": exception while reading: " + e.toString());
				} catch (FactoryConfigurationError e) {
					/* Ignore - if the package reading fails, we just don't care about that file. */ 
					logError(packageFile.getName() + ": exception while reading: " + e.toString());
				}
			}
			saveCounter();
		}
	}

	/**
	 * Thread used for handling the package upload. It connects to given host
	 * and port, downloads the package file form there, validates its contents
	 * and adds the package into the Software Repository.
	 * 
	 * @author David Majda
	 */
	private class PackageUploadThread extends Thread {
		/**
		 * How many times we should try to create a temp file. See
		 * <code>createTempFile</code> method for explanation, why we need this
		 * constant.
		 */
		private static final int MAX_CREATE_TEMP_FILE_TRY_COUNT = 1024;
		
		/** Upload handle. */ 
		private UploadHandle handle;
		/** IP address of host to download the package from. */ 
		private InetAddress ip;
		/** Port to download the package from. */ 
		private int port;
		
		/** @return returns the handle */
		public UploadHandle getHandle() {
			return handle;
		}
		
		/** @return returns the IP addresss */
		public InetAddress getIp() {
			return ip;
		}
		
		/** @return returns the port */
		public int getPort() {
			return port;
		}
		
		/**
		 * Allocates a new <code>PackageUploadThread</code> object with specified
		 * parameters.
		 * 
		 * @param handle upload handle
		 * @param ip IP address of host to download the package from
		 * @param port port to download the package from
		 */
		public PackageUploadThread(UploadHandle handle, InetAddress ip, int port) {
			super();
			this.handle = handle;
			this.ip = ip;
			this.port = port;
			setStatus(UploadStatus.INITIALIZING);
			setErrorMessages(new String[] {});
		}
		       
		/**
		 * Sets the upload status to given value.
		 * 
		 * @param status upload status
		 */
		private void setStatus(UploadStatus status) {
			uploadStatuses.put(handle, status);
		}

		/**
		 * Sets the error messages.
		 * 
		 * @param errorMessages error messages
		 */
		private void setErrorMessages(String[] errorMessages) {
			uploadErrorMessages.put(handle, errorMessages);
		}

		/**
		 * Uploads the package from the uploader to the specified file.
		 * 
		 * @param file file to download
		 * @throws IOException if some I/O error occurs
		 */
		private void uploadPackage(File file) throws IOException {
			byte[] buffer = new byte[UPLOAD_BUFFER_SIZE];
			int bytesRead;
			Socket socket = new Socket(ip, port);
			try {
				InputStream inputStream = new BufferedInputStream(
					socket.getInputStream(),
					UPLOAD_BUFFER_SIZE
				);
				OutputStream outputStream = new BufferedOutputStream(
					new FileOutputStream(file.getPath()),
					UPLOAD_BUFFER_SIZE
				);
				try {
					while ((bytesRead = inputStream.read(buffer)) != -1) {
						outputStream.write(buffer, 0, bytesRead);
					}
				} finally {
					inputStream.close();
					outputStream.close();
				}
			} finally {
				socket.close();
			}                 
		}
		
		/**
		 * Deletes specified file if it exists.
		 * 
		 * @param file file to delete
		 */
		private void deleteFileIfExists(File file) {
			if (file.exists()) {
				file.delete();
			}
		}
		
		/**
		 * Creates teporary file in the <code>tmpDir</code> directory.
		 * 
		 * @return the temporary file
		 * 
		 * @throws IOException if the temporary file can not be created
		 */
		private File createTempFile() throws IOException {
			/*
			 * There is a bug in the Sun's JVM which causes the File.createTempFile
			 * call throw IOException with message "Access is denied" occasionally on
			 * Windows. We work around the bug by simply retrying the call multiple
			 * times. After a while we give up, to avoid hanging-up. 
			 * 
			 * See http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6325169 for more
			 * information about the bug.
			 */
			File result = null;
			int tryCount = 0;
			boolean failed;
			IOException thrownException = null;
			
			do {
				failed = false;
				try {
					result = File.createTempFile(TEMPFILE_PREFIX, TEMPFILE_SUFFIX, new File(tempDir));
				} catch (IOException e) {
					failed = true;
					thrownException = e;
				}
				tryCount++;
			} while (failed && tryCount < MAX_CREATE_TEMP_FILE_TRY_COUNT);
			
			if (result != null) {
				return result;
			} else {
				throw thrownException;
			}
		}
		
		/**
		 * Connects to given host and port, downloads the package file form there,
		 * validates its contents and adds the package into the Software Repository.
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			setStatus(UploadStatus.UPLOADING);
			try {
				/* Create the temp file. */
				File tempFile = createTempFile();
				try {
					/* Download the file form specified host and port to the temp file. */
					uploadPackage(tempFile);
					
					/* OK, now let's dig in and see if it is a valid package. */
					String[] errorMessages = validatePackage(tempFile.getPath());
					if (errorMessages.length > 0) {
						throw new PackageUploadException("Tried to upload invalid package "
								+ "(host = " + ip.getCanonicalHostName() + " (" + ip
								+ "), port = " + port + ".", errorMessages);
					}
					
					/* Extract the metadata from the file and create an unique file name.
					 * Corresponding method needs to be synchronized as two uploads can try
					 * to create unique name in the same time.
					 */
					PackageMetadata metadata = readPackageMetadata(tempFile.getPath());
					String packageFilename = createPackageFilename(metadata);
					
					/* Try to move the file. If the move fails, bail everything and delete
					 * the temp file. The file name won't be reused and is lost forever.
					 */
					File packageFile = new File(dataDir + File.separator + packageFilename);
					if (!tempFile.renameTo(packageFile)) {
						throw new IOException("Cant't rename file \""
								+ tempFile.getPath() + "\" to \""
								+ packageFile.getPath() + "\".");
					}
					
					/* If copying succeded, re-read the metadata (this is hack only to change
					 * the filename) and try adding the package metadata and serialize it. 
					 */
					metadata = readPackageMetadata(packageFile.getPath());
					addMetadata(metadata);
					logInfo("Successfully uploaded package \"" + packageFile.getPath() + "\".");
					setStatus(UploadStatus.ACCEPTED);

				/* After all operations finish (either succesfully or due to some error),
				 * delete the temporary file if it still exists. 
				 */
				} finally {
					deleteFileIfExists(tempFile);
				}
			} catch (IOException e) {
				logError(e.getMessage());
				setStatus(UploadStatus.ERROR);
			} catch (PackageUploadException e) {
				String[] errorMessages = e.getErrorMessages();
				logError(e.getMessage() + " " + Integer.toString(errorMessages.length)
					+ " messages reported:");
				for (String errorMessage: errorMessages) {
					logError("  " + errorMessage);
				}
				setErrorMessages(errorMessages);
				setStatus(UploadStatus.REJECTED);
			} catch (ParserConfigurationException e) {
				logError(e.getMessage());
				setStatus(UploadStatus.ERROR);
			} catch (FactoryConfigurationError e) {
				logError(e.getMessage());
				setStatus(UploadStatus.ERROR);
			}
		}
		
	}
	
	/**
	 * Thread used for handling the package download.
	 * 
	 * @author David Majda
	 */
	private class PackageDownloadThread extends Thread {
		/** Download handle. */
		private DownloadHandle handle;
		/** Filename of the package. */ 
		private String filename;
		/** IP of host to upload the package to. */ 
		private InetAddress ip;
		/** Port to upload the package to. */ 
		private int port;
		
		/** @return returns the download handle */
		public DownloadHandle getHandle() {
			return handle;
		}

		/** @return returns the filename */
		public String getFilename() {
			return filename;
		}
		
		/** @return returns the IP address */
		public InetAddress getIp() {
			return ip;
		}
		
		/** @return returns the port */
		public int getPort() {
			return port;
		}
		
		/**
		 * Allocates a new <code>PackageDownloadThread</code> object
		 * with specified parameters.
		 *
		 * @param handle download handle
		 * @param filename filename of the package
		 * @param ip IP address of host to download the package from
		 * @param port port to download the package from
		 */
		public PackageDownloadThread(DownloadHandle handle, String filename, InetAddress ip, int port) {
			super();
			this.handle = handle;
			this.filename = filename;
			this.ip = ip;
			this.port = port;
			setStatus(DownloadStatus.INITIALIZING);
		}
		
		/**
		 * Sets the download status to given value.
		 * 
		 * @param status download status
		 */
		private void setStatus(DownloadStatus status) {
			downloadStatuses.put(handle, status);
		}

		/**
		 * Uploads the package to the downloader from the specified file.
		 * 
		 * @param file file to upload
		 * @throws IOException if some I/O error occurs
		 */
		private void downloadPackage(File file) throws IOException {
			byte[] buffer = new byte[DOWNLOAD_BUFFER_SIZE];
			int bytesRead;
			Socket socket = new Socket(ip, port);
			try {
				OutputStream outputStream = new BufferedOutputStream(
					socket.getOutputStream(),
					DOWNLOAD_BUFFER_SIZE
				);
				InputStream inputStream = new BufferedInputStream(
					new FileInputStream(file.getPath()),
					DOWNLOAD_BUFFER_SIZE
				);
				try {
					while ((bytesRead = inputStream.read(buffer)) != -1) {
						outputStream.write(buffer, 0, bytesRead);     
					}
				} finally {
					outputStream.close();
					inputStream.close();
				}
			} finally {
				socket.close();
			}                 
		}
		
		/**
		 * Connects to given host and port and uploads the package file there.
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			setStatus(DownloadStatus.DOWNLOADING);
			File packageFile = new File(dataDir + File.separator + filename);
			try {
				downloadPackage(packageFile);
				setStatus(DownloadStatus.SUCCEEDED);
			} catch (UnknownHostException e) {
				setStatus(DownloadStatus.ERROR);
				logError(e.getMessage());
			} catch (IOException e) {
				setStatus(DownloadStatus.ERROR);
				logError(e.getMessage());
			}
		}
		
	}
	
	/**
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#beginPackageDownload(java.lang.String, java.net.InetAddress, int)
	 */
	public DownloadHandle beginPackageDownload(String filename, InetAddress ip, int port) throws RemoteException {
		File packageFile = new File(dataDir + File.separator + filename);
		if (!packageFile.exists()) {
			throw new IllegalArgumentException("Package \"" + filename + "\" doesn't exist.");
		}
		
		DownloadHandle handle = DownloadHandle.createDownloadHandle();
		PackageDownloadThread downloader = new PackageDownloadThread(handle, filename, ip, port);
		downloader.start();
		return handle;
	}

	/**
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#getPackageDownloadStatus(cz.cuni.mff.been.common.DownloadHandle)
	 */
	public DownloadStatus getPackageDownloadStatus(DownloadHandle handle)
			throws RemoteException {
		DownloadStatus result = downloadStatuses.get(handle);
		if (result != null) {
			return result;
		} else {
			throw new IllegalArgumentException("Invalid handle.");
		}
	}

	/**
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#endPackageDownload(cz.cuni.mff.been.common.DownloadHandle)
	 */
	public void endPackageDownload(DownloadHandle handle) throws RemoteException {
		downloadStatuses.remove(handle);
	}
	
	/**
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#beginPackageUpload(java.net.InetAddress, int)
	 */
	public UploadHandle beginPackageUpload(InetAddress ip, int port) throws RemoteException {
		UploadHandle handle = UploadHandle.createUploadHandle();
		PackageUploadThread uploader = new PackageUploadThread(handle, ip, port);
		uploader.start();
		return handle;
	}

	/**
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#getPackageUploadStatus(cz.cuni.mff.been.common.UploadHandle)
	 */
	public UploadStatus getPackageUploadStatus(UploadHandle handle) throws RemoteException {
		UploadStatus result = uploadStatuses.get(handle);
		if (result != null) {
			return result;
		} else {
			throw new IllegalArgumentException("Invalid handle.");
		}
	}

	/**
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#getRejectedPackageUploadErrorMessages(cz.cuni.mff.been.common.UploadHandle)
	 */
	public String[] getRejectedPackageUploadErrorMessages(UploadHandle handle) throws RemoteException {
		UploadStatus uploadStatus = uploadStatuses.get(handle);
		if (uploadStatus == null) {
			throw new IllegalArgumentException("Invalid handle.");
		}
		if (uploadStatus != UploadStatus.REJECTED) {
			throw new IllegalStateException("Package not rejected.");
		}
		return uploadErrorMessages.get(handle);
	}

	/**
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#endPackageUpload(cz.cuni.mff.been.common.UploadHandle)
	 */
	public void endPackageUpload(UploadHandle handle) throws RemoteException {
		uploadStatuses.remove(handle);
		uploadErrorMessages.remove(handle);
	}
	
	/**
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#deletePackage(java.lang.String)
	 */
	public boolean deletePackage(String filename) {
		File packageFile = new File(dataDir + File.separator + filename);
		if (!packageFile.exists()) {
			return false;
		}
		try {
			deleteMetadataForFilename(filename);
		} catch (IllegalArgumentException e) {
			return false;
		}
		if (packageFile.delete()) {
			logInfo("Successfully deleted package \"" + packageFile.getPath() + "\".");
			return true;
		} else {
			logError("Error deleting package \"" + packageFile.getPath() + "\".");
			return true;
		}
	}
	
	/** 
	 * @see cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface#queryPackages(cz.cuni.mff.been.softwarerepository.PackageQueryCallbackInterface)
	 */
	public PackageMetadata[] queryPackages(PackageQueryCallbackInterface callback)
			throws RemoteException, MatchException {
		ArrayList<PackageMetadata> resultList = new ArrayList<PackageMetadata>();
		for (int i = 0; i < packageMetadata.size(); i++) {
			PackageMetadata metadata = packageMetadata.get(i);
			if (callback.match(metadata)) {
				resultList.add(metadata);  
			}
		}
		PackageMetadata[] result = new PackageMetadata[resultList.size()];
		return (PackageMetadata[]) resultList.toArray(result);
	}
	
	/**
	 * Determines whether the <code>COUNTER_FILE</code> exists.
	 * 
	 * @return <code>true</code> if the file with autoincremented counter exists;
	 *          <code>false</code> otherwise
	 */
	private boolean counterFileExists() {
		File hashFile = new File(dataDir + File.separator + COUNTER_FILE);
		return hashFile.exists();
	}
	
	/**
	 * Loads autoincremented counter form the <code>COUNTER_FILE</code>.
	 * 
	 * @throws IOException if some I/O error occurs
	 */
	@SuppressWarnings("unchecked")
	private void loadCounter() throws IOException {
		ObjectInputStream inputStream = new ObjectInputStream(
				new FileInputStream(dataDir + File.separator + COUNTER_FILE)
		);
		try {
			autoIncrementedCounter = inputStream.readLong();
		} finally { 
			inputStream.close();
		}
	}
	
	/**
	 * Saved autoincremented counter to the <code>COUNTER_FILE</code>. 
	 *
	 * @throws IOException if some I/O error occurs
	 */
	private void saveCounter() throws IOException {
		ObjectOutputStream outputStream = new ObjectOutputStream(
				new FileOutputStream(dataDir + File.separator + COUNTER_FILE)
		);
		try {
			outputStream.writeLong(autoIncrementedCounter);
		} finally {
			outputStream.close();
		}
	}
}
