/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.build.ant;

import java.io.File;
import java.util.ArrayList;
import java.util.Map;
import java.util.Scanner;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.common.ScannerExtended;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Runs the Apache Ant build tool. It uses Ant installed in 
 * the system, it doesn't distribute it.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #DIRECTORY}: <br>
 * 		- working directory where Ant will be executed <br>
 * 		- mandatory <br>
 * <li>{@value #TARGET}: <br>
 * 		- name of the target in the build file which will be called <br>
 * 		- if ommited, the default target is used <br>
 * <li>{@value #PARAMETERS}: <br>
 * 		- additional parameters that will be passed to Ant <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class AntBuild extends Job {
	//------------------------CONSTANTS--------------------
	/** Task property name for the directory where Ant should be run.*/
	public static final String DIRECTORY = "directory";
	/**
	 * Task property name for which target of the build file should be used.
	 */
	public static final String TARGET = "target";
	/**
	 * Task property name for additional command line argument for Ant.
	 */
	public static final String PARAMETERS = "parameters";
	
	//-----------------------ATTRIBUTES--------------------

	/** Directroy where Ant will be launched */
	private String directory;
	/** Build target that will be used */
	private String target;
	/** Additional command line arguments for Ant */
	private String parameters;
	
	/**
	 * @throws TaskInitializationException if some error occured during task 
	 * initialization. 
	 */
	public AntBuild() throws TaskInitializationException {
		super();
	}
	
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run() throws TaskException {
		directory = getTaskProperty(DIRECTORY);
		if (directory != null) {
			logInfo("Directory: " + directory);
		}
		
		target = getTaskProperty(TARGET);
		if (target != null) {
			logInfo("Target: " + target);
		}
		
		parameters = getTaskProperty(PARAMETERS);
		if (parameters != null) {
			logInfo("Parameters: " + parameters);
		}

		// create the command to launch Ant
		ArrayList<String> params = new ArrayList<String>();
		
		if (isRunningInWindows()) {
			params.add("cmd");
			params.add("/C");
			params.add("call");
		}
		params.add("ant");
		
		// parse the command line arguments for Ant
		if (parameters != null) {
			Scanner scanner = new Scanner(parameters);
			scanner.useDelimiter(" ");
			ScannerExtended scannerEx = new ScannerExtended(scanner);
			while (scannerEx.hasNext()) {
				params.add(scannerEx.next());
			}
		}

		if (target != null) {
			params.add(target);
		}

		String[] cmdArray = params.toArray(new String[params.size()]);
		
		try {
			logInfo("Starting Ant");
			logInfo("Executing: " + ArrayUtils.join(" ", cmdArray));
			
			ProcessBuilder builder = new ProcessBuilder(cmdArray);
			Map<String, String> env = builder.environment();
			if (env.containsKey("CLASSPATH")) {
				logDebug("Removing CLASSPATH");
				env.remove("CLASSPATH");
			}
			builder.directory(new File(directory));
			Process p = builder.start();
			p.getOutputStream().close();

			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(p.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(p.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			// wait untill the script finishes
			if (p.waitFor() != 0) {
				throw new TaskException("Build failed");
			}
		} catch (TaskException e) {
			throw e;
		} catch (Exception e) {
			throw new TaskException("Cannot run Ant", e);
		}
		
		logInfo("Build succesfull");
		checkPointReached("finish", null);
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(DIRECTORY);
	}
}
