/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.download;

import org.netbeans.lib.cvsclient.Client;
import org.netbeans.lib.cvsclient.admin.StandardAdminHandler;
import org.netbeans.lib.cvsclient.command.CommandAbortedException;
import org.netbeans.lib.cvsclient.command.CommandException;
import org.netbeans.lib.cvsclient.command.GlobalOptions;
import org.netbeans.lib.cvsclient.command.checkout.CheckoutCommand;
import org.netbeans.lib.cvsclient.connection.AuthenticationException;
import org.netbeans.lib.cvsclient.connection.ConnectionFactory;
import org.netbeans.lib.cvsclient.connection.PServerConnection;
import org.netbeans.lib.cvsclient.connection.StandardScrambler;

import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * Performs checkout from a CVS repository.
 * 
 * @author Michal Tomcanyi
 */
public class CVSDownload extends Job {
    // ----------------------- CONSTANTS -------------------------------
	/** Task package identifier */
	public static final String TASK_NAME = "cvs-download";
	
	/** Property for remote CVS repository */
	public static final String CVS_REPOSITORY = "repository";
	/** Property for CVS module */
	public static final String CVS_MODULE = "module";
	/** Property for CVS password */
    public static final String CVS_PASSWORD = "password";
    /** Property for CVS revision (tag/revision) */
    public static final String CVS_REVISION = "revision";
    /** Property for CVS date to checkout for */
    public static final String CVS_DATE = "date";
    /** Checkpoint that will be reached upon sucesfull completition of task */
    public static final String CHECKPOINT = "finish";
    
    
    // ------------------------ ATTRIBUTES -------------------------------
	/** CVS client */
    private Client cvsClient = null;
	/** CVS client-wide options */
    private GlobalOptions globalOptions = null;
    /** Single CVS connection to a server (should be re-created after usage */
	private PServerConnection connection = null;
	
	/**
	 * Allocates a new <code>CVSDownload</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public CVSDownload() throws TaskInitializationException {
		super();
	}
	
	public void setup() {
		globalOptions = new GlobalOptions();
		globalOptions.setCVSRoot(getTaskProperty(CVS_REPOSITORY));
		connection = (PServerConnection) ConnectionFactory.getConnection(getTaskProperty(CVS_REPOSITORY));
		
		//user specified an cvs password 
		if (getTaskProperty(CVS_PASSWORD) != null) {
			connection.setEncodedPassword(StandardScrambler.getInstance().scramble(getTaskProperty(CVS_PASSWORD)));
		}
		
		cvsClient = new Client(connection, new StandardAdminHandler());
		cvsClient.setLocalPath(getWorkingDirectory());
		cvsClient.getEventManager().addCVSListener(new LoggingCVSListener(this));
	}
	
	/** 
	 * Downloads module from remote CVS repository by performing the checkout command.
	 * @throws CommandAbortedException		 if an error occurs when executing the command
	 * @throws CommandException			 if the command is aborted
	 * @throws AuthenticationException			thrown when a connection with the server cannot be made, for whatever reason
	 */
	public void download() throws CommandAbortedException, CommandException, AuthenticationException {
		CheckoutCommand checkout = new CheckoutCommand();
		checkout.setPruneDirectories(false);
		checkout.setBuilder(null);
		checkout.setRecursive(true);
		checkout.setModule(getTaskProperty(CVS_MODULE));
		
		// checkout revision if set
		if (getTaskProperty(CVS_REVISION) != null) {
			checkout.setCheckoutByRevision(getTaskProperty(CVS_REVISION));
		}
		
		// checkout to given date if set
		if (getTaskProperty(CVS_DATE) != null) {
			checkout.setCheckoutByDate(getTaskProperty(CVS_DATE));
		}
		
		cvsClient.executeCommand(checkout, globalOptions);		
	}
	
	@Override
	protected void run() throws TaskException {
		// prevent javacvs library warnings about reusing connection (which is doing itself)
		System.setProperty("javacvs.multiple_commands_warning", "false");
		
		setup();
		
		try {
			// download specified module
			download();
			checkPointReached(CHECKPOINT, "true");
		} catch (CommandException e) {
			logFatal("Checkout command aborted: " + e.getMessage());
			throw new TaskException("Checkout command aborted",e);
		} catch (AuthenticationException e) {
			logFatal("Authenticion error: " + e.getMessage());
			throw new TaskException("Authentication error",e);
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{CVS_MODULE,CVS_REPOSITORY});
	}

	public static void main(String[] args) throws Exception {
		CVSDownload download = new CVSDownload();
		download.run();
		
	}
}

