/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.jboss.run;

import java.io.File;
import java.util.Map;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Starts JBoss.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #JBOSS_DIR}: <br>
 * 		- path to the JBoss installation directory <br>
 * 		- mandatory <br>
 * </ul>
 *
 * <b>Checkpoints:</b><br>
 * <ul>
 * <li>{@value #CHECKPOINT_RUNNING}: <br>
 * 		- set when JBoss has succefully started<br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class JBossRun extends Job {
	//------------------------CONSTANTS--------------------
	/** Task property name for JBoss installation directory. */
	public static final String JBOSS_DIR = "jboss.dir";
	/** Name of the checkpoint which is set when JBoss is succesfully started.*/
	public static final String CHECKPOINT_RUNNING = "running";

	/** Time in seconds how long does it wait for JBoss to startup */
	private static final int STARTUP_TIMEOUT = 300;
	/** Time in seconds between the checks of JBoss' startup */
	private static final int TIMEOUT_INCREMENT = 20;
	/** Name of the environment variable for JBoss' root directory */
	private static final String JBOSS_HOME_VARIABLE = "JBOSS_HOME";
	
	//-----------------------ATTRIBUTES--------------------
	/** Path to directory where JBoss is installed */
	private String jbossDir = null;

	/**
	 * 
	 * Allocates a new <code>JBossRun</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public JBossRun() throws TaskInitializationException {
		super();
	}
	
	
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run() throws TaskException {
		logDebug("Timeout for JBoss' startup: " + STARTUP_TIMEOUT + " seconds");
		logDebug("Frequency of testing if JBoss has started: " + TIMEOUT_INCREMENT + " seconds");
		
		jbossDir = getTaskProperty(JBOSS_DIR);

		Process jbossProcess = null;
		
		String fSep = File.separator;
		String[] cmdArray = null;
		if (isRunningInLinux()) {
			String[] command = {
					jbossDir + fSep + "bin" + fSep + "run.sh"
			};
			cmdArray = command;
		}
		if (isRunningInWindows()) {
			String[] command = {
					"cmd",
					"/C",
					"call",
					jbossDir + fSep + "bin" + fSep + "run.bat"
			};
			cmdArray = command;
		}
		
		logInfo("Starting JBoss");
		logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));
		try {
			ProcessBuilder builder = new ProcessBuilder(cmdArray);
			Map<String, String> env = builder.environment();
			if (env.containsKey(JBOSS_HOME_VARIABLE)) {
				logDebug("Environment variable " + JBOSS_HOME_VARIABLE + " was set." +
				" It's removed from the new process.");
				env.remove(JBOSS_HOME_VARIABLE);
			}
			builder.directory(new File(jbossDir));
			jbossProcess = builder.start();
			jbossProcess.getOutputStream().close();
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(jbossProcess.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(jbossProcess.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			
			/*
			 * Check if JBoss started
			 */

			// object with does the actuall test if JBoss has started
			JBossStartupTest test = new JBossStartupTest(jbossDir, 1097);
			boolean jbossStarted = false;
			
			// look if JBoss started for 5 minutes
			// if it's doesn't start up until then, give up
			for (int i = 0; i < STARTUP_TIMEOUT; i = i + TIMEOUT_INCREMENT) {
				logInfo("Checking if JBoss has started");
				// call the test
				if (test.jbossRunning()) {
					jbossStarted = true;
					break;
				}
				logInfo("JBoss not started yet");
				
				// wait 20 seconds
				try {
					Thread.sleep(TIMEOUT_INCREMENT * 1000);
				} catch (InterruptedException e) {
					// nothing bad happened
					Thread.currentThread().interrupt();
				}
			}
			
			if (jbossStarted) {
				logInfo("JBoss started");
				checkPointReached(CHECKPOINT_RUNNING, null);
			} else {
				throw new TaskException("JBoss not started");
			}
			
			// wait untill JBoss finishes
			jbossProcess.waitFor();
		} catch (Exception e) {
			throw new TaskException("Cannot run JBoss", e);
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(JBOSS_DIR);
	}
}
