/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.jboss.run;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Map;
import java.util.logging.Logger;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.task.Task;

/**
 * Class used to test if JBoss has started succesfully.
 * 
 * @author Jaroslav Urban
 */
public class JBossStartupTest {
	/** Name of the environment variable for JBoss' root directory */
	private static final String JBOSS_HOME_VARIABLE = "JBOSS_HOME";

	/** Directory with installed JBoss */
	private String directory;
	/** port where JBoss is listening */
	private int port;
	/** Handle to the current Task */
	private Task taskHandle = Task.getTaskHandle();

	
	/**
	 * Allocates a new <code>JBossStartupTest</code> object.
	 *
	 * @param directory root directory of a JBoss installation.
	 * @param port port of the JBoss' JNP service.
	 */
	public JBossStartupTest(String directory, int port) {
		if (directory == null) {
			taskHandle.logFatal("directory is null");
			throw new NullPointerException("directory is null");
		}
		if (directory.equals("")) {
			taskHandle.logFatal("directory is an empty string");
			throw new IllegalArgumentException("directory is an empty String");
		}
		
		this.directory = directory;
		taskHandle.logDebug("directory: " + directory);
		this.port = port;
		taskHandle.logDebug("port: " + port);
	}
	
	/**
	 * @return true if JBoss is running.
	 */
	public boolean jbossRunning() {
		boolean started = false;

		Process twiddleProcess = null;
		String fSep = File.separator;
		
		String[] cmdArray = null;
		if (System.getProperty("os.name").toLowerCase().indexOf("linux") != -1) { 
			String[] commands = {directory  + fSep + "bin" + fSep + "twiddle.sh",
					"-s",
					"localhost:" + port,
					"get",
					"jboss.system:type=Server",
					"Started"
			};
			cmdArray = commands;
		}
		if (System.getProperty("os.name").toLowerCase().indexOf("windows") != -1) { 
			String[] commands = {
					"cmd",
					"/C",
					"call",
					directory  + fSep + "bin" + fSep + "twiddle.bat",
					"-s",
					"localhost:" + port,
					"get",
					"\"jboss.system:type=Server\"",
					"Started"
			};
			cmdArray = commands;
		}

		try {
			taskHandle.logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));

			ProcessBuilder builder = new ProcessBuilder(cmdArray);
			Map<String, String> env = builder.environment();
			if (env.containsKey(JBOSS_HOME_VARIABLE)) {
				taskHandle.logDebug("Environment variable " + JBOSS_HOME_VARIABLE + " was set." +
				" It's removed from the new process.");
				env.remove(JBOSS_HOME_VARIABLE);
			}
			builder.directory(new File(directory));
			twiddleProcess = builder.start();

			// read the output of the twiddle program, and test if it
			// is equall to "Started=true"
			BufferedReader reader = new BufferedReader(
					new InputStreamReader(twiddleProcess.getInputStream()));
			String line;
			while ((line = reader.readLine()) == null) {
			}
			if (line.equals("Started=true")) {
				started = true;	
			} else {
				// JBoss not started, print the output of twiddle
				// usually it will print some exceptions which are normal
				// to occur while JBoss is starting, but in case of some problem
				// the output can help
				System.err.println(" *** twiddle output (JBoss startup test)");
				System.err.println(" *** some exceptions are usuall while JBoss is starting up");
				do {
					System.err.println(line);
				} while ((line = reader.readLine()) != null);
				System.err.println(" *** twiddle output end");
				reader.close();
			}
			
			try {
				taskHandle.logDebug("waiting for twiddle to finish");
				twiddleProcess.waitFor();
				taskHandle.logDebug("twiddle finished");
			} catch (InterruptedException e) {
				taskHandle.logDebug("output readers interrupted");
				System.err.println(e);	// shouldn't happen
			}
		} catch (IOException e)	{
			taskHandle.logError("JBoss startup test aborted: " + e.getMessage());
			System.err.println("FATAL: JBoss startup test aborted: " + e.getMessage());
			e.printStackTrace();
		} catch (SecurityException e) { 
			// Subprocess creation not allowed
			taskHandle.logError("Cannot create JBoss startup test process: " + e.getMessage());
			System.err.println("FATAL: Cannot create JBoss startup test process: " + e.getMessage());
			e.printStackTrace();
		} catch (Exception e) {
			// Invalid command in exec
			taskHandle.logError("Invalid command in exec");
			e.printStackTrace();
		}

		return started;
	}
}
