/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.jboss.shutdown;

import java.io.File;
import java.util.ArrayList;
import java.util.Map;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Stops JBoss.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #JBOSS_DIR}: <br>
 * 		- path to the JBoss installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #JBOSS_JNP_PORT}: <br>
 * 		- port number of the JBoss' JNP service <br>
 * 		- if ommited, default JNP port will be used <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class JBossShutdown extends Job {
	//------------------------CONSTANTS--------------------
	/** Task property name for JBoss installation directory. */
	public static final String JBOSS_DIR = "jboss.dir";
	/**
	 * Port number of the JBoss' JNP service. It's not mandatory, default
	 * value is 1097.
	 */
	public static final String JBOSS_JNP_PORT = "jboss.jnp.port";
	
	/** Name of the environment variable for JBoss' root directory */
	private static final String JBOSS_HOME_VARIABLE = "JBOSS_HOME";

	
	//-----------------------ATTRIBUTES--------------------
	/** Path to directory where JBoss is installed */
	private String jbossDir = null;
	/** Port number of JBoss' JNP service. */
	private int jnpPort = 1097;

	/**
	 * Allocates a new <code>JBossShutdown</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public JBossShutdown() throws TaskInitializationException {
		super();
	}
	
	/**
	 * Shuts down JBoss.
	 * 
	 * @param url URL where JBoss is listening.
	 * @throws TaskException if an error occured during JBoss' shutdown.
	 */
	public void shutdown(String url) throws TaskException {
		Process shutdownProcess = null;
		
		String fSep = File.separator;
		ArrayList<String> params = new ArrayList<String>();
		if (System.getProperty("os.name").toLowerCase().indexOf("linux") != -1) {
			params.add(jbossDir + fSep + "bin" + fSep + "shutdown.sh");
		}
		if (System.getProperty("os.name").toLowerCase().indexOf("windows") != -1) {
			params.add("cmd");
			params.add("/C");
			params.add("call");
			params.add(jbossDir + fSep + "bin" + fSep + "shutdown.bat");
		}
		params.add("-s");
		params.add(url);
		params.add("-S");
		
		String[] cmdArray = (String[]) params.toArray(new String[params.size()]);
		try {
			logInfo("Shutting down JBoss");
			logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));

			ProcessBuilder builder = new ProcessBuilder(cmdArray);
			Map<String, String> env = builder.environment();
			if (env.containsKey(JBOSS_HOME_VARIABLE)) {
				logDebug("Environment variable " + JBOSS_HOME_VARIABLE + " was set." +
				" It's removed from the new process.");
				env.remove(JBOSS_HOME_VARIABLE);
			}
			builder.directory(new File(jbossDir));
			shutdownProcess = builder.start();
			shutdownProcess.getOutputStream().close();
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(shutdownProcess.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(shutdownProcess.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			
			// wait untill JBoss finishes
			shutdownProcess.waitFor();
			
			logInfo("JBoss is shut down");
		} catch (Exception e) {
			throw new TaskException("Cannot shutdown JBoss: " + e.getMessage());
		}
	}

	
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run()throws TaskException {
		jbossDir = getTaskProperty(JBOSS_DIR);
		logInfo("JBoss directory: " + jbossDir);
		
		String port = getTaskProperty(JBOSS_JNP_PORT);
		if (port != null) {
			jnpPort = Integer.valueOf(port);
			logInfo("JNP port: " + port);
		}

		shutdown("jnp://localhost:" + jnpPort);
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(JBOSS_DIR);
	}
}
