/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.jonas.run;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Map;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;


/**
 * <p>Starts Jonas.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #JONAS_DIR}: <br>
 * 		- path to the Jonas installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #TOMCAT_DIR}: <br>
 * 		- path to the Tomcat installation directory <br>
 * 		- mandatory <br>
 * </ul>
 *
 * <b>Checkpoints:</b><br>
 * <ul>
 * <li>{@value #CHECKPOINT_RUNNING}: <br>
 * 		- set when Jonas has succefully started<br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class JonasRun extends Job {
	//------------------------CONSTANTS--------------------
	/** Task property name for Jonas installation directory. */
	public static final String JONAS_DIR = "jonas.dir";
	/** Task property name for Tomcat installation directory. */
	public static final String TOMCAT_DIR = "tomcat.dir";
	/** Name of the checkpoint which is set when Jonas is succesfully started.*/
	public static final String CHECKPOINT_RUNNING = "running";
	
	/** Name of the environment variable for Jonas' root directory */
	private static final String JONAS_HOME_VARIABLE = "JONAS_ROOT";
	/** Name of the environment variable for Tomcat root directory */
	private static final String CATALINA_HOME_VARIABLE = "CATALINA_HOME";


	
	//-----------------------ATTRIBUTES--------------------
	/** Path to directory where Jonas is installed */
	private String jonasDir = null;
	/** Path to directory where Tomcat is installed */
	private String tomcatDir = null;

	/**
	 * Allocates a new <code>JonasRun</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public JonasRun() throws TaskInitializationException {
		super();
	}
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run() throws TaskException {
		jonasDir = getTaskProperty(JONAS_DIR);
		logInfo("Jonas directory: " + jonasDir);
		
		tomcatDir = getTaskProperty(TOMCAT_DIR);
		logInfo("Tomcat directory: " + tomcatDir);

		/*
		 * Run Jonas
		 */

		String fSep = File.separator;
		Process jonasProcess = null;
		String[] cmdArray = null;
		if (isRunningInLinux()) {
			String[] cmd = {jonasDir + fSep + "bin" + fSep + "unix" + fSep + "jonas",
					"start",
					"-fg",
					"-Dcatalina.home=" + tomcatDir
			};
			cmdArray = cmd;
		}

		if (isRunningInWindows()) {
			String[] cmd = {"cmd",
					"/C",
					"call",
					jonasDir + fSep + "bin" + fSep + "nt" + fSep + "jonas.bat",
					"start",
					"-fg",
					"-Dcatalina.home=" + tomcatDir
			};
			cmdArray = cmd;
		}
		
		try {
			logInfo("Starting Jonas");
			logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));

			ProcessBuilder builder = new ProcessBuilder(cmdArray);
			Map<String, String> env = builder.environment();
			if (env.containsKey(JONAS_HOME_VARIABLE)) {
				logDebug("Environment variable " + JONAS_HOME_VARIABLE + " was set." +
				" It's removed from the new process.");
				env.remove(JONAS_HOME_VARIABLE);
			}
			if (env.containsKey(CATALINA_HOME_VARIABLE)) {
				logDebug("Environment variable " + CATALINA_HOME_VARIABLE + " was set." +
				" It's removed from the new process.");
				env.remove(CATALINA_HOME_VARIABLE);
			}
			builder.directory(new File(jonasDir));
			jonasProcess = builder.start();
			jonasProcess.getOutputStream().close();

			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(jonasProcess.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(jonasProcess.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();

		} catch (Exception e) {
			throw new TaskException("Cannot run Jonas", e);
		}
		
		/*
		 * Periodically check if Jonas has started
		 */ 

		// process of the jonas admin tool
		Process adminProcess = null;
		String[] adminCmdArray = null;
		if (isRunningInLinux()) {
			String[] cmd = {jonasDir + fSep + "bin" + fSep + "unix" + fSep + "jonas",
					"admin",
					"-l",
			};
			adminCmdArray = cmd;
		}

		if (isRunningInWindows()) {
			String[] cmd = {"cmd",
					"/C",
					"call",
					jonasDir + fSep + "bin" + fSep + "nt" + fSep + "jonas.bat",
					"admin",
					"-l",
			};
			adminCmdArray = cmd;
		}

		boolean jonasStarted = false;
		// keep checking if Jonas started for 5 minutes
		for (int i = 0; i < 15; i++) {
			logInfo("Checking if Jonas has started");
			logDebug("Executing: " + ArrayUtils.join(" ", adminCmdArray));
			// launch the Jonas admin tool a ask it to list deployed beans,
			// it will write an error message if Jonas didn't start yet
			try {
				adminProcess = Runtime.getRuntime().exec(adminCmdArray, 
						null, 
						new File(jonasDir));
				adminProcess.getOutputStream().close();
			} catch (Exception e) {
				throw new TaskException("Cannot run Jonas startup check", e);
			}
			
			try {
				/* Read in the output from the subprocess */
				BufferedReader reader = new BufferedReader(new InputStreamReader(adminProcess.getInputStream()));
				String line = reader.readLine();
				if (line != null) {
					StringBuffer output = new StringBuffer(line);
					if (!output.equals("JOnAS server jonas unreachable")) {
						jonasStarted = true;
						break;
					}
				}
			} catch (IOException e) {
				throw new TaskException("Cannot read output from jonas admin tool", e);
			}

			// wait 20 seconds
			try {
				Thread.sleep(20000);
			} catch (InterruptedException e) {
				// nothing evil happened
				Thread.currentThread().interrupt();
			}
		}
		
		if (jonasStarted) {
			logInfo("Jonas started");
			checkPointReached(CHECKPOINT_RUNNING, null);
		} else {
			throw new TaskException("Jonas not started");
		}
		
		try {
			// wait untill Jonas finishes
			jonasProcess.waitFor();
		} catch (InterruptedException e) {
			throw new TaskException("Wait for Jonas' finish interrupted", e);
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{JONAS_DIR,TOMCAT_DIR});
	}
}
