/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.mysql.initialize;

import java.io.File;
import java.io.IOException;

import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Initializes a fresh MySQL installation. This initialization is needed
 * before the first start of MySQL.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #MYSQL_ROOT}: <br>
 * 		- path to the MySQL installation directory <br>
 * 		- mandatory <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class MySQLInitialize extends Job {
	//---------------- CONSTANTS ---------------

	/** Task property name for the root directory of the MySQL installation */
	public static final String MYSQL_ROOT = "mysql.root";
	
	//---------------- ATTRIBUTES --------------
	
	/** Root directory of the MySQL installation */
	private String mysqlRoot;
	
	/**
	 * Allocates a new <code>MySQLInitialize</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public MySQLInitialize() throws TaskInitializationException{
		super();
	}
	
	@Override
	protected void run() throws TaskException {
		mysqlRoot = getTaskProperty(MYSQL_ROOT);
		logInfo("MySQL installation root directory: " + mysqlRoot);

		if (System.getProperty("os.name").toLowerCase().indexOf("linux") != -1) {
			// run a script that initializes MySQL
			Process scriptProcess = null;
			String fSep = File.separator;
			String scriptPath = mysqlRoot + fSep + "scripts" + fSep + "mysql_install_db";
			logDebug("Executing: " + scriptPath);
			try {
				scriptProcess = Runtime.getRuntime().exec(scriptPath, null, new File(mysqlRoot));

				/* Read in the output from the subprocess */
				OutputReader stdoutReader = 
					new OutputReader(scriptProcess.getInputStream(), OutputType.STDOUT);
				OutputReader stderrReader = 
					new OutputReader(scriptProcess.getErrorStream(), OutputType.STDERR);
				stdoutReader.start();
				stderrReader.start();
				// wait untill the script finishes
				scriptProcess.waitFor();
			} catch (IOException e) {
				throw new TaskException("MySQL initialization aborted", e);
			} catch (SecurityException e) {
				// Subprocess creation not allowed
				throw new TaskException("Cannot create MySQL initialization process", e);
			} catch (InterruptedException e) {
				throw new TaskException("Output readers interrupted", e);
			} catch (Exception e) {
				throw new TaskException(e);
			}
		}
		if (System.getProperty("os.name").toLowerCase().indexOf("windows") != -1) {
			logInfo("Running in Windows, no post-install initialization required");
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(MYSQL_ROOT);
	}
}
