/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.mysql.shutdown;

import java.io.File;

import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Shuts down MySQL.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #MYSQL_ROOT}: <br>
 * 		- path to the MySQL installation directory <br>
 * 		- mandatory <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class MySQLShutdown extends Job {
	//---------------- CONSTANTS ---------------
	
	/** Task property name for the root directory of the MySQL installation */
	public static final String MYSQL_ROOT = "mysql.root";
	
	/** The time we give to MySQL to shut down */
	private static final int MYSQL_SHUTDOWN_TIMEOUT = 20000;

	//---------------- ATTRIBUTES --------------
	
	/** Root directory of the MySQL installation */
	private String mysqlRoot;

	/**
	 * Allocates a new <code>MySQLShutdown</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public MySQLShutdown() throws TaskInitializationException {
		super();
	}

	@Override
	protected void run() throws TaskException {
		mysqlRoot = getTaskProperty(MYSQL_ROOT);
		logInfo("MySQL root: " + mysqlRoot);

		try {
			logInfo("Shutting down MySQL");
			String[] cmdArray = { mysqlRoot + File.separator + "bin" + File.separator + "mysqladmin" ,
					"-u",
					"root",
					"shutdown"};
			logDebug("Executing : " + cmdArray[0] + " " + cmdArray[1] + " " + cmdArray[2] + " " + 
					cmdArray[3]);
			Process p = Runtime.getRuntime().exec(cmdArray, null, new File(mysqlRoot));
			p.getOutputStream().close();
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(p.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(p.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			
			if (p.waitFor() != 0) {
				logFatal("Shutdown unsuccessfull");
				exitError();
			}
		} catch (Exception e) {
			throw new TaskException("Cannot shutdown MySQL", e);
		}

		// give MySQL a bit of time to shutdown 
		try {
			Thread.sleep(MYSQL_SHUTDOWN_TIMEOUT);
		} catch (InterruptedException e) {
			// nothing bad happened
			Thread.currentThread().interrupt();
		}
		
		logInfo("MySQL shut down");
	}

	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(MYSQL_ROOT);
	}
}
