/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.omniorb.compile.linux;

import java.io.File;
import java.io.IOException;

import cz.cuni.mff.been.common.BeenException;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.common.StringUtils;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Chmod;
import cz.cuni.mff.been.common.anttasks.Untar;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

public class OmniOrbLinuxCompile extends Job {
	
	// source directory with omniorb archive
	private static final String PROPERTY_SOURCE_DIR = "source.dir";
	// omniorb installation root name (will be placed in task working directory)
	private static final String PROPERTY_OMNIORB_ROOT = "omniorb.root";
	private String baseName;
	
	public OmniOrbLinuxCompile() throws TaskInitializationException {
		super();
	}
	
	@Override
	protected void run() throws TaskException {
	
		try {
			
			// dearchive the source
			dearchive();

			// determine the root directory for dearchived sources
			baseName = findSourceRoot();
			logDebug("Sources root directory set to: " + baseName);
			
			// create build directory
			createBuildDir();
			
			// set execute permission on configure script
			chmodBinDir();
			
			// run configure script
			runConfigure();
			
			// build omniorb
			runBuild();
			
			// install
			runInstall();
			
		} catch (BeenException e) {
			throw new TaskException("Build failed", e);
		} catch (InterruptedException e) {
			throw new TaskException("Build failed", e);
		} catch (IOException e) {
			throw new TaskException("Build failed", e);
		} 
		
	}

	private void dearchive() throws AntTaskException {
		File sourceDir = new File(getTaskProperty(PROPERTY_SOURCE_DIR));
		// existence checked in task initialization
		String fileName = sourceDir.list()[0];
		File absoluteFile = new File(sourceDir, fileName);
		Untar.untar(absoluteFile.getAbsolutePath(), getTempDirectory(), "gzip");
	}
	
	private void createBuildDir() {
		File f = new File(getTempDirectory(), "build");
		f.mkdir();
	}
	
	private void chmodBinDir() throws AntTaskException {
		String binDir = getTempDirectory() + File.separatorChar
			+	baseName + File.separatorChar + "bin";
		logDebug("Changing permissions for " + binDir);
		Chmod.recursiveDirectoryChmod(binDir, "ugo+rx");
	}
	
	private void runConfigure() 
	throws IOException, InterruptedException, AntTaskException {
		logDebug("Running configure script");
		
		String configScript = getTempDirectory() + File.separatorChar 
			+ baseName + File.separatorChar + "configure";
		Chmod.chmod(configScript, "ugo+rx");
		
		
		String prefix = getWorkingDirectory() +  File.separatorChar
			+ getTaskProperty(PROPERTY_OMNIORB_ROOT);
		
		try {
			int result = runCommand(configScript,
					new String[]{
					"--prefix=" + prefix,
					"--with-omniORB-config=omniORB.cfg",
					"--with-omniNames-logdir=log"
			});
		
			if (result != 0) {
				logFatal("Configure script failed, see log for details");
			}
		} catch (InterruptedException e) {
			throw new InterruptedException("Configure script interrupted");
		}		
	}
	
	private void runBuild() throws IOException, InterruptedException {
		logDebug("Running compilation");
		try {
			int result = runCommand("make", null);
		
			if (result != 0) {
				logFatal("'make' failed, see log for details");
			}
		} catch (InterruptedException e) {
			throw new InterruptedException("'make' interrupted");
		}
		logDebug("Build finished");
	}
	
	private void runInstall() throws IOException, InterruptedException {
		logDebug("Running installation");
		try {
			int result = runCommand("make", new String[]{"install"});
		
			if (result != 0) {
				logFatal("'install' failed, see log for details");
			}
		} catch (InterruptedException e) {
			throw new InterruptedException("'install' interrupted");
		}
		logDebug("Installation finished");
	}
	

	private int runCommand(String command, String[] params) throws IOException, InterruptedException {
		ProcessBuilder procBuilder = new ProcessBuilder();
		procBuilder.directory(new File(getTempDirectory(), "build"));
		
		int cmdArrayLength = params == null ? 1 : params.length + 1;
		String[] cmdArray = new String[cmdArrayLength];
		cmdArray[0] = command;
		if (params != null) {
			System.arraycopy(params, 0, cmdArray, 1, params.length);
		}
		
		procBuilder.command(cmdArray);
		
		Process p = procBuilder.start();
		
		OutputReader stdOutReader = new OutputReader(p.getInputStream(), OutputType.STDOUT);
		stdOutReader.setOutputStream(System.out);
		stdOutReader.start();
		OutputReader stdErrReader = new OutputReader(p.getErrorStream(), OutputType.STDERR);
		stdErrReader.setOutputStream(System.err);
		stdErrReader.start();
		
		int result = -1;
		try {
			result = p.waitFor();
		} catch (InterruptedException e) {
			p.destroy();
			throw e;
		} finally {
			// stop reader threads in any case
			
			if (stdOutReader.isAlive()) {
				stdOutReader.interrupt();
			}
			if (stdErrReader.isAlive()) {
				stdErrReader.interrupt();
			}
		}
		
		return result;
	}
	
	private String findSourceRoot() throws TaskException {
		File tempDir = new File(getTempDirectory());
		for (File candidate : tempDir.listFiles()) {
			if (candidate.isDirectory()) {
				return candidate.getName();
			}
		}
		throw new TaskException("Incorrect omniorb package format (must use single directory)");
	}
	
	
	protected void checkRequiredProperties() throws TaskException {
		if (StringUtils.isEmpty(getTaskProperty(PROPERTY_OMNIORB_ROOT))) {
			throw new TaskException(PROPERTY_OMNIORB_ROOT
				+ " property not set");
		}
		
		if (StringUtils.isEmpty(getTaskProperty(PROPERTY_SOURCE_DIR))) {
			throw new TaskException(PROPERTY_SOURCE_DIR 
				+ " property not set");
		}
		
		File sourceDir = new File(getTaskProperty(PROPERTY_SOURCE_DIR));
		if (!sourceDir.exists() || sourceDir.list().length == 0) {
			throw new TaskException("Source file does not exist in "
				+ sourceDir.getAbsolutePath());
		}
	}
}
