/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.results;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipOutputStream;

import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.resultsrepository.Zipper;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.Task;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

public abstract class CheckAndConvertTask extends Job {

	/** Key of command-line value */
	public static final String RESULTS_PATHS_PROPERTY = "results.paths";

	/** Key of sample counts for each logical results table */
	public static final String SAMPLE_COUNTS_PROPERTY = "samples.counts";

	/** Key of results role name */
	public static final String RESULTS_ROLE_PROPERTY = "results.role";

	/** Key of suite name */
	public static final String SUITE_ARGUMENT = "suite";

	/** Ascii of tab character */
	protected static final String ASCII_TAB = "\t";
	
	/** Task argument containing table with locations of results for runs */
	private String srcDirectories;

	/** Samples count for each table */
	private long sampleCount;

	/** Results role - now the only distinguisher between tables */
	private String resultsRole;
	
	/** Host name - name of host we're running on */
	private String hostName;

	/** Suite name */
	protected String suiteName;
	
	/** Task working directory */
	private String workDir;
	
	/** Writer for results collecting metadata table */ 
	private PrintWriter outputWriter;

	/**
	 * Allocates a new <code>XamplerResultsCollectionTask</code> object. 
	 * 
	 * @throws TaskInitializationException
	 *             if something fails during the task creation
	 */
	public CheckAndConvertTask() throws TaskInitializationException {
		super();
	}

	/*
	 * ********* Results checking and converting functions (the core of the task) *********
	 */

	/**
	 * Converts ascii data format of one run into netcdf format, making it ready
	 * to upload to Results Repository
	 * 
	 * @param bid	Binary Id 
	 * @param rid 	Run Id
	 * @param sourceName	Source filename (ASCII)
	 * @param destinatioName	Destination filename (NetCDF)
	 * @param resultsRole	Identification of role in benchmark 
	 * @param suiteName 	Name of the benchmark suite
	 * @param samples 	Sample counts per each table
	 * 
	 * @return 	True iff succesful
	 * 
	 * @throws IOException
	 * @throws TaskException 
	 */
	public abstract boolean checkAndConvertOneRun(BID bid, RID rid, 
			String sourceName, String destinatioName, String resultsRole, 
			String suiteName, long samples) throws IOException, TaskException;

	/**
	 * Returns list of files to zip, when run is invalid (has crashed), this is sent 
	 * to results repository for logging purposes
	 * @param srcName Source directory of crashed run
	 * 
	 * @return	List of filenames to be packed and sent to RR for storage in case of crashed run 
	 */
	public abstract List<String> getCrashList(String srcName);
	
	/**
	 * Get canonical name of the localhost.
	 * 
	 * @return Canonical name of the localhost.
	 */
	public static String getCanonicalLocalhostName() {
		
		try {
			return InetAddress.getLocalHost().getCanonicalHostName();
		} catch (Exception e) {
			return "localhost";
		}
	}

	/**
	 * Packs all files into one crash zip to be sent to results repository
	 * 
	 * @param filesToPack
	 *            List of files to pack into that zip
	 * @param dstName filename, where to put zip
	 * @return Returns false in case if zip file could not be created
	 */
	public boolean createCrashFile(List<String> filesToPack, String dstName) {
		try {
			File file = new File(dstName);
			if (file.exists()) {
				file.delete();
			} 
			
			FileOutputStream f = new FileOutputStream(file);
			ZipOutputStream zos = new ZipOutputStream(f);
			BufferedOutputStream out = new BufferedOutputStream(zos);

			for (String filename : filesToPack) {
				logDebug("Adding " + filename + " into " + dstName);
				Zipper.zipFile(zos, out, new File(filename), "");
			}
			out.close();

		} catch (IOException e) {
			Task.getTaskHandle().logError(
					"i/o error generating zip with crashed results");
			e.printStackTrace();
			return false;
		}
		return true;
	}	

	/**
	 * Method to be called after one table of run is convert in descendant, which performs results
	 * checking and convertin. Run table is put in the list for upload.  
	 * 
	 * @param tableIndex	Index of table to be uploaded
	 * @param tableTotal	Total number of tables for this run
	 * @param bid			Binary Id of table
	 * @param rid			Run Id of table
	 * @param valid		Validity of run (i.e. correctness of results)
	 */
	protected void notifyTableCreation(int tableIndex, int tableTotal, BID bid, RID rid, boolean valid) {
		
		/* Add a line to tables.meta file containing info for collecting task */
		/* Number of table (per run), TAB, total number of tables, TAB, binaryId, 
		 * TAB, runId, TAB, runValidity, TAB, results role, TAB hostname */
		outputWriter.println(tableIndex + ASCII_TAB + tableTotal + ASCII_TAB 
				+ bid + ASCII_TAB + rid + ASCII_TAB
				+ ((valid) ? "1" : "0") + ASCII_TAB + resultsRole + ASCII_TAB + hostName);
	}
	
	/**
	 * Overriden run method to run task
	 * @throws TaskException 
	 */
	public void run() throws TaskException {

		workDir = getWorkingDirectory();
		srcDirectories = getTaskProperty(RESULTS_PATHS_PROPERTY);
		String sampleCountString = getTaskProperty(SAMPLE_COUNTS_PROPERTY); 
		if (sampleCountString != null) { 
			sampleCount = Long.valueOf(sampleCountString);
		} else {
			sampleCount = 0;
		}

		resultsRole = getTaskProperty(RESULTS_ROLE_PROPERTY);
		hostName = getCanonicalLocalhostName();
		suiteName = getTaskProperty(SUITE_ARGUMENT);
	
		/*
		 * We suppose all methods addAnalysis, addExperiment, and addBinary has
		 * been called. Otherwise, attempt to upload results will result in an
		 * "analysis/experiment/binary doesn't exist" error, or metadata reading error
		 */
		
		/** Table with paths to files with results of runs */ 
		ArrayList<RunResultsPosition> resultsTable = new ArrayList<RunResultsPosition>();
		
		String[] lineTokens;
		String line;
		BufferedReader inputReader;

		inputReader = new BufferedReader(new StringReader(srcDirectories));

		/* In srcDirectories (task property read in constructor), there's table 
		 * containing info, where we should find results of runs. Table has got
		 * lines - one per each run to check and convert.
		 * 
		 * One line has got 4 items:
		 *   table index, BID, RID, Validity
		 *   
		 * Following code parses table.
		 * convert it from string to object (with Miso + Jaro) 
		 * - tracked as todo in RunResultsPosition.java
		 */
		
		try {
			while ((line = inputReader.readLine()) != null) {
				lineTokens = line.split(ASCII_TAB);
				if (lineTokens.length == 3) {
					RunResultsPosition temp = new RunResultsPosition(new BID(
							new Integer(lineTokens[0])), new RID(new Integer(
							lineTokens[1])), lineTokens[2]);

					resultsTable.add(temp);
				} else {					
					throw new TaskException("invalid task arguments (results directory table)");
				}
			}
			inputReader.close();
		} catch (IOException e) {
			throw new TaskException("Cannot read results", e);
		}

		
		/* Now, iterate through the table of all runs and check them all one by one */
		Iterator it = resultsTable.iterator();

			
		try {
			outputWriter = new PrintWriter(
					new BufferedWriter(new FileWriter(workDir
							+ File.separator + "tables.meta")));
		} catch (IOException e) {
			throw new TaskException("I/O error during results conversion,"
					+ " couldn't create 'tables.meta' file", e);			
		}
			
			/* Iterate through the table with items for runs */
			while (it.hasNext()) {				
				
				boolean validRun;
				
				RunResultsPosition currentRun = (RunResultsPosition) it.next();
			
				try {
					validRun = checkAndConvertOneRun(currentRun.getBid(),
							currentRun.getRid(), currentRun.getPath(), 
							workDir + File.separator + currentRun.getRid(), 
							resultsRole, suiteName, sampleCount);
				} catch (IOException e) {
					logError("I/O error during results conversion, BID(" + currentRun.getBid() + "), "
							+ "RID (" + currentRun.getRid() + ")");
					notifyTableCreation(0, 1, currentRun.getBid(), currentRun.getRid(), false);
					validRun = false;
				}
				
				/* If run is invalid, we collect the output of benchmark 
				 * to send to results repository */
				if (!validRun) {
					createCrashFile(getCrashList(currentRun.getPath()),
							workDir + File.separator + currentRun.getRid()); 
				}						

			}
			
			outputWriter.close();

		logInfo("Results checked and converted.");

	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{
				RESULTS_PATHS_PROPERTY,
				RESULTS_ROLE_PROPERTY,
				SUITE_ARGUMENT
		});
	}

}
