/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.results.collect;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.ServerSocket;
import java.net.Socket;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;

import ucar.ma2.ArrayInt;
import ucar.ma2.Index;
import ucar.nc2.Dimension;
import ucar.nc2.NetcdfFileWriteable;
import cz.cuni.mff.been.benchmarkmanager.BenchmarkManagerService;
import cz.cuni.mff.been.benchmarkmanager.gui.BenchmarkManagerGUIInterface;
import cz.cuni.mff.been.common.BeenUtils;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.hostmanager.InvalidArgumentException;
import cz.cuni.mff.been.hostmanager.load.HardwareDescription;
import cz.cuni.mff.been.hostmanager.load.LoadFile;
import cz.cuni.mff.been.hostmanager.load.LoadFileParserInterface;
import cz.cuni.mff.been.hostmanager.load.LoadMonitorInterface;
import cz.cuni.mff.been.hostmanager.load.LoadSample;
import cz.cuni.mff.been.hostmanager.util.TimeUtils;
import cz.cuni.mff.been.resultsrepository.Database;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryBusyException;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryException;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryService;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.Task;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;
import cz.cuni.mff.been.taskmanager.TaskManagerInterface;

/**
 * This task collects all the results, load and logs and sends them to Results Repository
 * 
 * @author Jakub Lehotsky
 *
 */
public class ResultsCollectionTask extends Job {

	/** Index of table number item in metadata table line */
	private static final int TABLE_NUMBER_INDEX = 0;
	
	/** Index of total table number item in metadata table line */
	private static final int TOTAL_TABLES_INDEX = 1;

	/** Index of binary Id item in metadata table line */
	private static final int BINARY_ID_INDEX = 2;

	/** Index of run Id item in metadata table line */
	private static final int RUN_ID_INDEX = 3;

	/** Index of validity item in metadata table line */
	private static final int VALIDITY_INDEX = 4;
	
	/** Index of results role item in metadata table line */
	private static final int RESULTS_ROLE_INDEX = 5;
	
	/** Index of hostname item in metadata table line */
	private static final int HOSTNAME_INDEX = 6;
	
	/** Key of command-line value */
	public static final String AID_PROPERTY = "analysis.id";
	
	/** Key of command-line value */
	public static final String EID_PROPERTY = "experiment.id";
	
	/** Task name */
	public static final String TASK_NAME = "results-collect";
	
	/** Key of command-line value */
	public static final String RESULTS_PATHS_PROPERTY = "results.paths";

	/** Filename of title for load graph*/
	private static final String TITLE_FILE = "title";

	/** Filename for names of network devices for load */
	private static final String NET_NAMES_FILE = "netnames";

	/** Filename for names of disk devices for load */
	private static final String DISK_NAMES_FILE = "disknames";

	/** Filename with memory usage table */
	private static final String MEMORY_FILE = "memory";
	
	/** Filename with total memory information */
	private static final String MAX_MEMORY_FILE = "totalmemory";

	/** Filename with processor usage information */
	private static final String PROCESSOR_FILE = "processor";

	/** Filename with disks access information */ 
	private static final String DISK_FILE = "disk";

	/** Filename with network traffic information */
	private static final String NET_FILE = "net";

	/** The TAB string */
	private static final String ASCII_TAB = "\t";

	/** Socket to use for file transfer */
	private static ServerSocket srvSocket;
	
	/** Analysis identification (of results being uploaded) */
	private AID analysisId;
	
	/** Experiment identification (of results being uploaded) */ 
	private EID experimentId;	
	
	/** Running instance of Results Repository to connect to */
	private static ResultsRepositoryInterface resultsRepositoryInstance;
	
	/** Running instance of Benchmark Manager to connect to */
	private static BenchmarkManagerGUIInterface benchmarkManagerInstance;
	
	/** Load monitor reference */
	private static LoadMonitorInterface loadMonitorReference;
	
	private String hostName;
	
	/** Task argument containing table with locations of results for runs */ 
	private String srcDirectories;
	
	/** Task temporary directory */
	private String tempDir;
	
	private PrintWriter memoryWriter = null;
	private PrintWriter maxMemoryWriter = null;	
	private PrintWriter processorWriter = null;
	private PrintWriter diskNamesWriter = null;
	private PrintWriter diskWriter = null;
	private PrintWriter netNamesWriter = null; 
	private PrintWriter netWriter = null;
	private PrintWriter titleWriter = null;	
	
	public ResultsCollectionTask() throws TaskInitializationException {
		super();
	}
	
	/**
	 * Initialization of Results Repository upload - connect to Results Repository through RMI and
	 * create socket for network transfer
	 * 
	 * @throws IOException
	 * @throws MalformedURLException
	 * @throws NotBoundException
	 * @throws ResultsRepositoryException
	 */ 
	private void initUpload() throws IOException, MalformedURLException, NotBoundException,
			ResultsRepositoryException {
				
		resultsRepositoryInstance = (ResultsRepositoryInterface) getTasksPort().serviceFind(
				ResultsRepositoryService.SERVICE_NAME, ResultsRepositoryInterface.RMI_NAME);		
	  	
		benchmarkManagerInstance = (BenchmarkManagerGUIInterface) getTasksPort().serviceFind(
				BenchmarkManagerService.SERVICE_NAME, BenchmarkManagerService.RMI_MAIN_IFACE);		
	  	
		
	  	srvSocket = new ServerSocket(0);
	  	logInfo("server socket intialized");
	}
	
	/**
	 * Get canonical name of the localhost.
	 * 
	 * @return Canonical name of the localhost.
	 */
	public static String getCanonicalLocalhostName() {
		
		try {
			return InetAddress.getLocalHost().getCanonicalHostName();
		} catch (Exception e) {
			return "localhost";
		}
	}

	/**
	 * Converts ascii data format of one run into netcdf format, 
	 * making it ready to upload to Results Repository
	 * 
	 * @param srcName	Source filename (ASCII)
	 * @param dstName	Destination filename (NetCDF)
	 * @throws IOException
	 */
	private void convertOneRunTable(String srcName, String dstName, String suiteName, String[] columnNames)
		throws IOException {
		int columnCount = columnNames.length;
		String debugMessage;
		debugMessage = "Converting ";
		for (int i = 0; i < columnNames.length; i++) {
			debugMessage += columnNames[i] + "   ";
		}
		logDebug(debugMessage);
		int[][] value = new int[20][columnCount];
		int[] origin = new int[2];
		int counter;
		File file = new File(srcName);
		String line;
		String[] lineTokens;
		
		logDebug("Column count for converting run is " + columnCount);
			
		/*BufferedReader inputReader = new BufferedReader(new FileReader(file));
		NetcdfFileWriteable ncfile = NetcdfFileWriteable.createNew(dstName, false);*/
		BufferedReader inputReader = new BufferedReader(new FileReader(file));
		NetcdfFileWriteable ncfile = new NetcdfFileWriteable();
		
		if (file.exists()) {
			try {								
				ncfile.setName(dstName);
								
				Dimension columnDim = ncfile.addDimension("columns", columnCount);
				Dimension measurementDim = ncfile.addDimension("measurements", -1);
				
				Dimension[] dim2 = new Dimension[2];
				
				dim2[0] = measurementDim;
				dim2[1] = columnDim;
							
				/*ncfile.addVariable("values", DataType.Int, Arrays.asList(dim2));*/
				ncfile.addVariable("values", int.class, dim2);
				ncfile.addVariableAttribute("values", "long_name", "Table of measured values");
				
				ncfile.create();				
				
				origin[0] = 0; counter = 0;			
				while ((line = inputReader.readLine()) != null) {
										
					lineTokens = line.split("\t");
					
					for (int i = 0; i < columnCount; i++) {
						try {
							value[(counter++) % 20][i] = Integer.valueOf(lineTokens[i]);
							//logDebug("stored value " + Integer.valueOf(lineTokens[i]) + "into array");
						} catch (NumberFormatException e) {
							value[counter % 20][i] = new Integer(0);
							logInfo("invalid number format - column " + i + " sample " + counter);
							
						}
					}
					
					if ((counter % 20) == 0) {
						ArrayInt valueA = new ArrayInt.D2(20, columnDim.getLength());
						Index ima = valueA.getIndex();
						for (int i = 0; i < 20; i++) {
							for (int j = 0; j < columnCount; j++) {
								valueA.setInt(ima.set(i, j), value[i][j]);
							}
						}
						
//						try {
							ncfile.write("values", origin, valueA);
						/*} catch (InvalidRangeException e) {
							logInfo("Invlaid ");
						}*/
						origin[0] += 20;
						
					}
				}
								
				/*if ((counter % 20) > 0) {
				    long[][] value2 = new long[counter % 20][columnCount];
				    for (int i = 0; i < counter % 20; i++)				    	
				    	value2[i] = value[i];
					ArrayInt valueA = new ArrayInt.D2(20, columnDim.getLength());
					Index ima = valueA.getIndex();
					for (int i = 0; i < 20; i++) {
						for (int j=0; j < columnCount; j++)
							valueA.setLong(ima.set(i,j), value[i][j]);
					}

					ncfile.write("values", origin, valueA);
					ncfile.write("values", origin, ArrayAbstract.factory(value2));
				}*/
				
			} catch (IOException e) {
				e.printStackTrace();
				logError("io error converting " + srcName);				
			} finally {
				ncfile.close();
				inputReader.close();
			}
		}			
	}
	


	
	/**
	 * Uploads one binary file (one run) to Results Repository
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 * @param sourceName	filename of source
	 * @param targetName	filename of target (in Results Repository)
	 * @throws ResultsRepositoryBusyException
	 * @throws IOException
	 */	
	private void uploadBinaryFile(AID aid, EID eid, BID bid, RID rid, boolean validRun, 
			String sourceName, String roleName, String hostName, int tableIndex, int totalTables) 
	throws ResultsRepositoryBusyException, IOException {
	
		byte[] buffer = new byte[1024];
	
	  	UploadHandle upHandle = resultsRepositoryInstance.uploadRunResults(
	  			aid, eid, bid, rid, roleName,
	  			 hostName, srvSocket.getLocalPort(), validRun, false, tableIndex, totalTables);
	  	Socket clientSocket = srvSocket.accept();		  	
	  		
	  	try {
			OutputStream outputStream = clientSocket.getOutputStream();
			InputStream inputStream = new FileInputStream(sourceName);
			try {
				int bytesRead;
				while ((bytesRead = inputStream.read(buffer)) != -1) {
					outputStream.write(buffer, 0, bytesRead);
				}
			} finally {
				inputStream.close();
			}
		} finally {
			clientSocket.close();
		}
		
	  	//logInfo("send data finished, waiting for Results Repository to acknowlege... ");
		String chkPntValue;
		do {
			/* Wait one hour - infinite would be more suitable */ 
			chkPntValue = (String) Task.getTaskHandle().getTasksPort().checkPointWait("system", 
				"resultsrepository-tid", "upload-finish-" + upHandle.hashCode(), 
				TaskManagerInterface.INFINITE_TIME);
			Task.getTaskHandle().logDebug("Value of checkpoint scanned = " + chkPntValue);
		} while ((chkPntValue == null) || (!chkPntValue.equals("1")));
        if (resultsRepositoryInstance.getFileUploadStatus(upHandle) 
        		== UploadStatus.UPLOADING) {
        	logError("Error in synchronization of upload with Results Repository - state still uploading");
        }
	  	
		switch (resultsRepositoryInstance.getFileUploadStatus(upHandle)) {
			case ACCEPTED:
				logInfo("uploaded: " + sourceName);
				resultsRepositoryInstance.endRunUpload(aid, eid, bid, rid, hostName, upHandle);
				break;
			case ERROR:
				logError("error uploading " + sourceName);
				resultsRepositoryInstance.endRunUpload(aid, eid, bid, rid, hostName, upHandle);
				break;
			default:
				logError("error synchronizing upload of " + sourceName);
				resultsRepositoryInstance.endRunUpload(aid, eid, bid, rid, hostName, upHandle);
				break;
		}
	}

	
	/**
	 * Uploads one load information file to Results Repository
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 * @param rid	Run Id
	 * @param sourceFile	filename of source - full path
	 * @param loadDataFile	filename, where to store load data in Results Repository
	 * @param targetName	filename of target (in Results Repository)
	 * @param hostName	hostname from which load information is uploaded
	 * @throws ResultsRepositoryBusyException
	 * @throws IOException
	 */
	private void uploadLoadFile(AID aid, EID eid, BID bid, RID rid, 
			String sourceFile, String loadDataFile, String hostName)
		throws IOException, ResultsRepositoryException {
		byte[] buffer = new byte[1024];
		
	  	UploadHandle upHandle = resultsRepositoryInstance.uploadLoadInfo(
	  			aid, eid, bid, rid, loadDataFile, 
	  			 hostName, srvSocket.getLocalPort(), false);
		
	  	Socket clientSocket = srvSocket.accept();		  	
	  		
	  	try {
			OutputStream outputStream = clientSocket.getOutputStream();
			InputStream inputStream = new FileInputStream(sourceFile);
			try {
				int bytesRead;
				while ((bytesRead = inputStream.read(buffer)) != -1) {
					outputStream.write(buffer, 0, bytesRead);
				}
			} finally {
				inputStream.close();
			}
		} finally {
			clientSocket.close();
		}
		
	  	//logInfo("send data finished, waiting for Results Repository to acknowlege... ");
		String chkPntValue;
		do {
			/* Wait one hour - infinite would be more suitable */ 
			chkPntValue = (String) Task.getTaskHandle().getTasksPort().checkPointWait("system", 
				"resultsrepository-tid", "upload-finish-" + upHandle.hashCode(), 
				TaskManagerInterface.INFINITE_TIME);
			Task.getTaskHandle().logDebug("Value of checkpoint scanned = " + chkPntValue);
		} while ((chkPntValue == null) || (!chkPntValue.equals("1")));
        if (resultsRepositoryInstance.getFileUploadStatus(upHandle) 
        		== UploadStatus.UPLOADING) {
        	logError("Error in synchronization of upload with Results Repository - state still uploading");
        }
	  	
		switch (resultsRepositoryInstance.getFileUploadStatus(upHandle)) {
			case ACCEPTED:
				logInfo("uploaded: " + sourceFile);
				resultsRepositoryInstance.endRunUpload(aid, eid, bid, rid, hostName, upHandle);
				break;
			case ERROR:
				logError("error uploading " + sourceFile);
				resultsRepositoryInstance.endRunUpload(aid, eid, bid, rid, hostName, upHandle);
				break;
			default:
				logError("error synchronizing upload of " + sourceFile);
				resultsRepositoryInstance.endRunUpload(aid, eid, bid, rid, hostName, upHandle);
				break;
		}		
	}
	
	/**
	 * Closes all files containing load upload information
	 */
	private void closeFiles() {
		
		if (maxMemoryWriter != null) maxMemoryWriter.close();
		if (memoryWriter != null) memoryWriter.close();
		if (processorWriter != null) processorWriter.close();
		if (diskWriter != null) diskWriter.close();
		if (netWriter != null) netWriter.close();
		if (diskNamesWriter != null) diskNamesWriter.close();
		if (netNamesWriter != null) netNamesWriter.close();
		if (titleWriter != null) titleWriter.close();
		
	}
	
	/**
	 * Upload all files that contain information about load from a given run
	 * 
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * @param loadDir	Directory with load files
	 * @param loadDataFiles	List of load files (filenames)
	 * @param hostname	Hostname from where load is being uploaded
	 * @throws IOException
	 * @throws ResultsRepositoryException
	 */
	private void uploadLoadFiles(BID bid, RID rid, String loadDir, String[] loadDataFiles,
			String hostname) 
		throws IOException, ResultsRepositoryException {
		
		for (String loadDataFile : loadDataFiles) {
			uploadLoadFile(analysisId, experimentId, bid, rid, loadDir 
					+ File.separator + loadDataFile, loadDataFile, hostname);			
		}		
	}
	
	/**
	 * Executes load reading and upload for a given run
	 * 
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * @return	True if successful 
	 * @throws TaskException
	 */
	private boolean doLoadUpload(BID bid, RID rid) throws TaskException {

		if (loadMonitorReference == null)
			return false;
				
		LoadFileParserInterface<LoadSample> loadParser;

		String loadDir = tempDir + File.separator + bid.toString() + File.separator + rid.toString();
		
		new File(loadDir).mkdirs();		 
		
		String contextName;
		String[] taskNames;
		String taskId = null;
		
		try {
			contextName = benchmarkManagerInstance.getAnalysisInterface()
					.getSubentityInterface(analysisId).getContext(experimentId);
		} catch (Exception e) {
			logError("Couldn't get contextName from Benchmark Manager");
			return false;
		}

		try {
			
			taskNames = BeenUtils.extractTaskIDs(benchmarkManagerInstance.getAnalysisInterface()
				.getSubentityInterface(analysisId)
				.getSubentityInterface(experimentId)
				.getSubentityInterface(bid).getAllTasks(rid));
		} catch (RemoteException e) {
			logError("RMI error contacting Benchmark Manager when uploading load");
			return false;
		}
		
		logDebug("Candidate tasks with logs : " + taskNames.length);
		
		try {
			for (int i = 0; i < taskNames.length; i++) {
				/*if (loadresultsRepositoryInstance.performLogUpload(analysisId, 
						experimentId, contextName, taskNames[i]);*/
				if (loadMonitorReference.hasDetailedModeData(contextName, taskNames[i])) {
					taskId = taskNames[i];
				}
			}
		} catch (RemoteException e) {
			logError("RMI error contacting Load Monitor when uploading load");
			return false;
		}
		
		if (taskId == null) {
			return false;
		}
				
		try {
			logDebug("LOAD: Context is " + contextName);
			logDebug("LOAD: Task name is " + taskId);
			loadParser = LoadFile.createSampleParser(loadMonitorReference.getDetailedModeLoadData(
					contextName, taskId));
		} catch (FileNotFoundException e) {
			logError("File with load not found.");
			return false;
		} catch (InvalidArgumentException e) {
			logError("Invalid argument when getting LoadFile : " + e.getMessage());
			return false;
		} catch (RemoteException e) {
			logError("Error contacting load monitor RMI.");
			return false;
		}
		
		logDebug("Starting load reading...");
		
		boolean first = true;
		long startTimestamp = 0;
		long endTimestamp = 0;
		
		int driveCount = 0;
		int netIfaceCount = 0;
		int processorCount = 0;
		
		try {
		
			memoryWriter = new PrintWriter(new BufferedWriter(
					new FileWriter(loadDir + File.separator + MEMORY_FILE)));
			maxMemoryWriter = new PrintWriter(new BufferedWriter(
					new FileWriter(loadDir + File.separator + MAX_MEMORY_FILE)));			
			processorWriter = new PrintWriter(new BufferedWriter(
					new FileWriter(loadDir + File.separator + PROCESSOR_FILE)));
			diskWriter = new PrintWriter(new BufferedWriter(
					new FileWriter(loadDir + File.separator + DISK_FILE)));
			netWriter = new PrintWriter(new BufferedWriter(
					new FileWriter(loadDir + File.separator + NET_FILE)));
			diskNamesWriter = new PrintWriter(new BufferedWriter(
					new FileWriter(loadDir + File.separator + DISK_NAMES_FILE)));
			netNamesWriter = new PrintWriter(new BufferedWriter(
					new FileWriter(loadDir + File.separator + NET_NAMES_FILE)));
			titleWriter = new PrintWriter(new BufferedWriter(
					new FileWriter(loadDir + File.separator + TITLE_FILE)));
			
		} catch (IOException e) {
			
			logError("Error initializting files for storing load");
			closeFiles();			 
			return false;			
		}

		/* Conversion of load information into Results Repository format */
		while(true) {
			try {
				if (!loadParser.hasNext()) {
					
					logDebug("Load parser didn't have next sample.");
					break;
				}
			} catch (Exception e){
				e.printStackTrace();
				continue;
			}
						
			try {
						
				try {
					
					LoadSample sample = loadParser.getNext();
										
					short[] processorUsage = sample.getProcessorUsage();
					
					long memoryFree = sample.getMemoryFree();
					
					long[] diskRead = sample.getDiskReadBytesPerSecond();
					long[] diskWrite = sample.getDiskWriteBytesPerSecond();
					
					int[] netReceive = sample.getNetworkBytesReceivedPerSecond();
					int[] netSend = sample.getNetworkBytesSentPerSecond();
					
					
					if (first) {
						
						driveCount = sample.getDriveCount();
						
						netIfaceCount = sample.getNetworkInterfaceCount();
						
						processorCount = sample.getProcessorCount();
						
						startTimestamp = sample.getTimeStamp();
						first = false;
						
					} else {
						
						if (driveCount != sample.getDriveCount()) {
							logError("Load error : drive count changed.");
							return false;
						}
						
						if (netIfaceCount != sample.getNetworkInterfaceCount()) {
							logError("Load error : interface count changed.");
							return false;
						}
						
						if (processorCount != sample.getProcessorCount()) {
							logError("Load error : processor count changed.");
							return false;
						}
						
						endTimestamp = sample.getTimeStamp();
						
					}
					
					/* Store processors percentage */
					for (int i = 0; i < processorUsage.length; i++) {
						
						if (i == 0) {
							processorWriter.print(processorUsage[i]);
						} else {
							processorWriter.print(ASCII_TAB + processorUsage[i]);
						}
					}
					processorWriter.println();

					/* Store memory usage */
					memoryWriter.println(memoryFree);

					/* Store disks usage */
					for (int i = 0; i < diskRead.length; i++) {
						
						if (i == 0) {
							diskWriter.print(diskRead[i]);
							diskWriter.print(ASCII_TAB + diskWrite[i]);
						} else {
							diskWriter.print(ASCII_TAB + diskRead[i]);
							diskWriter.print(ASCII_TAB + diskWrite[i]);
						}
					}					
					diskWriter.println();

					/* Store net usage */
					for (int i = 0; i < netReceive.length; i++) {
						
						if (i == 0) {
							netWriter.print(netReceive[i]);
							netWriter.print(ASCII_TAB + netSend[i]);						
						} else {
							netWriter.print(ASCII_TAB + netReceive[i]);
							netWriter.print(ASCII_TAB + netSend[i]);
						}
					}					
					netWriter.println();
					
				} catch (IOException e) {
					
					logError("I/O error converting load information : " + e.getMessage());					
					closeFiles();
					
					return false;
					
				}				
			} catch (Exception e) {
				e.printStackTrace();
				closeFiles();
				
				continue;
			}
		}
		
		HardwareDescription hwDesc;
		
		try { 
			 hwDesc = loadMonitorReference.getHardwareDescription();
		} catch (RemoteException e) {
			logError("RMI error contacting Load Monitor when uploading load");
			return false;
		}
		
		long memorySize = hwDesc.getMemorySize();
		maxMemoryWriter.println(memorySize);

		String[] discNames = hwDesc.getDrives();
		
		for (String driveName : discNames) {
			diskNamesWriter.println(driveName);
		}

		String[] netNames = hwDesc.getAdapters();
		
		for (String netName : netNames) {
			netNamesWriter.println(netName);
		}
		
		Date startTime = TimeUtils.convertWindowsTimeToJavaDate(startTimestamp);
		Date endTime = TimeUtils.convertWindowsTimeToJavaDate(endTimestamp);
		
		SimpleDateFormat format = 
            new SimpleDateFormat("dd MMM yyyy HH:mm:ss.SSS");
		
		String timeTitle = "from " + format.format(startTime) + " to " + format.format(endTime);
		
		titleWriter.println(timeTitle);		
		
		closeFiles();
			
		try {
			uploadLoadFiles(bid, rid, loadDir, new String[] { TITLE_FILE, NET_NAMES_FILE, DISK_NAMES_FILE, MEMORY_FILE, 
					MAX_MEMORY_FILE, PROCESSOR_FILE, DISK_FILE, NET_FILE }, hostName);
		} catch (ResultsRepositoryException e) {
			logError("Results repository error : " + e.getMessage());
			return false;
		} catch (IOException e) {
			logError("I/O error during load upload : " + e.getMessage());
			e.printStackTrace();
		}
		
		return true;
	}

	/**
	 * Runs the task
	 */
	@SuppressWarnings("unchecked")
	protected void run() throws TaskException {
		tempDir = getWorkingDirectory();
		analysisId = new AID(new Integer(getTaskProperty(AID_PROPERTY))); 
		experimentId = new EID(new Integer(getTaskProperty(EID_PROPERTY)));
		srcDirectories = getTaskProperty(RESULTS_PATHS_PROPERTY);
		hostName = getCanonicalLocalhostName();
		
		/* We suppose all methods addAnalysis, addExperiment, and addBinary has been called. 
		 * Otherwise, attempt to upload results will result in an "analysis/experiment/binary 
		 * doesn't exist" error */
		
		String contextName;
		
		try {
			initUpload();
			contextName = benchmarkManagerInstance.getAnalysisInterface().getSubentityInterface(analysisId)
					.getContext(experimentId);
			
		} catch (Exception e) {
			throw new TaskException("Results Repository upload initialization has failed (results collection task)", e);
		}
		
		try {
			loadMonitorReference = (LoadMonitorInterface) Naming.lookup("rmi://localhost/" 
					+ LoadMonitorInterface.RMI_NAME);
		} catch (RemoteException e) {
			logError("Couldn't connect to Load Monitor : " + e.getMessage());
		} catch (MalformedURLException e) {
			logError("Couldn't connect to Load Monitor : " + e.getMessage());
		} catch (NotBoundException e) {
			logError("Couldn't connect to Load Monitor : " + e.getMessage());
		}
						
		BufferedReader inputReader;
		
		/* Create list of paths to working directories of all results converting tasks */
		inputReader = new BufferedReader(new StringReader(srcDirectories));
		
		ArrayList<String>resultsTable = new ArrayList<String>();
		String[] lineTokens;
		String line;
		
		try {
		
			while ((line = inputReader.readLine()) != null) {								
				lineTokens = line.split("\t");			
					if (lineTokens.length == 1) {						 
						String temp = new String(lineTokens[0]);
								
						resultsTable.add(temp); 
					} else {
						throw new TaskException("Invalid task arguments (results directory table)");
					}										
			}
			inputReader.close();
		} catch (IOException e) {
			throw new TaskException("Cannot read results...", e);
		}		

		Iterator it = resultsTable.iterator();

		
		try {
			/* For each task, which was converting */
			while (it.hasNext()) {
			
				BID bid;
				RID rid;
				String currentRun = (String) it.next();
				/* Read tables with all metdata about data to upload */ 
				BufferedReader tablesReader = new BufferedReader(new FileReader(
						currentRun + File.separator + "tables.meta"));
				
				
				while ((line = tablesReader.readLine()) != null) {
					logDebug("Read line from tables.meta:" + line);
					lineTokens = line.split("\t");
					if (lineTokens.length == 7) {

						BufferedReader metaReader = new BufferedReader(new FileReader(
								currentRun + File.separator + lineTokens[RUN_ID_INDEX] + "-"
								+ lineTokens[TABLE_NUMBER_INDEX] + ".meta"));
						String suiteTitle = (line = metaReader.readLine()); 
						
						ArrayList<String> columnNames = new ArrayList<String>();						
						
						while ((line = metaReader.readLine()) != null) {
							if (line.compareTo("") != 0 && line.compareTo("\n") != 0) {
								columnNames.add(line);
							}
						}
						
						resultsRepositoryInstance.reportExperimentColumnNames(analysisId, 
								experimentId, columnNames);

						metaReader.close();
						
						bid = new BID(Integer.valueOf(lineTokens[BINARY_ID_INDEX]));
						rid = new RID(Integer.valueOf(lineTokens[RUN_ID_INDEX]));
						
						if (lineTokens[VALIDITY_INDEX].compareTo("0") == 0) {							
							uploadBinaryFile(analysisId, experimentId,
									bid, rid, false,
									currentRun + File.separator + lineTokens[RUN_ID_INDEX],
									lineTokens[RESULTS_ROLE_INDEX], lineTokens[HOSTNAME_INDEX],
									Integer.valueOf(lineTokens[TABLE_NUMBER_INDEX]),
									Integer.valueOf(lineTokens[TOTAL_TABLES_INDEX]));
																		
						} else {
							
							convertOneRunTable(currentRun + File.separator 
									+ lineTokens[RUN_ID_INDEX] + "-" + lineTokens[TABLE_NUMBER_INDEX], 
									tempDir + File.separator + lineTokens[RUN_ID_INDEX] + "-" 
									+ lineTokens[TABLE_NUMBER_INDEX], 
									suiteTitle, columnNames.toArray(new String[] { null }));
							uploadBinaryFile(analysisId, experimentId,
									bid, rid, true,
									tempDir + File.separator + lineTokens[RUN_ID_INDEX] + "-" 
									+ lineTokens[TABLE_NUMBER_INDEX],
									lineTokens[RESULTS_ROLE_INDEX], lineTokens[HOSTNAME_INDEX],
									Integer.valueOf(lineTokens[TABLE_NUMBER_INDEX]),
									Integer.valueOf(lineTokens[TOTAL_TABLES_INDEX]));
						}
					} else {
						throw new TaskException("Invalid 'tables.meta' file generated from checking tasks");
					}
					
					try {																		
						String[] taskNames = BeenUtils.extractTaskIDs(benchmarkManagerInstance.getAnalysisInterface()
								.getSubentityInterface(analysisId)
								.getSubentityInterface(experimentId).getAllTasks(bid));						
						for (int i = 0; i < taskNames.length; i++) {
							resultsRepositoryInstance.performLogUpload(analysisId, 
								experimentId, contextName, taskNames[i]);
						}
					} catch (RemoteException e) {
						logError("RMI error contacting task manager when uploading logs : " + e.getMessage());
					} catch (ResultsRepositoryException e) {
						logError("Results repository error in logs upload : " + e.getMessage());
					}
					
					logInfo("Uploading load for " 
							+ Database.getRunReferenceString(analysisId, experimentId, bid, rid));
					
					resultsRepositoryInstance.initLoadUpload(analysisId, experimentId, bid, rid);
					
					if (doLoadUpload(bid, rid)) {		
						resultsRepositoryInstance.closeLoadUpload(analysisId, experimentId, 
								bid, rid, true);
						logInfo("Load uploaded for " 
								+ Database.getRunReferenceString(analysisId, experimentId, bid, rid));
					} else {
						resultsRepositoryInstance.closeLoadUpload(analysisId, experimentId, 
								bid, rid, false);
						logInfo("No load info to upload for " 
								+ Database.getRunReferenceString(analysisId, experimentId, bid, rid));
					}

				}
				tablesReader.close();
				
			}
		} catch (IOException e) {
			e.printStackTrace();
			throw new TaskException("I/O error during results upload", e);
		}  catch (ResultsRepositoryBusyException e) {
			throw new TaskException("Results Repository overloaded (too many network connections)", e);
		}
		
		logInfo("Results, logs and load uploaded.");

	}

	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{AID_PROPERTY, EID_PROPERTY, RESULTS_PATHS_PROPERTY});
	}
	
}
