/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.results.xampler;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.List;

import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;
import cz.cuni.mff.been.task.results.CheckAndConvertTask;

/**
 * Task to check and convert xampler results
 * 
 * @author Jakub Lehotsky
 *
 */
public class CheckAndConvertXamplerTask extends CheckAndConvertTask {
	

	private enum XamplerSuite {
		Concurrency_Client_Connections("Concurrency_Client_Connections", new String[] { 
				"Value" }),
		Dispatch_Servant_Instances("Dispatch_Servant_Instances", new String[] { 
				"Value" }),
		Dispatch_Servant_Lifecycle("Dispatch_Servant_Lifecycle", new String[] { 
				"CreateInvoke", "CreateReturn", "CreateFinish", "DeleteInvoke", 
				"DeleteReturn", "DeleteFinish", "Value" }),
		Invocation_Dynamic("Invocation_Dynamic", new String[] { 
		        "Halfway", "Value" }),
		Invocation_Initial("Invocation_Initial", new String[] { 
	            "RoundtripFirst", "RoundtripSecond", "RoundtripThird", "Value" }),
		Invocation_Static("Invocation_Static", new String[] {
	            "Halfway", "Value" }),
		Ping("Ping", new String[] { 
				"Value" }),
		Void("Void", new String[] { 
				"Halfway", "Value" }),
		Marshal_Types_Array("Marshal_Types_Array",	new String[] { 
				"Value" }),
		Marshal_Types_Single("Marshal_Types_Single",  new String[]{ 
				"Value" }),				
		Marshal_Octet_Sequence("Marshal_Octet_Sequence",  new String[]{ 
				"Value" }),				
		Marshal_Delayed_Sequence("Marshal_Delayed_Sequence",  new String[]{ 
				"Value" }),				
		Marshal_Delayed_Any("Marshal_Delayed_Any",  new String[]{ 
				"Value" });
		private String name;
		private String[] columnNames;
		
		XamplerSuite(String suiteName, String[] columnNames) {
			this.name = suiteName;
			this.columnNames = columnNames;
		}

		public String getName() {
			return name;
		}
						
		public String[] getColumnNames() {
			return columnNames;
		}				
	}
	
	/** The only xampler role identification */
	public static final String ROLE_CLIENT = "client";
	
	/** Xampler results filename */
	private static final String RESULTS_FILENAME = "client.out";
		
	/** Task name */
	public static final String TASK_NAME = "check-and-convert-xampler";

	/**
	 * Xampler subsuite - set only when suite has subsuites
	 * e.g. for suite Marshal_Octet_Sequence one of IN, OUT, INOUT, RETURN
	 */
	public static final String XAMPLER_SUBSUITE = "xampler.subsuite";

	private static final Object INVALID_VALUE = -1;
	
	/**
	 * Allocates a new <code>XamplerResultsCollectionTask</code> object. It is here
	 * only to shut up the compiler.
	 * 
	 * @throws TaskInitializationException if something fails during the task
	 *          creation
	 **/
	public CheckAndConvertXamplerTask() throws TaskInitializationException {
		super();
	}	
				
	/* ********* Results checking and converting functions (the core of the task) ********* */
	
	/**
	 * Converts ascii data format of one run into netcdf format, 
	 * making it ready to upload to Results Repository
	 * 
	 * @param sourceName	Source filename (ASCII)
	 * @param destinatioName	Destination filename (NetCDF)
	 * @param resultsRole	The results role of the machine in benchmark 
	 * @param suiteName 	Identification of used suite
	 * @param bid		Binary Id 
	 * @param rid		Run Id
	 * @param samples 	Count of samples for each table 
	 * @return 	True if sucesful
	 * @throws IOException
	 * @throws TaskException 
	 */
	public boolean checkAndConvertOneRun(BID bid, RID rid, String sourceName, String destinatioName, String resultsRole, 
			String suiteName, long samples) throws  IOException, TaskException {
		
		int counter;
//		int value;
		File file = new File(sourceName);
		String line;
		String[] lineTokens;
		String[] suiteColumns = null;
	
		for (XamplerSuite candidate : XamplerSuite.values()) {
			if (candidate.getName().equalsIgnoreCase(suiteName)) {
				
				break;
			}
		}
		
		for (XamplerSuite candidate : XamplerSuite.values()) {
			try {
				if (candidate.getName().equals(suiteName.substring(0, candidate.getName().length()))) {				
					suiteColumns = candidate.getColumnNames();
				}
			} catch (StringIndexOutOfBoundsException e) {
				//note: empty catch -> if that string is larger, nothing matters, they just don't fit
			}
		}
		
		if (suiteColumns == null) {
			
			logError("Suite not identified!");
			notifyTableCreation(0, 1, bid, rid, false);
			return false;
			
		}
		
		BufferedReader inputReader = new BufferedReader(new FileReader(file + File.separator 
				+ RESULTS_FILENAME));
		PrintWriter outputWriter;
		PrintWriter resultsWriter;
				
		resultsWriter = new PrintWriter(new BufferedWriter(new FileWriter(destinatioName + "-0")));
		
		int columnCount = suiteColumns.length;
		
		if (file.exists()) {
			try {										
				counter = 0;			
				while ((line = inputReader.readLine()) != null) {
					counter++;
					lineTokens = line.split(ASCII_TAB);					
						try {

							if (lineTokens.length < columnCount) {
								if (counter == 1) {
									columnCount = lineTokens.length;
								} else {
									logError("results converting error - invalid column count for given suite");
									notifyTableCreation(0, 1, bid, rid, false);
									return false;
								}
							}
							
							for (int i = 0; i < columnCount; i++) {				
								try {
									resultsWriter.print(Long.valueOf(lineTokens[i]));
								} catch (NumberFormatException e) {							
									resultsWriter.print(INVALID_VALUE);
								}
								
								if (i < columnCount) { 
									resultsWriter.print("\t");
								}
							}
							
							resultsWriter.println("");
						
						} catch (NullPointerException e) {
							logError("results checking: null pointer exception when trying to read sample" + counter);
							notifyTableCreation(0, 1, bid, rid, false);
							return false; // Run is corrupted
						} catch (ArrayIndexOutOfBoundsException e) {							
							logInfo("invalid item format - sample " + counter);
							notifyTableCreation(0, 1, bid, rid, false);
							return false; // Run is corrupted
						}
				}
				
			} catch (IOException e) {
				e.printStackTrace();
				logError("io error converting " + sourceName);
				notifyTableCreation(0, 1, bid, rid, false);
				return false; // Run is corrupted
			} finally {
				inputReader.close();
				resultsWriter.close();
			}
			
			outputWriter = new PrintWriter(new BufferedWriter(new FileWriter(destinatioName + "-0.meta")));
			outputWriter.println(suiteName);
			
			for (int i = 0; i < columnCount; i++) {
				outputWriter.println(suiteColumns[i]);
			}
			
			outputWriter.close();
			
			logInfo(counter + " samples from " + samples + " were read when converting.");
			
			if (counter != samples) {
				notifyTableCreation(0, 1, bid, rid, false);
				return false;
			}
			
			notifyTableCreation(0, 1, bid, rid, true);
			return true;
			
		} else { // file doesn't exist
			notifyTableCreation(0, 1, bid, rid, false);
			return false;
		}		
	}
	
	public List<String> getCrashList(String srcName) {
		return Arrays.asList(new String[] { srcName + File.separator + "client.out", 
				srcName + File.separator + "client.err"});		
	}
}
