/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.rubis.buildprepare;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.regex.PatternSyntaxException;

import cz.cuni.mff.been.common.RegexSubstitute;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Prepares the Rubis' source code for compilation. It sets the correct values
 * of properties in the build.properties file and edits the required source files.
 * </p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #RUBIS_ROOT}: <br>
 * 		- path to the Rubis source code <br>
 * 		- mandatory <br>
 * <li>{@value #EJB_SERVER}: <br>
 * 		- which EJB server will be used by Rubis. Valid values are "jboss"
 * 		and "jonas"<br>
 * 		- mandatory <br>
 * <li>{@value #J2EE_ROOT}: <br>
 * 		- path to a directory that contains J2EE API jar files <br>
 * 		- mandatory <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class RubisBuildPrepare extends Job {
	//------------ CONSTANTS ---------------
	/** Task property name for the root directory of the Rubis source code. */
	public static final String RUBIS_ROOT = "rubis.root";
	/** Task property name for the name of the EJB server. Valid values are
	 * <code>jboss</code> and <code>jonas</code>.
	 */
	public static final String EJB_SERVER = "ejb.server";
	/** Task property name for the directory with J2EE API jar files.*/
	public static final String J2EE_ROOT = "j2ee";

	/** Name of the property for EJB server in the build.properties file */
	private static final String EJB_SERVER_PROPERTY = "ejb.server";
	/** Name of the property for J2EE path in the build.properties file */
	private static final String J2EE_ROOT_PROPERTY = "j2ee";
	/** 
	 * Name of the property for the directory with web interface in the 
	 * build.properties file 
	 */
	private static final String WEB_DIR_PROPERTY = "web.dir";
	
	// ------------ ATTRIBUTES -------------
	
	/** Path to the root directory of the Rubis source code. */
	private String rubisRoot;
	/** Name of the EJB server. Valid values are <code>jboss</code> and
	 * <code>jonas</code>.
	 */
	private String ejbServer;
	/** Path to the root directory of a J2EE installation. */
	private String j2eeRoot;
	
	/**
	 * Allocates a new <code>RubisBuildPrepare</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public RubisBuildPrepare() throws TaskInitializationException {
		super();
	}

	@Override
	protected void run() throws TaskException {
		rubisRoot = getTaskProperty(RUBIS_ROOT);
		logInfo("Rubis root: " + rubisRoot);
		
		ejbServer = getTaskProperty(EJB_SERVER);
		logInfo("EJB server: " + ejbServer);
		
		j2eeRoot = getTaskProperty(J2EE_ROOT);
		logInfo("J2EE path: " + j2eeRoot);

		Properties buildProperties = new Properties();
		
		// load the build.properties file
		String path = rubisRoot + File.separator + "build.properties";
		File buildPropertiesFile = new File(path);
		try {
			FileInputStream is = new FileInputStream(buildPropertiesFile);
			buildProperties.load(is);
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while reading file: " + path, e);
		}
		
		// set the properties
		buildProperties.setProperty(EJB_SERVER_PROPERTY, ejbServer);
		buildProperties.setProperty(J2EE_ROOT_PROPERTY, j2eeRoot);
		buildProperties.setProperty(WEB_DIR_PROPERTY, "../ejb_rubis_web");
		
		// save to the build.properties file
		try {
			FileOutputStream os = new FileOutputStream(buildPropertiesFile);
			buildProperties.store(os, "This file is generated by BEEN");
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while writing to file: " + path, e);
		}
		
		// edit the servlets config files in every Rubis suite
		
		String fs = File.separator;
		
		editConfigFile(rubisRoot + fs + "EJB_CMP2.0" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJBoss.java");
		editConfigFile(rubisRoot + fs + "EJB_CMP2.0" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJOnAS.java");
		
		editConfigFile(rubisRoot + fs + "EJB_EntityBean_id" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJBoss.java");
		editConfigFile(rubisRoot + fs + "EJB_EntityBean_id" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJOnAS.java");
		
		editConfigFile(rubisRoot + fs + "EJB_EntityBean_id_BMP" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJBoss.java");
		editConfigFile(rubisRoot + fs + "EJB_EntityBean_id_BMP" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJOnAS.java");
		
		editConfigFile(rubisRoot + fs + "EJB_local_remote" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJBoss.java");
		editConfigFile(rubisRoot + fs + "EJB_local_remote" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJOnAS.java");
		
		editConfigFile(rubisRoot + fs + "EJB_Session_facade" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJBoss.java");
		editConfigFile(rubisRoot + fs + "EJB_Session_facade" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJOnAS.java");
		
		editConfigFile(rubisRoot + fs + "EJB_SessionBean" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJBoss.java");
		editConfigFile(rubisRoot + fs + "EJB_SessionBean" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJOnAS.java");
		
		editConfigFile(rubisRoot + fs + "MDB" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJBoss.java");
		editConfigFile(rubisRoot + fs + "MDB" + fs + "edu" + fs + "rice" + fs + 
				"rubis" + fs + "beans" + fs + "servlets" + fs +
				"ConfigJOnAS.java");
	}
	
	private void editConfigFile(String path) throws TaskException {
		try {
			// must escape backslashes
			String substitution = (rubisRoot + File.separator + "ejb_rubis_web").replaceAll("\\\\", "\\\\\\\\");
			RegexSubstitute.substitute(path, 
					"/home/margueri/workspace/RUBiS/ejb_rubis_web", 
					substitution);
		} catch (IOException e) {
			throw new TaskException("IO error while editing the file " + path, e);
		} catch (PatternSyntaxException e) {
			// should never happen
			assert true: "Error in the regular expression";
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{RUBIS_ROOT,EJB_SERVER,J2EE_ROOT});
	}
}
