/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.rubis.mysql.backup;

import java.io.File;

import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Copy;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Backups the Rubis' database in MySQL to it's working
 * directory. The backup can later be used to quickly restore the default
 * state of the Rubis' database.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #MYSQL_ROOT}: <br>
 * 		- path to the MySQL installation directory <br>
 * 		- mandatory <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class RubisMySQLBackup extends Job {
	/** Task property name for the root directory of MySQL */
	public static final String MYSQL_ROOT = "mysql.root";

	/** Root directory of MySQL */
	private String mysqlRoot;
	
	/**
	 * 
	 * Allocates a new <code>RubisMySQLBackup</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public RubisMySQLBackup() throws TaskInitializationException {
		super();
	}

	@Override
	protected void run() throws TaskException {
		if ((mysqlRoot = getTaskProperty(MYSQL_ROOT)) == null) {
			throw new TaskException(MYSQL_ROOT + " task property not set");
		}
		
		logInfo("MySQL installation root directory: " + mysqlRoot);

		try {
			Copy.directoryCopy(
					mysqlRoot + File.separator + "data" + File.separator + "rubis",
					getWorkingDirectory() + File.separator + "rubis");
			Copy.copyToDir(mysqlRoot + File.separator + "data" + File.separator + "ib_logfile0", 
					getWorkingDirectory());
			Copy.copyToDir(mysqlRoot + File.separator + "data" + File.separator + "ib_logfile1", 
					getWorkingDirectory());
			Copy.copyToDir(mysqlRoot + File.separator + "data" + File.separator + "ibdata1", 
					getWorkingDirectory());
		} catch (AntTaskException e) {
			throw new TaskException("Cannot copy the Rubis' database to my working directory", e); 
		}

		logInfo("Rubis database backup successfull");
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(MYSQL_ROOT);
	}

}
