/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.rubis.mysql.initialize;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Initializes the MySQL database for usage with Rubis. It creates
 * the needed databases and tables, and fills them with data from 
 * a MySQL dump.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #MYSQL_ROOT}: <br>
 * 		- path to the MySQL installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #DUMP_ROOT}: <br>
 * 		- path to the directory with the MySQL database dump of the Rubis 
 * 		database<br>
 * 		- mandatory <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class RubisMySQLInitialize extends Job {
	//---------------- CONSTANTS ---------------
	
	/** Task property name for the root directory of the MySQL installation */
	public static final String MYSQL_ROOT = "mysql.root";
	/** Task property name for the root directory of the MySQL dump */
	public static final String DUMP_ROOT = "dump.root";
	
	/** Name of the database that will be created for Rubis */
	private static final String DATABASE_NAME = "rubis";
	/** Size of the buffer for piping a file as input for the mysql command */
	private static final int BUFFER_SIZE = 1024;

	//---------------- ATTRIBUTES --------------
	
	/** Root directory of the MySQL installation */
	private String mysqlRoot;
	/** Root directory of the MySQL dump */
	private String dumpRoot;


	/**
	 * Allocates a new <code>RubisMySQLInitialize</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public RubisMySQLInitialize() throws TaskInitializationException {
		super();
	}
	
	private void createDatabase() throws TaskException {
		String fSep = File.separator;
		
		// create the database for Rubis
		logInfo("Creating database");
		try {
			String[] cmdArray = { mysqlRoot + fSep + "bin" + fSep + "mysql" ,
					"-u", "root",
					"-e",  "CREATE DATABASE " + DATABASE_NAME};

			logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));

			Process p = Runtime.getRuntime().exec(cmdArray, null, new File(mysqlRoot));
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(p.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(p.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			// wait untill the script finishes
			if (p.waitFor() != 0) {
				throw new TaskException("Database creation failed, see the standard "
						+ "output of this task.");
			}
		} catch (Exception e) {
			throw new TaskException("Cannot create the \""
					+ DATABASE_NAME + "\" database", 
					e);
		}
	
		logInfo("Database created: " + DATABASE_NAME);
	}
	
	private void createTables() throws TaskException {
		String fSep = File.separator;
		
		// create tables in the database
		logInfo("Creating tables in the database: " + DATABASE_NAME);
		try {
			String[] cmdArray = { mysqlRoot + fSep + "bin" + fSep + "mysql" ,
					"-u", "root",
					DATABASE_NAME};

			logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));
			logDebug("Piping file as input for mysql: " + dumpRoot + fSep
					+ "create_tables.sql");

			Process p = Runtime.getRuntime().exec(cmdArray, null, new File(mysqlRoot));
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(p.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(p.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			
			// pipe a file as input to the mysql process, the input
			// creates the needed tables for rubis
			
			InputStream in = new FileInputStream(dumpRoot + fSep
					+ "create_tables.sql");
			BufferedReader reader = new BufferedReader(
					new InputStreamReader(in),
					BUFFER_SIZE);
			OutputStream out = p.getOutputStream();
			BufferedWriter writer = new BufferedWriter(
					new OutputStreamWriter(out), 
					BUFFER_SIZE);

			char[] buf = new char[BUFFER_SIZE];
			int read = 0;
			try {
				while ((read = reader.read(buf)) != -1) {
					writer.write(buf, 0, read);
				}
				reader.close();
				writer.close();
			} catch (IOException e) {
				throw new TaskException("Cannot pipe file as input for the mysql command",
						e);
			}
			
			// wait untill the script finishes
			if (p.waitFor() != 0) {
				throw new Exception("Failed creating tables in the database,"
						+ " see the standard output of this task.");
			}
		} catch (Exception e) {
			throw new TaskException("Cannot create tables in the database", e);
		}
	
		logInfo("Created tables in the database: " + DATABASE_NAME);
	}
	
	private void fillDatabase() throws TaskException {
		String fSep = File.separator;
		
		// filling database with data
		logInfo("Filling database: " + DATABASE_NAME);
		
		String[] files = {
				"bids.txt",
				"buy_now.txt",
				"categories.txt",
				"comments.txt",
				"ids.txt",
				"items.txt",
				"old_items.txt",
				"regions.txt",
				"users.txt",
			};
		
		// fill tables in a cycle, because putting all those filenames in
		// one command might exceed the length of the commad line
		for (int i = 0; i < files.length; i++) {
			try {
				String[] cmdArray = { mysqlRoot + fSep + "bin" + fSep + "mysqlimport" ,
						"-u", "root",
						"--local",
						"--fields-terminated-by=,",
						"--fields-optionally-enclosed-by='",
						"--fields-escaped-by=\\",
						DATABASE_NAME,
						dumpRoot + fSep + files[i],
				};
				
				logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));
				
				Process p = Runtime.getRuntime().exec(cmdArray, null, new File(mysqlRoot));
				
				/* Read in the output from the subprocess */
				OutputReader stdoutReader = 
					new OutputReader(p.getInputStream(), OutputType.STDOUT);
				OutputReader stderrReader = 
					new OutputReader(p.getErrorStream(), OutputType.STDERR);
				stdoutReader.start();
				stderrReader.start();
				
				// wait untill the script finishes
				if (p.waitFor() != 0) {
					throw new Exception("Failed filling the database, see the standard "
							+ "output of this task.");
				}
			} catch (Exception e) {
				throw new TaskException("Cannot fill the \"" + DATABASE_NAME + "\" database", 
						e);
			}
		}
	
		logInfo("Database filled: " + DATABASE_NAME);
	}
	
	private void updateOldItems() throws TaskException {
		String fSep = File.separator;
		
		logInfo("Updating old items");
		try {
			String[] cmdArray = { mysqlRoot + fSep + "bin" + fSep + "mysql" ,
					"-u", "root",
					DATABASE_NAME,
					"-e",  "INSERT INTO old_items SELECT * FROM items WHERE"
					+ " end_date < \"2001-10-18 16:17:00\"; "
					+ "DELETE FROM items WHERE end_date < \"2001-10-18 16:17:00\""};

			logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));

			Process p = Runtime.getRuntime().exec(cmdArray, null, new File(mysqlRoot));
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(p.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(p.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			// wait untill the script finishes
			if (p.waitFor() != 0) {
				throw new Exception("Old items updating failed, see the standard "
						+ "output of this task.");
			}
		} catch (Exception e) {
			throw new TaskException("Cannot update old items in the database",
					e);
		}
		
		logInfo("Updated old items");
	}
	
	@Override
	protected void run() throws TaskException {
		mysqlRoot = getTaskProperty(MYSQL_ROOT);
		logInfo("MySQL installation root directory: " + mysqlRoot);
		
		dumpRoot = getTaskProperty(DUMP_ROOT);
		logInfo("MySQL dump root directory: " + dumpRoot);

		createDatabase();
		createTables();
		fillDatabase();
		updateOldItems();
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{MYSQL_ROOT, DUMP_ROOT});
	}
}
