/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.rubis.mysql.restore;

import java.io.File;

import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Copy;
import cz.cuni.mff.been.common.anttasks.Delete;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Restores the Rubis' database in MySQL from a backup.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #MYSQL_ROOT}: <br>
 * 		- path to the MySQL installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #BACKUP_PATH}: <br>
 * 		- path to the directory with the backup of the Rubis' MySQL database<br>
 * 		- mandatory <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class RubisMySQLRestore extends Job {
	/** Task property name for the path to the backup. */
	public static final String BACKUP_PATH = "backup.path";
	/** Task property name for the root directory of MySQL */
	public static final String MYSQL_ROOT = "mysql.root";


	/** Path to the backup */
	private String backupPath;
	/** Root directory of MySQL */
	private String mysqlRoot;

	/**
	 * Allocates a new <code>RubisMySQLRestore</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public RubisMySQLRestore() throws TaskInitializationException {
		super();
	}

	@Override
	protected void run() throws TaskException {
		backupPath = getTaskProperty(BACKUP_PATH);
		logInfo("Path to the backup of Rubis' database: " + backupPath);
		
		mysqlRoot = getTaskProperty(MYSQL_ROOT);
		logInfo("MySQL installation root directory: " + mysqlRoot);

		// delete the old Rubis' database
		try {
			Delete.deleteDirectory(mysqlRoot + File.separator + "data"
					+ File.separator + "rubis");
		} catch (AntTaskException e) {
			throw new TaskException("Cannot delete the old Rubis' database", e);
		}
		logInfo("Succesfully deleted the old Rubis' database");
		
		// copy the backup of the database in the MySQL directory
		try {
			Copy.directoryCopy(backupPath + File.separator + "rubis", 
					mysqlRoot + File.separator + "data" + File.separator + "rubis");
			Copy.copyToDir(backupPath + File.separator + "ib_logfile0", 
					mysqlRoot + File.separator + "data");
			Copy.copyToDir(backupPath + File.separator + "ib_logfile1", 
					mysqlRoot + File.separator + "data");
			Copy.copyToDir(backupPath + File.separator + "ibdata1", 
					mysqlRoot + File.separator + "data");
		} catch (AntTaskException e) {
			throw new TaskException("Cannot copy the backup of the database to the MySQL directory", e);
		}
		
		logInfo("Database successfully restored");
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{MYSQL_ROOT, BACKUP_PATH});
	}
}
