/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.svn.checkout;

import java.io.File;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;

import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.SVNURL;
import org.tmatesoft.svn.core.auth.ISVNAuthenticationManager;
import org.tmatesoft.svn.core.internal.io.dav.DAVRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.svn.SVNRepositoryFactoryImpl;
import org.tmatesoft.svn.core.wc.SVNClientManager;
import org.tmatesoft.svn.core.wc.SVNRevision;
import org.tmatesoft.svn.core.wc.SVNUpdateClient;
import org.tmatesoft.svn.core.wc.SVNWCUtil;

import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Performs a checkout from a Subversion repository. If no revision is
 * specified, then it will checkout the HEAD.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #URL}: <br>
 * 		- URL of the SVN repository <br>
 * 		- mandatory <br>
 * <li>{@value #REVISION_NUMBER}: <br>
 * 		- if this task property is set, then the revision number in it's value
 * 		will be checked out from the SVN repository<br>
 * <li>{@value #REVISION_TIME}: <br>
 * 		- if this task property is set, then the revision from the time specified
 * 		in it's value will be checked out from the SVN repository. The time
 * 		must be specified in milliseconds from epoch. <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class SVNCheckout extends Job {
	/** Task property name for the URL of the SVN repository. */
	public static final String URL = "url";
	/** Task property name for revision number. */
	public static final String REVISION_NUMBER = "revision.number";
	/** Task property name for revision date. */
	public static final String REVISION_TIME = "revision.time";


	/** URL to checkout. */
	private String url;
	/** <tt>true</tt> if HEAD revision should be checked out.*/
	private boolean revisionHead = false;
	/** Revision number to checkout. */
	private long revisionNumber = -1;
	/** Revision date to checkout. */
	private Date revisionDate;


	/**
	 * Allocates a new <code>SVNCheckout</code> object.
	 * 
	 * @throws TaskInitializationException
	 */
	public SVNCheckout() throws TaskInitializationException {
		super();
	}


	@Override
	protected void checkRequiredProperties() throws TaskException {
	}
	
	/**
	 * Performs checkout from SVN.
	 * 
	 * @throws TaskException
	 */
	private void checkout() throws TaskException {
		// initialize the JavaSVN library to work with http:// and https:// 
		DAVRepositoryFactory.setup();
		// initialize the JavaSVN library to work with svn:// and svn+ssh://
		SVNRepositoryFactoryImpl.setup();
		
		SVNClientManager clientManager = SVNClientManager.newInstance();
		SVNUpdateClient client = clientManager.getUpdateClient();
		
		SVNRevision revision = null;
		if (revisionHead) {
			revision = SVNRevision.HEAD;
		}
		if (revisionDate != null) {
			revision = SVNRevision.create(revisionDate);
		}
		if (revisionNumber != -1) {
			revision = SVNRevision.create(revisionNumber);
		}
		
		try {
			client.doCheckout(SVNURL.parseURIDecoded(url), 
				new File(getWorkingDirectory()), 
				revision, 
				revision, 
				true);
		} catch (SVNException e) {
			throw new TaskException("Cannot checkout: " + e.getMessage(), e);
		}
	}
	
	/**
	 * Parses the task properties.
	 * 
	 * @throws TaskException
	 */
	private void parseTaskProperties() throws TaskException {
		if ((url = getTaskProperty(URL)) == null) {
			throw new TaskException(URL + " task priperty not set");
		}
		logInfo("URL: " + url);
		String date;
		if ((date = getTaskProperty(REVISION_TIME)) != null) {
			revisionDate = new Date(Long.valueOf(date));
			DateFormat formatter = DateFormat.getDateTimeInstance();
			logInfo("Using revision date: " + formatter.format(revisionDate) 
					+ " timestamp: " + date);
		}
		
		String number;
		if ((number = getTaskProperty(REVISION_NUMBER)) != null) {
			if (date != null) {
				throw new TaskException(REVISION_TIME + " and " 
						+ REVISION_NUMBER + " task properties specified at the " +
								"same time");
			}
			
			revisionNumber = Integer.valueOf(number);
			logInfo("Using revision number: " + revisionNumber);
		}
		
		if ((date == null) && (number == null)) {
			revisionHead = true;
			logInfo("Using HEAD revision");
		}
	}


	@Override
	protected void run() throws TaskException {
		parseTaskProperties();
		logInfo("Checking out from SVN");
		checkout();
	}
}
