/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Antonin Tomecek
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.taskmanager;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;
import java.rmi.Naming;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Delete;
import cz.cuni.mff.been.hostmanager.HostManagerException;
import cz.cuni.mff.been.hostmanager.HostManagerInterface;
import cz.cuni.mff.been.hostmanager.HostManagerService;
import cz.cuni.mff.been.hostmanager.database.HostInfoInterface;
import cz.cuni.mff.been.hostmanager.database.RSLRestriction;
import cz.cuni.mff.been.hostmanager.database.RestrictionInterface;
import cz.cuni.mff.been.hostruntime.HostRuntimeException;
import cz.cuni.mff.been.hostruntime.HostRuntimeInterface;
import cz.cuni.mff.been.hostruntime.TaskInterface;
import cz.cuni.mff.been.logging.FilesystemLogStorage;
import cz.cuni.mff.been.logging.LogLevel;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorage;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.logging.OutputHandle;
import cz.cuni.mff.been.softwarerepository.MatchException;
import cz.cuni.mff.been.softwarerepository.PackageMetadata;
import cz.cuni.mff.been.softwarerepository.RSLPackageQueryCallback;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryService;
import cz.cuni.mff.been.task.Service;
import cz.cuni.mff.been.taskmanager.data.CheckPointEntry;
import cz.cuni.mff.been.taskmanager.data.ContextEntry;
import cz.cuni.mff.been.taskmanager.data.Data;
import cz.cuni.mff.been.taskmanager.data.HostRuntimeEntry;
import cz.cuni.mff.been.taskmanager.data.TaskData;
import cz.cuni.mff.been.taskmanager.data.TaskEntry;
import cz.cuni.mff.been.taskmanager.data.TaskExclusivity;
import cz.cuni.mff.been.taskmanager.data.TaskState;


/**
 * Implementation of Task Manager (main class).
 * 
 * @author Antonin Tomecek
 */
public class TaskManagerImplementation extends UnicastRemoteObject
implements TaskManagerInterface, HostRuntimesPortInterface {
	
	private static final String RESCUE_DIR_NAME = "rescue";
	
	/* Path where TM's configuration file should be stored.
	 * Initialized by constructor. */
	private final File configurationFile;
	
	/* Object storing data of Task Manager. */
	private Data data;
	
	/* Path to the root directory for Task Manager's files. */
	private String rootDirectory = null;
	
	/* Log storage component */
	private LogStorage logStorage = null;
	
	/* Log level of the Task Manager. */
	private LogLevel logLevel;
	
	private LinkedList<ServiceEntry> serviceRegistry
		= new LinkedList<ServiceEntry>();
	
	private CheckPoint checkPointWaitingObject
		= new CheckPoint(null, null, null, null);

	/**
	 * Size limit of the Host Runtime's package cache.
	 */
	private long maxPackageCacheSize = DEFAULT_MAX_PACKAGE_CACHE_SIZE;
	
	/**
	 * Number of closed contexts, for which the Host Runtime should keep data on
	 * the disk.
	 */
	private int keptClosedContextCount = DEFAULT_KEPT_CLOSED_CONTEXT_COUNT;
	
	/**
	 * Listeners on Host Runtime registrations and unregistration events.
	 */
	private ArrayList<HostRuntimeRegistrationListener> registrationListeners;
	
	/** Random number generator */
	private static final Random random = new Random(System.currentTimeMillis());
	
	/**
	 * Delete file or directory (recursively) specified by <code>file</code>.
	 * 
	 * @param file File or directory to be removed.
	 * @throws NullPointerException If input parameter is <code>null</code>.
	 * @throws IllegalArgumentException If file is unknown thing...
	 */
	private static void deleteRecursive(File file) {
		/* Check input parameters. */
		if (file == null) {
			throw new NullPointerException("directory is null");
		}
		
		/* Delete (recursively). */
		if (file.isFile()) {
			file.delete();
		} else if (file.isDirectory()) {
			File[] items = file.listFiles();
			for (File item : items) {
				deleteRecursive(item);
			}
			file.delete();
		}
	}
	
	/**
	 * Store configuration to configuration file.
	 * 
	 * @param conigurationFile XML file for storing configuration.
	 */
	private void storeConfiguration(File configurationFile) {
		Document document;
		try {
			document = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new RuntimeException("Could not create Document for XML. "
					+ "This should not occur.", e);
		}
		
		/* Add root element. */
		Element configurationElement = document.createElement("configuration");
		document.appendChild(configurationElement);
		
		/* Add hostRuntimes element. */
		Element hostRuntimesElement
			= document.createElement("hostRuntimes");
		configurationElement.appendChild(hostRuntimesElement);
		/* ... set attributes for hostRuntimes element. */
		hostRuntimesElement.setAttribute("maxPackageCacheSize",
				String.valueOf(this.maxPackageCacheSize));
		hostRuntimesElement.setAttribute("keptClosedContextCount",
				String.valueOf(this.keptClosedContextCount));
		
		/* Store to file. */
		Transformer transformer;
		try {
			transformer = TransformerFactory.newInstance()
				.newTransformer();
		} catch (TransformerConfigurationException e) {
			throw new RuntimeException("Could not create Transformer for XML. "
					+ "This should not occur.", e);
		}
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		transformer.setOutputProperty(
				OutputKeys.DOCTYPE_SYSTEM, "configuration.dtd");
		try {
			transformer.transform(new DOMSource(document),
					new StreamResult(configurationFile));
		} catch (TransformerException e) {
			System.err.println("Could not store configuration of Task "
					+ "Manager to XML file.");
		}
	}
	
	/**
	 * Load configuration from configuration file.
	 * 
	 * @param configurationFile XML file for loading configuration.
	 * @throws IllegalArgumentException If <code>configurationFile</code> could
	 * 	not be parsed for some reason.
	 */
	private void loadConfiguration(File configurationFile) {
		if (!configurationFile.exists() || !configurationFile.isFile()) {
			throw new IllegalArgumentException("Configuration file (\""
					+ configurationFile.getPath() + "\") not found");
		}
		
		DocumentBuilderFactory documentBuilderFactory
			= DocumentBuilderFactory.newInstance();
		documentBuilderFactory.setValidating(true);
		
		Document document;
		try {
			document = documentBuilderFactory
				.newDocumentBuilder().parse(configurationFile);
		} catch (SAXException e) {
			throw new IllegalArgumentException("Parse error occured.", e);
		} catch (IOException e) {
			throw new IllegalArgumentException("IO error occurred.", e);
		} catch (ParserConfigurationException e) {
			throw new IllegalArgumentException("DocumentBuilder can not be "
					+ "created", e);
		}
		
		/* Process element configuration... */
		Element configurationElement = document.getDocumentElement();
		if (!configurationElement.getTagName().equals("configuration")) {
			logWarning("Unknown format of configuration file "
					+ "(will not be loaded).");
		}
		
		/* Process element hostRuntimes... */
		Element hostRuntimesElement = (Element)
			configurationElement.getElementsByTagName("hostRuntimes").item(0);
		/* ... attribute maxPackageCacheSize. */
		String attributeMaxPackageCacheSize
			= hostRuntimesElement.getAttribute("maxPackageCacheSize");
		if (!attributeMaxPackageCacheSize.equals("")) {
			this.maxPackageCacheSize
				= Long.parseLong(attributeMaxPackageCacheSize);
		}
		/* ... attribute keptClosedContextCount. */
		String attributeKeptClosedContextCount
			= hostRuntimesElement.getAttribute("keptClosedContextCount");
		if (!attributeKeptClosedContextCount.equals("")) {
			this.keptClosedContextCount
				= Integer.parseInt(attributeKeptClosedContextCount);
		}
	}
	
	/**
	 * This method should be called after <code>Data</code> object is rescued.
	 * It fills (updates) remote interfaces for tasks.
	 */
	private void reinitializeDataWhenRescue() {
		HostRuntimeEntry[] hostRuntimes = this.data.getHostRuntimes();
		
		for (HostRuntimeEntry hostRuntime : hostRuntimes) {
			/* Construct URI of Host Runtime control interface. */
			URI hostRuntimeUri;
			try {
				hostRuntimeUri = new URI("rmi", null, hostRuntime.getHostName(),
						Registry.REGISTRY_PORT,
						HostRuntimeInterface.URL, null, null);
			} catch (URISyntaxException e) {
				System.err.println("Could not construct hierarchical URI from "
						+ "the given components (hostName \""
						+ hostRuntime.getHostName() + "\"):" + e.getMessage());
				continue;
			}
			
			/* Obtain RMI reference to Host Runtime. */
			HostRuntimeInterface hostRuntimeInterface;
			try {
				hostRuntimeInterface = (HostRuntimeInterface)
				Naming.lookup(hostRuntimeUri.toString());
			} catch (Exception e) {
				System.err.println("Could not get RMI reference for host "
						+ "runtime (URI \"" + hostRuntimeUri.toString()
						+ "\")");
				
				/* Set all tasks of that hostRuntime to ABORTED and remove
				 * hostRuntime */
				TaskEntry[] tasks
					= this.data.getTasksOnHost(hostRuntime.getHostName());
				for (TaskEntry task : tasks) {
					this.data.changeTaskState(
							task.getTaskId(), task.getContextId(),
							TaskState.ABORTED);
				}
				this.data.removeHostRuntime(hostRuntime);
				
				continue;
			}
			
			/* Get all RMI interfaces to tasks from each Host Runtime and set
			 * Task Manager's data. */
			try {
				TaskInterface[] taskInterfaces;
				taskInterfaces = hostRuntimeInterface.getRunningTaskInterfaces();
				for (TaskInterface taskInterface : taskInterfaces) {
					String taskId = taskInterface.getTaskID();
					String contextId = taskInterface.getContextID();
					/* Set TaskInterface in data. */
					try {
						this.data.setTaskInterface(taskId, contextId,
								taskInterface);
					} catch (Exception e) {
						System.err.println("Problem when setting "
								+ "taskInterface: " + e.getMessage());
					}
				}
			} catch (RemoteException e) {
				System.out.println("Could not get interfaces of running tasks "
						+ "on hostRuntime (URI \"" + hostRuntimeUri.toString()
						+ "\"): " + e.getMessage());
			}
		}
		
	}
	
	/**
	 * Creates a new Task Manager instance and
	 * handles RemoteException thrown by implicit super constructor.
	 * 
	 * @param rootDirectory Root directory for Task Manager.
	 * @param level Log level of the Task Manager.
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	protected TaskManagerImplementation(String rootDirectory, LogLevel level)
	throws RemoteException {
		super();
		
		/* Is Task Manager initialized from rescue? */
		boolean rescued = false;
		
		this.rootDirectory = rootDirectory;
		this.logLevel = level;
		
		File rescueDirectory = new File(rootDirectory, RESCUE_DIR_NAME);
//		if (rescueDirectory.exists()) {
//			rescued = true;
//			/* Rescue... */
//			System.err.println("Reload to last state...");
//			File rescueDirectoryOld
//				= new File(rootDirectory, RESCUE_DIR_NAME + "_OLD");
//			rescueDirectory.renameTo(rescueDirectoryOld);
//			this.data = Rescue.rescueData(rescueDirectory, rescueDirectoryOld);
//			this.reinitializeDataWhenRescue();
//			deleteRecursive(rescueDirectoryOld);
//			System.err.println("Reloaded.");
//		} else {
			this.data = new Data(rescueDirectory);
//		}
		
		this.registrationListeners = new ArrayList<HostRuntimeRegistrationListener>();
		
		/* Prepare and load configuration from XML file. */
		this.configurationFile = new File(rootDirectory, CONFIGURATION_FILE);
		try {
			this.loadConfiguration(configurationFile);
		} catch (IllegalArgumentException e) {
			System.err.println("Could not load configuration file. New "
					+ "configuration file will be created (with default values "
					+ "set).");
			this.storeConfiguration(configurationFile);
		}
		
		/* Create and initialize the log storage. */
		try {
			String logDirPath = rootDirectory + File.separator + "logs";
			File logDir = new File(logDirPath);
			if (logDir.exists()) {
				Delete.deleteDirectory(logDirPath);
			}
			
			logStorage = new FilesystemLogStorage(rootDirectory + File.separator + "logs"); 
		} catch (LogStorageException e) {
			System.err.println("Cannot create the log storage: " + e.getMessage());
			System.exit(1);
		} catch (AntTaskException e) {
			System.err.println("Cannot delete the log directory: " + e.getMessage());
			System.exit(1);
		}
		
//		if (!rescued) {
			/* Create system context. */
			this.newContext(SYSTEM_CONTEXT_ID,
					SYSTEM_CONTEXT_NAME, SYSTEM_CONTEXT_DESCRIPTION, null);

			try {
				this.logStorage.addTask(SYSTEM_CONTEXT_ID, TASKMANAGER_TASKNAME);
				this.logStorage.setTaskHostname(SYSTEM_CONTEXT_ID, TASKMANAGER_TASKNAME, 
						InetAddress.getLocalHost().getCanonicalHostName());
			} catch (Exception e) {
				logFatal("Cannot store Task Manager's logs: " + e.getMessage());
				System.exit(1);
			}
//		}
		
		/* If the Task Manager shuts down, do some work. */ 
//		Runtime.getRuntime().addShutdownHook(new Thread() {
//			public void run() {
////				deleteRecursive(data.getRescueRootDir());
//			}
//		});
}
	
	/**
	 * End working of Task Manager.
	 */
	public void stopTaskManager() throws RemoteException {
		System.exit(0);
	}
	
	/**
	 * End working of Task Manager and of all known Host Runtimes.
	 */
	public void stopTaskManagerAndHostRuntimes() throws RemoteException {
		HostRuntimeEntry[] hostRuntimes = this.data.getHostRuntimes();
		
		for (HostRuntimeEntry hostRuntime : hostRuntimes) {
			/* Construct URI of Host Runtime control interface. */
			URI hostRuntimeUri;
			try {
				hostRuntimeUri = new URI("rmi", null, hostRuntime.getHostName(),
						Registry.REGISTRY_PORT,
						HostRuntimeInterface.URL, null, null);
			} catch (URISyntaxException e) {
				throw new RemoteException("Could not construct URI of Host "
						+ "Runtime");
			}
			
			/* Obtain RMI reference to Host Runtime. */
			HostRuntimeInterface hostRuntimeInterface;
			try {
				hostRuntimeInterface = (HostRuntimeInterface)
				Naming.lookup(hostRuntimeUri.toString());
			} catch (Exception e) {
				throw new RemoteException("Could not connect to required Host "
						+ "Runtime (URI=\"" + hostRuntimeUri.toString()
						+ "\")");
			}
			
			/* Kill Host Runtime... */
			hostRuntimeInterface.terminate();
		}
		
		System.exit(0);
	}
	
	/**
	 * Selects a Host Runtime for a task.
	 * 
	 * @param task the task.
	 * @param hosts	possible hosts.
	 * @return hostname of the selected Host Runtime.
	 */
	private String selectHostRuntime(TaskData task, String[] hosts) {
		String hostName;
		TaskDescriptor taskDescriptor = task.getTaskDescriptor();
		String otherTask;
		if ((otherTask = taskDescriptor.getHostRuntimesAsTask()) != null) {
			hostName = this.data.getTaskHost(
					otherTask,
					taskDescriptor.getContextId());
		} else {
			String[] hostnames = taskDescriptor.getHostRuntimesName();
			hostName = hosts[random.nextInt(hostnames.length)];
		}
		return hostName;
	}
	
	/**
	 * Test if task is ready to run (exclusivity, dependencies, ...).
	 * 
	 * @param taskEntry <code>TaskEntry</code> of checked task.
	 * @param taskData <code>TaskData</code> of checked task.
	 * @return <code>true</code> if task is ready to run, <code>false</code>
	 * 	otherwise.
	 */
	private boolean isTaskReadyToRun(TaskEntry taskEntry, TaskData taskData) {
		/* Test for reached checkPoints from task's dependencies. */
		if (!this.checkDependencies(taskData)) {
			return false;
		}

		/* Test task's HostRuntime exclusivity... */
		String contextId = taskEntry.getContextId();
		
		/* Get hostName. */
		String hostName;

		if (taskData.getTaskDescriptor().getHostRuntimesName() == null) {
			this.prepareHostName(taskData.getTaskDescriptor());
			hostName = selectHostRuntime(taskData, 
					taskData.getTaskDescriptor().getHostRuntimesName());
			String[] hostNames = new String[] {hostName}; 
			taskData.getTaskDescriptor().setHostRuntimesName(hostNames);
		} else {
			hostName = selectHostRuntime(taskData, 
					taskData.getTaskDescriptor().getHostRuntimesName());
			String[] hostNames = new String[] {hostName}; 
			taskData.getTaskDescriptor().setHostRuntimesName(hostNames);
		}
		
		HostRuntimeEntry hostRuntimeEntry
			= this.data.getHostRuntimeByName(hostName);
		if (hostRuntimeEntry != null) {
			String hostReservation = hostRuntimeEntry.getReservation();

			TaskExclusivity taskExclusivity = taskEntry.getExclusivity();
			switch (taskExclusivity) {
				case NON_EXCLUSIVE:
					if (hostReservation != null) {
						if (!hostReservation.equals(contextId)) {
							return false;
						}
					}
					break;
				case CONTEXT_EXCLUSIVE:
					if ((hostReservation != null)
							&& !hostReservation.equals(contextId)) {
						return false;
					} else {
						TaskEntry[] tasksOnHost
						= this.data.getTasksOnHost(hostName);
						for (TaskEntry taskOnHost : tasksOnHost) {
							TaskState taskState = taskOnHost.getState();
							if ((taskState == TaskState.RUNNING)
									|| (taskState == TaskState.SLEEPING)) {
								if (!taskOnHost.getContextId()
										.equals(contextId)) {
									return false;
								}
							}
						}
					}
					break;
				case EXCLUSIVE:
					if (hostReservation != null) {
						return false;
					} else {
						TaskEntry[] tasksOnHost
						= this.data.getTasksOnHost(hostName);
						for (TaskEntry taskOnHost : tasksOnHost) {
							TaskState taskState = taskOnHost.getState();
							if ((taskState == TaskState.RUNNING)
									|| (taskState == TaskState.SLEEPING)) {
							return false;
							}
						}
					}
					break;
				default:
					// no other variants...
					break;
			}
		}
		
		return true;
	}
	
	/**
	 * Test if task is ready to run and start it.
	 * 
	 * @param taskEntry <code>TaskEntry</code> of checked task.
	 * @param taskData <code>TaskData</code> of checked task.
	 * @return <code>true</code> if task started, <code>false</code> otherwise.
	 */
	private boolean runTaskIfReady(TaskEntry taskEntry, TaskData taskData)
	throws TaskManagerException {
		boolean taskReady;
		
		synchronized (this.data) {
			taskReady = this.isTaskReadyToRun(taskEntry, taskData);
			if (taskReady) {
				String taskId = taskEntry.getTaskId();
				String contextId = taskEntry.getContextId();

				data.changeTaskState(taskId, contextId, TaskState.SCHEDULED);
			}
		}
		if (taskReady) {
			this.taskStart(taskData);
		}
		
		return taskReady;
	}
	
	/**
	 * Test all <code>SUBMITTED</code> tasks and run all ready of them.
	 */
	private void runAllReadyTasks() throws TaskManagerException {
		synchronized (data) {
			TaskEntry[] tasksSubmitted
			= this.data.getTasksByState(TaskState.SUBMITTED);
		
			for (TaskEntry taskEntry : tasksSubmitted) {
				String taskId = taskEntry.getTaskId();
				String contextId = taskEntry.getContextId();

				TaskData taskData = this.data.getTaskData(taskId, contextId);

				this.runTaskIfReady(taskEntry, taskData);
			}
		}
	}
	
	/**
	 * Run (schedule) one or more new tasks specified by their task
	 * descriptors (XML form).
	 * 
	 * @param taskDescriptorPaths Array containing paths to the XML
	 * 	representation of Task Descriptors.
	 * @throws RemoteException If something failed during this operation.
	 */
	public void runTask(String[] taskDescriptorPaths)
	throws RemoteException {
		/* Paths to TaskDescriptors in Task Manager's root
		 * directory. */
		String[] taskDescriptors
			= new String[taskDescriptorPaths.length];
		
		/* Copy TaskDescriptor XML files to the Task Manager's
		 * root directory. */
		for (int i = 0; i < taskDescriptorPaths.length; i++) {
			try {
				File oldFile = new File(taskDescriptorPaths[i]);
				File newFile = new File(this.rootDirectory,
						oldFile.getName());
				taskDescriptors[i]
					= newFile.getCanonicalPath();
				
				/* Transfer data to the newFile... */
				BufferedInputStream in
				= new BufferedInputStream(
						new FileInputStream(oldFile));
				BufferedOutputStream out
				= new BufferedOutputStream(
						new FileOutputStream(newFile));
				final int bufferSize = 1024;
				byte [] buffer = new byte[bufferSize];
				int len;
				while ((len = in.read(buffer, 0, bufferSize)) != -1) {
					out.write(buffer, 0, len);
				}
				
				in.close();
				out.close();
			} catch (Exception e) {
				throw new RemoteException("Cannot copy task descriptor", e);
			}
		}
		
		/* Create task for each Task Descriptor. */
		for (int i = 0; i < taskDescriptors.length; i++) {
			TaskDescriptor taskDescriptor = null;
			
			try {
				taskDescriptor = TaskDescriptorXmlParser
					.parseXmlTaskDescriptor(new File(taskDescriptors[i]));
			} catch (TaskDescriptorException e) {
				throw new RemoteException("Could not parse XML taskDescriptor",
						e);
			}
			
			this.runTask(taskDescriptor);
		}
	}
	
	/**
	 * Get canonical name of the host. Names returned by this methods are used in the database.
	 * 
	 * @param hostName Name of the host to resolve.
	 * 
	 * @return Canonical name of the host if available.
	 * 
	 * @throws UnknownHostException If host was not found on the network.
	 */
	private static String getCanonicalHostName(String hostName) throws UnknownHostException {

		String canonicalName = null;
		
		if (hostName.equalsIgnoreCase("localhost")) {
			canonicalName = InetAddress.getLocalHost().getCanonicalHostName();
		} else {
			canonicalName = InetAddress.getByName(hostName).getCanonicalHostName();
		}
		
		if (canonicalName.equalsIgnoreCase("localhost")
				|| canonicalName.equalsIgnoreCase("127.0.0.1")) {
			canonicalName = InetAddress.getLocalHost().getCanonicalHostName();
		}
		
		return canonicalName;
	}
	
	/**
	 * Run (schedule) one new Task specified by its task descriptor.
	 * 
	 * @param taskDescriptor Task descriptor of new task to run.
	 * @throws RemoteException If something failed during this operation.
	 */
	public void runTask(TaskDescriptor taskDescriptor)
	throws RemoteException {
		if (taskDescriptor.getHostRuntimesName() != null) {
			String []canonicalNames = new String[taskDescriptor.getHostRuntimesName().length];

			for (int i = 0; i < taskDescriptor.getHostRuntimesName().length; ++i) {
				try {
					canonicalNames[i] = getCanonicalHostName(taskDescriptor.getHostRuntimesName()[i]);
				} catch (UnknownHostException e) {
					throw new RemoteException("Unable to resolve host \""
							+ taskDescriptor.getHostRuntimesName()[i] + "\".", e);
				}
			}

			taskDescriptor.setHostRuntimesName(canonicalNames);
		}
		
		try {
			this.taskNew(taskDescriptor);
		} catch (TaskManagerException e) {
			throw new RemoteException("Cannot start task", e);
		}
	}
	
	/**
	 * Run (schedule) one or more new tasks specified by their task
	 * descriptors.
	 * 
	 * @param taskDescriptors Task descriptors of new tasks to run.
	 * @throws RemoteException If something failed during this operation.
	 */
	public void runTask(TaskDescriptor[] taskDescriptors)
	throws RemoteException {
		synchronized (data) {
			for (int i = 0; i < taskDescriptors.length; i++) {
				this.runTask(taskDescriptors[i]);
			}
		}
	}
	
	/**
	 * Run (schedule) on new task specified by its task descriptor. If task can
	 * be started immediately then it is, otherwise
	 * {@link IllegalStateException} exception is thrown.
	 *  
	 * @param taskDescriptor Task descriptor of new task to run.
	 * @throws RemoteException If something failed during this operation.
	 * @throws IllegalStateException If task can not be started immediately.
	 */
	public void runTaskImmediately(TaskDescriptor taskDescriptor)
	throws RemoteException {
		if (taskDescriptor.getHostRuntimesName() != null) {
			String []canonicalNames
				= new String[taskDescriptor.getHostRuntimesName().length];

			for (int i = 0; i < taskDescriptor.getHostRuntimesName().length;
			++i) {
				try {
					canonicalNames[i] = getCanonicalHostName(
							taskDescriptor.getHostRuntimesName()[i]);
				} catch (UnknownHostException e) {
					throw new RemoteException("Unable to resolve host \""
							+ taskDescriptor.getHostRuntimesName()[i] + "\".",
							e);
				}
			}

			taskDescriptor.setHostRuntimesName(canonicalNames);
		}
		
		boolean started = false;
		try {
			this.taskNew(taskDescriptor);
		} catch (TaskManagerException e) {
			throw new RemoteException("Cannot start task", e);
		}
		
		if (!started) {
			throw new IllegalStateException("Task (taskId \""
					+ taskDescriptor.getTaskTid() + "\", contextId \""
					+ taskDescriptor.getContextId() + "\") could not be "
					+ "started immediately");
		}
	}
	
	/* Index of last context. */
	private static long lastContextIndex = 0;
	
	/**
	 * Create new ID for context.
	 * 
	 * @return New ID for context.
	 */
	private static String getNewContextId() {
		lastContextIndex++;
		
		return lastContextIndex + "_" + System.currentTimeMillis();
	}
	
	/**
	 * Create new context.
	 * 
	 * @param name Human readable name of context.
	 * @param description Human readable description of context.
	 * @param magicObject Some magic object (Serializable and Cloneable).
	 * @return ID of context.
	 * @throws RemoteException If something failed during this operation.
	 */
	public String newContext(String name, String description,
			Serializable magicObject)
	throws RemoteException {
		String id = getNewContextId();
		
		this.newContext(id, name, description, magicObject);
		
		return id;
	}
	
	/**
	 * Create new context. If context with specified ID exists then if it is
	 * already deactivated, it is deleted and newly created otherwise exception
	 * is thrown.
	 * 
	 * @param id ID of context.
	 * @param name Human readable name of context.
	 * @param description Human readable description of context.
	 * @param magicObject Some magic object (Serializable and Cloneable).
	 * @throws IllegalStateException If context with specified ID already
	 * 	exists.
	 * @throws RemoteException If something failed during this operation.
	 */
	public void newContext(String id, String name, String description,
			Serializable magicObject)
	throws RemoteException {
		/* Test if context currently exists. */
		ContextEntry currentContextEntry = this.data.getContextById(id);
		if (currentContextEntry != null) {
//			if (currentContextEntry.isOpen()) {
//				throw new IllegalStateException("Context with id \"" + id
//						+ "\" already exists and is still opened");
//			}
			throw new IllegalStateException("Context with id \"" + id
					+ "\" already exists");
		}
		
		ContextEntry contextEntry
			= new ContextEntry(id, name, description, magicObject);
		this.data.newContext(contextEntry);
		
		/* Add new context to log storage. */
		try {
			this.logStorage.addContext(id);
		} catch (LogStorageException e) {
			throw new RemoteException("Unable to add context to the log storage", e);
		}
	}
	
	/**
	 * Close context.
	 * 
	 * @param id ID of context.
	 * @throws RemoteException If something failed during this operation.
	 */
	public void closeContext(String id) throws RemoteException {
		this.data.closeContext(id);
	}
	
	/**
	 * Return informations about all tasks known inside Task Manager.
	 * 
	 * @return Array containing TaskEntry for each task.
	 * @throws RemoteException If something failed during this operation.
	 */
	public TaskEntry[] getTasks() throws RemoteException {
		return this.data.getTasks();
	}
	
	/**
	 * Return informations about all tasks known inside Task Manager as member
	 * of specified context.
	 * 
	 * @param contextId ID of requested context.
	 * @return Array conraining TaskEntry for each convenient task.
	 * @throws RemoteException If something failed during this operation.
	 */
	public TaskEntry[] getTasksInContext(String contextId)
	throws RemoteException {
		return this.data.getTasksInContext(contextId);
	}
	
	/**
	 * Return informations about all tasks known inside Task Manager as
	 * scheduled on specified HostRuntime.
	 * 
	 * @param hostName URI of requested Host Runtime.
	 * @return Array containing TaskEntry for each convenient task.
	 * @throws RemoteException If something failed during this operation.
	 */
	public TaskEntry[] getTasksOnHost(String hostName)
	throws RemoteException {
		return this.data.getTasksOnHost(hostName);
	}
	
	/**
	 * Return informations about all contexts known inside Task Manager.
	 * 
	 * @return Array containing ContextEntry for each context.
	 * @throws RemoteException If something failed during this operation.
	 */
	public ContextEntry[] getContexts() throws RemoteException {
		return this.data.getContexts();
	}
	
	/**
	 * Return informations about one task specified by its ID.
	 * 
	 * @param taskId ID of requested task.
	 * @return TaskEntry filled in by informations about requested task (null
	 * 	if requested task not found).
	 * @throws RemoteException If something failed during this operation.
	 */
	public TaskEntry getTaskById(String taskId, String contextId)
	throws RemoteException {
		TaskEntry result = data.getTaskById(taskId, contextId);
		if (result == null) {
			throw new IllegalArgumentException("Invalid contextId (\"" + contextId
				+ "\") or taskId (\"" + taskId + "\").");
		}
		return result;
	}
	
	/**
	 * Return informations about one context specified by its ID.
	 * 
	 * @param contextId ID of requested context.
	 * @return ContextEntry filled in by informations about requested context
	 * 	(null if requested task not found).
	 * @throws RemoteException If something failed during this operation.
	 */
	public ContextEntry getContextById(String contextId)
	throws RemoteException {
		ContextEntry result = data.getContextById(contextId); 
		if (result == null) {
			throw new IllegalArgumentException("Invalid contextId (\"" + contextId + "\").");
		}
		return result;
	}
	
	/**
	 * Kill task specified by its ID.
	 * 
	 * @param taskId ID of task.
	 * @param contextId ID of contex.
	 * @throws IllegalArgumentException If task not found.
	 * @throws RemoteException If something failed during this operation.
	 */
	public void killTaskById(String taskId, String contextId)
	throws RemoteException {
		TaskInterface taskInterface = null;
		
		synchronized (data) {
			TaskData taskData = this.data.getTaskData(taskId, contextId);
			if (taskData == null) {
				throw new IllegalArgumentException("task (taskId \"" + taskId
						+ "\", contextId \"" + contextId + "\") not found");
			}

			taskInterface = taskData.getTaskInterface();
			if (taskInterface == null) {
				this.taskReachedState(taskId, contextId, TaskState.ABORTED);
				this.data.changeTaskState(taskId, contextId, TaskState.ABORTED);
			}
		}
		
		if (taskInterface != null) {
			taskInterface.kill();
		}
		
		unregisterFinishedService(contextId, taskId);
	}
	
	/**
	 * Kill all tasks within specified context and that context.
	 * 
	 * @param contextId ID of context.
	 * @throws RemoteException If something failed during this operation.
	 */
	public void killContextById(String contextId) throws RemoteException {
		ContextEntry contextEntry = this.data.getContextById(contextId);
		if (contextEntry == null) {
			throw new IllegalArgumentException("context (contextId \""
					+ contextId + "\") not found");
		}
		
		TaskEntry[] taskEntries = this.data.getTasksInContext(contextId);
		for (TaskEntry taskEntry : taskEntries) {
			this.killTaskById(taskEntry.getTaskId(), taskEntry.getContextId());
		}
	}
	
	/**
	 * Used by Host Runtime for forwarding of check points reached by
	 * tasks.
	 * 
	 * @param name Name of checkPoint.
	 * @param value Value of checkPoint.
	 * @param taskId ID of task.
	 * @param contextId ID of context.
	 * @param hostName Name of host on which this checkPoint was reached.
	 * @param magicObject <code>MagicObject</code> of this checkPoint.
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public void checkPointReached(String name, String value,
			String taskId, String contextId, String hostName,
			Serializable magicObject)
	throws RemoteException {
		this.checkPointReached(new CheckPoint(taskId, contextId,
				name, value));
	}
	
	/**
	 * Used by Host Runtime for forwarding of checkpoint reached by task.
	 * 
	 * @param checkPoint CheckPoint representation of reached checkpoint.
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public void checkPointReached(CheckPoint checkPoint)
	throws RemoteException {
		logDebug("[" + checkPoint.getContextId() + ":" 
			+ checkPoint.getTaskId() + "] Checkpoint \"" + checkPoint.getName() 
			+ "\" set to \"" + checkPoint.getValue() + "\"");
		
		/* Add information about this checkpoint to the list of
		 * reached checkpoints. */
		String name = checkPoint.getName();
		String taskId = checkPoint.getTaskId();
		String contextId = checkPoint.getContextId();
		String hostName = checkPoint.getHostName();
		Serializable magicObject = checkPoint.getValue();
		CheckPointEntry checkPointEntry = new CheckPointEntry(name,
				taskId, contextId, hostName, magicObject);
		
		synchronized (data) {
			this.data.newCheckPointOver(checkPointEntry);
		}

		/* Test dependencies of all waiting tasks if they were
		 * already reached. */
		
		try {
			this.runAllReadyTasks();
		} catch (TaskManagerException e) {
			logError("Some tasks cannot be started: " + e.getMessage());
		}
		
		/* Notify all tasks waiting for checkpoint in checkPointLook(). */
		synchronized (this.checkPointWaitingObject) {
			this.checkPointWaitingObject.notifyAll();
		}
	}
	
	/**
	 * Test if specified context is still open or if there is specified task and
	 * is not finished yet.
	 * 
	 * @param taskId ID of task.
	 * @param contextId ID of context.
	 * @return <code>true</code> if checkPoint with specified
	 * 	<code>taskId</code> and <code>contextId</code> can be still reached;
	 * 	<code>false</code> otherwise.
	 */
	private boolean isCheckPointReachPossible(String taskId, String contextId) {
		/* Test if specified context exists and is still open. */
		ContextEntry contextEntry = this.data.getContextById(contextId);
		if (contextEntry == null) {
			return false;
		}
		if (contextEntry.isOpen()) {
			return true;
		}
		
		/* If context is not open so check if there is not-ended task with
		 * specified taskId. */
		TaskEntry taskEntry = this.data.getTaskById(taskId, contextId);
		if (taskEntry == null) {
			return false;
		}
		if ((taskEntry.getState() == TaskState.FINISHED)
				|| (taskEntry.getState() == TaskState.ABORTED)) {
			return false;
		}
		
		/* If there is not-ended task (specified by its taskId and contextId) in
		 * closed context... */
		return true;
	}
	
	/**
	 * Returns array containing all reached checkpoints matching specified
	 * parameters (taskId, contextId, name).
	 * All filled in values must match. Values set to null are arbitrary.
	 * Calling of this method is non-blocking.
	 * 
	 * @param name Name of checkpoint.
	 * @param value Value of checkpoint.
	 * @param taskId ID of task which reached checkpoint.
	 * @param contextId ID of context in which checkpoint was reached.
	 * @return Array containing all checkpoint matching specified
	 * 	checkpointTemplate.
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	private CheckPoint[] checkPointLook(String name, Serializable value,
			String taskId, String contextId)
	throws RemoteException {
		CheckPointEntry[] checkPointEntries
			= this.data.getCheckPoints(name, taskId, contextId, value);
		
		CheckPoint[] matchingCheckPoints
			= new CheckPoint[checkPointEntries.length];
		
		for (int i = 0; i < checkPointEntries.length; i++) {
			matchingCheckPoints[i] = new CheckPoint(
					checkPointEntries[i].getTaskId(),
					checkPointEntries[i].getContextId(),
					checkPointEntries[i].getName(),
					checkPointEntries[i].getMagicObject());
		}
		
		return matchingCheckPoints;
	}
	
	/**
	 * Returns array containing all reached checkpoints matching specified
	 * parameters (taskId, contextId, name).
	 * All filled in values must match. Values set to null are arbitrary.
	 * Calling of this method is blocking.
	 * 
	 * @param name Name of checkpoint.
	 * @param value Value of checkpoint.
	 * @param taskId ID of task which reached checkpoint.
	 * @param contextId ID of context in which checkpoint was reached.
	 * @param timeout Maximum time to wait in milliseconds.
	 * @return Array containing all checkpoint matching specified
	 * 	checkpointTemplate.
	 * @throws TaskManagerException If Required checkPoint can not be reached
	 * 	anyway
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	private CheckPoint[] checkPointLook(String name, Serializable value,
			String taskId, String contextId, long timeout)
	throws TaskManagerException, RemoteException {
		long startTimeNano = System.nanoTime();
		CheckPoint[] result = null;
		
		synchronized (this.checkPointWaitingObject) {
			while ((result = this.checkPointLook(
					name, value, taskId, contextId)).length < 1) {
				
				if (!this.isCheckPointReachPossible(taskId, contextId)) {
					throw new TaskManagerException("Required checkPoint can "
							+ "not be reached anyway");
				}
				
				if (timeout == INFINITE_TIME) {
					try {
						this.checkPointWaitingObject.wait();
					} catch (InterruptedException e) {
						// do nothing
					}
				} else {
					long elapsedTime
					= (System.nanoTime() - startTimeNano) / 1000000;
					long remainTime = timeout - elapsedTime;

					if (remainTime <= 0) {
						return new CheckPoint[0];
					}

					try {
						this.checkPointWaitingObject.wait(remainTime);
					} catch (InterruptedException e) {
						// do nothing
					}
				}
			}
		}
		
		return result;
	}
	
	/**
	 * Do check point lookup. Tasks should not call this.
	 * Returns last value of reached checkpoints matching specified taskId,
	 * contextId and name. All values must be non-null and must match.
	 * 
	 * Calling of this method is blocking. If timeout is set to zero, then
	 * return immediately.
	 * @param name Name of checkpoint.
	 * @param taskId ID of task which reached checkpoint.
	 * @param contextId ID of context in which checkpoint was reached.
	 * @param timeout Maximum time to wait in milliseconds.
	 * 
	 * @return Value of specified checkpoint (can be <code>null</code>).
	 * @throws NullPointerException If some input parameter is null.
	 * @throws IllegalArgumentException If checkpoint not found.
	 * @throws TaskManagerException If Required checkPoint can not be reached
	 * 	anyway
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public Serializable checkPointLook(String name,
			String taskId, String contextId, long timeout)
	throws TaskManagerException, RemoteException {
		/* Check input parameters. */
		if (name == null) {
			throw new NullPointerException("name is null");
		}
		if (taskId == null) {
			throw new NullPointerException("taskId is null");
		}
		if (contextId == null) {
			throw new NullPointerException("contextId is null");
		}
		if (name == null) {
			throw new NullPointerException("name is null");
		}
		
		CheckPoint[] foundCheckPoints
			= this.checkPointLook(name, null, taskId, contextId, timeout);
		
		if (foundCheckPoints.length == 0) {
			throw new IllegalArgumentException("specified checkpoint not foud");
		}
		
		return foundCheckPoints[0].getValue();
	}
	
	/**
	 * Used by Host Runtime for forwarding of lookup request from task.
	 * Returns array containing all reached checkpoints matching specified
	 * checkpointTemplate. All filled in values must match. Values set to
	 * null are arbitrary.
	 * Calling of this method is blocking. If timeout is set to zero, then
	 * return immediately.
	 * 
	 * @param checkPointTemplate Prepared template for checkpoint match.
	 * @param timeout Maximum time to wait in milliseconds.
	 * @return Array containing all checkpoint matching specified
	 * 	checkpointTemplate.
	 * @throws TaskManagerException If Required checkPoint can not be reached
	 * 	anyway
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public CheckPoint[] checkPointLook(CheckPoint checkPointTemplate,
			long timeout)
	throws TaskManagerException, RemoteException {
		String name = checkPointTemplate.getName();
		Serializable value = checkPointTemplate.getValue();
		String taskId = checkPointTemplate.getTaskId();
		String contextId = checkPointTemplate.getContextId();
		
		return this.checkPointLook(name, value, taskId, contextId, timeout);
	}
	
	/**
	 * Used by Host Runtime for forwarding of log messages from tasks.
	 * 
	 * @param contextId Id of the task's context.
	 * @param taskId TID of task.
 	 * @param level log level of this log message 
	 * @param timestamp timestamp of this log message
	 * @param message Message to log.
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public void log(String contextId, String taskId, LogLevel level, Date timestamp,
			String message)
	throws RemoteException {
		if (!logLevel.isGreaterOrEqual(LogLevel.WARN)) {
			System.out.println("[" + contextId + ":" + taskId + "] " + level
					+ " " + message);
		}
		
		try {
			logStorage.log(contextId, taskId, timestamp, level, message);
		} catch (LogStorageException e) {
			throw new RemoteException("Cannot store the log message in the log "
					+ "storage", e);
		}
	}
	

	/**
	 * Used by Host Runtime for forwarding of task's request for new
	 * registration of service.
	 * 
	 * @param service Object describing service to register.
	 * @throws RemoteException If something failed during the execution
	 * 	of the remote method call.
	 * @throws IllegalArgumentException If <code>service</code> is not correctly
	 * 	filled in.
	 */
	public void serviceRegister(ServiceEntry service)
	throws RemoteException {
		/* Check all values in ServiceEntry. */
		if (service.getServiceName() == null) {
			throw new IllegalArgumentException("service.serviceName can not be "
					+ "null");
		}
		if (service.getInterfaceName() == null) {
			throw new IllegalArgumentException("service.InterfaceName can not "
					+ "be null");
		}
		if (service.getRmiAddress() == null) {
			throw new IllegalArgumentException("service.rmiAddress can not be "
					+ "null");
		}
//		if ((service.getServiceName() == null)
//				|| (service.getInterfaceName() == null)
//				|| (service.getRmiAddress() == null)) {
//			logWarning("Warning: Registered service entry must have "
//				+ "all fields filled in (i.e. serviceName, interfaceName, "
//				+ "rmiAddress, remoteInterface). This entry ["
//				+ service.getServiceName() + "," + service.getInterfaceName()
//				+ "," + service.getRmiAddress() + ","
//				+ service.getRemoteInterface() + "] won't be registered!!");
//			return;
//		}
		
		ServiceEntry newEntry = (ServiceEntry) service.clone();
		
		synchronized (this.serviceRegistry) {
			this.serviceRegistry.add(newEntry);
		}
	}
	
	/**
	 * Used by Host Runtime for forwarding of task's request for 
	 * deregistration of services.
	 * All entries matching specified template are removed from the
	 * registry.
	 * 
	 * @param serviceTemplate Object describing services to remove (using
	 * 	regular expressions).
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public void serviceUnregister(ServiceEntry serviceTemplate)
	throws RemoteException {
		/* Make patterns for regex matching. */
		Pattern serviceNamePattern = null;
		Pattern interfaceNamePattern = null;
		Pattern rmiAddressPattern = null;
		
		String serviceNameTemplate = serviceTemplate.getServiceName();
		if (serviceNameTemplate == null) {
			/* Match everything. */
			serviceNamePattern = Pattern.compile(".*");
		} else {
			serviceNamePattern
				= Pattern.compile(serviceNameTemplate);
		}
		
		String interfaceNameTemplate
			= serviceTemplate.getInterfaceName();
		if (interfaceNameTemplate == null) {
			interfaceNamePattern = Pattern.compile(".*");
		} else {
			interfaceNamePattern
				= Pattern.compile(interfaceNameTemplate);
		}
		
		URI rmiAddressTemplate = serviceTemplate.getRmiAddress();
		if (rmiAddressTemplate == null) {
			rmiAddressPattern = Pattern.compile(".*");
		} else {
			rmiAddressPattern = Pattern.compile(rmiAddressTemplate
					.normalize().toString());
					// normalized and converted to String
		}
		
		synchronized (this.serviceRegistry) {
			Iterator registryIterator = this.serviceRegistry.iterator();	
			while (registryIterator.hasNext()) {
				ServiceEntry serviceEntry
					= (ServiceEntry) registryIterator.next();
				
				/* Do matching... */
				Matcher serviceNameMatcher = serviceNamePattern
					.matcher(serviceEntry.getServiceName());
				if (!serviceNameMatcher.matches()) {
					continue;
				}
				
				Matcher interfaceNameMatcher = interfaceNamePattern
					.matcher(serviceEntry.getInterfaceName());
				if (!interfaceNameMatcher.matches()) {
					continue;
				}
				
				Matcher rmiAddressMatcher = rmiAddressPattern
					.matcher(serviceEntry.getRmiAddress()
							.normalize().toString());
				if (!rmiAddressMatcher.matches()) {
					continue;
				}
				
				/* If everything matches so remove this entry from
				 * registry. */
				registryIterator.remove();
			}
		}
	}
	
	/**
	 * Used by Host Runtime for forwarding of task's request for looking in
	 * service entries.
	 * Returns array containing all service entries matching specified
	 * template (using regular expressions).
	 * 
	 * @param serviceTemplate Object looked describing services (using
	 * 	regular expressions).
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public ServiceEntry[] serviceLook(ServiceEntry serviceTemplate)
	throws RemoteException {
		LinkedList<ServiceEntry> matchingEntries
			= new LinkedList<ServiceEntry>();
		
		/* Make patterns for regex matching. */
		Pattern serviceNamePattern = null;
		Pattern interfaceNamePattern = null;
		Pattern rmiAddressPattern = null;
		
		String serviceNameTemplate = serviceTemplate.getServiceName();
		if (serviceNameTemplate == null) {
			/* Match everything. */
			serviceNamePattern = Pattern.compile(".*");
		} else {
			serviceNamePattern
				= Pattern.compile(serviceNameTemplate);
		}
		
		String interfaceNameTemplate
			= serviceTemplate.getInterfaceName();
		if (interfaceNameTemplate == null) {
			interfaceNamePattern = Pattern.compile(".*");
		} else {
			interfaceNamePattern
				= Pattern.compile(interfaceNameTemplate);
		}
		
		URI rmiAddressTemplate = serviceTemplate.getRmiAddress();
		if (rmiAddressTemplate == null) {
			rmiAddressPattern = Pattern.compile(".*");
		} else {
			rmiAddressPattern = Pattern.compile(rmiAddressTemplate
					.normalize().toString());
					// normalized and converted to String
		}
		
		String taskId = serviceTemplate.getTaskId();
		String contextId = serviceTemplate.getContextId();
		
		synchronized (this.serviceRegistry) {
			Iterator registryIterator = this.serviceRegistry.iterator();
			while (registryIterator.hasNext()) {
				ServiceEntry serviceEntry
					= (ServiceEntry) registryIterator.next();
				
				/* Do matching... */
				Matcher serviceNameMatcher = serviceNamePattern
					.matcher(serviceEntry.getServiceName());
				if (!serviceNameMatcher.matches()) {
					continue;
				}
				
				Matcher interfaceNameMatcher = interfaceNamePattern
					.matcher(serviceEntry.getInterfaceName());
				if (!interfaceNameMatcher.matches()) {
					continue;
				}
				
				Matcher rmiAddressMatcher = rmiAddressPattern
					.matcher(serviceEntry.getRmiAddress()
							.normalize().toString());
				if (!rmiAddressMatcher.matches()) {
					continue;
				}
				
				if ((taskId != null) && (!taskId.equals(serviceEntry.getTaskId()))) {
					continue;
				}

				if ((contextId != null) && (!contextId.equals(serviceEntry.getContextId()))) {
					continue;
				}
				
				/* If everything matches so add clone of this entry to
				 * the created list. */
				ServiceEntry matchingEntry = serviceEntry.clone();
				matchingEntries.add(matchingEntry);
			}
		}
		
		return (ServiceEntry[]) matchingEntries.toArray(
				new ServiceEntry[matchingEntries.size()]);
	}
	
	/**
	 * Used by tasks for finding some registered remote interface.
	 * Given names are compared for exact match (doesn't use regular
	 * expressions).
	 * 
	 * @param serviceName Name of service.
	 * @param interfaceName Name of service's interface.
	 * @return Remote representation of one from all matching interfaces
	 * 	or null if none.
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public Remote serviceFind(String serviceName, String interfaceName)
	throws RemoteException {
		/* Check input parameters (null is not allowed). */
		if ((serviceName == null) || (interfaceName == null)) {
			return null;
		}
		
		/* Do search. */
		ServiceEntry templateEntry = new ServiceEntry(
			Pattern.quote(serviceName),
			Pattern.quote(interfaceName), null, null, null, null);
		ServiceEntry[] matchingEntries
			= this.serviceLook(templateEntry);
		
		/* If found then return the first. */
		if (matchingEntries.length > 0) {
			return matchingEntries[0].getRemoteInterface();
		}
		
		/* Return null by default (if nothing found). */
		return null;
	}
	
	/**
	 * Used by tasks for finding some registered remote interface.
	 * Given names are compared for exact match (doesn't use regular
	 * expressions).
	 * 
	 * @param serviceName Name of service.
	 * @param interfaceName Name of service's interface.
	 * @return URI representation of one from all matching interfaces or
	 * 	null if none.
	 * @throws RemoteException If something failed during the execuiton of
	 * 	the remote method call.
	 */
	public URI serviceFindURI(String serviceName, String interfaceName)
	throws RemoteException {
		/* Check input parameters (null is not allowed). */
		if ((serviceName == null) || (interfaceName == null)) {
			return null;
		}
		
		/* Do search. */
		ServiceEntry templateEntry = new ServiceEntry(
			Pattern.quote(serviceName),
			Pattern.quote(interfaceName), null, null, null, null);
		ServiceEntry[] matchingEntries
			= this.serviceLook(templateEntry);
		
		/* If found then return the first. */
		if (matchingEntries.length > 0) {
			return matchingEntries[0].getRmiAddress();
		}
		
		/* Return null by default (if nothing found). */
		return null;
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#getLogsForTask(java.lang.String, java.lang.String)
	 */
	public LogRecord[] getLogsForTask(String context, String taskID) 
	throws RemoteException, LogStorageException, IllegalArgumentException, NullPointerException {
		return logStorage.getLogsForTask(context, taskID);
	}

	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#isContextRegistered(java.lang.String)
	 */
	public boolean isContextRegistered(String name) throws RemoteException, LogStorageException, IllegalArgumentException, NullPointerException {
		return logStorage.isContextRegistered(name);
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#isTaskRegistered(java.lang.String, java.lang.String)
	 */
	public boolean isTaskRegistered(String context, String taskID) throws RemoteException, LogStorageException, IllegalArgumentException, NullPointerException {
		return logStorage.isTaskRegistered(context, taskID);
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#addErrorOutput(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void addErrorOutput(String context, String taskID, String output) 
	throws RemoteException, LogStorageException, IllegalArgumentException, NullPointerException {
		logStorage.addErrorOutput(context, taskID, output);
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#addStandardOutput(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void addStandardOutput(String context, String taskID, String output) 
	throws RemoteException, LogStorageException, IllegalArgumentException, NullPointerException {
		logStorage.addStandardOutput(context, taskID, output);
	}

	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#getErrorOutput(java.lang.String, java.lang.String)
	 */
	public OutputHandle getErrorOutput(String context, String taskID) throws RemoteException, LogStorageException, IllegalArgumentException, NullPointerException {
		return logStorage.getErrorOutput(context, taskID);
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#getStandardOutput(java.lang.String, java.lang.String)
	 */
	public OutputHandle getStandardOutput(String context, String taskID) throws RemoteException, LogStorageException, IllegalArgumentException, NullPointerException {
		return logStorage.getStandardOutput(context, taskID);
	}
	
	/**
	 * Test if context is finished (i.e. closed and all its tasks are ended).
	 * 
	 * @param contextId ID of context.
	 * @return <code>true</code> if context is finished; <code>false</code>
	 * 	otherwise.
	 * @throws IllegalArgumentException If context with specified ID does not
	 * 	exist.
	 */
	private boolean isContextFinished(String contextId) {
		/* Check if system context. */
		if (SYSTEM_CONTEXT_ID.equals(contextId)) {
			return false;
		}
		
		synchronized (this.data) {
			/* Check if context exists. */
			ContextEntry contextEntry = this.data.getContextById(contextId);
			if (contextEntry == null) {
				throw new IllegalArgumentException("Context \"" + contextId + "\" "
						+ "does not exist");
			}

			/* Check if context is open. */
			if (contextEntry.isOpen()) {
				return false;
			}

			/* Check if some task with specified contextId is not ended (FINISHED or
			 * ABORTED). */
			TaskEntry[] taskEntries = this.data.getTasksInContext(contextId);
			for (TaskEntry taskEntry : taskEntries) {
				if ((taskEntry.getState() != TaskState.FINISHED)
						&& (taskEntry.getState() != TaskState.ABORTED)) {
					return false;
				}
			}
		}
		
		/* Otherwise context is finished... */
		return true;
	}
	
	/**
	 * Remove specified context from Task Manager and notify all Host Runtimes.
	 * Do nothing if specified context is system context of BEEN.
	 * 
	 * @param contextId ID of context.
	 */
	private void removeContext(String contextId) {
		/* Check if system context. */
		if (SYSTEM_CONTEXT_ID.equals(contextId)) {
			return;
		}
		
		/* Get names of hosts used by this context. */
		HashSet<String> hostNames = new HashSet<String>();
		TaskEntry[] taskEntries = this.data.getTasksInContext(contextId);
		for (TaskEntry taskEntry : taskEntries) {
			String hostName = taskEntry.getHostName();
			if (hostName != null) {
				hostNames.add(hostName);
			}
		}
		
		/* Remove context from data of TaskManager. */
		this.data.delContextByForce(contextId);
		
		/* Remove context from logStorage. */
		try {
			this.logStorage.removeContext(contextId);
		} catch (LogStorageException e) {
			System.err.println("Cannot remove context from the log storage: "
					+ e.getMessage());
		}
		
		/* Remove context from each used HostRuntime. */
		Iterator<String> hostNameIterator = hostNames.iterator();
		while (hostNameIterator.hasNext()) {
			String hostName = hostNameIterator.next();
			
			/* Construct URI of Host Runtime control interface. */
			URI hostRuntimeUri;
			try {
				hostRuntimeUri = new URI("rmi", null, hostName,
						Registry.REGISTRY_PORT,
						HostRuntimeInterface.URL, null, null);
			} catch (URISyntaxException e) {
				logError("Constructing of hierarchical URI from the given "
						+ "components failed (hostName \"" + hostName + "\": "
						+ e.getMessage());
				break;
			}
			
			/* Obtain RMI reference to Host Runtime. */
			HostRuntimeInterface hostRuntimeInterface;
			try {
				hostRuntimeInterface = (HostRuntimeInterface)
				Naming.lookup(hostRuntimeUri.toString());
			} catch (Exception e) {
				logError("Obtaining of RMI reference to hostRuntime failed "
						+ "(URI \"" + hostRuntimeUri.toString() + "\"): "
						+ e.getMessage());
				break;
			}
			
			/* Close context for Host Runtime. */
			try {
				hostRuntimeInterface.closeContext(contextId);
			} catch (HostRuntimeException e) {
				logError("Deleting of context data failed:" + e.getMessage());
			} catch (RemoteException e) {
				logError("Some error occured when deleting data of context:"
						+ e.getMessage());
			}
		}
	}

	/**
	 * Notification from Host Runtime, that specified task was restarted.
	 * 
	 * @param taskId ID of task.
	 * @param contextId ID of context.
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public void taskRestarted(String taskId, String contextId)
	throws RemoteException {
		this.data.notifyTaskRestarted(taskId, contextId);
	}
			
	/**
	 * Used by Host Runtime for notify of task's reached state.
	 * 
	 * @param taskId TID of task.
	 * @param state String representation of reached state.
	 * @throws RemoteException If something failed during the execution of
	 * 	the remote method call.
	 */
	public void taskReachedState(String taskId, String contextId, TaskState state)
	throws RemoteException {
		if (state == TaskState.FINISHED || state == TaskState.ABORTED) {
			synchronized (this.data) {
				
				unregisterFinishedService(contextId, taskId);
				
				/* Set state of task. */
				this.data.changeTaskState(taskId, contextId,state);
				
				/* Unset reservation of Host Runtime if possible and set... */
				TaskEntry taskEntry = this.data.getTaskById(taskId, contextId);
				TaskExclusivity taskExclusivity = taskEntry.getExclusivity();
				
				String hostName = taskEntry.getHostName();
				
				if (hostName != null) {
					switch (taskExclusivity) {
						case CONTEXT_EXCLUSIVE:
							unset_exclusivity: {
							TaskEntry[] tasksOnHost
								= this.data.getTasksOnHost(hostName);
							for (TaskEntry taskOnHost : tasksOnHost) {
								TaskState taskState = taskOnHost.getState();
								if ((taskState == TaskState.RUNNING)
										|| (taskState == TaskState.SLEEPING)) {
									TaskExclusivity taskExclusivity2
										= taskOnHost.getExclusivity();
									if (taskExclusivity2 != TaskExclusivity.NON_EXCLUSIVE) {
										break unset_exclusivity;
									}
								}
							}
							this.data.changeHostRuntimeReservation(
									hostName, null);
						}
						break;
						case EXCLUSIVE:
							this.data.changeHostRuntimeReservation(hostName, null);
							break;
						default:
							break;  // No work needs to be done...
					}
				}
			}
			
			try {
				this.runAllReadyTasks();
			} catch (TaskManagerException e) {
				logError("Some tasks cannot be started: " + e.getMessage());
			}
			
			/* Remove context if finished. */
			// This feature is disabled, it's confusing to the user.
//			if (this.isContextFinished(contextId)) {
//				this.removeContext(contextId);
//			}
		}
	}
	
	/**
	 * For specified taskDescriptor prepare its list of useable hostRuntimes
	 * (based on RSL or asTask attribute) if needed.
	 * 
	 * @param taskDescriptor TaskDescriptor to process.
	 */
	private void prepareHostName(TaskDescriptor taskDescriptor) {
		if (taskDescriptor.getHostRuntimesName() == null) {
			if (taskDescriptor.getHostRuntimesAsTask() != null) {
				TaskEntry determiningTask = this.data.getTaskById(
						taskDescriptor.getHostRuntimesAsTask(),
						taskDescriptor.getContextId());
				if (determiningTask != null) {
					String[] hostNames = { determiningTask.getHostName() };
					taskDescriptor.setHostRuntimesName(hostNames);
				}
			} else {
				RSLRestriction restriction = new RSLRestriction(
						taskDescriptor.getHostRuntimesRsl().toString());
				RestrictionInterface[] restrictionInterface
				= new RestrictionInterface [] {restriction};

				try {
					HostManagerInterface hostManagerInterface
					= (HostManagerInterface) this.serviceFind(
							HostManagerService.SERVICE_NAME,
							Service.RMI_MAIN_IFACE);
					HostInfoInterface[] hostInfoInterface
					= hostManagerInterface.queryHosts(restrictionInterface);

					/* Set taskDescriptor. */
					String[] hostNames = new String[hostInfoInterface.length];
					for (int i = 0; i < hostNames.length; i++) {
						hostNames[i] = hostInfoInterface[i].getHostName();
					}

					if (hostNames.length > 0) {
						taskDescriptor.setHostRuntimesName(hostNames);
					}
				} catch (RemoteException e) {
					logError("Could not find hostManager service: "
							+ e.getMessage());
				} catch (HostManagerException e) {
					logError("Could not resolve RSL to hostNames (RSL \""
							+ taskDescriptor.getHostRuntimesRsl().toString()
							+ "\"): " + e.getMessage());
					return;
				}
			}
		}
	}
	
	/**
	 * For specified taskDescriptor prepare its definition of package (based on
	 * RSL) if needed.
	 * 
	 * @param taskDescriptor TaskDescriptor to process.
	 * @throws TaskManagerException if anything goes wrong.
	 */
	private void preparePackageName(TaskDescriptor taskDescriptor) 
	throws TaskManagerException {
		if (taskDescriptor.getPackageName() == null) {
			RSLPackageQueryCallback packageQueryCallback
				= new RSLPackageQueryCallback(taskDescriptor.getPackageRsl());
			
			try {
				SoftwareRepositoryInterface softwareRepositoryInterface
					= (SoftwareRepositoryInterface) this.serviceFind(
							SoftwareRepositoryService.SERVICE_NAME,
							Service.RMI_MAIN_IFACE);
				if (softwareRepositoryInterface == null) {
					throw new TaskManagerException("Software Repository not found");
				}
				
				PackageMetadata[] metadata
					= softwareRepositoryInterface
					.queryPackages(packageQueryCallback);
				
				/* Set taskDescriptor. */
				// use only the first... */
				if (metadata.length < 1) {
					throw new IllegalArgumentException("Name of package not "
							+ "found: " + taskDescriptor.getPackageRsl());
				}
				taskDescriptor.setPackageName(metadata[0].getFilename());
			} catch (RemoteException e) {
				throw new TaskManagerException ("Could not find "
						+ "softwareRepository service", e);
			} catch (MatchException e) {
				throw new IllegalArgumentException("Could not get package "
						+ "names", e);
			}
		}
	}
	
	/**
	 * Starts execution of specified task.
	 * 
	 * @param task Task to start.
	 */
	private void taskStart(TaskData taskData) throws TaskManagerException {
		URI hostRuntimeUri;
		HostRuntimeInterface hostRuntime;
		TaskDescriptor taskDescriptor = taskData.getTaskDescriptor();
		
		/* Prepare hostName. */
		this.prepareHostName(taskDescriptor);
		
		/* Get hostName. */
		String hostName;
		try {
			String[] hostnames = taskDescriptor.getHostRuntimesName();
			hostName = InetAddress.getByName(hostnames[0])
					.getCanonicalHostName();
		} catch (UnknownHostException e) {
			throw new TaskManagerException("Getting of canonical host name "
					+ "failed", e);
		}
		
		if (this.data.getHostRuntimeByName(hostName) == null) {
			throw new TaskManagerException("Host runtime not registered: " + hostName);
		}
		
		try {
			hostRuntimeUri = new URI("rmi", null, hostName,
					Registry.REGISTRY_PORT,
					HostRuntimeInterface.URL, null, null);
		} catch (URISyntaxException e) {
			throw new TaskManagerException("Could not construct URI of Host Runtime");
		}
		
		try {
			hostRuntime = (HostRuntimeInterface)
				Naming.lookup(hostRuntimeUri.toString());
		} catch (Exception e) {
			throw new TaskManagerException("Could not connect to required Host "
				+ "Runtime (URI=\"" + hostRuntimeUri.toString() + "\")");
		}
		
		try {
			String taskId = taskData.getTaskDescriptor().getTaskTid();
			String contextId = taskData.getTaskDescriptor().getContextId();
			
			/* Add task to logStorage. */
			logStorage.setTaskHostname(contextId, taskId, hostName);
			
			/* Initialize hostRuntime. */
			hostRuntime.initialize(this, maxPackageCacheSize, keptClosedContextCount);
			
			synchronized (this.data) {
				/* Set task's state. */
				this.data.changeTaskState(taskId, contextId, TaskState.RUNNING);
				
				/* Run task on Host Runtime. */
				taskData.setTaskInterface(
						hostRuntime.createTask(taskData.getTaskDescriptor()));
			
				/* Set HostRuntime for running task. */
				this.data.setTaskHostRuntime(taskId, contextId, hostName);
				
				/* Set reservation of Host Runtime if needed. */
				TaskExclusivity taskExclusivity
					= taskData.getTaskDescriptor().getTaskExclusive();
				switch (taskExclusivity) {
					case CONTEXT_EXCLUSIVE:
						this.data.changeHostRuntimeReservation(
								hostName, contextId);
						break;
					case EXCLUSIVE:
						this.data.changeHostRuntimeReservation(hostName, "");
					default:
						break;  // No work needs to be done...
				}
			}
			
			/* Set task's directory paths. */
			String taskDirectory
				= taskData.getTaskInterface().getTaskDirectory();
			String workingDirectory
				= taskData.getTaskInterface().getWorkingDirectory();
			String temporaryDirectory
				= taskData.getTaskInterface().getTemporaryDirectory();
			this.data.setTaskDirectories(taskId, contextId,
					taskDirectory, workingDirectory, temporaryDirectory);
			
			logTrace("[" + taskData.getTaskDescriptor().getContextId()
					+ ":" + taskData.getTaskDescriptor().getTaskTid() + "] "
					+ "Started.");
		} catch (Exception e) {
			throw new TaskManagerException(e);
		}
	}
	
	/**
	 * Tests if specified dependencyCheckpoint was already reached.
	 * 
	 * @param dependencyCheckpoint Tested CheckPoint.
	 * @return True if CheckPoint was already reached, false otherwise.
	 */
	private boolean checkDependency(Dependency dependencyCheckpoint, String contextId) {
		String type = dependencyCheckpoint.getType();
		String taskId = dependencyCheckpoint.getTaskId();
		Serializable magicObject = dependencyCheckpoint.getValue();
		CheckPointEntry[] checkPointEntries
			= this.data.getCheckPoints(type, taskId, contextId, magicObject);
		
		return ((checkPointEntries.length > 0) ? true : false);
	}
	
	/**
	 * Tests if all CheckPoints on which this task depends was already
	 * reached.
	 * 
	 * @param task Task to test.
	 * @return True if all dependencies were already reached, false
	 * 	otherwise.
	 */
	private boolean checkDependencies(TaskData taskData) {
		Dependency[] dependencyCheckPoints =
			taskData.getTaskDescriptor().getDependencyCheckPoints();
		
		for (int i = 0; i < dependencyCheckPoints.length; i++) {
			if (!this.checkDependency(dependencyCheckPoints[i],
					taskData.getTaskDescriptor().getContextId())) {
				logTrace("[" + taskData.getTaskDescriptor().getContextId()
						+ ":" + taskData.getTaskDescriptor().getTaskTid()
						+ "] Testing dependencies: failed.");
				return false;
			}
		}
		
		logTrace("[" + taskData.getTaskDescriptor().getContextId()
				+ ":" + taskData.getTaskDescriptor().getTaskTid()
				+ "] Testing dependencies: succeded.");
		return true;
	}
	
	/**
	 * Adds new task to list of tasks and starts it.
	 * 
	 * @param taskDescriptor Object representing task descriptor of new
	 * 	task.
	 * @throws IllegalArgumentException If taskDescriptor is not valid.
	 */
	private boolean taskNew(TaskDescriptor taskDescriptor)
	throws TaskManagerException {
		/* Check validity of taskDescriptor. */
		if (!taskDescriptor.validate()) {
			throw new IllegalArgumentException("Task descriptor is not valid.");
		}
		
		String taskId = taskDescriptor.getTaskTid();
		String contextId = taskDescriptor.getContextId();
		
		/* Test if task already exists. */
		TaskEntry foundTaskEntry = this.data.getTaskById(taskId, contextId);
		synchronized (data) {
			if (foundTaskEntry != null) {
				if ((foundTaskEntry.getState() == TaskState.FINISHED)
						|| (foundTaskEntry.getState() == TaskState.ABORTED)) {
					/* Remove this task. */
					this.data.delTask(taskId, contextId);
				}
			}

			/* Prepare packageName. */
			this.preparePackageName(taskDescriptor);

			/* Prepare TaskEntry. */
			TaskEntry taskEntry = new TaskEntry();
			taskEntry.setTaskId(taskId);
			taskEntry.setContextId(contextId);
			taskEntry.setTaskName(taskDescriptor.getTaskName());
			taskEntry.setTaskDescription(taskDescriptor.getTaskDescription());
			taskEntry.setPackageName(taskDescriptor.getPackageName());
			taskEntry.setExclusivity(taskDescriptor.getTaskExclusive());
			taskEntry.setRestartMax(taskDescriptor.getRestartMax());
			taskEntry.setTaskProperties(taskDescriptor.getTaskProperties());

			/* Prepare TaskData. */
			TaskData taskData = new TaskData(taskDescriptor);

			/* Add new task to TM's Data. */
			this.data.newTask(taskEntry, taskData);

			if (contextId.equals(SYSTEM_CONTEXT_ID)
					&& taskId.equals(TASKMANAGER_TASKNAME)) {
				throw new TaskManagerException("Task ID " + TASKMANAGER_TASKNAME 
						+ " in the context " + SYSTEM_CONTEXT_ID + " is reserved");
			}

			/* Add new task to log storage. */
			try {
				this.logStorage.addTask(contextId, taskId);
			} catch (LogStorageException e) {
				throw new TaskManagerException("Cannot register the task in the "
						+ "log storage", e);
			}

			try {
				return this.runTaskIfReady(taskEntry, taskData);
			} catch (TaskManagerException e) {
				logError("Cannot run task (contextId \"" + contextId
						+ "\", taskId \"" + taskId + "\"): " + e.getMessage());
				this.data.delTask(taskId, contextId);
				throw e;
			}
		}
	}
	
//	/**
//	 * Starts main loop of Task Manager.
//	 * 
//	 * @param taskDescriptors List of task descriptor files.
//	 */
//	private void run(String[] taskDescriptors) {
//		try {
//			runTask(taskDescriptors);
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//	}
	
	/**
	 * Returns RMI interface of hostRuntime based on hostName.
	 * 
	 * @param hostName Name of host running hostRuntime.
	 * @return RMI interface of hostRuntime.
	 */
	private HostRuntimeInterface getHostRuntimeInterface(String hostName) {
		URI hostRuntimeUri;
		
		try {
			hostRuntimeUri = new URI("rmi", null, hostName,
					Registry.REGISTRY_PORT,
					HostRuntimeInterface.URL, null, null);
		} catch (URISyntaxException e) {
			logError("Could not construct URI of Host Runtime.");
			return null;
		}
		
		HostRuntimeInterface hostRuntimeInterface;
		
		try {
			hostRuntimeInterface = (HostRuntimeInterface)
				Naming.lookup(hostRuntimeUri.toString());
		} catch (Exception e) {
			logError("Could not connect to required Host "
				+ "Runtime (URI=\"" + hostRuntimeUri.toString() + "\").");
			return null;
		}
		
		return hostRuntimeInterface;
	}
	
	/**
	 * Returns the size limit of the Host Runtime's package cache.
	 * 
	 * @return size limit of the Host Runtime's package cache.
	 * @throws RemoteException if something failed during the execution of
	 *          the remote method call.
	 */
	public long getMaxPackageCacheSize() throws RemoteException {
		return maxPackageCacheSize;
	}
	
	/**
	 * Set MaxPackageCacheSize option for hostRuntimes.
	 * 
	 * @param maxPackageCacheSize New value for maxPackageCacheSize option.
	 */
	public void setMaxPackageCacheSize(long maxPackageCacheSize) throws RemoteException {
		this.maxPackageCacheSize = maxPackageCacheSize;
		this.storeConfiguration(configurationFile);
		
		/* Distribute to all known hostRuntimes. */
		HostRuntimeEntry[] hostRuntimes = this.data.getHostRuntimes();
		for (HostRuntimeEntry hostRuntime : hostRuntimes) {
			String hostName = hostRuntime.getHostName();
			HostRuntimeInterface hostRuntimeInterface
				= this.getHostRuntimeInterface(hostName);
			if (hostRuntimeInterface != null) {
				hostRuntimeInterface
					.setMaxPackageCacheSize(maxPackageCacheSize);
			}
		}
	}
	
	/**
	 * Returns the number of closed contexts, for which the Host Runtime should
	 * keep data on the disk
	 * 
	 * @return number of closed contexts, for which the Host Runtime should keep
	 *          data on the disk
	 * @throws RemoteException if something failed during the execution of
	 *          the remote method call.
	 */
	public int getKeptClosedContextCount() throws RemoteException {
		return keptClosedContextCount;
	}
	
	/**
	 * Set KeptClosedContextCount option for hostRuntimes.
	 * 
	 * @param keptClosedContextCount New value for keptClosedContextCount
	 * 	option.
	 */
	public void setKeptClosedContextCount(int keptClosedContextCount) throws RemoteException {
		this.keptClosedContextCount = keptClosedContextCount;
		this.storeConfiguration(this.configurationFile);
		
		/* Distribute to all known hostRuntimes. */
		HostRuntimeEntry[] hostRuntimes = this.data.getHostRuntimes();
		for (HostRuntimeEntry hostRuntime : hostRuntimes) {
			String hostName = hostRuntime.getHostName();
			HostRuntimeInterface hostRuntimeInterface
				= this.getHostRuntimeInterface(hostName);
			if (hostRuntimeInterface != null) {
				hostRuntimeInterface
					.setKeptClosedContextCount(keptClosedContextCount);
			}
		}
	}

	/**
	 * @see cz.cuni.mff.been.taskmanager.HostRuntimesPortInterface#getTaskManager()
	 */
	public TaskManagerInterface getTaskManager() throws RemoteException {
		return this;
	}

	/**
	 * Searches the service registry and if it find a service with the same 
	 * task ID and context ID, it unregisteres it.
	 * 
	 * @param contextId
	 * @param taskId
	 */
	private void unregisterFinishedService(String contextId, String taskId) 
	throws RemoteException {
		ServiceEntry template = new ServiceEntry();
		template.setContextId(contextId);
		template.setTaskId(taskId);
		ServiceEntry[] registeredServices = serviceLook(template);
		if (registeredServices.length > 0) {
			for (ServiceEntry entry : registeredServices) {
				serviceRegistry.remove(entry);
			}
		}
	}
	
	/**
	 * Logs a log message.
	 * 
	 * @param level log level of the log message.
	 * @param message log message.
	 */
	private void log(LogLevel level, String message) {
		if (!level.isGreaterOrEqual(logLevel)) {
			return;
		}
		
		Date timestamp = new Date(System.currentTimeMillis());

		SimpleDateFormat format = 
			(SimpleDateFormat) DateFormat.getDateTimeInstance();
		format.applyPattern("dd.MM.yyyy HH:mm:ss.SSS");

		System.out.println(level + " " + message);

		try {
			logStorage.log(SYSTEM_CONTEXT_ID, 
					TASKMANAGER_TASKNAME, 
					timestamp,
					level,
					message);
		} catch (Exception e) {
			System.err.println("Unable to store log message: "
					+ e.getMessage());
		}
	}
	
	/**
	 * Logs a log message with the FATAL log level.
	 *  
	 * @param message log message.
	 * @throws TaskManagerException if something goes wrong.
	 */
	private void logFatal(String message) {
		log(LogLevel.FATAL, message);
	}

	/**
	 * Logs a log message with the ERROR log level.
	 *  
	 * @param message log message.
	 * @throws TaskManagerException if something goes wrong.
	 */
	private void logError(String message) {
		log(LogLevel.ERROR, message);
	}

	/**
	 * Logs a log message with the WARN log level.
	 *  
	 * @param message log message.
	 * @throws TaskManagerException if something goes wrong.
	 */
	private void logWarning(String message) {
		log(LogLevel.WARN, message);
	}

	/**
	 * Logs a log message with the INFO log level.
	 *  
	 * @param message log message.
	 * @throws TaskManagerException if something goes wrong.
	 */
	private void logInfo(String message) {
		log(LogLevel.INFO, message);
	}

	/**
	 * Logs a log message with the DEBUG log level.
	 *  
	 * @param message log message.
	 * @throws TaskManagerException if something goes wrong.
	 */
	private void logDebug(String message) {
		log(LogLevel.DEBUG, message);
	}

	/**
	 * Logs a log message with the TRACE log level.
	 *  
	 * @param message log message.
	 * @throws TaskManagerException if something goes wrong.
	 */
	private void logTrace(String message) {
		log(LogLevel.TRACE, message);
	}

	/*
	 * (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#registerEventListener(cz.cuni.mff.been.taskmanager.HostRuntimeRegistrationListener)
	 */
	public synchronized void registerEventListener(HostRuntimeRegistrationListener listener) throws RemoteException {
		if (listener == null) {
			throw new NullPointerException("Listener is null");
		}
		if (registrationListeners.contains(listener)) {
			throw new IllegalArgumentException("Listener already registered");
		}
		registrationListeners.add(listener);
	}

	/*
	 * (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#registerHostRuntime(java.lang.String)
	 */
	public synchronized void registerHostRuntime(String hostname) throws RemoteException {
		if (hostname == null) {
			throw new NullPointerException("Hostname is null");
		}
		
		String canonicalHostName; 
		try {
			canonicalHostName = InetAddress.getByName(hostname)
					.getCanonicalHostName();
		} catch (UnknownHostException e) {
			throw new RemoteException("Getting of canonical host name failed",
					e);
		}
		
		HostRuntimeEntry hostRuntime = new HostRuntimeEntry(canonicalHostName);
		this.data.addHostRuntime(hostRuntime);
		
		for (HostRuntimeRegistrationListener listener : registrationListeners) {
			listener.hostRuntimeRegistered(hostname);
		}
		
		logInfo("Host Runtime registered for host: " + hostname);
	}

	/*
	 * (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#unregisterEventListener(cz.cuni.mff.been.taskmanager.HostRuntimeRegistrationListener)
	 */
	public synchronized void unregisterEventListener(HostRuntimeRegistrationListener listener) throws RemoteException {
		if (listener == null) {
			throw new NullPointerException("Listener is null");
		}
		if (!registrationListeners.remove(listener)) {
			throw new IllegalArgumentException("Listener not registered");
		}
	}

	/*
	 * (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#unregisterHostRuntime(java.lang.String)
	 */
	public synchronized void unregisterHostRuntime(String hostName) throws RemoteException {
		if (hostName == null) {
			throw new NullPointerException("Hostname is null");
		}
		
		String canonicalHostName; 
		try {
			canonicalHostName = InetAddress.getByName(hostName)
					.getCanonicalHostName();
		} catch (UnknownHostException e) {
			throw new RemoteException("Getting of canonical host name failed",
					e);
		}
		
		/* Abort all non-finished tasks. */
		TaskEntry[] tasks = this.data.getTasksOnHost(hostName);
		for (TaskEntry task : tasks) {
			TaskState taskState = task.getState();
			if ((taskState != TaskState.FINISHED)
					&& (taskState != TaskState.ABORTED)) {
				this.taskReachedState(task.getTaskId(), task.getContextId(),
						TaskState.ABORTED);
			}
		}
		
		HostRuntimeEntry hostRuntime = new HostRuntimeEntry(canonicalHostName);
		this.data.removeHostRuntime(hostRuntime);
		
		for (HostRuntimeRegistrationListener listener : registrationListeners) {
			listener.hostRuntimeUnregistered(hostName);
		}
		
		logInfo("Host Runtime unregistered for host: " + hostName);
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#getRegisteredHostRuntimes()
	 */
	public String[] getRegisteredHostRuntimes() throws RemoteException {
		HostRuntimeEntry[] hostRuntimes = this.data.getHostRuntimes();
		String[] hostnames = new String[hostRuntimes.length];
		for (int i = 0; i < hostRuntimes.length; i++) {
			hostnames[i] = hostRuntimes[i].getHostName();
		}
		return hostnames;
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.taskmanager.TaskManagerInterface#deleteContext(java.lang.String)
	 */
	public void deleteContext(String id) throws RemoteException {
		if (id.equals(TaskManagerInterface.SYSTEM_CONTEXT_ID)) {
			// do not delete the "system" context
			return;
		}
		
		killContextById(id);
		removeContext(id);
	}
}
