/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Antonin Tomecek
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.taskmanager.data;

import java.io.Serializable;
import java.util.regex.Pattern;

/**
 * Class representing one entry (hostRuntime) in {@link
 * cz.cuni.mff.been.taskmanager.TaskManagerImplementation
 * <code>Task Manager</code>}.
 * 
 * @author Antonin Tomecek
 */
public class HostRuntimeEntry implements Cloneable, Serializable {
	
	/**
	 * Regular expression for match <code>hostName</code>.
	 */
	public static final String REGEXP_HOST_NAME = "^[a-zA-Z_0-9.-]+$";
	
	/* Current time on system creating this entry in milliseconds.
	 * Difference between current time and midnight, January 1, 1970 UTC. */
	private long currentTimeMillis;
	
	/* Name of host with this Host Runtime. */
	private String hostName = "";
	
	/* ID of context with reservation of this Host Runtime (task's exclusivity)
	 * null ... for any
	 * "" ... for none (Host Runtime is running exclusive task)
	 */
	private String reservation = null;
	
	/**
	 * Creates a new (empty - default inicialized)
	 * <code>HostRuntimeEntry</code>.
	 * This constructor is neded by <code>Serializable</code>.
	 */
	public HostRuntimeEntry() {
		this.currentTimeMillis = System.currentTimeMillis();
		
//		this.hostName = "";
	}
	
	/**
	 * Creates a new (and inicialized) <code>HostRuntimeEntry</code>.
	 * 
	 * @param hostName Name of host with this <code>Host Runtime</code>.
	 */
	public HostRuntimeEntry(String hostName) {
		this.currentTimeMillis = System.currentTimeMillis();
		
		/* Inicialization of variables with public write access. */
		this.setHostName(hostName);
	}
	
	/**
	 * Set name of host with this <code>Host Runtime</code>.
	 * 
	 * @param hostName Name of host with this <code>Host Runtime</code>.
	 */
	public void setHostName(String hostName) {
		if (Pattern.matches(REGEXP_HOST_NAME, hostName)) {
			this.hostName = hostName;
		} else {
			throw new IllegalArgumentException("hostName is not valid");
		}
	}
	
	/**
	 * Set ID of context with reservation of this Host Runtime (task's
	 * exclusivity).
	 * 
	 * @param reservation ID of context, <code>null</code> for any,
	 * 	<code>""</code> for none.
	 */
	public void setReservation(String reservation) {
		if ((reservation == null) || (reservation.equals(""))
				|| (Pattern.matches(
						ContextEntry.REGEXP_CONTEXT_ID, reservation))) {
			this.reservation = reservation;
		}
	}
	
	/**
	 * Get time of creation of this object (according with system creating this
	 * entry) in milliseconds. (Difference between current time and midnight,
	 * January 1, 1970 UTC.
	 * 
	 * @return Time of creation of this object.
	 */
	public long getCurentTime() {
		return this.currentTimeMillis;
	}
	
	/**
	 * Get name of host with this <code>Host Runtime</code>.
	 * 
	 * @return Name of host with this <code>Host Runtime</code>.
	 */
	public String getHostName() {
		return this.hostName;
	}
	
	/**
	 * Get ID of context with reservation of this Host Runtime (task's
	 * exclusivity).
	 * 
	 * @return ID of context, <code>null</code> if any, <code>""</code> if
	 * 	none.
	 */
	public String getReservation() {
		return this.reservation;
	}
	
	/**
	 * Creates and returns a copy of this object.
	 * Does not copy <code>currentTime</code> but uses real current time to
	 * inicialize it as used when constructing new object (i.e. clone is not
	 * fully identical with this object!).
	 */
	public HostRuntimeEntry clone() throws CloneNotSupportedException {
		HostRuntimeEntry hostRuntimeEntry = (HostRuntimeEntry) super.clone();
		
		/* Set current time. */
		hostRuntimeEntry.currentTimeMillis = System.currentTimeMillis();
		
		return hostRuntimeEntry;
	}
}
