/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Antonin Tomecek
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.taskmanager.data;

/**
 * Enum representing state of task in Task Manager.
 * 
 * @author Antonin Tomecek
 */
public enum TaskState {
	/**
	 * Task was submitted and not scheduled yet.
	 */
	SUBMITTED,
	/**
	 * Task was scheduled and not started yet.
	 */
	SCHEDULED,
	/**
	 * Task was started and not finished yet and currently is running.
	 */
	RUNNING,
	/**
	 * Task was started and not finished yet and currently is waiting (for
	 * something).
	 */
	SLEEPING,
	/**
	 * Task was finisched and not removed from TaskManager yet.
	 */
	FINISHED,
	/**
	 * Task was aborted and not removed from TaskManager yet.
	 */
	ABORTED;
	
	/* All enum values. */
	private static final TaskState[] VALUES = {
		SUBMITTED,
		SCHEDULED,
		RUNNING,
		SLEEPING,
		FINISHED,
		ABORTED
	};
	
	/* String representation for each task state. */
	private static final String[] STRINGS = {
		"submitted",
		"scheduled",
		"running",
		"sleeping",
		"finished",
		"aborted"
	};
	
	/* Human readable form of enum values. */
	private static final String[] HUMAN_READABLE = STRINGS;
	
	/**
	 * Returns a string representation of the object.
	 * 
	 * @return A string representation of the object.
	 */
	public String toString() {
		return HUMAN_READABLE[this.ordinal()];
	}
	
	/**
	 * Returns enum value represented by <code>String</code>.
	 * 
	 * @param string String representation of enum value.
	 * @return Enum value represented by <code>string</code>.
	 * @throws NullPointerException If input parameter is <code>null</code>.
	 * @throws IllegalArgumentException If value for specified
	 * 	<code>string</code> not found.
	 */
	public static TaskState fromString(String string) {
		/* Check input parameters. */
		if (string == null) {
			throw new NullPointerException("string is null");
		}
		
		/* Do search. */
		for (int i = 0; i < VALUES.length; i++) {
			if (string.equals(STRINGS[i])) {
				return VALUES[i];
			}
		}
		
		/* Throw exception if not found. */
		throw new IllegalArgumentException("No enum value has its string "
				+ "representation equal to \"" + string + "\"");
	}
}
