package cz.cuni.mff.been.webinterface.benchmarks;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import cz.cuni.mff.been.benchmarkmanager.EntityStatus;
import cz.cuni.mff.been.benchmarkmanager.ExperimentEntity;
import cz.cuni.mff.been.benchmarkmanager.gui.Role;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;
import cz.cuni.mff.been.taskmanager.CheckPoint;
import cz.cuni.mff.been.taskmanager.data.TaskEntry;
import cz.cuni.mff.been.taskmanager.data.TaskState;

/**
 * Wrapper class for the experiment entity in the Benchmark Manager.
 * 
 * @author David Majda
 */
public class Experiment {
	/** Wrapped entity. */
	private ExperimentEntity entity;
	/** Binaries in this experiment. */
	private Binary[] binaries;
	/** Tasks in this experiment. */
	private TaskEntry[] tasks;
	/** Reached checkpoints of tasks in this experiment. */ 
	private Map<TaskEntry, CheckPoint[]> checkpoints;
	
	/** @return experiment identifier */
	public EID getId() {
		return entity.getId();
	}
	
	/** @return experiment name */
	public String getName() {
		return entity.getName();
	}
	
	/** @return experiment status */
	public EntityStatus getStatus() {
		return entity.getStatus();
	}
	
	/** @return benchmarking roles of this experiment */
	public Role[] getRoles() {
		return entity.getBenchmarkRoles();
	}
	
	/** @return callback R scripts of this experiment */ 
	public CallbackRScripts getCallbackRScripts() {
		return entity.getScripts();
	}
	
	/**
	 * @return number of tasks in this experiment, contained binaries and runs
	 */
	public int aggregatedTaskCount() {
		int result = tasks.length;
		for (Binary binary: binaries) {
			result += binary.aggregatedTaskCount();
		}
		return result;
	}
	
	/**
	 * @return number of finished tasks in this experiment, contained binaries
	 *          and runs
	 */
	public int aggregatedFinishedTaskCount() {
		int result = 0;
		for (TaskEntry task: tasks) {
			if (task.getState().equals(TaskState.FINISHED)) {
				result++;
			}
		}
		for (Binary binary: binaries) {
			result += binary.aggregatedFinishedTaskCount();
		}
		return result;
	}

	/** @return number of runs in this experiment */
	public int runCount() {
		int result = 0;
		for (Binary binary: binaries) {
			result += binary.runCount();
		}
		return result;
	}

	/** @return number of binaries in this experiment */
	public int binaryCount() {
		return binaries.length;
	}

	/** @return bnaries in this experiment */
	public Binary[] getBinaries() {
		return binaries;
	}
	
	/** @return tasks in this experiment */
	public TaskEntry[] getTasks() {
		return tasks;
	}

	/** @return tasks in this experiment, contained binaries and runs */
	public TaskEntry[] getAggregatedTasks() {
		List<TaskEntry> result = new LinkedList<TaskEntry>();
		result.addAll(Arrays.asList(tasks));
		for (Binary binary: binaries) {
			result.addAll(Arrays.asList(binary.getAggregatedTasks()));
		}
		return result.toArray(new TaskEntry[result.size()]);
	}

	/** @return reached checkpoints of tasks in this experiment */
	public Map<TaskEntry, CheckPoint[]> getCheckpoints() {
		return checkpoints;
	}

	/** @return context ID of context associated with this experiment */
	public String getContextId() {
		return entity.getAnalysis() + "-" + entity.getId();
	}
	
	/**
	 * Allocates a new <code>Experiment</code> object.
	 * 
	 * @param entity wrapped entity
	 * @param binaries binaries in this experiment
	 * @param tasks tasks in this experiment
	 * @param checkpoints reached checkpoints of tasks in this experiment
	 */
	public Experiment(ExperimentEntity entity, Binary[] binaries,
			TaskEntry[] tasks, Map<TaskEntry, CheckPoint[]> checkpoints) {
		this.entity = entity;
		this.binaries = binaries;
		this.tasks = tasks;
		this.checkpoints = checkpoints;
	}
}
