/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.webinterface.modules;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.rmi.ConnectException;
import java.rmi.RemoteException;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.DiskFileUpload;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;

import cz.cuni.mff.been.common.ComponentInitializationException;
import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.resultsrepository.DeleteLockedException;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryBusyException;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryException;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryService;
import cz.cuni.mff.been.resultsrepository.gui.DataFormat;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.EntityInterface;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.GraphType;
import cz.cuni.mff.been.resultsrepository.gui.RFunctionHelp;
import cz.cuni.mff.been.resultsrepository.gui.RScriptPackage;
import cz.cuni.mff.been.resultsrepository.gui.ResultsRepositoryInterface;
import cz.cuni.mff.been.webinterface.InvalidParamValueException;
import cz.cuni.mff.been.webinterface.LogEntityColumn;
import cz.cuni.mff.been.webinterface.LogUtils;
import cz.cuni.mff.been.webinterface.MissingParamException;
import cz.cuni.mff.been.webinterface.Routines;
import cz.cuni.mff.been.webinterface.UploadResult;
import cz.cuni.mff.been.webinterface.Page.LayoutType;
import cz.cuni.mff.been.webinterface.event.Event;
import cz.cuni.mff.been.webinterface.event.EventListener;
import cz.cuni.mff.been.webinterface.ref.ServiceReference;
import cz.cuni.mff.been.webinterface.ref.TaskManagerReference;

/**
 * Web interface module for the Results Repository.
 * 
 * @author David Majda
 */
public class ResultsModule extends Module implements EventListener {
	private static final String RESULTS_REPOSITORY_BUSY_IMAGE
		= "img/results-repository-busy.png";
	private static final String RESULTS_REPOSITORY_ERROR_IMAGE
		= "img/results-repository-error.png";

	/** Class instance (singleton pattern). */
	private static ResultsModule instance;

	private TaskManagerReference taskManager = new TaskManagerReference();
	private ServiceReference<ResultsRepositoryInterface> resultsRepository
		= new ServiceReference<ResultsRepositoryInterface>(
			taskManager,
			ResultsRepositoryService.SERVICE_NAME,
			cz.cuni.mff.been.resultsrepository.gui.ResultsRepositoryInterface.RMI_NAME,
			ResultsRepositoryService.SERVICE_HUMAN_NAME
		);

	/**
	 * Allocates a new <code>ResultsModule</code> object. Construcor is private so only
	 * instance in <code>instance</code> field can be constructed (singleton
	 * pattern).
	 */
	private ResultsModule() {
		super();
		
		/* Initialize general module info... */
		id = "results";
		name = "Results";
		defaultAction = "analysis-list";
		
		menu = new MenuItem[] {
				new MenuItem("analysis-list", "Analyses"), 
				new MenuItem("package-list", "R script packages"), 
				new MenuItem("package-upload", "Upload R script package"), 
		};
	
		eventManager.registerEventListener(this);
	}

	/**
	 * Returns the only class instance (singleton pattern).
	 * 
	 * @return class instance
	 */
	public static ResultsModule getInstance() {
		if (instance == null) {
			 instance = new ResultsModule();
		}
		return instance;
	}

	/**
	 * Called by the event manager when status of some service changes
	 * programatically or when the configuation changes.
	 * 
	 * We invalidate remote reference because they could be meaningless now.
	 * 
	 * @param event sent event 
	 */
	public void receiveEvent(Event event) {
		taskManager.drop();
		resultsRepository.drop();
	}

	/**
	 * Invokes method for given action, which is found by reflection.
	 * 
	 * The method is overriden in this class to allow catching and processing
	 * exceptions thrown in executed methods in one place (so no big ugly
	 * <code>try { ... } catch { ... }</code> is needed in each method).
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @param action action to invoke
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if some kind of method invocation error occurs 
	 * @throws InvocationTargetException wraps an exception thrown by the invoked
	 *                                    method
	 *                                      
	 * @see cz.cuni.mff.been.webinterface.modules.Module#invokeMethodForAction(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, java.lang.String)
	 */
	@Override
	public void invokeMethodForAction(HttpServletRequest request,
			HttpServletResponse response, String action) throws ServletException,
			IOException, InvocationTargetException {
		try {
			super.invokeMethodForAction(request, response, action);
		} catch (InvocationTargetException e) {
			if (e.getCause() instanceof ConnectException) {
				throw new InvocationTargetException(
					new ConnectException(
						"<strong>Can't execute remote call to the Results Repository."
						+ "</strong><br /><br />"
						+ "Try to reload the page. If the error persists after multiple reloads, "
						+ "go to the <a href=\"../../services/>Services</a> tab and make "
						+ "sure the Results Repository is running.<br /><br/>"
						+ "Most probale causes of this error are network-related problems or "
						+ "crash of the service."
					),
					e.getMessage()
				);
			} else {
				throw e;
			}
		}
	}

	private AID ensureAndGetAid(HttpServletRequest request,
			EntityInterface<AID> analysisInterface)
			throws InvalidParamValueException, RemoteException,
			ComponentInitializationException {
		AID aid = null; // to shut up the compiler
		try {
			aid = AID.valueOf(request.getParameter("aid"));
		} catch (NumberFormatException e) {
			params.ensureCondition("aid", false);
		}
		return aid;
	}		

	private EID ensureAndGetEid(HttpServletRequest request,
			EntityInterface<EID> experimentInterface)
			throws InvalidParamValueException, RemoteException,
			ComponentInitializationException {
		EID eid = null; // to shut up the compiler
		try {
			eid = EID.valueOf(request.getParameter("eid"));
		} catch (NumberFormatException e) {
			params.ensureCondition("eid", false);
		}
		return eid;
	}

	private BID ensureAndGetBid(HttpServletRequest request,
			EntityInterface<BID> binaryInterface) throws InvalidParamValueException,
			RemoteException, ComponentInitializationException {
		BID bid = null; // to shut up the compiler
		try {
			bid = BID.valueOf(request.getParameter("bid"));
		} catch (NumberFormatException e) {
			params.ensureCondition("bid", false);
		}
		return bid;
	}

	private RID ensureAndGetRid(HttpServletRequest request,
			EntityInterface<RID> runInterface) throws InvalidParamValueException,
			RemoteException, ComponentInitializationException {
		RID rid = null; // to shut up the compiler
		try {
			rid = RID.valueOf(request.getParameter("rid"));
		} catch (NumberFormatException e) {
			params.ensureCondition("rid", false);
		}
		return rid;
	}
	
	private int ensureAndGetIndex(HttpServletRequest request)
			throws InvalidParamValueException {
		int result = 0; // to shut up the compiler
		try {
			result = Integer.valueOf(request.getParameter("index"));
		} catch (NumberFormatException e) {
			params.ensureCondition("index", false);
		}
		return result;
	}		

	@SuppressWarnings("unchecked") // Necessary to convert to generic interface.
	private EntityInterface<EID> getExperimentInterface(
			EntityInterface<AID> analysisInterface, AID aid)
			throws RemoteException, InvalidParamValueException {
		try {
			return analysisInterface.getSubentityInterface(aid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid", false);
			return null;
		}
	}	

	@SuppressWarnings("unchecked") // Necessary to convert to generic interface.
	private EntityInterface<BID> getBinaryInterface(
			EntityInterface<EID> experimentInterface, EID eid)
			throws RemoteException, InvalidParamValueException {
		try {
			return experimentInterface.getSubentityInterface(eid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("eid", false);
			return null;
		}
	}

	@SuppressWarnings("unchecked") // Necessary to convert to generic interface.
	private EntityInterface<RID> getRunInterface(
			EntityInterface<BID> binaryInterface, BID bid)
			throws RemoteException, InvalidParamValueException {
		try {
			return binaryInterface.getSubentityInterface(bid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("bid", false);
			return null;
		}
	}

	private Entity<AID> getAnalysis(EntityInterface<AID> analysisInterface, AID aid)
			throws RemoteException, InvalidParamValueException {
		try {
			return analysisInterface.getEntity(aid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid", false);
			return null;
		}
	}

	private Entity<RID> getRun(EntityInterface<RID> runInterface, RID rid)
			throws RemoteException, InvalidParamValueException {
		try {
			return runInterface.getEntity(rid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("rid", false);
			return null;
		}
	}

	private Entity<BID> getBinary(EntityInterface<BID> binaryInterface, BID bid)
			throws RemoteException, InvalidParamValueException {
		try {
			return binaryInterface.getEntity(bid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("bid", false);
			return null;
		}
	}

	private Entity<EID> getExperiment(EntityInterface<EID> experimentInterface, EID eid)
			throws RemoteException, InvalidParamValueException {
		try {
			return experimentInterface.getEntity(eid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("eid", false);
			return null;
		}
	}

	private <ID extends cz.cuni.mff.been.common.id.ID> void downloadGraph(
			GraphType type, EntityInterface<ID> entityInterface, ID id, int index,
			HttpServletResponse response) throws IOException,
			ResultsRepositoryBusyException, ResultsRepositoryException {
		ServerSocket serverSocket = new ServerSocket(0); // 0 = use any port
		DownloadHandle handle = null; 
		handle = entityInterface.beginGraphDownload(
			type, 
			id,
			InetAddress.getLocalHost(),
			serverSocket.getLocalPort(),
			index
		);
		byte[] buffer = new byte[ResultsRepositoryInterface.GRAPH_BUFFER_SIZE];
		int bytesRead;
		Socket socket = serverSocket.accept();
		try {
			InputStream inputStream = new BufferedInputStream(
				socket.getInputStream(),
				ResultsRepositoryInterface.GRAPH_BUFFER_SIZE
			);
			OutputStream outputStream = new BufferedOutputStream(
				response.getOutputStream(),
				ResultsRepositoryInterface.GRAPH_BUFFER_SIZE
			);
			try {
				while ((bytesRead = inputStream.read(buffer)) != -1) {
					outputStream.write(buffer, 0, bytesRead);     
				}
			}	finally {
				inputStream.close();
				outputStream.close();
			}
		} finally {
			socket.close();
			serverSocket.close();
			entityInterface.endGraphDownload(handle);
		}
	}

	private <ID extends cz.cuni.mff.been.common.id.ID> void sendGraphOrBusyOrErrorImage(
			GraphType type, EntityInterface<ID> entityInterface, ID id, int index,
			HttpServletResponse response) throws IOException,
			ComponentInitializationException {
		response.setContentType(ResultsRepositoryInterface.GRAPH_MIME_TYPE);
		try {
			downloadGraph(type, entityInterface, id, index, response);
		} catch (ResultsRepositoryBusyException e) {
			response.sendRedirect(page.getRootPath() + RESULTS_REPOSITORY_BUSY_IMAGE);
		} catch (ResultsRepositoryException e) {
			response.sendRedirect(page.getRootPath() + RESULTS_REPOSITORY_ERROR_IMAGE);
		}
	}	

	private <ID extends cz.cuni.mff.been.common.id.ID> GraphInfo[] getGraphInfos(
			GraphType type, EntityInterface<ID> entityInterface, ID id)
			throws RemoteException {
		try {
			GraphInfo[] result = new GraphInfo[entityInterface.graphCount(type, id)];
			for (int i = 0; i < result.length; i++) {
				result[i] = entityInterface.getGraphInfo(type, id, i);
			}
			return result;
		} catch (ResultsRepositoryException e) {
			/* In case of exception we are silent and just don't send info about any
			 * graphs.
			 */
			return new GraphInfo[0];
		}
	}

	private <ID extends cz.cuni.mff.been.common.id.ID> void downloadRawData(
			EntityInterface<ID> entityInterface, ID id, String filename,
			HttpServletResponse response) throws IOException,
			ResultsRepositoryBusyException, ResultsRepositoryException {
		ServerSocket serverSocket = new ServerSocket(0); // 0 = use any port
		DownloadHandle handle = null;
		handle = entityInterface.beginRawDataDownload(
			id,
			InetAddress.getLocalHost(),
			serverSocket.getLocalPort()
		);
		byte[] buffer = new byte[ResultsRepositoryInterface.RAW_DATA_BUFFER_SIZE];
		int bytesRead;
		Socket socket = serverSocket.accept();
		try {
			response.setContentType(ResultsRepositoryInterface.RAW_DATA_MIME_TYPE);
			response.setHeader("Content-Disposition", "attachment; filename=\""
				+ filename + "\"");
			InputStream inputStream = new BufferedInputStream(
				socket.getInputStream(),
				ResultsRepositoryInterface.RAW_DATA_BUFFER_SIZE
			);
			OutputStream outputStream = new BufferedOutputStream(
				response.getOutputStream(),
				ResultsRepositoryInterface.RAW_DATA_BUFFER_SIZE
			);
			try {
				while ((bytesRead = inputStream.read(buffer)) != -1) {
					outputStream.write(buffer, 0, bytesRead);     
				}
			}	finally {
				inputStream.close();
				outputStream.close();
			}
		} finally {
			socket.close();
			serverSocket.close();
			entityInterface.endRawDataDownload(handle);
		}
	}
	
	public static <ID extends cz.cuni.mff.been.common.id.ID> LogRecord[] 
			getLogRecordsForTasksFromEntity(EntityInterface<ID> entityInterface, ID id)
			throws RemoteException, LogStorageException {
		String[] tids = entityInterface.getTasks(id);
		
		List<LogRecord[]> logRecordsList = new LinkedList<LogRecord[]>();
		for (String tid: tids) {
			logRecordsList.add(entityInterface.getLogsForTask(id, tid));
		}
		
		return LogUtils.aggregateLogs(logRecordsList);
	}

	public static <ID extends cz.cuni.mff.been.common.id.ID> LogRecord[] 
		  getLogRecordsForTasksFromEntities(EntityInterface<ID> entityInterface)
			throws RemoteException, LogStorageException {
		List<LogRecord[]> logRecordsList = new LinkedList<LogRecord[]>();
		List<Entity<ID>> entities = entityInterface.getEntities();
		for (Entity<ID> entity: entities) {
			String[] tids = entityInterface.getTasks(entity.getId());

			for (String tid: tids) {
				logRecordsList.add(entityInterface.getLogsForTask(entity.getId(), tid));
			}
		}
		return LogUtils.aggregateLogs(logRecordsList);
	}

	public static <ID extends cz.cuni.mff.been.common.id.ID> LogEntityColumn<ID>
			buildEntityColumn(String title, String urlTemplate,
			EntityInterface<ID> entityInterface) throws RemoteException {
		LogEntityColumn<ID> result = new LogEntityColumn<ID>(title, urlTemplate);
		List<Entity<ID>> entities = entityInterface.getEntities();
		
		for (Entity<ID> entity: entities) {
			ID id = entity.getId();
			String name = entity.getName();
			/* Note: Runs have null name, so we pass the stringified ID as the name
			 * in that case.
			 */ 
			LogEntityColumn.Value<ID> value = new LogEntityColumn.Value<ID>(
				id,
				name != null ? name: id.toString()
			);
			String contextID = entityInterface.getContext(id);
			for (String taskID: entityInterface.getTasks(id)) {
				result.addValue(contextID, taskID, value);
			}
		}
		
		return result;
	}
	
	public void testRScriptingEnabled() throws RemoteException,
			ComponentInitializationException {
		if (!resultsRepository.get().rScriptingEnabled()) {
			page.getWarningMessages().addHTMLMessage("R scripting is not enabled in"
				+ " the Results Repository - statistic computations will not be "
				+ "performed. See <a href=\"../../services/logs/?service=resultsrepository\">"
				+ "service logs</a> for the reason.");
		}
	}
	
	private void handleDeleteLockedException() throws ServletException, IOException {
		errorMessages.addTextMessage("Unable to delete entity at the moment, it is "
			+	"being processed. Retry deleting later.");
		
		page.setTitle("Error deleting entity");
		page.writeHeader();
		page.writeErrorMessages();
		page.writeFooter();
		return;
	}

	/**
	 * Handles the "analysis-list" action. 
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	public void analysisList(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException, InvalidParamValueException,
			ComponentInitializationException {
		testRScriptingEnabled();

		if (params.exists("action")) {
			String action = request.getParameter("action");
			if (action.equals("deleted")) {
				infoMessages.addTextMessage("Analysis deleted successfully.");
			} else {
				throw new InvalidParamValueException("Parameter \"action\" has invalid value.");
			}
		}
		
		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
					
		HashMap<String, Object> data = new HashMap<String, Object>();
		data.put("analyses", analysisInterface.getEntities());
		data.put("file", "");
		data.put("importFormats", analysisInterface.getImportFormats());
		data.put("exportFormats", analysisInterface.getExportFormats());
				
		page.setShowTitle(false);
		page.setTitle("Analyses");
		page.writeHeader();
		page.writeTemplate("results-analysis-list", data);
		page.writeFooter();
	}

	/**
	 * Handles the "analysis-details" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void analysisDetails(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			ServletException, IOException, ComponentInitializationException {
		testRScriptingEnabled();

		String activeSheet = "statistics-sheet";
		if (params.exists("action")) {
			String action = request.getParameter("action");
			if (action.equals("changed-comment")) {
				infoMessages.addTextMessage("Comment changed successfully.");
				activeSheet = "metadata-sheet";
			} else if (action.equals("deleted")) {
				infoMessages.addTextMessage("Experiment deleted successfully.");
			} else {
				throw new InvalidParamValueException("Parameter \"action\" has invalid value.");
			}
		}

		params.ensureExists("aid");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		Entity<AID> analysis = getAnalysis(analysisInterface, aid);
		
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid); 
		
		if (params.exists("change-comment")) {
			params.ensureExists("comment");

			analysisInterface.setEntityComment(
				aid,
				request.getParameter("comment")
			);
			
			if (errorMessages.isEmpty()) {
				HashMap<String, String> actionParams = new HashMap<String, String>();
				actionParams.put("aid", aid.toString());
				actionParams.put("action", "changed-comment");
				page.redirectToAction("analysis-details", actionParams);
				return;
			}
		}

		LogRecord[] logRecords = null;
		try {
			logRecords = getLogRecordsForTasksFromEntities(experimentInterface);
		} catch (LogStorageException e) {
			errorMessages.addTextMessage("Error retrieving logs: " + e.getMessage());
		}
		
		LogEntityColumn<EID> experimentColumn = buildEntityColumn(
			"Experiment",
			"../experiment-details/?aid=" + aid.toString() + "&eid=%s",
			experimentInterface
		);

		HashMap<String, Object> data = new HashMap<String, Object>();
		data.put("analysis", analysis);
		data.put("experiments", experimentInterface.getEntities());
		data.put("graphInfos", getGraphInfos(GraphType.RESULTS, analysisInterface, aid));
		data.put("logRecords", logRecords);
		data.put("logFields", EnumSet.of(
			LogRecord.Fields.CONTEXT,
			LogRecord.Fields.TASK_ID,
			LogRecord.Fields.HOSTNAME,
			LogRecord.Fields.TIMESTAMP,
			LogRecord.Fields.LEVEL,
			LogRecord.Fields.MESSAGE
		));
		data.put("logEntityColumns", new LogEntityColumn[] { experimentColumn });
		data.put("file", "");
		data.put("importFormats", analysisInterface.getImportFormats());
		data.put("exportFormats", analysisInterface.getExportFormats());
		data.put("activeSheet", activeSheet);
					
		page.setShowTitle(false);
		page.setTitle("Analysis details: " + analysis.getName());
		page.writeHeader();
		page.writeTemplate("results-analysis-details", data);
		page.writeFooter();
	}
	
	/**
	 * Handles the "analysis-results-graph" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	public void analysisResultsGraph(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			ServletException, IOException, ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExist("aid", "index");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		int index = ensureAndGetIndex(request);

		try {
			sendGraphOrBusyOrErrorImage(GraphType.RESULTS, analysisInterface, aid, index, response);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or index", false);
		}
	}
	
	private void ensureFormat(HttpServletRequest request,
			DataFormat[] formats) throws InvalidParamValueException {
		String format = request.getParameter("format");
		for (DataFormat f: formats) {
			if (f.getId().equals(format)) {
				return;
			}
		}
		params.ensureCondition("format", false);
	}

	private <T> void writeExportHeaders(String filenameBase, T[] ids,
			String formatId, HttpServletResponse response) {
		response.setContentType(formatId);
		response.setHeader("Content-Disposition", "attachment; filename=\""
				+ filenameBase + "-" + Routines.join(",", ids) + "\"");
	}

	private void doEntityExport(HttpServletResponse response,
			ServerSocket serverSocket, DownloadHandle handle)
			throws IOException {
		byte[] buffer = new byte[ResultsRepositoryInterface.EXPORT_BUFFER_SIZE];
		int bytesRead;
		Socket socket = serverSocket.accept();
		try {
			InputStream inputStream = new BufferedInputStream(
				socket.getInputStream(),
				ResultsRepositoryInterface.EXPORT_BUFFER_SIZE
  		);
			OutputStream outputStream = new BufferedOutputStream(
				response.getOutputStream(),
				ResultsRepositoryInterface.EXPORT_BUFFER_SIZE
			);
			try {
				while ((bytesRead = inputStream.read(buffer)) != -1) {
					outputStream.write(buffer, 0, bytesRead);     
				}
			}	finally {
				inputStream.close();
				outputStream.close();
			}
		} finally {
			socket.close();
			serverSocket.close();
		}
	}

	private void exportAnalyses(EntityInterface<AID> analysisInterface,
			AID[] aids, String formatId, HttpServletResponse response)
			throws IOException, ResultsRepositoryBusyException,
			ComponentInitializationException {
		ServerSocket serverSocket = new ServerSocket(0); // 0 = use any port
		DownloadHandle handle = analysisInterface.beginExport(
			aids,
			formatId,
			InetAddress.getLocalHost(),
			serverSocket.getLocalPort()
		);
		try {
			writeExportHeaders("analyses", aids, formatId, response);
			doEntityExport(response, serverSocket, handle);
		} finally {
			analysisInterface.endExport(handle);
		}
	}
	
	/**
	 * Handles the "analysis-export" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	public void analysisExport(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			IOException, ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExist("aids", "format");
		
		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		ensureFormat(request, analysisInterface.getExportFormats());
		
		String[] aidStrings = Routines.split2(",", request.getParameter("aids"));
		AID[] aids = new AID[aidStrings.length];
		for (int i = 0; i < aidStrings.length; i++) {
			try {
				aids[i] = AID.valueOf(aidStrings[i]);
			} catch (NumberFormatException e) {
				params.ensureCondition("aids", false);
			}
		}
		
		try {
			exportAnalyses(analysisInterface, aids, request.getParameter("format"),
				response);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aids", false);
		} catch (ResultsRepositoryBusyException e) {
			errorMessages.addTextMessage("Results Repository is currently busy and "
				+ "can't export selected entities. Try waiting for a while and reload "
				+ "the page. If the problem persists, see the logs for explanation.");
		}
	}

	/**
 	 * Handles the "analysis-delete" action.
 	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	public void analysisDelete(HttpServletRequest request, HttpServletResponse response)
			throws IOException, MissingParamException, InvalidParamValueException,
			ComponentInitializationException, ServletException {
		testRScriptingEnabled();

		params.ensureExists("aid");
		
		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);

		try {
			analysisInterface.deleteEntity(aid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid", false);
		} catch (DeleteLockedException e) {
			handleDeleteLockedException();
			return;
		}
		
		HashMap<String, String> actionParams = new HashMap<String, String>();
		actionParams.put("action", "deleted");
		page.redirectToAction("analysis-list", actionParams);
	}

	/**
	 * Handles the "experiment-details" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void experimentDetails(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			ServletException, IOException, ComponentInitializationException {
		testRScriptingEnabled();

		String activeSheet = "statistics-sheet";
		if (params.exists("action")) {
			String action = request.getParameter("action");
			if (action.equals("changed-comment")) {
				infoMessages.addTextMessage("Comment changed successfully.");
				activeSheet = "metadata-sheet";
			} else if (action.equals("deleted")) {
				infoMessages.addTextMessage("Binary deleted successfully.");
			} else {
				throw new InvalidParamValueException("Parameter \"action\" has invalid value.");
			}
		}

		params.ensureExist("aid", "eid");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		Entity<AID> analysis = getAnalysis(analysisInterface, aid);
		
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		Entity<EID> experiment = getExperiment(experimentInterface, eid);
		
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		
		if (params.exists("change-comment")) {
			params.ensureExists("comment");
				
			experimentInterface.setEntityComment(
				eid,
				request.getParameter("comment")
			);
			
			if (errorMessages.isEmpty()) {
				HashMap<String, String> actionParams = new HashMap<String, String>();
				actionParams.put("aid", aid.toString());
				actionParams.put("eid", eid.toString());
				actionParams.put("action", "changed-comment");
				page.redirectToAction("experiment-details", actionParams);
				return;
			}
		}
		
		LogRecord[] logRecords = null;
		try {
			logRecords = getLogRecordsForTasksFromEntity(experimentInterface, eid);
		} catch (LogStorageException e) {
			errorMessages.addTextMessage("Error retrieving logs: " + e.getMessage());
		}

		LogEntityColumn<BID> binaryColumn = buildEntityColumn(
			"Binary",
			"../binary-details/?aid=" + aid.toString() + "&eid=" + eid.toString()
				+ "&bid=%s",
			binaryInterface
		);

		HashMap<String, Object> data = new HashMap<String, Object>();
		data.put("analysis", analysis);
		data.put("experiment", experiment);
		data.put("binaries", binaryInterface.getEntities());
		data.put("graphInfos", getGraphInfos(GraphType.RESULTS, experimentInterface, eid));
		data.put("logRecords", logRecords);
		data.put("logFields", EnumSet.of(
			LogRecord.Fields.CONTEXT,
			LogRecord.Fields.TASK_ID,
			LogRecord.Fields.HOSTNAME,
			LogRecord.Fields.TIMESTAMP,
			LogRecord.Fields.LEVEL,
			LogRecord.Fields.MESSAGE
		));
		data.put("logEntityColumns", new LogEntityColumn[] { binaryColumn });
		data.put("file", "");
		data.put("importFormats", experimentInterface.getImportFormats());
		data.put("exportFormats", experimentInterface.getExportFormats());
		data.put("activeSheet", activeSheet);
			
		page.setShowTitle(false);
		page.setTitle("Experiment details: " + experiment.getName());
		page.writeHeader();
		page.writeTemplate("results-experiment-details", data);
		page.writeFooter();
	}
	
	/**
	 * Handles the "experiment-results-graph" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void experimentResultsGraph(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
				ServletException, IOException, ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eid", "index");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		int index = ensureAndGetIndex(request);

		try {
			sendGraphOrBusyOrErrorImage(GraphType.RESULTS, experimentInterface, eid, index, response);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eid or index", false);
		}
	}


	private void exportExperiments(EntityInterface<EID> experimentInterface,
			AID aid, EID[] eids, String formatId, HttpServletResponse response)
			throws IOException, ResultsRepositoryBusyException,
			ComponentInitializationException {
		ServerSocket serverSocket = new ServerSocket(0); // 0 = use any port
		DownloadHandle handle = experimentInterface.beginExport(
			eids, formatId,
			InetAddress.getLocalHost(),
			serverSocket.getLocalPort()
		);
		try {
			writeExportHeaders("experiments", eids, formatId, response);
			doEntityExport(response, serverSocket, handle);
		} finally {
			experimentInterface.endExport(handle);
		}
	}

	/**
	 * Handles the "experiment-export" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void experimentExport(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			IOException, ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eids", "format");
		
		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		ensureFormat(request, experimentInterface.getExportFormats());
		
		String[] eidStrings = Routines.split2(",", request.getParameter("eids"));
		EID[] eids = new EID[eidStrings.length];
		for (int i = 0; i < eidStrings.length; i++) {
			try {
				eids[i] = EID.valueOf(eidStrings[i]);
			} catch (NumberFormatException e) {
				params.ensureCondition("eids", false);
			}
		}
		
		try {
			exportExperiments(experimentInterface, aid, eids,
				request.getParameter("format"), response);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eids", false);
		} catch (ResultsRepositoryBusyException e) {
			errorMessages.addTextMessage("Results Repository is currently busy and "
				+ "can't export selected entities. Try waiting for a while and reload "
				+ "the page. If the problem persists, see the logs for explanation.");
		}
	}

	/**
 	 * Handles the "experiment-delete" action.
 	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void experimentDelete(HttpServletRequest request, HttpServletResponse response)
			throws IOException, MissingParamException, InvalidParamValueException,
			ComponentInitializationException, ServletException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eid");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		
		try {
			experimentInterface.deleteEntity(eid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eid", false);
		} catch (DeleteLockedException e) {
			handleDeleteLockedException();
			return;
		}
		
		HashMap<String, String> actionParams = new HashMap<String, String>();
		actionParams.put("aid", aid.toString());
		actionParams.put("action", "deleted");
		page.redirectToAction("analysis-details", actionParams);
	}	
	
	/**
	 * Handles the "binary-details" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void binaryDetails(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			ServletException, IOException, ComponentInitializationException {
		testRScriptingEnabled();

		String activeSheet = "statistics-sheet";
		if (params.exists("action")) {
			String action = request.getParameter("action");
			if (action.equals("changed-comment")) {
				infoMessages.addTextMessage("Comment changed successfully.");
				activeSheet = "metadata-sheet";
			} else if (action.equals("deleted")) {
				infoMessages.addTextMessage("Run deleted successfully.");
			} else {
				throw new InvalidParamValueException("Parameter \"action\" has invalid value.");
			}
		}

		params.ensureExist("aid", "eid", "bid");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		Entity<AID> analysis = getAnalysis(analysisInterface, aid);
		
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		Entity<EID> experiment = getExperiment(experimentInterface, eid);
		
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		BID bid = ensureAndGetBid(request, binaryInterface);
		Entity<BID> binary = getBinary(binaryInterface, bid);
		EntityInterface<RID> runInterface = getRunInterface(binaryInterface, bid);
	
		if (params.exists("change-comment")) {
			params.ensureExists("comment");
				
			binaryInterface.setEntityComment(
				bid,
				request.getParameter("comment")
			);
			
			if (errorMessages.isEmpty()) {
				HashMap<String, String> actionParams = new HashMap<String, String>();
				actionParams.put("aid", aid.toString());
				actionParams.put("eid", eid.toString());
				actionParams.put("bid", bid.toString());
				actionParams.put("action", "changed-comment");
				page.redirectToAction("binary-details", actionParams);
				return;
			}
		}

		LogRecord[] logRecords = null;
		try {
			logRecords = getLogRecordsForTasksFromEntity(binaryInterface, bid);
		} catch (LogStorageException e) {
			errorMessages.addTextMessage("Error retrieving logs: " + e.getMessage());
		}

		LogEntityColumn<RID> runColumn = buildEntityColumn(
			"Run",
			"../run-details/?aid=" + aid.toString() + "&eid=" + eid.toString()
				+ "&bid=" + bid.toString() + "&rid=%s",
			runInterface
		);

		HashMap<String, Object> data = new HashMap<String, Object>();
		data.put("analysis", analysis);
		data.put("experiment", experiment);
		data.put("binary", binary);
		data.put("runs", runInterface.getEntities());
		data.put("logRecords", logRecords);
		data.put("logFields", EnumSet.of(
			LogRecord.Fields.CONTEXT,
			LogRecord.Fields.TASK_ID,
			LogRecord.Fields.HOSTNAME,
			LogRecord.Fields.TIMESTAMP,
			LogRecord.Fields.LEVEL,
			LogRecord.Fields.MESSAGE
		));
		data.put("logEntityColumns", new LogEntityColumn[] { runColumn });
		data.put("file", "");
		data.put("importFormats", binaryInterface.getImportFormats());
		data.put("exportFormats", binaryInterface.getExportFormats());
		data.put("activeSheet", activeSheet);
			
		page.setShowTitle(false);
		page.setTitle("Binary details: " + binary.getName());
		page.writeHeader();
		page.writeTemplate("results-binary-details", data);
		page.writeFooter();
	}

	private void exportBinaries(EntityInterface<BID> binaryInterface,
			AID aid, EID eid, BID[] bids, String formatId,
			HttpServletResponse response)
			throws IOException, ResultsRepositoryBusyException,
			ComponentInitializationException {
		ServerSocket serverSocket = new ServerSocket(0); // 0 = use any port
		DownloadHandle handle = binaryInterface.beginExport(
			bids, formatId,
			InetAddress.getLocalHost(),
			serverSocket.getLocalPort()
		);
		try {
			writeExportHeaders("binaries", bids, formatId, response);
			doEntityExport(response, serverSocket, handle);
		} finally {
			binaryInterface.endExport(handle);
		}
	}

	/**
	 * Handles the "binary-export" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void binaryExport(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			IOException, ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eid", "bids", "format");
		
		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		ensureFormat(request, binaryInterface.getExportFormats());

		String[] bidStrings = Routines.split2(",", request.getParameter("bids"));
		BID[] bids = new BID[bidStrings.length];
		for (int i = 0; i < bidStrings.length; i++) {
			try {
				bids[i] = BID.valueOf(bidStrings[i]);
			} catch (NumberFormatException e) {
				params.ensureCondition("bids", false);
			}
		}
		
		try {
			exportBinaries(binaryInterface, aid, eid, bids,
				request.getParameter("format"), response);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eid or bids", false);
		} catch (ResultsRepositoryBusyException e) {
			errorMessages.addTextMessage("Results Repository is currently busy and "
				+ "can't export selected entities. Try waiting for a while and reload "
				+ "the page. If the problem persists, see the logs for explanation.");
		}
	}

	/**
 	 * Handles the "binary-delete" action.
 	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void binaryDelete(HttpServletRequest request, HttpServletResponse response)
			throws IOException, MissingParamException, InvalidParamValueException,
			ComponentInitializationException, ServletException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eid", "bid");
	
		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		BID bid = ensureAndGetBid(request, binaryInterface);

		try {
			binaryInterface.deleteEntity(bid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eid or bid", false);
		} catch (DeleteLockedException e) {
			handleDeleteLockedException();
			return;
		}
		
		HashMap<String, String> actionParams = new HashMap<String, String>();
		actionParams.put("aid", aid.toString());
		actionParams.put("eid", eid.toString());
		actionParams.put("action", "deleted");
		page.redirectToAction("experiment-details", actionParams);
	}		

	/**
	 * Handles the "run-details" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void runDetails(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			ServletException, IOException, ComponentInitializationException {
		testRScriptingEnabled();

		String activeSheet = null;
		if (params.exists("action")) {
			String action = request.getParameter("action");
			if (action.equals("changed-comment")) {
				infoMessages.addTextMessage("Comment changed successfully.");
				activeSheet = "metadata-sheet";
			} else {
				throw new InvalidParamValueException("Parameter \"action\" has invalid value.");
			}
		}

		params.ensureExist("aid", "eid", "bid", "rid");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		Entity<AID> analysis = getAnalysis(analysisInterface, aid);
		
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		Entity<EID> experiment = getExperiment(experimentInterface, eid);
		
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		BID bid = ensureAndGetBid(request, binaryInterface);
		Entity<BID> binary = getBinary(binaryInterface, bid);
		
		EntityInterface<RID> runInterface = getRunInterface(binaryInterface, bid);
		RID rid = ensureAndGetRid(request, runInterface);
		Entity<RID> run = getRun(runInterface, rid);

		if (params.exists("change-comment")) {
			params.ensureExists("comment");
				
			runInterface.setEntityComment(
				rid,
				request.getParameter("comment")
			);
			
			if (errorMessages.isEmpty()) {
				HashMap<String, String> actionParams = new HashMap<String, String>();
				actionParams.put("aid", aid.toString());
				actionParams.put("eid", eid.toString());
				actionParams.put("bid", bid.toString());
				actionParams.put("rid", rid.toString());
				actionParams.put("action", "changed-comment");
				page.redirectToAction("run-details", actionParams);
				return;
			}
		}

		LogRecord[] logRecords = null;
		try {
			logRecords = getLogRecordsForTasksFromEntity(runInterface, rid);
		} catch (LogStorageException e) {
			errorMessages.addTextMessage("Error retrieving logs: " + e.getMessage());
		}

		if (activeSheet == null) {
			activeSheet = run.isValid() ? "statistics-sheet" : "raw-data-sheet";
		}
		
		HashMap<String, Object> data = new HashMap<String, Object>();
		data.put("analysis", analysis);
		data.put("experiment", experiment);
		data.put("binary", binary);
		data.put("run", run);
		data.put("graphInfos", getGraphInfos(GraphType.LOAD, runInterface, rid));
		data.put("logRecords", logRecords);
		data.put("logFields", EnumSet.of(
			LogRecord.Fields.CONTEXT,
			LogRecord.Fields.TASK_ID,
			LogRecord.Fields.HOSTNAME,
			LogRecord.Fields.TIMESTAMP,
			LogRecord.Fields.LEVEL,
			LogRecord.Fields.MESSAGE
		));
		data.put("logEntityColumns", new LogEntityColumn[] {});
		data.put("activeSheet", activeSheet);

		page.setShowTitle(false);
		page.setTitle("Run details: " + run.getId().toString());
		page.writeHeader();
		page.writeTemplate("results-run-details", data);
		page.writeFooter();
	}

	/**
	 * Handles the "run-load-graph" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void runLoadGraph(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
				ServletException, IOException, ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eid", "bid", "rid", "index");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		BID bid = ensureAndGetBid(request, binaryInterface);
		EntityInterface<RID> runInterface = getRunInterface(binaryInterface, bid);
		RID rid = ensureAndGetRid(request, runInterface);
		int index = ensureAndGetIndex(request);

		try {
			sendGraphOrBusyOrErrorImage(GraphType.LOAD, runInterface, rid, index, response);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eid or bid or rid or index", false);
		}
	}	
		
	/**
	 * Handles the "run-raw-data" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void runRawData(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
				ServletException, IOException, ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eid", "bid", "rid");

		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		BID bid = ensureAndGetBid(request, binaryInterface);
		EntityInterface<RID> runInterface = getRunInterface(binaryInterface, bid);
		RID rid = ensureAndGetRid(request, runInterface);

		try {
			downloadRawData(
				runInterface,
				rid,
				"raw-run-data-"
					+ Routines.join(
						"-",
						new cz.cuni.mff.been.common.id.ID[] { aid, eid, bid, rid}
					)
					+ ".zip",
				response);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eid or bid or rid", false);
		} catch (ResultsRepositoryBusyException e) {
			errorMessages.addTextMessage("Results Repository is currently busy and "
				+ "can't send raw entity data. Try waiting for a while and reload "
				+ "the page. If the problem persists, see the logs for explanation.");
		} catch (ResultsRepositoryException e) {
			errorMessages.addTextMessage("Can't retrieve graph from the Results "
				+ "Repository. Try waiting for a while and reload the page. If the "
				+ "problem persists, see the logs for explanation.");
		}
	}	

	private void exportRuns(EntityInterface<RID> runInterface,
			AID aid, EID eid, BID bid, RID[] rids, String formatId,
			HttpServletResponse response)
			throws IOException, ResultsRepositoryBusyException,
			ComponentInitializationException {
		ServerSocket serverSocket = new ServerSocket(0); // 0 = use any port
		DownloadHandle handle = runInterface.beginExport(
			rids, formatId,
			InetAddress.getLocalHost(),
			serverSocket.getLocalPort()
		);
		try {
			writeExportHeaders("runs", rids, formatId, response);
			doEntityExport(response, serverSocket, handle);
		} finally {
			runInterface.endExport(handle);
		}
	}

	/**
	 * Handles the "run-export" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws IOException if some I/O error occurs
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void runExport(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, InvalidParamValueException, 
			IOException, ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eid", "bid", "rids", "format");
		
		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		BID bid = ensureAndGetBid(request, binaryInterface);
		EntityInterface<RID> runInterface = getRunInterface(binaryInterface, bid);
		ensureFormat(request, runInterface.getExportFormats());

		String[] ridStrings = Routines.split2(",", request.getParameter("rids"));
		RID[] rids = new RID[ridStrings.length];
		for (int i = 0; i < ridStrings.length; i++) {
			try {
				rids[i] = RID.valueOf(ridStrings[i]);
			} catch (NumberFormatException e) {
				params.ensureCondition("rids", false);
			}
		}
		
		try {
			exportRuns(runInterface, aid, eid, bid, rids,
				request.getParameter("format"), response);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eid or bid or rids", false);
		} catch (ResultsRepositoryBusyException e) {
			errorMessages.addTextMessage("Results Repository is currently busy and "
				+ "can't export selected entities. Try waiting for a while and reload "
				+ "the page. If the problem persists, see the logs for explanation.");
		}
	}

	/**
 	 * Handles the "run-delete" action.
 	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws IOException if some I/O error occurs
	 * @throws ServletException if including the template file fails
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	/* Needed for EntityInferfce.getSubentityInterface method. */
	@SuppressWarnings("unchecked") 
	public void runDelete(HttpServletRequest request, HttpServletResponse response)
			throws IOException, MissingParamException, InvalidParamValueException,
			ComponentInitializationException, ServletException {
		testRScriptingEnabled();

		params.ensureExist("aid", "eid", "bid", "rid");
	
		EntityInterface<AID> analysisInterface = resultsRepository.get()
			.getAnalysisInterface();
		AID aid = ensureAndGetAid(request, analysisInterface);
		EntityInterface<EID> experimentInterface = getExperimentInterface(analysisInterface, aid);
		EID eid = ensureAndGetEid(request, experimentInterface);
		EntityInterface<BID> binaryInterface = getBinaryInterface(experimentInterface, eid);
		BID bid = ensureAndGetBid(request, binaryInterface);
		EntityInterface<RID> runInterface = getRunInterface(binaryInterface, bid);
		RID rid = ensureAndGetRid(request, runInterface);
		
		try {
			runInterface.deleteEntity(rid);
		} catch (IllegalArgumentException e) {
			params.ensureCondition("aid or eid or bid or rid", false);
		} catch (DeleteLockedException e) {
			handleDeleteLockedException();
			return;
		}
		
		HashMap<String, String> actionParams = new HashMap<String, String>();
		actionParams.put("aid", aid.toString());
		actionParams.put("eid", eid.toString());
		actionParams.put("bid", bid.toString());
		actionParams.put("action", "deleted");
		page.redirectToAction("binary-details", actionParams);
	}

	/**
	 * Handles the "package-list" action. 
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws IOException if some I/O error occurs
	 * @throws MissingParamException if some required parameter is missing
	 * @throws ServletException if including the template file fails
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	public void packageList(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException, InvalidParamValueException,
			ComponentInitializationException, MissingParamException {
		testRScriptingEnabled();

		if (params.exists("action")) {
			String action = request.getParameter("action");
			if (action.equals("deleted")) {
				infoMessages.addTextMessage("R script package deleted successfully.");
			} else {
				throw new InvalidParamValueException("Parameter \"action\" has invalid value.");
			}
		}

		RScriptPackage[] packages = resultsRepository.get().getRScriptPackages();
		
		HashMap<String, Object> data = new HashMap<String, Object>();
		data.put("packages", packages);

		page.setTitle("R script packages");
		page.writeHeader();
		page.writeTemplate("results-package-list", data);
		page.writeFooter();
	}

	/**
	 * Uploads the package from specified <code>InputStream</code>
	 * to the Results Repository.
	 * 
	 * @param name package name
	 * @param inputStream input stream providing package contents
	 * @return package upload result (value form <code>UploadStatus</code>
	 *          enumeration and error messages)
	 * @throws IOException if some I/O error occurs
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	private UploadResult uploadPackage(String name, InputStream inputStream)
			throws IOException, ComponentInitializationException {
		UploadStatus status = UploadStatus.ACCEPTED;
		String[] errorMessages = new String[0];
	    
		ServerSocket serverSocket = new ServerSocket(0); // 0 = use any port
		UploadHandle handle = resultsRepository.get().beginRScriptPackageUpload(
			name,
			InetAddress.getLocalHost(), 
			serverSocket.getLocalPort()
		);
		byte[] buffer = new byte[ResultsRepositoryInterface.UPLOAD_BUFFER_SIZE];
		int bytesRead;
		Socket socket = serverSocket.accept();
		try {
			OutputStream outputStream = new BufferedOutputStream(
				socket.getOutputStream(),
				ResultsRepositoryInterface.UPLOAD_BUFFER_SIZE
			);
			InputStream bufferedInputStream = new BufferedInputStream(
				inputStream,
				ResultsRepositoryInterface.UPLOAD_BUFFER_SIZE
			);
			try {
				while ((bytesRead = bufferedInputStream.read(buffer)) != -1) {
					outputStream.write(buffer, 0, bytesRead);
				}
			} finally {
				outputStream.close();
				bufferedInputStream.close();
			}
		} finally {
			socket.close();
			serverSocket.close();
	
			/* Wait until the Results Repository finishes its processing and sets some
			 * meaningful state. Finish the upload then.
			 */
			do {
				status = resultsRepository.get().getRScriptPackageUploadStatus(handle);
			} while (status == UploadStatus.UPLOADING || status == UploadStatus.INITIALIZING);
			if (status == UploadStatus.REJECTED) {
				errorMessages = resultsRepository.get().getRejectedRScriptPackageUploadErrorMessages(handle);
			}
			resultsRepository.get().endRScriptPackageUpload(handle);
		}
		return new UploadResult(status, errorMessages);
	}

	/**
	 * Handles the "package-upload" action.
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws MissingParamException if some required parameter is missing
	 * @throws IOException if some I/O error occurs
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ServletException if including the template file fails
	 * @throws ComponentInitializationException if the Software Repository can't
	 *                                          be initialized
	 */
	public void packageUpload(HttpServletRequest request, HttpServletResponse response)
			throws MissingParamException, ServletException, IOException,
			ComponentInitializationException, InvalidParamValueException {	
		testRScriptingEnabled();

		HashMap<String, Object> data = new HashMap<String, Object>();
	
		if (params.exists("action")) {
			String action = request.getParameter("action");
			if (action.equals("uploaded")) {
				infoMessages.addTextMessage("R script package uploaded successfully.");
			} else {
				throw new InvalidParamValueException("Parameter \"action\" has invalid value.");
			}
		}
	
		/* Ideally, the presence of the query parameter "upload" should be checked
		 * here - for consistency with other forms. Unfortunately, when the form
		 * data is sent as "multipart/form-data" (necessary to upload files), Tomcat
		 * doesnt' understand it and leaves the request unparsed => we can't
		 * directly access the query parameters.
		 * 
		 * Workaround is to check the request type - if POSTing, we're uploading,
		 * if GETting, we're only viewing the form.
		 */ 
		if (params.requestMethodIsPost()) {
			DiskFileUpload upload = new DiskFileUpload();
			
			List fileItems = null;
			try {
				fileItems = upload.parseRequest(request);
			} catch (FileUploadException e) {
				errorMessages.addTextMessage("Error uploading R script package:" + e.getMessage());
			}
			
			FileItem fileItem = null;
			if (errorMessages.isEmpty()) {
				/* Index "0" is the index of the field in the upload form. Needs to be
				 * synchronized with /webinterface/templates/results-package-upload.jsp.
				 */
				fileItem = (FileItem) fileItems.get(0);
				params.checkCondition(!fileItem.getName().equals(""), "Enter the package file.");
			}

			if (errorMessages.isEmpty()) {
				InputStream inputStream = fileItem.getInputStream();
				try {
					try {
						UploadResult result = uploadPackage(fileItem.getName(), inputStream);
						switch (result.getStatus()) {
							case REJECTED:
								String message = "<strong>R script package was "
									+ "rejected by the Results Repository. Reported errors:</strong><br /><br />";
								message += "<ul>";
								for (String errorMessage: result.getErrorMessages()) {
									message += "<li>";
									message += Routines.htmlspecialchars(errorMessage);
									message += "</li>";
								}
								message += "</ul>";
								errorMessages.addHTMLMessage(message);
								break;
							case ERROR:
								errorMessages.addTextMessage("Error uploading R script package: Results Repository reports error.");
								break;
							default:
								/* Just pass... */
						}
					} finally {
						inputStream.close();
					}
				} catch (IllegalArgumentException e) {
					/* Most probably the package does already exist. */
					errorMessages.addTextMessage(e.getMessage());
				} catch (IOException e) {
					errorMessages.addTextMessage("Error uploading R script package:" + e.getMessage());
				}
			}
	
			if (errorMessages.isEmpty()) {
				HashMap<String, String> actionParams = new HashMap<String, String>();
				actionParams.put("action", "uploaded");
				page.redirectToAction("package-upload", actionParams);
				return;
			} else {
				data.put("file", fileItem.getName());
			}
		} else {
			data.put("file", "");
		}
		
		page.setTitle("Upload R script package");
		page.setFocusedElement(0, "file");
		page.writeHeader();
		page.writeTemplate("results-package-upload", data);
		page.writeFooter();
	}

	/**
 	 * Handles the "package-delete" action.
 	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws IOException if some I/O error occurs
	 * @throws MissingParamException if some required parameter is missing
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	public void packageDelete(HttpServletRequest request, HttpServletResponse response)
			throws IOException, MissingParamException, InvalidParamValueException,
			ComponentInitializationException {
		testRScriptingEnabled();

		params.ensureExists("package");
		
		resultsRepository.get().deleteRScriptPackage(request.getParameter("package"));
		
		HashMap<String, String> actionParams = new HashMap<String, String>();
		actionParams.put("action", "deleted");
		page.redirectToAction("package-list", actionParams);
	}

	/**
	 * Handles the "r-functions-help" action. 
	 * 
	 * @param request HTTP request
	 * @param response HTTP response
	 * @throws IOException if some I/O error occurs
	 * @throws MissingParamException if some required parameter is missing
	 * @throws ServletException if including the template file fails
	 * @throws InvalidParamValueException if required parameter contains invalid
	 *          value
	 * @throws ComponentInitializationException if the Results Repository can't
	 *                                           be initialized
	 */
	public void rFunctionsHelp(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException, InvalidParamValueException,
			ComponentInitializationException, MissingParamException {
		testRScriptingEnabled();

		RFunctionHelp[] helpItems = resultsRepository.get().getRFunctionsHelp();
		
		HashMap<String, Object> data = new HashMap<String, Object>();
		data.put("helpItems", helpItems);

		page.setTitle("R functions help");
		page.setLayoutType(LayoutType.SIMPLE);
		page.writeHeader();
		page.writeTemplate("results-r-functions-help", data);
		page.writeFooter();
	}

}
